function Gb(a,o){for(var i=0;i<o.length;i++){const c=o[i];if(typeof c!="string"&&!Array.isArray(c)){for(const u in c)if(u!=="default"&&!(u in a)){const f=Object.getOwnPropertyDescriptor(c,u);f&&Object.defineProperty(a,u,f.get?f:{enumerable:!0,get:()=>c[u]})}}}return Object.freeze(Object.defineProperty(a,Symbol.toStringTag,{value:"Module"}))}(function(){const o=document.createElement("link").relList;if(o&&o.supports&&o.supports("modulepreload"))return;for(const u of document.querySelectorAll('link[rel="modulepreload"]'))c(u);new MutationObserver(u=>{for(const f of u)if(f.type==="childList")for(const h of f.addedNodes)h.tagName==="LINK"&&h.rel==="modulepreload"&&c(h)}).observe(document,{childList:!0,subtree:!0});function i(u){const f={};return u.integrity&&(f.integrity=u.integrity),u.referrerPolicy&&(f.referrerPolicy=u.referrerPolicy),u.crossOrigin==="use-credentials"?f.credentials="include":u.crossOrigin==="anonymous"?f.credentials="omit":f.credentials="same-origin",f}function c(u){if(u.ep)return;u.ep=!0;const f=i(u);fetch(u.href,f)}})();function Zv(a){return a&&a.__esModule&&Object.prototype.hasOwnProperty.call(a,"default")?a.default:a}var Ps={exports:{}},Er={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Wh;function Yb(){if(Wh)return Er;Wh=1;var a=Symbol.for("react.transitional.element"),o=Symbol.for("react.fragment");function i(c,u,f){var h=null;if(f!==void 0&&(h=""+f),u.key!==void 0&&(h=""+u.key),"key"in u){f={};for(var v in u)v!=="key"&&(f[v]=u[v])}else f=u;return u=f.ref,{$$typeof:a,type:c,key:h,ref:u!==void 0?u:null,props:f}}return Er.Fragment=o,Er.jsx=i,Er.jsxs=i,Er}var Fh;function Vb(){return Fh||(Fh=1,Ps.exports=Yb()),Ps.exports}var d=Vb(),$s={exports:{}},ge={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Ih;function Xb(){if(Ih)return ge;Ih=1;var a=Symbol.for("react.transitional.element"),o=Symbol.for("react.portal"),i=Symbol.for("react.fragment"),c=Symbol.for("react.strict_mode"),u=Symbol.for("react.profiler"),f=Symbol.for("react.consumer"),h=Symbol.for("react.context"),v=Symbol.for("react.forward_ref"),b=Symbol.for("react.suspense"),g=Symbol.for("react.memo"),p=Symbol.for("react.lazy"),S=Symbol.iterator;function w(A){return A===null||typeof A!="object"?null:(A=S&&A[S]||A["@@iterator"],typeof A=="function"?A:null)}var M={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},T=Object.assign,N={};function _(A,X,F){this.props=A,this.context=X,this.refs=N,this.updater=F||M}_.prototype.isReactComponent={},_.prototype.setState=function(A,X){if(typeof A!="object"&&typeof A!="function"&&A!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,A,X,"setState")},_.prototype.forceUpdate=function(A){this.updater.enqueueForceUpdate(this,A,"forceUpdate")};function j(){}j.prototype=_.prototype;function q(A,X,F){this.props=A,this.context=X,this.refs=N,this.updater=F||M}var L=q.prototype=new j;L.constructor=q,T(L,_.prototype),L.isPureReactComponent=!0;var Y=Array.isArray,B={H:null,A:null,T:null,S:null,V:null},J=Object.prototype.hasOwnProperty;function P(A,X,F,$,ee,he){return F=he.ref,{$$typeof:a,type:A,key:X,ref:F!==void 0?F:null,props:he}}function Z(A,X){return P(A.type,X,void 0,void 0,void 0,A.props)}function I(A){return typeof A=="object"&&A!==null&&A.$$typeof===a}function re(A){var X={"=":"=0",":":"=2"};return"$"+A.replace(/[=:]/g,function(F){return X[F]})}var ue=/\/+/g;function de(A,X){return typeof A=="object"&&A!==null&&A.key!=null?re(""+A.key):X.toString(36)}function Ee(){}function be(A){switch(A.status){case"fulfilled":return A.value;case"rejected":throw A.reason;default:switch(typeof A.status=="string"?A.then(Ee,Ee):(A.status="pending",A.then(function(X){A.status==="pending"&&(A.status="fulfilled",A.value=X)},function(X){A.status==="pending"&&(A.status="rejected",A.reason=X)})),A.status){case"fulfilled":return A.value;case"rejected":throw A.reason}}throw A}function ve(A,X,F,$,ee){var he=typeof A;(he==="undefined"||he==="boolean")&&(A=null);var oe=!1;if(A===null)oe=!0;else switch(he){case"bigint":case"string":case"number":oe=!0;break;case"object":switch(A.$$typeof){case a:case o:oe=!0;break;case p:return oe=A._init,ve(oe(A._payload),X,F,$,ee)}}if(oe)return ee=ee(A),oe=$===""?"."+de(A,0):$,Y(ee)?(F="",oe!=null&&(F=oe.replace(ue,"$&/")+"/"),ve(ee,X,F,"",function(it){return it})):ee!=null&&(I(ee)&&(ee=Z(ee,F+(ee.key==null||A&&A.key===ee.key?"":(""+ee.key).replace(ue,"$&/")+"/")+oe)),X.push(ee)),1;oe=0;var ie=$===""?".":$+":";if(Y(A))for(var Me=0;Me<A.length;Me++)$=A[Me],he=ie+de($,Me),oe+=ve($,X,F,he,ee);else if(Me=w(A),typeof Me=="function")for(A=Me.call(A),Me=0;!($=A.next()).done;)$=$.value,he=ie+de($,Me++),oe+=ve($,X,F,he,ee);else if(he==="object"){if(typeof A.then=="function")return ve(be(A),X,F,$,ee);throw X=String(A),Error("Objects are not valid as a React child (found: "+(X==="[object Object]"?"object with keys {"+Object.keys(A).join(", ")+"}":X)+"). If you meant to render a collection of children, use an array instead.")}return oe}function D(A,X,F){if(A==null)return A;var $=[],ee=0;return ve(A,$,"","",function(he){return X.call(F,he,ee++)}),$}function K(A){if(A._status===-1){var X=A._result;X=X(),X.then(function(F){(A._status===0||A._status===-1)&&(A._status=1,A._result=F)},function(F){(A._status===0||A._status===-1)&&(A._status=2,A._result=F)}),A._status===-1&&(A._status=0,A._result=X)}if(A._status===1)return A._result.default;throw A._result}var H=typeof reportError=="function"?reportError:function(A){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var X=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof A=="object"&&A!==null&&typeof A.message=="string"?String(A.message):String(A),error:A});if(!window.dispatchEvent(X))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",A);return}console.error(A)};function W(){}return ge.Children={map:D,forEach:function(A,X,F){D(A,function(){X.apply(this,arguments)},F)},count:function(A){var X=0;return D(A,function(){X++}),X},toArray:function(A){return D(A,function(X){return X})||[]},only:function(A){if(!I(A))throw Error("React.Children.only expected to receive a single React element child.");return A}},ge.Component=_,ge.Fragment=i,ge.Profiler=u,ge.PureComponent=q,ge.StrictMode=c,ge.Suspense=b,ge.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=B,ge.__COMPILER_RUNTIME={__proto__:null,c:function(A){return B.H.useMemoCache(A)}},ge.cache=function(A){return function(){return A.apply(null,arguments)}},ge.cloneElement=function(A,X,F){if(A==null)throw Error("The argument must be a React element, but you passed "+A+".");var $=T({},A.props),ee=A.key,he=void 0;if(X!=null)for(oe in X.ref!==void 0&&(he=void 0),X.key!==void 0&&(ee=""+X.key),X)!J.call(X,oe)||oe==="key"||oe==="__self"||oe==="__source"||oe==="ref"&&X.ref===void 0||($[oe]=X[oe]);var oe=arguments.length-2;if(oe===1)$.children=F;else if(1<oe){for(var ie=Array(oe),Me=0;Me<oe;Me++)ie[Me]=arguments[Me+2];$.children=ie}return P(A.type,ee,void 0,void 0,he,$)},ge.createContext=function(A){return A={$$typeof:h,_currentValue:A,_currentValue2:A,_threadCount:0,Provider:null,Consumer:null},A.Provider=A,A.Consumer={$$typeof:f,_context:A},A},ge.createElement=function(A,X,F){var $,ee={},he=null;if(X!=null)for($ in X.key!==void 0&&(he=""+X.key),X)J.call(X,$)&&$!=="key"&&$!=="__self"&&$!=="__source"&&(ee[$]=X[$]);var oe=arguments.length-2;if(oe===1)ee.children=F;else if(1<oe){for(var ie=Array(oe),Me=0;Me<oe;Me++)ie[Me]=arguments[Me+2];ee.children=ie}if(A&&A.defaultProps)for($ in oe=A.defaultProps,oe)ee[$]===void 0&&(ee[$]=oe[$]);return P(A,he,void 0,void 0,null,ee)},ge.createRef=function(){return{current:null}},ge.forwardRef=function(A){return{$$typeof:v,render:A}},ge.isValidElement=I,ge.lazy=function(A){return{$$typeof:p,_payload:{_status:-1,_result:A},_init:K}},ge.memo=function(A,X){return{$$typeof:g,type:A,compare:X===void 0?null:X}},ge.startTransition=function(A){var X=B.T,F={};B.T=F;try{var $=A(),ee=B.S;ee!==null&&ee(F,$),typeof $=="object"&&$!==null&&typeof $.then=="function"&&$.then(W,H)}catch(he){H(he)}finally{B.T=X}},ge.unstable_useCacheRefresh=function(){return B.H.useCacheRefresh()},ge.use=function(A){return B.H.use(A)},ge.useActionState=function(A,X,F){return B.H.useActionState(A,X,F)},ge.useCallback=function(A,X){return B.H.useCallback(A,X)},ge.useContext=function(A){return B.H.useContext(A)},ge.useDebugValue=function(){},ge.useDeferredValue=function(A,X){return B.H.useDeferredValue(A,X)},ge.useEffect=function(A,X,F){var $=B.H;if(typeof F=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return $.useEffect(A,X)},ge.useId=function(){return B.H.useId()},ge.useImperativeHandle=function(A,X,F){return B.H.useImperativeHandle(A,X,F)},ge.useInsertionEffect=function(A,X){return B.H.useInsertionEffect(A,X)},ge.useLayoutEffect=function(A,X){return B.H.useLayoutEffect(A,X)},ge.useMemo=function(A,X){return B.H.useMemo(A,X)},ge.useOptimistic=function(A,X){return B.H.useOptimistic(A,X)},ge.useReducer=function(A,X,F){return B.H.useReducer(A,X,F)},ge.useRef=function(A){return B.H.useRef(A)},ge.useState=function(A){return B.H.useState(A)},ge.useSyncExternalStore=function(A,X,F){return B.H.useSyncExternalStore(A,X,F)},ge.useTransition=function(){return B.H.useTransition()},ge.version="19.1.1",ge}var ev;function ju(){return ev||(ev=1,$s.exports=Xb()),$s.exports}var x=ju();const el=Zv(x),Kv=Gb({__proto__:null,default:el},[x]);var Js={exports:{}},Ar={},Ws={exports:{}},Fs={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var tv;function Qb(){return tv||(tv=1,(function(a){function o(D,K){var H=D.length;D.push(K);e:for(;0<H;){var W=H-1>>>1,A=D[W];if(0<u(A,K))D[W]=K,D[H]=A,H=W;else break e}}function i(D){return D.length===0?null:D[0]}function c(D){if(D.length===0)return null;var K=D[0],H=D.pop();if(H!==K){D[0]=H;e:for(var W=0,A=D.length,X=A>>>1;W<X;){var F=2*(W+1)-1,$=D[F],ee=F+1,he=D[ee];if(0>u($,H))ee<A&&0>u(he,$)?(D[W]=he,D[ee]=H,W=ee):(D[W]=$,D[F]=H,W=F);else if(ee<A&&0>u(he,H))D[W]=he,D[ee]=H,W=ee;else break e}}return K}function u(D,K){var H=D.sortIndex-K.sortIndex;return H!==0?H:D.id-K.id}if(a.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var f=performance;a.unstable_now=function(){return f.now()}}else{var h=Date,v=h.now();a.unstable_now=function(){return h.now()-v}}var b=[],g=[],p=1,S=null,w=3,M=!1,T=!1,N=!1,_=!1,j=typeof setTimeout=="function"?setTimeout:null,q=typeof clearTimeout=="function"?clearTimeout:null,L=typeof setImmediate<"u"?setImmediate:null;function Y(D){for(var K=i(g);K!==null;){if(K.callback===null)c(g);else if(K.startTime<=D)c(g),K.sortIndex=K.expirationTime,o(b,K);else break;K=i(g)}}function B(D){if(N=!1,Y(D),!T)if(i(b)!==null)T=!0,J||(J=!0,de());else{var K=i(g);K!==null&&ve(B,K.startTime-D)}}var J=!1,P=-1,Z=5,I=-1;function re(){return _?!0:!(a.unstable_now()-I<Z)}function ue(){if(_=!1,J){var D=a.unstable_now();I=D;var K=!0;try{e:{T=!1,N&&(N=!1,q(P),P=-1),M=!0;var H=w;try{t:{for(Y(D),S=i(b);S!==null&&!(S.expirationTime>D&&re());){var W=S.callback;if(typeof W=="function"){S.callback=null,w=S.priorityLevel;var A=W(S.expirationTime<=D);if(D=a.unstable_now(),typeof A=="function"){S.callback=A,Y(D),K=!0;break t}S===i(b)&&c(b),Y(D)}else c(b);S=i(b)}if(S!==null)K=!0;else{var X=i(g);X!==null&&ve(B,X.startTime-D),K=!1}}break e}finally{S=null,w=H,M=!1}K=void 0}}finally{K?de():J=!1}}}var de;if(typeof L=="function")de=function(){L(ue)};else if(typeof MessageChannel<"u"){var Ee=new MessageChannel,be=Ee.port2;Ee.port1.onmessage=ue,de=function(){be.postMessage(null)}}else de=function(){j(ue,0)};function ve(D,K){P=j(function(){D(a.unstable_now())},K)}a.unstable_IdlePriority=5,a.unstable_ImmediatePriority=1,a.unstable_LowPriority=4,a.unstable_NormalPriority=3,a.unstable_Profiling=null,a.unstable_UserBlockingPriority=2,a.unstable_cancelCallback=function(D){D.callback=null},a.unstable_forceFrameRate=function(D){0>D||125<D?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):Z=0<D?Math.floor(1e3/D):5},a.unstable_getCurrentPriorityLevel=function(){return w},a.unstable_next=function(D){switch(w){case 1:case 2:case 3:var K=3;break;default:K=w}var H=w;w=K;try{return D()}finally{w=H}},a.unstable_requestPaint=function(){_=!0},a.unstable_runWithPriority=function(D,K){switch(D){case 1:case 2:case 3:case 4:case 5:break;default:D=3}var H=w;w=D;try{return K()}finally{w=H}},a.unstable_scheduleCallback=function(D,K,H){var W=a.unstable_now();switch(typeof H=="object"&&H!==null?(H=H.delay,H=typeof H=="number"&&0<H?W+H:W):H=W,D){case 1:var A=-1;break;case 2:A=250;break;case 5:A=1073741823;break;case 4:A=1e4;break;default:A=5e3}return A=H+A,D={id:p++,callback:K,priorityLevel:D,startTime:H,expirationTime:A,sortIndex:-1},H>W?(D.sortIndex=H,o(g,D),i(b)===null&&D===i(g)&&(N?(q(P),P=-1):N=!0,ve(B,H-W))):(D.sortIndex=A,o(b,D),T||M||(T=!0,J||(J=!0,de()))),D},a.unstable_shouldYield=re,a.unstable_wrapCallback=function(D){var K=w;return function(){var H=w;w=K;try{return D.apply(this,arguments)}finally{w=H}}}})(Fs)),Fs}var nv;function Zb(){return nv||(nv=1,Ws.exports=Qb()),Ws.exports}var Is={exports:{}},rt={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var lv;function Kb(){if(lv)return rt;lv=1;var a=ju();function o(b){var g="https://react.dev/errors/"+b;if(1<arguments.length){g+="?args[]="+encodeURIComponent(arguments[1]);for(var p=2;p<arguments.length;p++)g+="&args[]="+encodeURIComponent(arguments[p])}return"Minified React error #"+b+"; visit "+g+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function i(){}var c={d:{f:i,r:function(){throw Error(o(522))},D:i,C:i,L:i,m:i,X:i,S:i,M:i},p:0,findDOMNode:null},u=Symbol.for("react.portal");function f(b,g,p){var S=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:u,key:S==null?null:""+S,children:b,containerInfo:g,implementation:p}}var h=a.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function v(b,g){if(b==="font")return"";if(typeof g=="string")return g==="use-credentials"?g:""}return rt.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=c,rt.createPortal=function(b,g){var p=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!g||g.nodeType!==1&&g.nodeType!==9&&g.nodeType!==11)throw Error(o(299));return f(b,g,null,p)},rt.flushSync=function(b){var g=h.T,p=c.p;try{if(h.T=null,c.p=2,b)return b()}finally{h.T=g,c.p=p,c.d.f()}},rt.preconnect=function(b,g){typeof b=="string"&&(g?(g=g.crossOrigin,g=typeof g=="string"?g==="use-credentials"?g:"":void 0):g=null,c.d.C(b,g))},rt.prefetchDNS=function(b){typeof b=="string"&&c.d.D(b)},rt.preinit=function(b,g){if(typeof b=="string"&&g&&typeof g.as=="string"){var p=g.as,S=v(p,g.crossOrigin),w=typeof g.integrity=="string"?g.integrity:void 0,M=typeof g.fetchPriority=="string"?g.fetchPriority:void 0;p==="style"?c.d.S(b,typeof g.precedence=="string"?g.precedence:void 0,{crossOrigin:S,integrity:w,fetchPriority:M}):p==="script"&&c.d.X(b,{crossOrigin:S,integrity:w,fetchPriority:M,nonce:typeof g.nonce=="string"?g.nonce:void 0})}},rt.preinitModule=function(b,g){if(typeof b=="string")if(typeof g=="object"&&g!==null){if(g.as==null||g.as==="script"){var p=v(g.as,g.crossOrigin);c.d.M(b,{crossOrigin:p,integrity:typeof g.integrity=="string"?g.integrity:void 0,nonce:typeof g.nonce=="string"?g.nonce:void 0})}}else g==null&&c.d.M(b)},rt.preload=function(b,g){if(typeof b=="string"&&typeof g=="object"&&g!==null&&typeof g.as=="string"){var p=g.as,S=v(p,g.crossOrigin);c.d.L(b,p,{crossOrigin:S,integrity:typeof g.integrity=="string"?g.integrity:void 0,nonce:typeof g.nonce=="string"?g.nonce:void 0,type:typeof g.type=="string"?g.type:void 0,fetchPriority:typeof g.fetchPriority=="string"?g.fetchPriority:void 0,referrerPolicy:typeof g.referrerPolicy=="string"?g.referrerPolicy:void 0,imageSrcSet:typeof g.imageSrcSet=="string"?g.imageSrcSet:void 0,imageSizes:typeof g.imageSizes=="string"?g.imageSizes:void 0,media:typeof g.media=="string"?g.media:void 0})}},rt.preloadModule=function(b,g){if(typeof b=="string")if(g){var p=v(g.as,g.crossOrigin);c.d.m(b,{as:typeof g.as=="string"&&g.as!=="script"?g.as:void 0,crossOrigin:p,integrity:typeof g.integrity=="string"?g.integrity:void 0})}else c.d.m(b)},rt.requestFormReset=function(b){c.d.r(b)},rt.unstable_batchedUpdates=function(b,g){return b(g)},rt.useFormState=function(b,g,p){return h.H.useFormState(b,g,p)},rt.useFormStatus=function(){return h.H.useHostTransitionStatus()},rt.version="19.1.1",rt}var av;function Pv(){if(av)return Is.exports;av=1;function a(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(a)}catch(o){console.error(o)}}return a(),Is.exports=Kb(),Is.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var rv;function Pb(){if(rv)return Ar;rv=1;var a=Zb(),o=ju(),i=Pv();function c(e){var t="https://react.dev/errors/"+e;if(1<arguments.length){t+="?args[]="+encodeURIComponent(arguments[1]);for(var n=2;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n])}return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function u(e){return!(!e||e.nodeType!==1&&e.nodeType!==9&&e.nodeType!==11)}function f(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do t=e,(t.flags&4098)!==0&&(n=t.return),e=t.return;while(e)}return t.tag===3?n:null}function h(e){if(e.tag===13){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function v(e){if(f(e)!==e)throw Error(c(188))}function b(e){var t=e.alternate;if(!t){if(t=f(e),t===null)throw Error(c(188));return t!==e?null:e}for(var n=e,l=t;;){var r=n.return;if(r===null)break;var s=r.alternate;if(s===null){if(l=r.return,l!==null){n=l;continue}break}if(r.child===s.child){for(s=r.child;s;){if(s===n)return v(r),e;if(s===l)return v(r),t;s=s.sibling}throw Error(c(188))}if(n.return!==l.return)n=r,l=s;else{for(var m=!1,y=r.child;y;){if(y===n){m=!0,n=r,l=s;break}if(y===l){m=!0,l=r,n=s;break}y=y.sibling}if(!m){for(y=s.child;y;){if(y===n){m=!0,n=s,l=r;break}if(y===l){m=!0,l=s,n=r;break}y=y.sibling}if(!m)throw Error(c(189))}}if(n.alternate!==l)throw Error(c(190))}if(n.tag!==3)throw Error(c(188));return n.stateNode.current===n?e:t}function g(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e;for(e=e.child;e!==null;){if(t=g(e),t!==null)return t;e=e.sibling}return null}var p=Object.assign,S=Symbol.for("react.element"),w=Symbol.for("react.transitional.element"),M=Symbol.for("react.portal"),T=Symbol.for("react.fragment"),N=Symbol.for("react.strict_mode"),_=Symbol.for("react.profiler"),j=Symbol.for("react.provider"),q=Symbol.for("react.consumer"),L=Symbol.for("react.context"),Y=Symbol.for("react.forward_ref"),B=Symbol.for("react.suspense"),J=Symbol.for("react.suspense_list"),P=Symbol.for("react.memo"),Z=Symbol.for("react.lazy"),I=Symbol.for("react.activity"),re=Symbol.for("react.memo_cache_sentinel"),ue=Symbol.iterator;function de(e){return e===null||typeof e!="object"?null:(e=ue&&e[ue]||e["@@iterator"],typeof e=="function"?e:null)}var Ee=Symbol.for("react.client.reference");function be(e){if(e==null)return null;if(typeof e=="function")return e.$$typeof===Ee?null:e.displayName||e.name||null;if(typeof e=="string")return e;switch(e){case T:return"Fragment";case _:return"Profiler";case N:return"StrictMode";case B:return"Suspense";case J:return"SuspenseList";case I:return"Activity"}if(typeof e=="object")switch(e.$$typeof){case M:return"Portal";case L:return(e.displayName||"Context")+".Provider";case q:return(e._context.displayName||"Context")+".Consumer";case Y:var t=e.render;return e=e.displayName,e||(e=t.displayName||t.name||"",e=e!==""?"ForwardRef("+e+")":"ForwardRef"),e;case P:return t=e.displayName||null,t!==null?t:be(e.type)||"Memo";case Z:t=e._payload,e=e._init;try{return be(e(t))}catch{}}return null}var ve=Array.isArray,D=o.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,K=i.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,H={pending:!1,data:null,method:null,action:null},W=[],A=-1;function X(e){return{current:e}}function F(e){0>A||(e.current=W[A],W[A]=null,A--)}function $(e,t){A++,W[A]=e.current,e.current=t}var ee=X(null),he=X(null),oe=X(null),ie=X(null);function Me(e,t){switch($(oe,t),$(he,e),$(ee,null),t.nodeType){case 9:case 11:e=(e=t.documentElement)&&(e=e.namespaceURI)?Mh(e):0;break;default:if(e=t.tagName,t=t.namespaceURI)t=Mh(t),e=Nh(t,e);else switch(e){case"svg":e=1;break;case"math":e=2;break;default:e=0}}F(ee),$(ee,e)}function it(){F(ee),F(he),F(oe)}function Qt(e){e.memoizedState!==null&&$(ie,e);var t=ee.current,n=Nh(t,e.type);t!==n&&($(he,e),$(ee,n))}function Zt(e){he.current===e&&(F(ee),F(he)),ie.current===e&&(F(ie),yr._currentValue=H)}var Kt=Object.prototype.hasOwnProperty,Rn=a.unstable_scheduleCallback,ki=a.unstable_cancelCallback,by=a.unstable_shouldYield,xy=a.unstable_requestPaint,Pt=a.unstable_now,Sy=a.unstable_getCurrentPriorityLevel,af=a.unstable_ImmediatePriority,rf=a.unstable_UserBlockingPriority,Br=a.unstable_NormalPriority,wy=a.unstable_LowPriority,of=a.unstable_IdlePriority,Ey=a.log,Ay=a.unstable_setDisableYieldValue,Na=null,vt=null;function _n(e){if(typeof Ey=="function"&&Ay(e),vt&&typeof vt.setStrictMode=="function")try{vt.setStrictMode(Na,e)}catch{}}var gt=Math.clz32?Math.clz32:Ty,My=Math.log,Ny=Math.LN2;function Ty(e){return e>>>=0,e===0?32:31-(My(e)/Ny|0)|0}var qr=256,Gr=4194304;function al(e){var t=e&42;if(t!==0)return t;switch(e&-e){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return e&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return e&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return e}}function Yr(e,t,n){var l=e.pendingLanes;if(l===0)return 0;var r=0,s=e.suspendedLanes,m=e.pingedLanes;e=e.warmLanes;var y=l&134217727;return y!==0?(l=y&~s,l!==0?r=al(l):(m&=y,m!==0?r=al(m):n||(n=y&~e,n!==0&&(r=al(n))))):(y=l&~s,y!==0?r=al(y):m!==0?r=al(m):n||(n=l&~e,n!==0&&(r=al(n)))),r===0?0:t!==0&&t!==r&&(t&s)===0&&(s=r&-r,n=t&-t,s>=n||s===32&&(n&4194048)!==0)?t:r}function Ta(e,t){return(e.pendingLanes&~(e.suspendedLanes&~e.pingedLanes)&t)===0}function Ry(e,t){switch(e){case 1:case 2:case 4:case 8:case 64:return t+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function cf(){var e=qr;return qr<<=1,(qr&4194048)===0&&(qr=256),e}function sf(){var e=Gr;return Gr<<=1,(Gr&62914560)===0&&(Gr=4194304),e}function Li(e){for(var t=[],n=0;31>n;n++)t.push(e);return t}function Ra(e,t){e.pendingLanes|=t,t!==268435456&&(e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0)}function _y(e,t,n,l,r,s){var m=e.pendingLanes;e.pendingLanes=n,e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0,e.expiredLanes&=n,e.entangledLanes&=n,e.errorRecoveryDisabledLanes&=n,e.shellSuspendCounter=0;var y=e.entanglements,E=e.expirationTimes,z=e.hiddenUpdates;for(n=m&~n;0<n;){var G=31-gt(n),Q=1<<G;y[G]=0,E[G]=-1;var U=z[G];if(U!==null)for(z[G]=null,G=0;G<U.length;G++){var k=U[G];k!==null&&(k.lane&=-536870913)}n&=~Q}l!==0&&uf(e,l,0),s!==0&&r===0&&e.tag!==0&&(e.suspendedLanes|=s&~(m&~t))}function uf(e,t,n){e.pendingLanes|=t,e.suspendedLanes&=~t;var l=31-gt(t);e.entangledLanes|=t,e.entanglements[l]=e.entanglements[l]|1073741824|n&4194090}function ff(e,t){var n=e.entangledLanes|=t;for(e=e.entanglements;n;){var l=31-gt(n),r=1<<l;r&t|e[l]&t&&(e[l]|=t),n&=~r}}function Hi(e){switch(e){case 2:e=1;break;case 8:e=4;break;case 32:e=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:e=128;break;case 268435456:e=134217728;break;default:e=0}return e}function Bi(e){return e&=-e,2<e?8<e?(e&134217727)!==0?32:268435456:8:2}function df(){var e=K.p;return e!==0?e:(e=window.event,e===void 0?32:Qh(e.type))}function Cy(e,t){var n=K.p;try{return K.p=e,t()}finally{K.p=n}}var Cn=Math.random().toString(36).slice(2),lt="__reactFiber$"+Cn,ut="__reactProps$"+Cn,Rl="__reactContainer$"+Cn,qi="__reactEvents$"+Cn,jy="__reactListeners$"+Cn,Oy="__reactHandles$"+Cn,mf="__reactResources$"+Cn,_a="__reactMarker$"+Cn;function Gi(e){delete e[lt],delete e[ut],delete e[qi],delete e[jy],delete e[Oy]}function _l(e){var t=e[lt];if(t)return t;for(var n=e.parentNode;n;){if(t=n[Rl]||n[lt]){if(n=t.alternate,t.child!==null||n!==null&&n.child!==null)for(e=Ch(e);e!==null;){if(n=e[lt])return n;e=Ch(e)}return t}e=n,n=e.parentNode}return null}function Cl(e){if(e=e[lt]||e[Rl]){var t=e.tag;if(t===5||t===6||t===13||t===26||t===27||t===3)return e}return null}function Ca(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e.stateNode;throw Error(c(33))}function jl(e){var t=e[mf];return t||(t=e[mf]={hoistableStyles:new Map,hoistableScripts:new Map}),t}function Je(e){e[_a]=!0}var hf=new Set,vf={};function rl(e,t){Ol(e,t),Ol(e+"Capture",t)}function Ol(e,t){for(vf[e]=t,e=0;e<t.length;e++)hf.add(t[e])}var Dy=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),gf={},pf={};function zy(e){return Kt.call(pf,e)?!0:Kt.call(gf,e)?!1:Dy.test(e)?pf[e]=!0:(gf[e]=!0,!1)}function Vr(e,t,n){if(zy(t))if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":e.removeAttribute(t);return;case"boolean":var l=t.toLowerCase().slice(0,5);if(l!=="data-"&&l!=="aria-"){e.removeAttribute(t);return}}e.setAttribute(t,""+n)}}function Xr(e,t,n){if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(t);return}e.setAttribute(t,""+n)}}function cn(e,t,n,l){if(l===null)e.removeAttribute(n);else{switch(typeof l){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(n);return}e.setAttributeNS(t,n,""+l)}}var Yi,yf;function Dl(e){if(Yi===void 0)try{throw Error()}catch(n){var t=n.stack.trim().match(/\n( *(at )?)/);Yi=t&&t[1]||"",yf=-1<n.stack.indexOf(`
    at`)?" (<anonymous>)":-1<n.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Yi+e+yf}var Vi=!1;function Xi(e,t){if(!e||Vi)return"";Vi=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var l={DetermineComponentFrameRoot:function(){try{if(t){var Q=function(){throw Error()};if(Object.defineProperty(Q.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(Q,[])}catch(k){var U=k}Reflect.construct(e,[],Q)}else{try{Q.call()}catch(k){U=k}e.call(Q.prototype)}}else{try{throw Error()}catch(k){U=k}(Q=e())&&typeof Q.catch=="function"&&Q.catch(function(){})}}catch(k){if(k&&U&&typeof k.stack=="string")return[k.stack,U.stack]}return[null,null]}};l.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var r=Object.getOwnPropertyDescriptor(l.DetermineComponentFrameRoot,"name");r&&r.configurable&&Object.defineProperty(l.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var s=l.DetermineComponentFrameRoot(),m=s[0],y=s[1];if(m&&y){var E=m.split(`
`),z=y.split(`
`);for(r=l=0;l<E.length&&!E[l].includes("DetermineComponentFrameRoot");)l++;for(;r<z.length&&!z[r].includes("DetermineComponentFrameRoot");)r++;if(l===E.length||r===z.length)for(l=E.length-1,r=z.length-1;1<=l&&0<=r&&E[l]!==z[r];)r--;for(;1<=l&&0<=r;l--,r--)if(E[l]!==z[r]){if(l!==1||r!==1)do if(l--,r--,0>r||E[l]!==z[r]){var G=`
`+E[l].replace(" at new "," at ");return e.displayName&&G.includes("<anonymous>")&&(G=G.replace("<anonymous>",e.displayName)),G}while(1<=l&&0<=r);break}}}finally{Vi=!1,Error.prepareStackTrace=n}return(n=e?e.displayName||e.name:"")?Dl(n):""}function Uy(e){switch(e.tag){case 26:case 27:case 5:return Dl(e.type);case 16:return Dl("Lazy");case 13:return Dl("Suspense");case 19:return Dl("SuspenseList");case 0:case 15:return Xi(e.type,!1);case 11:return Xi(e.type.render,!1);case 1:return Xi(e.type,!0);case 31:return Dl("Activity");default:return""}}function bf(e){try{var t="";do t+=Uy(e),e=e.return;while(e);return t}catch(n){return`
Error generating stack: `+n.message+`
`+n.stack}}function _t(e){switch(typeof e){case"bigint":case"boolean":case"number":case"string":case"undefined":return e;case"object":return e;default:return""}}function xf(e){var t=e.type;return(e=e.nodeName)&&e.toLowerCase()==="input"&&(t==="checkbox"||t==="radio")}function ky(e){var t=xf(e)?"checked":"value",n=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),l=""+e[t];if(!e.hasOwnProperty(t)&&typeof n<"u"&&typeof n.get=="function"&&typeof n.set=="function"){var r=n.get,s=n.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return r.call(this)},set:function(m){l=""+m,s.call(this,m)}}),Object.defineProperty(e,t,{enumerable:n.enumerable}),{getValue:function(){return l},setValue:function(m){l=""+m},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}function Qr(e){e._valueTracker||(e._valueTracker=ky(e))}function Sf(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),l="";return e&&(l=xf(e)?e.checked?"true":"false":e.value),e=l,e!==n?(t.setValue(e),!0):!1}function Zr(e){if(e=e||(typeof document<"u"?document:void 0),typeof e>"u")return null;try{return e.activeElement||e.body}catch{return e.body}}var Ly=/[\n"\\]/g;function Ct(e){return e.replace(Ly,function(t){return"\\"+t.charCodeAt(0).toString(16)+" "})}function Qi(e,t,n,l,r,s,m,y){e.name="",m!=null&&typeof m!="function"&&typeof m!="symbol"&&typeof m!="boolean"?e.type=m:e.removeAttribute("type"),t!=null?m==="number"?(t===0&&e.value===""||e.value!=t)&&(e.value=""+_t(t)):e.value!==""+_t(t)&&(e.value=""+_t(t)):m!=="submit"&&m!=="reset"||e.removeAttribute("value"),t!=null?Zi(e,m,_t(t)):n!=null?Zi(e,m,_t(n)):l!=null&&e.removeAttribute("value"),r==null&&s!=null&&(e.defaultChecked=!!s),r!=null&&(e.checked=r&&typeof r!="function"&&typeof r!="symbol"),y!=null&&typeof y!="function"&&typeof y!="symbol"&&typeof y!="boolean"?e.name=""+_t(y):e.removeAttribute("name")}function wf(e,t,n,l,r,s,m,y){if(s!=null&&typeof s!="function"&&typeof s!="symbol"&&typeof s!="boolean"&&(e.type=s),t!=null||n!=null){if(!(s!=="submit"&&s!=="reset"||t!=null))return;n=n!=null?""+_t(n):"",t=t!=null?""+_t(t):n,y||t===e.value||(e.value=t),e.defaultValue=t}l=l??r,l=typeof l!="function"&&typeof l!="symbol"&&!!l,e.checked=y?e.checked:!!l,e.defaultChecked=!!l,m!=null&&typeof m!="function"&&typeof m!="symbol"&&typeof m!="boolean"&&(e.name=m)}function Zi(e,t,n){t==="number"&&Zr(e.ownerDocument)===e||e.defaultValue===""+n||(e.defaultValue=""+n)}function zl(e,t,n,l){if(e=e.options,t){t={};for(var r=0;r<n.length;r++)t["$"+n[r]]=!0;for(n=0;n<e.length;n++)r=t.hasOwnProperty("$"+e[n].value),e[n].selected!==r&&(e[n].selected=r),r&&l&&(e[n].defaultSelected=!0)}else{for(n=""+_t(n),t=null,r=0;r<e.length;r++){if(e[r].value===n){e[r].selected=!0,l&&(e[r].defaultSelected=!0);return}t!==null||e[r].disabled||(t=e[r])}t!==null&&(t.selected=!0)}}function Ef(e,t,n){if(t!=null&&(t=""+_t(t),t!==e.value&&(e.value=t),n==null)){e.defaultValue!==t&&(e.defaultValue=t);return}e.defaultValue=n!=null?""+_t(n):""}function Af(e,t,n,l){if(t==null){if(l!=null){if(n!=null)throw Error(c(92));if(ve(l)){if(1<l.length)throw Error(c(93));l=l[0]}n=l}n==null&&(n=""),t=n}n=_t(t),e.defaultValue=n,l=e.textContent,l===n&&l!==""&&l!==null&&(e.value=l)}function Ul(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&n.nodeType===3){n.nodeValue=t;return}}e.textContent=t}var Hy=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function Mf(e,t,n){var l=t.indexOf("--")===0;n==null||typeof n=="boolean"||n===""?l?e.setProperty(t,""):t==="float"?e.cssFloat="":e[t]="":l?e.setProperty(t,n):typeof n!="number"||n===0||Hy.has(t)?t==="float"?e.cssFloat=n:e[t]=(""+n).trim():e[t]=n+"px"}function Nf(e,t,n){if(t!=null&&typeof t!="object")throw Error(c(62));if(e=e.style,n!=null){for(var l in n)!n.hasOwnProperty(l)||t!=null&&t.hasOwnProperty(l)||(l.indexOf("--")===0?e.setProperty(l,""):l==="float"?e.cssFloat="":e[l]="");for(var r in t)l=t[r],t.hasOwnProperty(r)&&n[r]!==l&&Mf(e,r,l)}else for(var s in t)t.hasOwnProperty(s)&&Mf(e,s,t[s])}function Ki(e){if(e.indexOf("-")===-1)return!1;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var By=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),qy=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function Kr(e){return qy.test(""+e)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":e}var Pi=null;function $i(e){return e=e.target||e.srcElement||window,e.correspondingUseElement&&(e=e.correspondingUseElement),e.nodeType===3?e.parentNode:e}var kl=null,Ll=null;function Tf(e){var t=Cl(e);if(t&&(e=t.stateNode)){var n=e[ut]||null;e:switch(e=t.stateNode,t.type){case"input":if(Qi(e,n.value,n.defaultValue,n.defaultValue,n.checked,n.defaultChecked,n.type,n.name),t=n.name,n.type==="radio"&&t!=null){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll('input[name="'+Ct(""+t)+'"][type="radio"]'),t=0;t<n.length;t++){var l=n[t];if(l!==e&&l.form===e.form){var r=l[ut]||null;if(!r)throw Error(c(90));Qi(l,r.value,r.defaultValue,r.defaultValue,r.checked,r.defaultChecked,r.type,r.name)}}for(t=0;t<n.length;t++)l=n[t],l.form===e.form&&Sf(l)}break e;case"textarea":Ef(e,n.value,n.defaultValue);break e;case"select":t=n.value,t!=null&&zl(e,!!n.multiple,t,!1)}}}var Ji=!1;function Rf(e,t,n){if(Ji)return e(t,n);Ji=!0;try{var l=e(t);return l}finally{if(Ji=!1,(kl!==null||Ll!==null)&&(Do(),kl&&(t=kl,e=Ll,Ll=kl=null,Tf(t),e)))for(t=0;t<e.length;t++)Tf(e[t])}}function ja(e,t){var n=e.stateNode;if(n===null)return null;var l=n[ut]||null;if(l===null)return null;n=l[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(l=!l.disabled)||(e=e.type,l=!(e==="button"||e==="input"||e==="select"||e==="textarea")),e=!l;break e;default:e=!1}if(e)return null;if(n&&typeof n!="function")throw Error(c(231,t,typeof n));return n}var sn=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),Wi=!1;if(sn)try{var Oa={};Object.defineProperty(Oa,"passive",{get:function(){Wi=!0}}),window.addEventListener("test",Oa,Oa),window.removeEventListener("test",Oa,Oa)}catch{Wi=!1}var jn=null,Fi=null,Pr=null;function _f(){if(Pr)return Pr;var e,t=Fi,n=t.length,l,r="value"in jn?jn.value:jn.textContent,s=r.length;for(e=0;e<n&&t[e]===r[e];e++);var m=n-e;for(l=1;l<=m&&t[n-l]===r[s-l];l++);return Pr=r.slice(e,1<l?1-l:void 0)}function $r(e){var t=e.keyCode;return"charCode"in e?(e=e.charCode,e===0&&t===13&&(e=13)):e=t,e===10&&(e=13),32<=e||e===13?e:0}function Jr(){return!0}function Cf(){return!1}function ft(e){function t(n,l,r,s,m){this._reactName=n,this._targetInst=r,this.type=l,this.nativeEvent=s,this.target=m,this.currentTarget=null;for(var y in e)e.hasOwnProperty(y)&&(n=e[y],this[y]=n?n(s):s[y]);return this.isDefaultPrevented=(s.defaultPrevented!=null?s.defaultPrevented:s.returnValue===!1)?Jr:Cf,this.isPropagationStopped=Cf,this}return p(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var n=this.nativeEvent;n&&(n.preventDefault?n.preventDefault():typeof n.returnValue!="unknown"&&(n.returnValue=!1),this.isDefaultPrevented=Jr)},stopPropagation:function(){var n=this.nativeEvent;n&&(n.stopPropagation?n.stopPropagation():typeof n.cancelBubble!="unknown"&&(n.cancelBubble=!0),this.isPropagationStopped=Jr)},persist:function(){},isPersistent:Jr}),t}var ol={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},Wr=ft(ol),Da=p({},ol,{view:0,detail:0}),Gy=ft(Da),Ii,ec,za,Fr=p({},Da,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:nc,button:0,buttons:0,relatedTarget:function(e){return e.relatedTarget===void 0?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==za&&(za&&e.type==="mousemove"?(Ii=e.screenX-za.screenX,ec=e.screenY-za.screenY):ec=Ii=0,za=e),Ii)},movementY:function(e){return"movementY"in e?e.movementY:ec}}),jf=ft(Fr),Yy=p({},Fr,{dataTransfer:0}),Vy=ft(Yy),Xy=p({},Da,{relatedTarget:0}),tc=ft(Xy),Qy=p({},ol,{animationName:0,elapsedTime:0,pseudoElement:0}),Zy=ft(Qy),Ky=p({},ol,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),Py=ft(Ky),$y=p({},ol,{data:0}),Of=ft($y),Jy={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},Wy={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},Fy={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function Iy(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):(e=Fy[e])?!!t[e]:!1}function nc(){return Iy}var e0=p({},Da,{key:function(e){if(e.key){var t=Jy[e.key]||e.key;if(t!=="Unidentified")return t}return e.type==="keypress"?(e=$r(e),e===13?"Enter":String.fromCharCode(e)):e.type==="keydown"||e.type==="keyup"?Wy[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:nc,charCode:function(e){return e.type==="keypress"?$r(e):0},keyCode:function(e){return e.type==="keydown"||e.type==="keyup"?e.keyCode:0},which:function(e){return e.type==="keypress"?$r(e):e.type==="keydown"||e.type==="keyup"?e.keyCode:0}}),t0=ft(e0),n0=p({},Fr,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),Df=ft(n0),l0=p({},Da,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:nc}),a0=ft(l0),r0=p({},ol,{propertyName:0,elapsedTime:0,pseudoElement:0}),o0=ft(r0),i0=p({},Fr,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0}),c0=ft(i0),s0=p({},ol,{newState:0,oldState:0}),u0=ft(s0),f0=[9,13,27,32],lc=sn&&"CompositionEvent"in window,Ua=null;sn&&"documentMode"in document&&(Ua=document.documentMode);var d0=sn&&"TextEvent"in window&&!Ua,zf=sn&&(!lc||Ua&&8<Ua&&11>=Ua),Uf=" ",kf=!1;function Lf(e,t){switch(e){case"keyup":return f0.indexOf(t.keyCode)!==-1;case"keydown":return t.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function Hf(e){return e=e.detail,typeof e=="object"&&"data"in e?e.data:null}var Hl=!1;function m0(e,t){switch(e){case"compositionend":return Hf(t);case"keypress":return t.which!==32?null:(kf=!0,Uf);case"textInput":return e=t.data,e===Uf&&kf?null:e;default:return null}}function h0(e,t){if(Hl)return e==="compositionend"||!lc&&Lf(e,t)?(e=_f(),Pr=Fi=jn=null,Hl=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return zf&&t.locale!=="ko"?null:t.data;default:return null}}var v0={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function Bf(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t==="input"?!!v0[e.type]:t==="textarea"}function qf(e,t,n,l){kl?Ll?Ll.push(l):Ll=[l]:kl=l,t=Bo(t,"onChange"),0<t.length&&(n=new Wr("onChange","change",null,n,l),e.push({event:n,listeners:t}))}var ka=null,La=null;function g0(e){xh(e,0)}function Ir(e){var t=Ca(e);if(Sf(t))return e}function Gf(e,t){if(e==="change")return t}var Yf=!1;if(sn){var ac;if(sn){var rc="oninput"in document;if(!rc){var Vf=document.createElement("div");Vf.setAttribute("oninput","return;"),rc=typeof Vf.oninput=="function"}ac=rc}else ac=!1;Yf=ac&&(!document.documentMode||9<document.documentMode)}function Xf(){ka&&(ka.detachEvent("onpropertychange",Qf),La=ka=null)}function Qf(e){if(e.propertyName==="value"&&Ir(La)){var t=[];qf(t,La,e,$i(e)),Rf(g0,t)}}function p0(e,t,n){e==="focusin"?(Xf(),ka=t,La=n,ka.attachEvent("onpropertychange",Qf)):e==="focusout"&&Xf()}function y0(e){if(e==="selectionchange"||e==="keyup"||e==="keydown")return Ir(La)}function b0(e,t){if(e==="click")return Ir(t)}function x0(e,t){if(e==="input"||e==="change")return Ir(t)}function S0(e,t){return e===t&&(e!==0||1/e===1/t)||e!==e&&t!==t}var pt=typeof Object.is=="function"?Object.is:S0;function Ha(e,t){if(pt(e,t))return!0;if(typeof e!="object"||e===null||typeof t!="object"||t===null)return!1;var n=Object.keys(e),l=Object.keys(t);if(n.length!==l.length)return!1;for(l=0;l<n.length;l++){var r=n[l];if(!Kt.call(t,r)||!pt(e[r],t[r]))return!1}return!0}function Zf(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function Kf(e,t){var n=Zf(e);e=0;for(var l;n;){if(n.nodeType===3){if(l=e+n.textContent.length,e<=t&&l>=t)return{node:n,offset:t-e};e=l}e:{for(;n;){if(n.nextSibling){n=n.nextSibling;break e}n=n.parentNode}n=void 0}n=Zf(n)}}function Pf(e,t){return e&&t?e===t?!0:e&&e.nodeType===3?!1:t&&t.nodeType===3?Pf(e,t.parentNode):"contains"in e?e.contains(t):e.compareDocumentPosition?!!(e.compareDocumentPosition(t)&16):!1:!1}function $f(e){e=e!=null&&e.ownerDocument!=null&&e.ownerDocument.defaultView!=null?e.ownerDocument.defaultView:window;for(var t=Zr(e.document);t instanceof e.HTMLIFrameElement;){try{var n=typeof t.contentWindow.location.href=="string"}catch{n=!1}if(n)e=t.contentWindow;else break;t=Zr(e.document)}return t}function oc(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&(t==="input"&&(e.type==="text"||e.type==="search"||e.type==="tel"||e.type==="url"||e.type==="password")||t==="textarea"||e.contentEditable==="true")}var w0=sn&&"documentMode"in document&&11>=document.documentMode,Bl=null,ic=null,Ba=null,cc=!1;function Jf(e,t,n){var l=n.window===n?n.document:n.nodeType===9?n:n.ownerDocument;cc||Bl==null||Bl!==Zr(l)||(l=Bl,"selectionStart"in l&&oc(l)?l={start:l.selectionStart,end:l.selectionEnd}:(l=(l.ownerDocument&&l.ownerDocument.defaultView||window).getSelection(),l={anchorNode:l.anchorNode,anchorOffset:l.anchorOffset,focusNode:l.focusNode,focusOffset:l.focusOffset}),Ba&&Ha(Ba,l)||(Ba=l,l=Bo(ic,"onSelect"),0<l.length&&(t=new Wr("onSelect","select",null,t,n),e.push({event:t,listeners:l}),t.target=Bl)))}function il(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var ql={animationend:il("Animation","AnimationEnd"),animationiteration:il("Animation","AnimationIteration"),animationstart:il("Animation","AnimationStart"),transitionrun:il("Transition","TransitionRun"),transitionstart:il("Transition","TransitionStart"),transitioncancel:il("Transition","TransitionCancel"),transitionend:il("Transition","TransitionEnd")},sc={},Wf={};sn&&(Wf=document.createElement("div").style,"AnimationEvent"in window||(delete ql.animationend.animation,delete ql.animationiteration.animation,delete ql.animationstart.animation),"TransitionEvent"in window||delete ql.transitionend.transition);function cl(e){if(sc[e])return sc[e];if(!ql[e])return e;var t=ql[e],n;for(n in t)if(t.hasOwnProperty(n)&&n in Wf)return sc[e]=t[n];return e}var Ff=cl("animationend"),If=cl("animationiteration"),ed=cl("animationstart"),E0=cl("transitionrun"),A0=cl("transitionstart"),M0=cl("transitioncancel"),td=cl("transitionend"),nd=new Map,uc="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");uc.push("scrollEnd");function qt(e,t){nd.set(e,t),rl(t,[e])}var ld=new WeakMap;function jt(e,t){if(typeof e=="object"&&e!==null){var n=ld.get(e);return n!==void 0?n:(t={value:e,source:t,stack:bf(t)},ld.set(e,t),t)}return{value:e,source:t,stack:bf(t)}}var Ot=[],Gl=0,fc=0;function eo(){for(var e=Gl,t=fc=Gl=0;t<e;){var n=Ot[t];Ot[t++]=null;var l=Ot[t];Ot[t++]=null;var r=Ot[t];Ot[t++]=null;var s=Ot[t];if(Ot[t++]=null,l!==null&&r!==null){var m=l.pending;m===null?r.next=r:(r.next=m.next,m.next=r),l.pending=r}s!==0&&ad(n,r,s)}}function to(e,t,n,l){Ot[Gl++]=e,Ot[Gl++]=t,Ot[Gl++]=n,Ot[Gl++]=l,fc|=l,e.lanes|=l,e=e.alternate,e!==null&&(e.lanes|=l)}function dc(e,t,n,l){return to(e,t,n,l),no(e)}function Yl(e,t){return to(e,null,null,t),no(e)}function ad(e,t,n){e.lanes|=n;var l=e.alternate;l!==null&&(l.lanes|=n);for(var r=!1,s=e.return;s!==null;)s.childLanes|=n,l=s.alternate,l!==null&&(l.childLanes|=n),s.tag===22&&(e=s.stateNode,e===null||e._visibility&1||(r=!0)),e=s,s=s.return;return e.tag===3?(s=e.stateNode,r&&t!==null&&(r=31-gt(n),e=s.hiddenUpdates,l=e[r],l===null?e[r]=[t]:l.push(t),t.lane=n|536870912),s):null}function no(e){if(50<ur)throw ur=0,ys=null,Error(c(185));for(var t=e.return;t!==null;)e=t,t=e.return;return e.tag===3?e.stateNode:null}var Vl={};function N0(e,t,n,l){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=l,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function yt(e,t,n,l){return new N0(e,t,n,l)}function mc(e){return e=e.prototype,!(!e||!e.isReactComponent)}function un(e,t){var n=e.alternate;return n===null?(n=yt(e.tag,t,e.key,e.mode),n.elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.type=e.type,n.flags=0,n.subtreeFlags=0,n.deletions=null),n.flags=e.flags&65011712,n.childLanes=e.childLanes,n.lanes=e.lanes,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n.refCleanup=e.refCleanup,n}function rd(e,t){e.flags&=65011714;var n=e.alternate;return n===null?(e.childLanes=0,e.lanes=t,e.child=null,e.subtreeFlags=0,e.memoizedProps=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.stateNode=null):(e.childLanes=n.childLanes,e.lanes=n.lanes,e.child=n.child,e.subtreeFlags=0,e.deletions=null,e.memoizedProps=n.memoizedProps,e.memoizedState=n.memoizedState,e.updateQueue=n.updateQueue,e.type=n.type,t=n.dependencies,e.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),e}function lo(e,t,n,l,r,s){var m=0;if(l=e,typeof e=="function")mc(e)&&(m=1);else if(typeof e=="string")m=Rb(e,n,ee.current)?26:e==="html"||e==="head"||e==="body"?27:5;else e:switch(e){case I:return e=yt(31,n,t,r),e.elementType=I,e.lanes=s,e;case T:return sl(n.children,r,s,t);case N:m=8,r|=24;break;case _:return e=yt(12,n,t,r|2),e.elementType=_,e.lanes=s,e;case B:return e=yt(13,n,t,r),e.elementType=B,e.lanes=s,e;case J:return e=yt(19,n,t,r),e.elementType=J,e.lanes=s,e;default:if(typeof e=="object"&&e!==null)switch(e.$$typeof){case j:case L:m=10;break e;case q:m=9;break e;case Y:m=11;break e;case P:m=14;break e;case Z:m=16,l=null;break e}m=29,n=Error(c(130,e===null?"null":typeof e,"")),l=null}return t=yt(m,n,t,r),t.elementType=e,t.type=l,t.lanes=s,t}function sl(e,t,n,l){return e=yt(7,e,l,t),e.lanes=n,e}function hc(e,t,n){return e=yt(6,e,null,t),e.lanes=n,e}function vc(e,t,n){return t=yt(4,e.children!==null?e.children:[],e.key,t),t.lanes=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}var Xl=[],Ql=0,ao=null,ro=0,Dt=[],zt=0,ul=null,fn=1,dn="";function fl(e,t){Xl[Ql++]=ro,Xl[Ql++]=ao,ao=e,ro=t}function od(e,t,n){Dt[zt++]=fn,Dt[zt++]=dn,Dt[zt++]=ul,ul=e;var l=fn;e=dn;var r=32-gt(l)-1;l&=~(1<<r),n+=1;var s=32-gt(t)+r;if(30<s){var m=r-r%5;s=(l&(1<<m)-1).toString(32),l>>=m,r-=m,fn=1<<32-gt(t)+r|n<<r|l,dn=s+e}else fn=1<<s|n<<r|l,dn=e}function gc(e){e.return!==null&&(fl(e,1),od(e,1,0))}function pc(e){for(;e===ao;)ao=Xl[--Ql],Xl[Ql]=null,ro=Xl[--Ql],Xl[Ql]=null;for(;e===ul;)ul=Dt[--zt],Dt[zt]=null,dn=Dt[--zt],Dt[zt]=null,fn=Dt[--zt],Dt[zt]=null}var ct=null,Be=null,Te=!1,dl=null,$t=!1,yc=Error(c(519));function ml(e){var t=Error(c(418,""));throw Ya(jt(t,e)),yc}function id(e){var t=e.stateNode,n=e.type,l=e.memoizedProps;switch(t[lt]=e,t[ut]=l,n){case"dialog":we("cancel",t),we("close",t);break;case"iframe":case"object":case"embed":we("load",t);break;case"video":case"audio":for(n=0;n<dr.length;n++)we(dr[n],t);break;case"source":we("error",t);break;case"img":case"image":case"link":we("error",t),we("load",t);break;case"details":we("toggle",t);break;case"input":we("invalid",t),wf(t,l.value,l.defaultValue,l.checked,l.defaultChecked,l.type,l.name,!0),Qr(t);break;case"select":we("invalid",t);break;case"textarea":we("invalid",t),Af(t,l.value,l.defaultValue,l.children),Qr(t)}n=l.children,typeof n!="string"&&typeof n!="number"&&typeof n!="bigint"||t.textContent===""+n||l.suppressHydrationWarning===!0||Ah(t.textContent,n)?(l.popover!=null&&(we("beforetoggle",t),we("toggle",t)),l.onScroll!=null&&we("scroll",t),l.onScrollEnd!=null&&we("scrollend",t),l.onClick!=null&&(t.onclick=qo),t=!0):t=!1,t||ml(e)}function cd(e){for(ct=e.return;ct;)switch(ct.tag){case 5:case 13:$t=!1;return;case 27:case 3:$t=!0;return;default:ct=ct.return}}function qa(e){if(e!==ct)return!1;if(!Te)return cd(e),Te=!0,!1;var t=e.tag,n;if((n=t!==3&&t!==27)&&((n=t===5)&&(n=e.type,n=!(n!=="form"&&n!=="button")||zs(e.type,e.memoizedProps)),n=!n),n&&Be&&ml(e),cd(e),t===13){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(c(317));e:{for(e=e.nextSibling,t=0;e;){if(e.nodeType===8)if(n=e.data,n==="/$"){if(t===0){Be=Yt(e.nextSibling);break e}t--}else n!=="$"&&n!=="$!"&&n!=="$?"||t++;e=e.nextSibling}Be=null}}else t===27?(t=Be,Kn(e.type)?(e=Hs,Hs=null,Be=e):Be=t):Be=ct?Yt(e.stateNode.nextSibling):null;return!0}function Ga(){Be=ct=null,Te=!1}function sd(){var e=dl;return e!==null&&(ht===null?ht=e:ht.push.apply(ht,e),dl=null),e}function Ya(e){dl===null?dl=[e]:dl.push(e)}var bc=X(null),hl=null,mn=null;function On(e,t,n){$(bc,t._currentValue),t._currentValue=n}function hn(e){e._currentValue=bc.current,F(bc)}function xc(e,t,n){for(;e!==null;){var l=e.alternate;if((e.childLanes&t)!==t?(e.childLanes|=t,l!==null&&(l.childLanes|=t)):l!==null&&(l.childLanes&t)!==t&&(l.childLanes|=t),e===n)break;e=e.return}}function Sc(e,t,n,l){var r=e.child;for(r!==null&&(r.return=e);r!==null;){var s=r.dependencies;if(s!==null){var m=r.child;s=s.firstContext;e:for(;s!==null;){var y=s;s=r;for(var E=0;E<t.length;E++)if(y.context===t[E]){s.lanes|=n,y=s.alternate,y!==null&&(y.lanes|=n),xc(s.return,n,e),l||(m=null);break e}s=y.next}}else if(r.tag===18){if(m=r.return,m===null)throw Error(c(341));m.lanes|=n,s=m.alternate,s!==null&&(s.lanes|=n),xc(m,n,e),m=null}else m=r.child;if(m!==null)m.return=r;else for(m=r;m!==null;){if(m===e){m=null;break}if(r=m.sibling,r!==null){r.return=m.return,m=r;break}m=m.return}r=m}}function Va(e,t,n,l){e=null;for(var r=t,s=!1;r!==null;){if(!s){if((r.flags&524288)!==0)s=!0;else if((r.flags&262144)!==0)break}if(r.tag===10){var m=r.alternate;if(m===null)throw Error(c(387));if(m=m.memoizedProps,m!==null){var y=r.type;pt(r.pendingProps.value,m.value)||(e!==null?e.push(y):e=[y])}}else if(r===ie.current){if(m=r.alternate,m===null)throw Error(c(387));m.memoizedState.memoizedState!==r.memoizedState.memoizedState&&(e!==null?e.push(yr):e=[yr])}r=r.return}e!==null&&Sc(t,e,n,l),t.flags|=262144}function oo(e){for(e=e.firstContext;e!==null;){if(!pt(e.context._currentValue,e.memoizedValue))return!0;e=e.next}return!1}function vl(e){hl=e,mn=null,e=e.dependencies,e!==null&&(e.firstContext=null)}function at(e){return ud(hl,e)}function io(e,t){return hl===null&&vl(e),ud(e,t)}function ud(e,t){var n=t._currentValue;if(t={context:t,memoizedValue:n,next:null},mn===null){if(e===null)throw Error(c(308));mn=t,e.dependencies={lanes:0,firstContext:t},e.flags|=524288}else mn=mn.next=t;return n}var T0=typeof AbortController<"u"?AbortController:function(){var e=[],t=this.signal={aborted:!1,addEventListener:function(n,l){e.push(l)}};this.abort=function(){t.aborted=!0,e.forEach(function(n){return n()})}},R0=a.unstable_scheduleCallback,_0=a.unstable_NormalPriority,Ke={$$typeof:L,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function wc(){return{controller:new T0,data:new Map,refCount:0}}function Xa(e){e.refCount--,e.refCount===0&&R0(_0,function(){e.controller.abort()})}var Qa=null,Ec=0,Zl=0,Kl=null;function C0(e,t){if(Qa===null){var n=Qa=[];Ec=0,Zl=Ms(),Kl={status:"pending",value:void 0,then:function(l){n.push(l)}}}return Ec++,t.then(fd,fd),t}function fd(){if(--Ec===0&&Qa!==null){Kl!==null&&(Kl.status="fulfilled");var e=Qa;Qa=null,Zl=0,Kl=null;for(var t=0;t<e.length;t++)(0,e[t])()}}function j0(e,t){var n=[],l={status:"pending",value:null,reason:null,then:function(r){n.push(r)}};return e.then(function(){l.status="fulfilled",l.value=t;for(var r=0;r<n.length;r++)(0,n[r])(t)},function(r){for(l.status="rejected",l.reason=r,r=0;r<n.length;r++)(0,n[r])(void 0)}),l}var dd=D.S;D.S=function(e,t){typeof t=="object"&&t!==null&&typeof t.then=="function"&&C0(e,t),dd!==null&&dd(e,t)};var gl=X(null);function Ac(){var e=gl.current;return e!==null?e:Ue.pooledCache}function co(e,t){t===null?$(gl,gl.current):$(gl,t.pool)}function md(){var e=Ac();return e===null?null:{parent:Ke._currentValue,pool:e}}var Za=Error(c(460)),hd=Error(c(474)),so=Error(c(542)),Mc={then:function(){}};function vd(e){return e=e.status,e==="fulfilled"||e==="rejected"}function uo(){}function gd(e,t,n){switch(n=e[n],n===void 0?e.push(t):n!==t&&(t.then(uo,uo),t=n),t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,yd(e),e;default:if(typeof t.status=="string")t.then(uo,uo);else{if(e=Ue,e!==null&&100<e.shellSuspendCounter)throw Error(c(482));e=t,e.status="pending",e.then(function(l){if(t.status==="pending"){var r=t;r.status="fulfilled",r.value=l}},function(l){if(t.status==="pending"){var r=t;r.status="rejected",r.reason=l}})}switch(t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,yd(e),e}throw Ka=t,Za}}var Ka=null;function pd(){if(Ka===null)throw Error(c(459));var e=Ka;return Ka=null,e}function yd(e){if(e===Za||e===so)throw Error(c(483))}var Dn=!1;function Nc(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function Tc(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,callbacks:null})}function zn(e){return{lane:e,tag:0,payload:null,callback:null,next:null}}function Un(e,t,n){var l=e.updateQueue;if(l===null)return null;if(l=l.shared,(Re&2)!==0){var r=l.pending;return r===null?t.next=t:(t.next=r.next,r.next=t),l.pending=t,t=no(e),ad(e,null,n),t}return to(e,l,t,n),no(e)}function Pa(e,t,n){if(t=t.updateQueue,t!==null&&(t=t.shared,(n&4194048)!==0)){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,ff(e,n)}}function Rc(e,t){var n=e.updateQueue,l=e.alternate;if(l!==null&&(l=l.updateQueue,n===l)){var r=null,s=null;if(n=n.firstBaseUpdate,n!==null){do{var m={lane:n.lane,tag:n.tag,payload:n.payload,callback:null,next:null};s===null?r=s=m:s=s.next=m,n=n.next}while(n!==null);s===null?r=s=t:s=s.next=t}else r=s=t;n={baseState:l.baseState,firstBaseUpdate:r,lastBaseUpdate:s,shared:l.shared,callbacks:l.callbacks},e.updateQueue=n;return}e=n.lastBaseUpdate,e===null?n.firstBaseUpdate=t:e.next=t,n.lastBaseUpdate=t}var _c=!1;function $a(){if(_c){var e=Kl;if(e!==null)throw e}}function Ja(e,t,n,l){_c=!1;var r=e.updateQueue;Dn=!1;var s=r.firstBaseUpdate,m=r.lastBaseUpdate,y=r.shared.pending;if(y!==null){r.shared.pending=null;var E=y,z=E.next;E.next=null,m===null?s=z:m.next=z,m=E;var G=e.alternate;G!==null&&(G=G.updateQueue,y=G.lastBaseUpdate,y!==m&&(y===null?G.firstBaseUpdate=z:y.next=z,G.lastBaseUpdate=E))}if(s!==null){var Q=r.baseState;m=0,G=z=E=null,y=s;do{var U=y.lane&-536870913,k=U!==y.lane;if(k?(Ae&U)===U:(l&U)===U){U!==0&&U===Zl&&(_c=!0),G!==null&&(G=G.next={lane:0,tag:y.tag,payload:y.payload,callback:null,next:null});e:{var fe=e,ce=y;U=t;var Oe=n;switch(ce.tag){case 1:if(fe=ce.payload,typeof fe=="function"){Q=fe.call(Oe,Q,U);break e}Q=fe;break e;case 3:fe.flags=fe.flags&-65537|128;case 0:if(fe=ce.payload,U=typeof fe=="function"?fe.call(Oe,Q,U):fe,U==null)break e;Q=p({},Q,U);break e;case 2:Dn=!0}}U=y.callback,U!==null&&(e.flags|=64,k&&(e.flags|=8192),k=r.callbacks,k===null?r.callbacks=[U]:k.push(U))}else k={lane:U,tag:y.tag,payload:y.payload,callback:y.callback,next:null},G===null?(z=G=k,E=Q):G=G.next=k,m|=U;if(y=y.next,y===null){if(y=r.shared.pending,y===null)break;k=y,y=k.next,k.next=null,r.lastBaseUpdate=k,r.shared.pending=null}}while(!0);G===null&&(E=Q),r.baseState=E,r.firstBaseUpdate=z,r.lastBaseUpdate=G,s===null&&(r.shared.lanes=0),Vn|=m,e.lanes=m,e.memoizedState=Q}}function bd(e,t){if(typeof e!="function")throw Error(c(191,e));e.call(t)}function xd(e,t){var n=e.callbacks;if(n!==null)for(e.callbacks=null,e=0;e<n.length;e++)bd(n[e],t)}var Pl=X(null),fo=X(0);function Sd(e,t){e=Sn,$(fo,e),$(Pl,t),Sn=e|t.baseLanes}function Cc(){$(fo,Sn),$(Pl,Pl.current)}function jc(){Sn=fo.current,F(Pl),F(fo)}var kn=0,pe=null,Ce=null,Ve=null,mo=!1,$l=!1,pl=!1,ho=0,Wa=0,Jl=null,O0=0;function Ge(){throw Error(c(321))}function Oc(e,t){if(t===null)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!pt(e[n],t[n]))return!1;return!0}function Dc(e,t,n,l,r,s){return kn=s,pe=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,D.H=e===null||e.memoizedState===null?am:rm,pl=!1,s=n(l,r),pl=!1,$l&&(s=Ed(t,n,l,r)),wd(e),s}function wd(e){D.H=xo;var t=Ce!==null&&Ce.next!==null;if(kn=0,Ve=Ce=pe=null,mo=!1,Wa=0,Jl=null,t)throw Error(c(300));e===null||We||(e=e.dependencies,e!==null&&oo(e)&&(We=!0))}function Ed(e,t,n,l){pe=e;var r=0;do{if($l&&(Jl=null),Wa=0,$l=!1,25<=r)throw Error(c(301));if(r+=1,Ve=Ce=null,e.updateQueue!=null){var s=e.updateQueue;s.lastEffect=null,s.events=null,s.stores=null,s.memoCache!=null&&(s.memoCache.index=0)}D.H=B0,s=t(n,l)}while($l);return s}function D0(){var e=D.H,t=e.useState()[0];return t=typeof t.then=="function"?Fa(t):t,e=e.useState()[0],(Ce!==null?Ce.memoizedState:null)!==e&&(pe.flags|=1024),t}function zc(){var e=ho!==0;return ho=0,e}function Uc(e,t,n){t.updateQueue=e.updateQueue,t.flags&=-2053,e.lanes&=~n}function kc(e){if(mo){for(e=e.memoizedState;e!==null;){var t=e.queue;t!==null&&(t.pending=null),e=e.next}mo=!1}kn=0,Ve=Ce=pe=null,$l=!1,Wa=ho=0,Jl=null}function dt(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Ve===null?pe.memoizedState=Ve=e:Ve=Ve.next=e,Ve}function Xe(){if(Ce===null){var e=pe.alternate;e=e!==null?e.memoizedState:null}else e=Ce.next;var t=Ve===null?pe.memoizedState:Ve.next;if(t!==null)Ve=t,Ce=e;else{if(e===null)throw pe.alternate===null?Error(c(467)):Error(c(310));Ce=e,e={memoizedState:Ce.memoizedState,baseState:Ce.baseState,baseQueue:Ce.baseQueue,queue:Ce.queue,next:null},Ve===null?pe.memoizedState=Ve=e:Ve=Ve.next=e}return Ve}function Lc(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function Fa(e){var t=Wa;return Wa+=1,Jl===null&&(Jl=[]),e=gd(Jl,e,t),t=pe,(Ve===null?t.memoizedState:Ve.next)===null&&(t=t.alternate,D.H=t===null||t.memoizedState===null?am:rm),e}function vo(e){if(e!==null&&typeof e=="object"){if(typeof e.then=="function")return Fa(e);if(e.$$typeof===L)return at(e)}throw Error(c(438,String(e)))}function Hc(e){var t=null,n=pe.updateQueue;if(n!==null&&(t=n.memoCache),t==null){var l=pe.alternate;l!==null&&(l=l.updateQueue,l!==null&&(l=l.memoCache,l!=null&&(t={data:l.data.map(function(r){return r.slice()}),index:0})))}if(t==null&&(t={data:[],index:0}),n===null&&(n=Lc(),pe.updateQueue=n),n.memoCache=t,n=t.data[t.index],n===void 0)for(n=t.data[t.index]=Array(e),l=0;l<e;l++)n[l]=re;return t.index++,n}function vn(e,t){return typeof t=="function"?t(e):t}function go(e){var t=Xe();return Bc(t,Ce,e)}function Bc(e,t,n){var l=e.queue;if(l===null)throw Error(c(311));l.lastRenderedReducer=n;var r=e.baseQueue,s=l.pending;if(s!==null){if(r!==null){var m=r.next;r.next=s.next,s.next=m}t.baseQueue=r=s,l.pending=null}if(s=e.baseState,r===null)e.memoizedState=s;else{t=r.next;var y=m=null,E=null,z=t,G=!1;do{var Q=z.lane&-536870913;if(Q!==z.lane?(Ae&Q)===Q:(kn&Q)===Q){var U=z.revertLane;if(U===0)E!==null&&(E=E.next={lane:0,revertLane:0,action:z.action,hasEagerState:z.hasEagerState,eagerState:z.eagerState,next:null}),Q===Zl&&(G=!0);else if((kn&U)===U){z=z.next,U===Zl&&(G=!0);continue}else Q={lane:0,revertLane:z.revertLane,action:z.action,hasEagerState:z.hasEagerState,eagerState:z.eagerState,next:null},E===null?(y=E=Q,m=s):E=E.next=Q,pe.lanes|=U,Vn|=U;Q=z.action,pl&&n(s,Q),s=z.hasEagerState?z.eagerState:n(s,Q)}else U={lane:Q,revertLane:z.revertLane,action:z.action,hasEagerState:z.hasEagerState,eagerState:z.eagerState,next:null},E===null?(y=E=U,m=s):E=E.next=U,pe.lanes|=Q,Vn|=Q;z=z.next}while(z!==null&&z!==t);if(E===null?m=s:E.next=y,!pt(s,e.memoizedState)&&(We=!0,G&&(n=Kl,n!==null)))throw n;e.memoizedState=s,e.baseState=m,e.baseQueue=E,l.lastRenderedState=s}return r===null&&(l.lanes=0),[e.memoizedState,l.dispatch]}function qc(e){var t=Xe(),n=t.queue;if(n===null)throw Error(c(311));n.lastRenderedReducer=e;var l=n.dispatch,r=n.pending,s=t.memoizedState;if(r!==null){n.pending=null;var m=r=r.next;do s=e(s,m.action),m=m.next;while(m!==r);pt(s,t.memoizedState)||(We=!0),t.memoizedState=s,t.baseQueue===null&&(t.baseState=s),n.lastRenderedState=s}return[s,l]}function Ad(e,t,n){var l=pe,r=Xe(),s=Te;if(s){if(n===void 0)throw Error(c(407));n=n()}else n=t();var m=!pt((Ce||r).memoizedState,n);m&&(r.memoizedState=n,We=!0),r=r.queue;var y=Td.bind(null,l,r,e);if(Ia(2048,8,y,[e]),r.getSnapshot!==t||m||Ve!==null&&Ve.memoizedState.tag&1){if(l.flags|=2048,Wl(9,po(),Nd.bind(null,l,r,n,t),null),Ue===null)throw Error(c(349));s||(kn&124)!==0||Md(l,t,n)}return n}function Md(e,t,n){e.flags|=16384,e={getSnapshot:t,value:n},t=pe.updateQueue,t===null?(t=Lc(),pe.updateQueue=t,t.stores=[e]):(n=t.stores,n===null?t.stores=[e]:n.push(e))}function Nd(e,t,n,l){t.value=n,t.getSnapshot=l,Rd(t)&&_d(e)}function Td(e,t,n){return n(function(){Rd(t)&&_d(e)})}function Rd(e){var t=e.getSnapshot;e=e.value;try{var n=t();return!pt(e,n)}catch{return!0}}function _d(e){var t=Yl(e,2);t!==null&&Et(t,e,2)}function Gc(e){var t=dt();if(typeof e=="function"){var n=e;if(e=n(),pl){_n(!0);try{n()}finally{_n(!1)}}}return t.memoizedState=t.baseState=e,t.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:vn,lastRenderedState:e},t}function Cd(e,t,n,l){return e.baseState=n,Bc(e,Ce,typeof l=="function"?l:vn)}function z0(e,t,n,l,r){if(bo(e))throw Error(c(485));if(e=t.action,e!==null){var s={payload:r,action:e,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(m){s.listeners.push(m)}};D.T!==null?n(!0):s.isTransition=!1,l(s),n=t.pending,n===null?(s.next=t.pending=s,jd(t,s)):(s.next=n.next,t.pending=n.next=s)}}function jd(e,t){var n=t.action,l=t.payload,r=e.state;if(t.isTransition){var s=D.T,m={};D.T=m;try{var y=n(r,l),E=D.S;E!==null&&E(m,y),Od(e,t,y)}catch(z){Yc(e,t,z)}finally{D.T=s}}else try{s=n(r,l),Od(e,t,s)}catch(z){Yc(e,t,z)}}function Od(e,t,n){n!==null&&typeof n=="object"&&typeof n.then=="function"?n.then(function(l){Dd(e,t,l)},function(l){return Yc(e,t,l)}):Dd(e,t,n)}function Dd(e,t,n){t.status="fulfilled",t.value=n,zd(t),e.state=n,t=e.pending,t!==null&&(n=t.next,n===t?e.pending=null:(n=n.next,t.next=n,jd(e,n)))}function Yc(e,t,n){var l=e.pending;if(e.pending=null,l!==null){l=l.next;do t.status="rejected",t.reason=n,zd(t),t=t.next;while(t!==l)}e.action=null}function zd(e){e=e.listeners;for(var t=0;t<e.length;t++)(0,e[t])()}function Ud(e,t){return t}function kd(e,t){if(Te){var n=Ue.formState;if(n!==null){e:{var l=pe;if(Te){if(Be){t:{for(var r=Be,s=$t;r.nodeType!==8;){if(!s){r=null;break t}if(r=Yt(r.nextSibling),r===null){r=null;break t}}s=r.data,r=s==="F!"||s==="F"?r:null}if(r){Be=Yt(r.nextSibling),l=r.data==="F!";break e}}ml(l)}l=!1}l&&(t=n[0])}}return n=dt(),n.memoizedState=n.baseState=t,l={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ud,lastRenderedState:t},n.queue=l,n=tm.bind(null,pe,l),l.dispatch=n,l=Gc(!1),s=Kc.bind(null,pe,!1,l.queue),l=dt(),r={state:t,dispatch:null,action:e,pending:null},l.queue=r,n=z0.bind(null,pe,r,s,n),r.dispatch=n,l.memoizedState=e,[t,n,!1]}function Ld(e){var t=Xe();return Hd(t,Ce,e)}function Hd(e,t,n){if(t=Bc(e,t,Ud)[0],e=go(vn)[0],typeof t=="object"&&t!==null&&typeof t.then=="function")try{var l=Fa(t)}catch(m){throw m===Za?so:m}else l=t;t=Xe();var r=t.queue,s=r.dispatch;return n!==t.memoizedState&&(pe.flags|=2048,Wl(9,po(),U0.bind(null,r,n),null)),[l,s,e]}function U0(e,t){e.action=t}function Bd(e){var t=Xe(),n=Ce;if(n!==null)return Hd(t,n,e);Xe(),t=t.memoizedState,n=Xe();var l=n.queue.dispatch;return n.memoizedState=e,[t,l,!1]}function Wl(e,t,n,l){return e={tag:e,create:n,deps:l,inst:t,next:null},t=pe.updateQueue,t===null&&(t=Lc(),pe.updateQueue=t),n=t.lastEffect,n===null?t.lastEffect=e.next=e:(l=n.next,n.next=e,e.next=l,t.lastEffect=e),e}function po(){return{destroy:void 0,resource:void 0}}function qd(){return Xe().memoizedState}function yo(e,t,n,l){var r=dt();l=l===void 0?null:l,pe.flags|=e,r.memoizedState=Wl(1|t,po(),n,l)}function Ia(e,t,n,l){var r=Xe();l=l===void 0?null:l;var s=r.memoizedState.inst;Ce!==null&&l!==null&&Oc(l,Ce.memoizedState.deps)?r.memoizedState=Wl(t,s,n,l):(pe.flags|=e,r.memoizedState=Wl(1|t,s,n,l))}function Gd(e,t){yo(8390656,8,e,t)}function Yd(e,t){Ia(2048,8,e,t)}function Vd(e,t){return Ia(4,2,e,t)}function Xd(e,t){return Ia(4,4,e,t)}function Qd(e,t){if(typeof t=="function"){e=e();var n=t(e);return function(){typeof n=="function"?n():t(null)}}if(t!=null)return e=e(),t.current=e,function(){t.current=null}}function Zd(e,t,n){n=n!=null?n.concat([e]):null,Ia(4,4,Qd.bind(null,t,e),n)}function Vc(){}function Kd(e,t){var n=Xe();t=t===void 0?null:t;var l=n.memoizedState;return t!==null&&Oc(t,l[1])?l[0]:(n.memoizedState=[e,t],e)}function Pd(e,t){var n=Xe();t=t===void 0?null:t;var l=n.memoizedState;if(t!==null&&Oc(t,l[1]))return l[0];if(l=e(),pl){_n(!0);try{e()}finally{_n(!1)}}return n.memoizedState=[l,t],l}function Xc(e,t,n){return n===void 0||(kn&1073741824)!==0?e.memoizedState=t:(e.memoizedState=n,e=Wm(),pe.lanes|=e,Vn|=e,n)}function $d(e,t,n,l){return pt(n,t)?n:Pl.current!==null?(e=Xc(e,n,l),pt(e,t)||(We=!0),e):(kn&42)===0?(We=!0,e.memoizedState=n):(e=Wm(),pe.lanes|=e,Vn|=e,t)}function Jd(e,t,n,l,r){var s=K.p;K.p=s!==0&&8>s?s:8;var m=D.T,y={};D.T=y,Kc(e,!1,t,n);try{var E=r(),z=D.S;if(z!==null&&z(y,E),E!==null&&typeof E=="object"&&typeof E.then=="function"){var G=j0(E,l);er(e,t,G,wt(e))}else er(e,t,l,wt(e))}catch(Q){er(e,t,{then:function(){},status:"rejected",reason:Q},wt())}finally{K.p=s,D.T=m}}function k0(){}function Qc(e,t,n,l){if(e.tag!==5)throw Error(c(476));var r=Wd(e).queue;Jd(e,r,t,H,n===null?k0:function(){return Fd(e),n(l)})}function Wd(e){var t=e.memoizedState;if(t!==null)return t;t={memoizedState:H,baseState:H,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:vn,lastRenderedState:H},next:null};var n={};return t.next={memoizedState:n,baseState:n,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:vn,lastRenderedState:n},next:null},e.memoizedState=t,e=e.alternate,e!==null&&(e.memoizedState=t),t}function Fd(e){var t=Wd(e).next.queue;er(e,t,{},wt())}function Zc(){return at(yr)}function Id(){return Xe().memoizedState}function em(){return Xe().memoizedState}function L0(e){for(var t=e.return;t!==null;){switch(t.tag){case 24:case 3:var n=wt();e=zn(n);var l=Un(t,e,n);l!==null&&(Et(l,t,n),Pa(l,t,n)),t={cache:wc()},e.payload=t;return}t=t.return}}function H0(e,t,n){var l=wt();n={lane:l,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null},bo(e)?nm(t,n):(n=dc(e,t,n,l),n!==null&&(Et(n,e,l),lm(n,t,l)))}function tm(e,t,n){var l=wt();er(e,t,n,l)}function er(e,t,n,l){var r={lane:l,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null};if(bo(e))nm(t,r);else{var s=e.alternate;if(e.lanes===0&&(s===null||s.lanes===0)&&(s=t.lastRenderedReducer,s!==null))try{var m=t.lastRenderedState,y=s(m,n);if(r.hasEagerState=!0,r.eagerState=y,pt(y,m))return to(e,t,r,0),Ue===null&&eo(),!1}catch{}finally{}if(n=dc(e,t,r,l),n!==null)return Et(n,e,l),lm(n,t,l),!0}return!1}function Kc(e,t,n,l){if(l={lane:2,revertLane:Ms(),action:l,hasEagerState:!1,eagerState:null,next:null},bo(e)){if(t)throw Error(c(479))}else t=dc(e,n,l,2),t!==null&&Et(t,e,2)}function bo(e){var t=e.alternate;return e===pe||t!==null&&t===pe}function nm(e,t){$l=mo=!0;var n=e.pending;n===null?t.next=t:(t.next=n.next,n.next=t),e.pending=t}function lm(e,t,n){if((n&4194048)!==0){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,ff(e,n)}}var xo={readContext:at,use:vo,useCallback:Ge,useContext:Ge,useEffect:Ge,useImperativeHandle:Ge,useLayoutEffect:Ge,useInsertionEffect:Ge,useMemo:Ge,useReducer:Ge,useRef:Ge,useState:Ge,useDebugValue:Ge,useDeferredValue:Ge,useTransition:Ge,useSyncExternalStore:Ge,useId:Ge,useHostTransitionStatus:Ge,useFormState:Ge,useActionState:Ge,useOptimistic:Ge,useMemoCache:Ge,useCacheRefresh:Ge},am={readContext:at,use:vo,useCallback:function(e,t){return dt().memoizedState=[e,t===void 0?null:t],e},useContext:at,useEffect:Gd,useImperativeHandle:function(e,t,n){n=n!=null?n.concat([e]):null,yo(4194308,4,Qd.bind(null,t,e),n)},useLayoutEffect:function(e,t){return yo(4194308,4,e,t)},useInsertionEffect:function(e,t){yo(4,2,e,t)},useMemo:function(e,t){var n=dt();t=t===void 0?null:t;var l=e();if(pl){_n(!0);try{e()}finally{_n(!1)}}return n.memoizedState=[l,t],l},useReducer:function(e,t,n){var l=dt();if(n!==void 0){var r=n(t);if(pl){_n(!0);try{n(t)}finally{_n(!1)}}}else r=t;return l.memoizedState=l.baseState=r,e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:e,lastRenderedState:r},l.queue=e,e=e.dispatch=H0.bind(null,pe,e),[l.memoizedState,e]},useRef:function(e){var t=dt();return e={current:e},t.memoizedState=e},useState:function(e){e=Gc(e);var t=e.queue,n=tm.bind(null,pe,t);return t.dispatch=n,[e.memoizedState,n]},useDebugValue:Vc,useDeferredValue:function(e,t){var n=dt();return Xc(n,e,t)},useTransition:function(){var e=Gc(!1);return e=Jd.bind(null,pe,e.queue,!0,!1),dt().memoizedState=e,[!1,e]},useSyncExternalStore:function(e,t,n){var l=pe,r=dt();if(Te){if(n===void 0)throw Error(c(407));n=n()}else{if(n=t(),Ue===null)throw Error(c(349));(Ae&124)!==0||Md(l,t,n)}r.memoizedState=n;var s={value:n,getSnapshot:t};return r.queue=s,Gd(Td.bind(null,l,s,e),[e]),l.flags|=2048,Wl(9,po(),Nd.bind(null,l,s,n,t),null),n},useId:function(){var e=dt(),t=Ue.identifierPrefix;if(Te){var n=dn,l=fn;n=(l&~(1<<32-gt(l)-1)).toString(32)+n,t="«"+t+"R"+n,n=ho++,0<n&&(t+="H"+n.toString(32)),t+="»"}else n=O0++,t="«"+t+"r"+n.toString(32)+"»";return e.memoizedState=t},useHostTransitionStatus:Zc,useFormState:kd,useActionState:kd,useOptimistic:function(e){var t=dt();t.memoizedState=t.baseState=e;var n={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return t.queue=n,t=Kc.bind(null,pe,!0,n),n.dispatch=t,[e,t]},useMemoCache:Hc,useCacheRefresh:function(){return dt().memoizedState=L0.bind(null,pe)}},rm={readContext:at,use:vo,useCallback:Kd,useContext:at,useEffect:Yd,useImperativeHandle:Zd,useInsertionEffect:Vd,useLayoutEffect:Xd,useMemo:Pd,useReducer:go,useRef:qd,useState:function(){return go(vn)},useDebugValue:Vc,useDeferredValue:function(e,t){var n=Xe();return $d(n,Ce.memoizedState,e,t)},useTransition:function(){var e=go(vn)[0],t=Xe().memoizedState;return[typeof e=="boolean"?e:Fa(e),t]},useSyncExternalStore:Ad,useId:Id,useHostTransitionStatus:Zc,useFormState:Ld,useActionState:Ld,useOptimistic:function(e,t){var n=Xe();return Cd(n,Ce,e,t)},useMemoCache:Hc,useCacheRefresh:em},B0={readContext:at,use:vo,useCallback:Kd,useContext:at,useEffect:Yd,useImperativeHandle:Zd,useInsertionEffect:Vd,useLayoutEffect:Xd,useMemo:Pd,useReducer:qc,useRef:qd,useState:function(){return qc(vn)},useDebugValue:Vc,useDeferredValue:function(e,t){var n=Xe();return Ce===null?Xc(n,e,t):$d(n,Ce.memoizedState,e,t)},useTransition:function(){var e=qc(vn)[0],t=Xe().memoizedState;return[typeof e=="boolean"?e:Fa(e),t]},useSyncExternalStore:Ad,useId:Id,useHostTransitionStatus:Zc,useFormState:Bd,useActionState:Bd,useOptimistic:function(e,t){var n=Xe();return Ce!==null?Cd(n,Ce,e,t):(n.baseState=e,[e,n.queue.dispatch])},useMemoCache:Hc,useCacheRefresh:em},Fl=null,tr=0;function So(e){var t=tr;return tr+=1,Fl===null&&(Fl=[]),gd(Fl,e,t)}function nr(e,t){t=t.props.ref,e.ref=t!==void 0?t:null}function wo(e,t){throw t.$$typeof===S?Error(c(525)):(e=Object.prototype.toString.call(t),Error(c(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e)))}function om(e){var t=e._init;return t(e._payload)}function im(e){function t(C,R){if(e){var O=C.deletions;O===null?(C.deletions=[R],C.flags|=16):O.push(R)}}function n(C,R){if(!e)return null;for(;R!==null;)t(C,R),R=R.sibling;return null}function l(C){for(var R=new Map;C!==null;)C.key!==null?R.set(C.key,C):R.set(C.index,C),C=C.sibling;return R}function r(C,R){return C=un(C,R),C.index=0,C.sibling=null,C}function s(C,R,O){return C.index=O,e?(O=C.alternate,O!==null?(O=O.index,O<R?(C.flags|=67108866,R):O):(C.flags|=67108866,R)):(C.flags|=1048576,R)}function m(C){return e&&C.alternate===null&&(C.flags|=67108866),C}function y(C,R,O,V){return R===null||R.tag!==6?(R=hc(O,C.mode,V),R.return=C,R):(R=r(R,O),R.return=C,R)}function E(C,R,O,V){var te=O.type;return te===T?G(C,R,O.props.children,V,O.key):R!==null&&(R.elementType===te||typeof te=="object"&&te!==null&&te.$$typeof===Z&&om(te)===R.type)?(R=r(R,O.props),nr(R,O),R.return=C,R):(R=lo(O.type,O.key,O.props,null,C.mode,V),nr(R,O),R.return=C,R)}function z(C,R,O,V){return R===null||R.tag!==4||R.stateNode.containerInfo!==O.containerInfo||R.stateNode.implementation!==O.implementation?(R=vc(O,C.mode,V),R.return=C,R):(R=r(R,O.children||[]),R.return=C,R)}function G(C,R,O,V,te){return R===null||R.tag!==7?(R=sl(O,C.mode,V,te),R.return=C,R):(R=r(R,O),R.return=C,R)}function Q(C,R,O){if(typeof R=="string"&&R!==""||typeof R=="number"||typeof R=="bigint")return R=hc(""+R,C.mode,O),R.return=C,R;if(typeof R=="object"&&R!==null){switch(R.$$typeof){case w:return O=lo(R.type,R.key,R.props,null,C.mode,O),nr(O,R),O.return=C,O;case M:return R=vc(R,C.mode,O),R.return=C,R;case Z:var V=R._init;return R=V(R._payload),Q(C,R,O)}if(ve(R)||de(R))return R=sl(R,C.mode,O,null),R.return=C,R;if(typeof R.then=="function")return Q(C,So(R),O);if(R.$$typeof===L)return Q(C,io(C,R),O);wo(C,R)}return null}function U(C,R,O,V){var te=R!==null?R.key:null;if(typeof O=="string"&&O!==""||typeof O=="number"||typeof O=="bigint")return te!==null?null:y(C,R,""+O,V);if(typeof O=="object"&&O!==null){switch(O.$$typeof){case w:return O.key===te?E(C,R,O,V):null;case M:return O.key===te?z(C,R,O,V):null;case Z:return te=O._init,O=te(O._payload),U(C,R,O,V)}if(ve(O)||de(O))return te!==null?null:G(C,R,O,V,null);if(typeof O.then=="function")return U(C,R,So(O),V);if(O.$$typeof===L)return U(C,R,io(C,O),V);wo(C,O)}return null}function k(C,R,O,V,te){if(typeof V=="string"&&V!==""||typeof V=="number"||typeof V=="bigint")return C=C.get(O)||null,y(R,C,""+V,te);if(typeof V=="object"&&V!==null){switch(V.$$typeof){case w:return C=C.get(V.key===null?O:V.key)||null,E(R,C,V,te);case M:return C=C.get(V.key===null?O:V.key)||null,z(R,C,V,te);case Z:var xe=V._init;return V=xe(V._payload),k(C,R,O,V,te)}if(ve(V)||de(V))return C=C.get(O)||null,G(R,C,V,te,null);if(typeof V.then=="function")return k(C,R,O,So(V),te);if(V.$$typeof===L)return k(C,R,O,io(R,V),te);wo(R,V)}return null}function fe(C,R,O,V){for(var te=null,xe=null,ae=R,se=R=0,Ie=null;ae!==null&&se<O.length;se++){ae.index>se?(Ie=ae,ae=null):Ie=ae.sibling;var Ne=U(C,ae,O[se],V);if(Ne===null){ae===null&&(ae=Ie);break}e&&ae&&Ne.alternate===null&&t(C,ae),R=s(Ne,R,se),xe===null?te=Ne:xe.sibling=Ne,xe=Ne,ae=Ie}if(se===O.length)return n(C,ae),Te&&fl(C,se),te;if(ae===null){for(;se<O.length;se++)ae=Q(C,O[se],V),ae!==null&&(R=s(ae,R,se),xe===null?te=ae:xe.sibling=ae,xe=ae);return Te&&fl(C,se),te}for(ae=l(ae);se<O.length;se++)Ie=k(ae,C,se,O[se],V),Ie!==null&&(e&&Ie.alternate!==null&&ae.delete(Ie.key===null?se:Ie.key),R=s(Ie,R,se),xe===null?te=Ie:xe.sibling=Ie,xe=Ie);return e&&ae.forEach(function(Fn){return t(C,Fn)}),Te&&fl(C,se),te}function ce(C,R,O,V){if(O==null)throw Error(c(151));for(var te=null,xe=null,ae=R,se=R=0,Ie=null,Ne=O.next();ae!==null&&!Ne.done;se++,Ne=O.next()){ae.index>se?(Ie=ae,ae=null):Ie=ae.sibling;var Fn=U(C,ae,Ne.value,V);if(Fn===null){ae===null&&(ae=Ie);break}e&&ae&&Fn.alternate===null&&t(C,ae),R=s(Fn,R,se),xe===null?te=Fn:xe.sibling=Fn,xe=Fn,ae=Ie}if(Ne.done)return n(C,ae),Te&&fl(C,se),te;if(ae===null){for(;!Ne.done;se++,Ne=O.next())Ne=Q(C,Ne.value,V),Ne!==null&&(R=s(Ne,R,se),xe===null?te=Ne:xe.sibling=Ne,xe=Ne);return Te&&fl(C,se),te}for(ae=l(ae);!Ne.done;se++,Ne=O.next())Ne=k(ae,C,se,Ne.value,V),Ne!==null&&(e&&Ne.alternate!==null&&ae.delete(Ne.key===null?se:Ne.key),R=s(Ne,R,se),xe===null?te=Ne:xe.sibling=Ne,xe=Ne);return e&&ae.forEach(function(qb){return t(C,qb)}),Te&&fl(C,se),te}function Oe(C,R,O,V){if(typeof O=="object"&&O!==null&&O.type===T&&O.key===null&&(O=O.props.children),typeof O=="object"&&O!==null){switch(O.$$typeof){case w:e:{for(var te=O.key;R!==null;){if(R.key===te){if(te=O.type,te===T){if(R.tag===7){n(C,R.sibling),V=r(R,O.props.children),V.return=C,C=V;break e}}else if(R.elementType===te||typeof te=="object"&&te!==null&&te.$$typeof===Z&&om(te)===R.type){n(C,R.sibling),V=r(R,O.props),nr(V,O),V.return=C,C=V;break e}n(C,R);break}else t(C,R);R=R.sibling}O.type===T?(V=sl(O.props.children,C.mode,V,O.key),V.return=C,C=V):(V=lo(O.type,O.key,O.props,null,C.mode,V),nr(V,O),V.return=C,C=V)}return m(C);case M:e:{for(te=O.key;R!==null;){if(R.key===te)if(R.tag===4&&R.stateNode.containerInfo===O.containerInfo&&R.stateNode.implementation===O.implementation){n(C,R.sibling),V=r(R,O.children||[]),V.return=C,C=V;break e}else{n(C,R);break}else t(C,R);R=R.sibling}V=vc(O,C.mode,V),V.return=C,C=V}return m(C);case Z:return te=O._init,O=te(O._payload),Oe(C,R,O,V)}if(ve(O))return fe(C,R,O,V);if(de(O)){if(te=de(O),typeof te!="function")throw Error(c(150));return O=te.call(O),ce(C,R,O,V)}if(typeof O.then=="function")return Oe(C,R,So(O),V);if(O.$$typeof===L)return Oe(C,R,io(C,O),V);wo(C,O)}return typeof O=="string"&&O!==""||typeof O=="number"||typeof O=="bigint"?(O=""+O,R!==null&&R.tag===6?(n(C,R.sibling),V=r(R,O),V.return=C,C=V):(n(C,R),V=hc(O,C.mode,V),V.return=C,C=V),m(C)):n(C,R)}return function(C,R,O,V){try{tr=0;var te=Oe(C,R,O,V);return Fl=null,te}catch(ae){if(ae===Za||ae===so)throw ae;var xe=yt(29,ae,null,C.mode);return xe.lanes=V,xe.return=C,xe}finally{}}}var Il=im(!0),cm=im(!1),Ut=X(null),Jt=null;function Ln(e){var t=e.alternate;$(Pe,Pe.current&1),$(Ut,e),Jt===null&&(t===null||Pl.current!==null||t.memoizedState!==null)&&(Jt=e)}function sm(e){if(e.tag===22){if($(Pe,Pe.current),$(Ut,e),Jt===null){var t=e.alternate;t!==null&&t.memoizedState!==null&&(Jt=e)}}else Hn()}function Hn(){$(Pe,Pe.current),$(Ut,Ut.current)}function gn(e){F(Ut),Jt===e&&(Jt=null),F(Pe)}var Pe=X(0);function Eo(e){for(var t=e;t!==null;){if(t.tag===13){var n=t.memoizedState;if(n!==null&&(n=n.dehydrated,n===null||n.data==="$?"||Ls(n)))return t}else if(t.tag===19&&t.memoizedProps.revealOrder!==void 0){if((t.flags&128)!==0)return t}else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}function Pc(e,t,n,l){t=e.memoizedState,n=n(l,t),n=n==null?t:p({},t,n),e.memoizedState=n,e.lanes===0&&(e.updateQueue.baseState=n)}var $c={enqueueSetState:function(e,t,n){e=e._reactInternals;var l=wt(),r=zn(l);r.payload=t,n!=null&&(r.callback=n),t=Un(e,r,l),t!==null&&(Et(t,e,l),Pa(t,e,l))},enqueueReplaceState:function(e,t,n){e=e._reactInternals;var l=wt(),r=zn(l);r.tag=1,r.payload=t,n!=null&&(r.callback=n),t=Un(e,r,l),t!==null&&(Et(t,e,l),Pa(t,e,l))},enqueueForceUpdate:function(e,t){e=e._reactInternals;var n=wt(),l=zn(n);l.tag=2,t!=null&&(l.callback=t),t=Un(e,l,n),t!==null&&(Et(t,e,n),Pa(t,e,n))}};function um(e,t,n,l,r,s,m){return e=e.stateNode,typeof e.shouldComponentUpdate=="function"?e.shouldComponentUpdate(l,s,m):t.prototype&&t.prototype.isPureReactComponent?!Ha(n,l)||!Ha(r,s):!0}function fm(e,t,n,l){e=t.state,typeof t.componentWillReceiveProps=="function"&&t.componentWillReceiveProps(n,l),typeof t.UNSAFE_componentWillReceiveProps=="function"&&t.UNSAFE_componentWillReceiveProps(n,l),t.state!==e&&$c.enqueueReplaceState(t,t.state,null)}function yl(e,t){var n=t;if("ref"in t){n={};for(var l in t)l!=="ref"&&(n[l]=t[l])}if(e=e.defaultProps){n===t&&(n=p({},n));for(var r in e)n[r]===void 0&&(n[r]=e[r])}return n}var Ao=typeof reportError=="function"?reportError:function(e){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var t=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof e=="object"&&e!==null&&typeof e.message=="string"?String(e.message):String(e),error:e});if(!window.dispatchEvent(t))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",e);return}console.error(e)};function dm(e){Ao(e)}function mm(e){console.error(e)}function hm(e){Ao(e)}function Mo(e,t){try{var n=e.onUncaughtError;n(t.value,{componentStack:t.stack})}catch(l){setTimeout(function(){throw l})}}function vm(e,t,n){try{var l=e.onCaughtError;l(n.value,{componentStack:n.stack,errorBoundary:t.tag===1?t.stateNode:null})}catch(r){setTimeout(function(){throw r})}}function Jc(e,t,n){return n=zn(n),n.tag=3,n.payload={element:null},n.callback=function(){Mo(e,t)},n}function gm(e){return e=zn(e),e.tag=3,e}function pm(e,t,n,l){var r=n.type.getDerivedStateFromError;if(typeof r=="function"){var s=l.value;e.payload=function(){return r(s)},e.callback=function(){vm(t,n,l)}}var m=n.stateNode;m!==null&&typeof m.componentDidCatch=="function"&&(e.callback=function(){vm(t,n,l),typeof r!="function"&&(Xn===null?Xn=new Set([this]):Xn.add(this));var y=l.stack;this.componentDidCatch(l.value,{componentStack:y!==null?y:""})})}function q0(e,t,n,l,r){if(n.flags|=32768,l!==null&&typeof l=="object"&&typeof l.then=="function"){if(t=n.alternate,t!==null&&Va(t,n,r,!0),n=Ut.current,n!==null){switch(n.tag){case 13:return Jt===null?xs():n.alternate===null&&qe===0&&(qe=3),n.flags&=-257,n.flags|=65536,n.lanes=r,l===Mc?n.flags|=16384:(t=n.updateQueue,t===null?n.updateQueue=new Set([l]):t.add(l),ws(e,l,r)),!1;case 22:return n.flags|=65536,l===Mc?n.flags|=16384:(t=n.updateQueue,t===null?(t={transitions:null,markerInstances:null,retryQueue:new Set([l])},n.updateQueue=t):(n=t.retryQueue,n===null?t.retryQueue=new Set([l]):n.add(l)),ws(e,l,r)),!1}throw Error(c(435,n.tag))}return ws(e,l,r),xs(),!1}if(Te)return t=Ut.current,t!==null?((t.flags&65536)===0&&(t.flags|=256),t.flags|=65536,t.lanes=r,l!==yc&&(e=Error(c(422),{cause:l}),Ya(jt(e,n)))):(l!==yc&&(t=Error(c(423),{cause:l}),Ya(jt(t,n))),e=e.current.alternate,e.flags|=65536,r&=-r,e.lanes|=r,l=jt(l,n),r=Jc(e.stateNode,l,r),Rc(e,r),qe!==4&&(qe=2)),!1;var s=Error(c(520),{cause:l});if(s=jt(s,n),sr===null?sr=[s]:sr.push(s),qe!==4&&(qe=2),t===null)return!0;l=jt(l,n),n=t;do{switch(n.tag){case 3:return n.flags|=65536,e=r&-r,n.lanes|=e,e=Jc(n.stateNode,l,e),Rc(n,e),!1;case 1:if(t=n.type,s=n.stateNode,(n.flags&128)===0&&(typeof t.getDerivedStateFromError=="function"||s!==null&&typeof s.componentDidCatch=="function"&&(Xn===null||!Xn.has(s))))return n.flags|=65536,r&=-r,n.lanes|=r,r=gm(r),pm(r,e,n,l),Rc(n,r),!1}n=n.return}while(n!==null);return!1}var ym=Error(c(461)),We=!1;function et(e,t,n,l){t.child=e===null?cm(t,null,n,l):Il(t,e.child,n,l)}function bm(e,t,n,l,r){n=n.render;var s=t.ref;if("ref"in l){var m={};for(var y in l)y!=="ref"&&(m[y]=l[y])}else m=l;return vl(t),l=Dc(e,t,n,m,s,r),y=zc(),e!==null&&!We?(Uc(e,t,r),pn(e,t,r)):(Te&&y&&gc(t),t.flags|=1,et(e,t,l,r),t.child)}function xm(e,t,n,l,r){if(e===null){var s=n.type;return typeof s=="function"&&!mc(s)&&s.defaultProps===void 0&&n.compare===null?(t.tag=15,t.type=s,Sm(e,t,s,l,r)):(e=lo(n.type,null,l,t,t.mode,r),e.ref=t.ref,e.return=t,t.child=e)}if(s=e.child,!as(e,r)){var m=s.memoizedProps;if(n=n.compare,n=n!==null?n:Ha,n(m,l)&&e.ref===t.ref)return pn(e,t,r)}return t.flags|=1,e=un(s,l),e.ref=t.ref,e.return=t,t.child=e}function Sm(e,t,n,l,r){if(e!==null){var s=e.memoizedProps;if(Ha(s,l)&&e.ref===t.ref)if(We=!1,t.pendingProps=l=s,as(e,r))(e.flags&131072)!==0&&(We=!0);else return t.lanes=e.lanes,pn(e,t,r)}return Wc(e,t,n,l,r)}function wm(e,t,n){var l=t.pendingProps,r=l.children,s=e!==null?e.memoizedState:null;if(l.mode==="hidden"){if((t.flags&128)!==0){if(l=s!==null?s.baseLanes|n:n,e!==null){for(r=t.child=e.child,s=0;r!==null;)s=s|r.lanes|r.childLanes,r=r.sibling;t.childLanes=s&~l}else t.childLanes=0,t.child=null;return Em(e,t,l,n)}if((n&536870912)!==0)t.memoizedState={baseLanes:0,cachePool:null},e!==null&&co(t,s!==null?s.cachePool:null),s!==null?Sd(t,s):Cc(),sm(t);else return t.lanes=t.childLanes=536870912,Em(e,t,s!==null?s.baseLanes|n:n,n)}else s!==null?(co(t,s.cachePool),Sd(t,s),Hn(),t.memoizedState=null):(e!==null&&co(t,null),Cc(),Hn());return et(e,t,r,n),t.child}function Em(e,t,n,l){var r=Ac();return r=r===null?null:{parent:Ke._currentValue,pool:r},t.memoizedState={baseLanes:n,cachePool:r},e!==null&&co(t,null),Cc(),sm(t),e!==null&&Va(e,t,l,!0),null}function No(e,t){var n=t.ref;if(n===null)e!==null&&e.ref!==null&&(t.flags|=4194816);else{if(typeof n!="function"&&typeof n!="object")throw Error(c(284));(e===null||e.ref!==n)&&(t.flags|=4194816)}}function Wc(e,t,n,l,r){return vl(t),n=Dc(e,t,n,l,void 0,r),l=zc(),e!==null&&!We?(Uc(e,t,r),pn(e,t,r)):(Te&&l&&gc(t),t.flags|=1,et(e,t,n,r),t.child)}function Am(e,t,n,l,r,s){return vl(t),t.updateQueue=null,n=Ed(t,l,n,r),wd(e),l=zc(),e!==null&&!We?(Uc(e,t,s),pn(e,t,s)):(Te&&l&&gc(t),t.flags|=1,et(e,t,n,s),t.child)}function Mm(e,t,n,l,r){if(vl(t),t.stateNode===null){var s=Vl,m=n.contextType;typeof m=="object"&&m!==null&&(s=at(m)),s=new n(l,s),t.memoizedState=s.state!==null&&s.state!==void 0?s.state:null,s.updater=$c,t.stateNode=s,s._reactInternals=t,s=t.stateNode,s.props=l,s.state=t.memoizedState,s.refs={},Nc(t),m=n.contextType,s.context=typeof m=="object"&&m!==null?at(m):Vl,s.state=t.memoizedState,m=n.getDerivedStateFromProps,typeof m=="function"&&(Pc(t,n,m,l),s.state=t.memoizedState),typeof n.getDerivedStateFromProps=="function"||typeof s.getSnapshotBeforeUpdate=="function"||typeof s.UNSAFE_componentWillMount!="function"&&typeof s.componentWillMount!="function"||(m=s.state,typeof s.componentWillMount=="function"&&s.componentWillMount(),typeof s.UNSAFE_componentWillMount=="function"&&s.UNSAFE_componentWillMount(),m!==s.state&&$c.enqueueReplaceState(s,s.state,null),Ja(t,l,s,r),$a(),s.state=t.memoizedState),typeof s.componentDidMount=="function"&&(t.flags|=4194308),l=!0}else if(e===null){s=t.stateNode;var y=t.memoizedProps,E=yl(n,y);s.props=E;var z=s.context,G=n.contextType;m=Vl,typeof G=="object"&&G!==null&&(m=at(G));var Q=n.getDerivedStateFromProps;G=typeof Q=="function"||typeof s.getSnapshotBeforeUpdate=="function",y=t.pendingProps!==y,G||typeof s.UNSAFE_componentWillReceiveProps!="function"&&typeof s.componentWillReceiveProps!="function"||(y||z!==m)&&fm(t,s,l,m),Dn=!1;var U=t.memoizedState;s.state=U,Ja(t,l,s,r),$a(),z=t.memoizedState,y||U!==z||Dn?(typeof Q=="function"&&(Pc(t,n,Q,l),z=t.memoizedState),(E=Dn||um(t,n,E,l,U,z,m))?(G||typeof s.UNSAFE_componentWillMount!="function"&&typeof s.componentWillMount!="function"||(typeof s.componentWillMount=="function"&&s.componentWillMount(),typeof s.UNSAFE_componentWillMount=="function"&&s.UNSAFE_componentWillMount()),typeof s.componentDidMount=="function"&&(t.flags|=4194308)):(typeof s.componentDidMount=="function"&&(t.flags|=4194308),t.memoizedProps=l,t.memoizedState=z),s.props=l,s.state=z,s.context=m,l=E):(typeof s.componentDidMount=="function"&&(t.flags|=4194308),l=!1)}else{s=t.stateNode,Tc(e,t),m=t.memoizedProps,G=yl(n,m),s.props=G,Q=t.pendingProps,U=s.context,z=n.contextType,E=Vl,typeof z=="object"&&z!==null&&(E=at(z)),y=n.getDerivedStateFromProps,(z=typeof y=="function"||typeof s.getSnapshotBeforeUpdate=="function")||typeof s.UNSAFE_componentWillReceiveProps!="function"&&typeof s.componentWillReceiveProps!="function"||(m!==Q||U!==E)&&fm(t,s,l,E),Dn=!1,U=t.memoizedState,s.state=U,Ja(t,l,s,r),$a();var k=t.memoizedState;m!==Q||U!==k||Dn||e!==null&&e.dependencies!==null&&oo(e.dependencies)?(typeof y=="function"&&(Pc(t,n,y,l),k=t.memoizedState),(G=Dn||um(t,n,G,l,U,k,E)||e!==null&&e.dependencies!==null&&oo(e.dependencies))?(z||typeof s.UNSAFE_componentWillUpdate!="function"&&typeof s.componentWillUpdate!="function"||(typeof s.componentWillUpdate=="function"&&s.componentWillUpdate(l,k,E),typeof s.UNSAFE_componentWillUpdate=="function"&&s.UNSAFE_componentWillUpdate(l,k,E)),typeof s.componentDidUpdate=="function"&&(t.flags|=4),typeof s.getSnapshotBeforeUpdate=="function"&&(t.flags|=1024)):(typeof s.componentDidUpdate!="function"||m===e.memoizedProps&&U===e.memoizedState||(t.flags|=4),typeof s.getSnapshotBeforeUpdate!="function"||m===e.memoizedProps&&U===e.memoizedState||(t.flags|=1024),t.memoizedProps=l,t.memoizedState=k),s.props=l,s.state=k,s.context=E,l=G):(typeof s.componentDidUpdate!="function"||m===e.memoizedProps&&U===e.memoizedState||(t.flags|=4),typeof s.getSnapshotBeforeUpdate!="function"||m===e.memoizedProps&&U===e.memoizedState||(t.flags|=1024),l=!1)}return s=l,No(e,t),l=(t.flags&128)!==0,s||l?(s=t.stateNode,n=l&&typeof n.getDerivedStateFromError!="function"?null:s.render(),t.flags|=1,e!==null&&l?(t.child=Il(t,e.child,null,r),t.child=Il(t,null,n,r)):et(e,t,n,r),t.memoizedState=s.state,e=t.child):e=pn(e,t,r),e}function Nm(e,t,n,l){return Ga(),t.flags|=256,et(e,t,n,l),t.child}var Fc={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function Ic(e){return{baseLanes:e,cachePool:md()}}function es(e,t,n){return e=e!==null?e.childLanes&~n:0,t&&(e|=kt),e}function Tm(e,t,n){var l=t.pendingProps,r=!1,s=(t.flags&128)!==0,m;if((m=s)||(m=e!==null&&e.memoizedState===null?!1:(Pe.current&2)!==0),m&&(r=!0,t.flags&=-129),m=(t.flags&32)!==0,t.flags&=-33,e===null){if(Te){if(r?Ln(t):Hn(),Te){var y=Be,E;if(E=y){e:{for(E=y,y=$t;E.nodeType!==8;){if(!y){y=null;break e}if(E=Yt(E.nextSibling),E===null){y=null;break e}}y=E}y!==null?(t.memoizedState={dehydrated:y,treeContext:ul!==null?{id:fn,overflow:dn}:null,retryLane:536870912,hydrationErrors:null},E=yt(18,null,null,0),E.stateNode=y,E.return=t,t.child=E,ct=t,Be=null,E=!0):E=!1}E||ml(t)}if(y=t.memoizedState,y!==null&&(y=y.dehydrated,y!==null))return Ls(y)?t.lanes=32:t.lanes=536870912,null;gn(t)}return y=l.children,l=l.fallback,r?(Hn(),r=t.mode,y=To({mode:"hidden",children:y},r),l=sl(l,r,n,null),y.return=t,l.return=t,y.sibling=l,t.child=y,r=t.child,r.memoizedState=Ic(n),r.childLanes=es(e,m,n),t.memoizedState=Fc,l):(Ln(t),ts(t,y))}if(E=e.memoizedState,E!==null&&(y=E.dehydrated,y!==null)){if(s)t.flags&256?(Ln(t),t.flags&=-257,t=ns(e,t,n)):t.memoizedState!==null?(Hn(),t.child=e.child,t.flags|=128,t=null):(Hn(),r=l.fallback,y=t.mode,l=To({mode:"visible",children:l.children},y),r=sl(r,y,n,null),r.flags|=2,l.return=t,r.return=t,l.sibling=r,t.child=l,Il(t,e.child,null,n),l=t.child,l.memoizedState=Ic(n),l.childLanes=es(e,m,n),t.memoizedState=Fc,t=r);else if(Ln(t),Ls(y)){if(m=y.nextSibling&&y.nextSibling.dataset,m)var z=m.dgst;m=z,l=Error(c(419)),l.stack="",l.digest=m,Ya({value:l,source:null,stack:null}),t=ns(e,t,n)}else if(We||Va(e,t,n,!1),m=(n&e.childLanes)!==0,We||m){if(m=Ue,m!==null&&(l=n&-n,l=(l&42)!==0?1:Hi(l),l=(l&(m.suspendedLanes|n))!==0?0:l,l!==0&&l!==E.retryLane))throw E.retryLane=l,Yl(e,l),Et(m,e,l),ym;y.data==="$?"||xs(),t=ns(e,t,n)}else y.data==="$?"?(t.flags|=192,t.child=e.child,t=null):(e=E.treeContext,Be=Yt(y.nextSibling),ct=t,Te=!0,dl=null,$t=!1,e!==null&&(Dt[zt++]=fn,Dt[zt++]=dn,Dt[zt++]=ul,fn=e.id,dn=e.overflow,ul=t),t=ts(t,l.children),t.flags|=4096);return t}return r?(Hn(),r=l.fallback,y=t.mode,E=e.child,z=E.sibling,l=un(E,{mode:"hidden",children:l.children}),l.subtreeFlags=E.subtreeFlags&65011712,z!==null?r=un(z,r):(r=sl(r,y,n,null),r.flags|=2),r.return=t,l.return=t,l.sibling=r,t.child=l,l=r,r=t.child,y=e.child.memoizedState,y===null?y=Ic(n):(E=y.cachePool,E!==null?(z=Ke._currentValue,E=E.parent!==z?{parent:z,pool:z}:E):E=md(),y={baseLanes:y.baseLanes|n,cachePool:E}),r.memoizedState=y,r.childLanes=es(e,m,n),t.memoizedState=Fc,l):(Ln(t),n=e.child,e=n.sibling,n=un(n,{mode:"visible",children:l.children}),n.return=t,n.sibling=null,e!==null&&(m=t.deletions,m===null?(t.deletions=[e],t.flags|=16):m.push(e)),t.child=n,t.memoizedState=null,n)}function ts(e,t){return t=To({mode:"visible",children:t},e.mode),t.return=e,e.child=t}function To(e,t){return e=yt(22,e,null,t),e.lanes=0,e.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},e}function ns(e,t,n){return Il(t,e.child,null,n),e=ts(t,t.pendingProps.children),e.flags|=2,t.memoizedState=null,e}function Rm(e,t,n){e.lanes|=t;var l=e.alternate;l!==null&&(l.lanes|=t),xc(e.return,t,n)}function ls(e,t,n,l,r){var s=e.memoizedState;s===null?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:l,tail:n,tailMode:r}:(s.isBackwards=t,s.rendering=null,s.renderingStartTime=0,s.last=l,s.tail=n,s.tailMode=r)}function _m(e,t,n){var l=t.pendingProps,r=l.revealOrder,s=l.tail;if(et(e,t,l.children,n),l=Pe.current,(l&2)!==0)l=l&1|2,t.flags|=128;else{if(e!==null&&(e.flags&128)!==0)e:for(e=t.child;e!==null;){if(e.tag===13)e.memoizedState!==null&&Rm(e,n,t);else if(e.tag===19)Rm(e,n,t);else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;e.sibling===null;){if(e.return===null||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}l&=1}switch($(Pe,l),r){case"forwards":for(n=t.child,r=null;n!==null;)e=n.alternate,e!==null&&Eo(e)===null&&(r=n),n=n.sibling;n=r,n===null?(r=t.child,t.child=null):(r=n.sibling,n.sibling=null),ls(t,!1,r,n,s);break;case"backwards":for(n=null,r=t.child,t.child=null;r!==null;){if(e=r.alternate,e!==null&&Eo(e)===null){t.child=r;break}e=r.sibling,r.sibling=n,n=r,r=e}ls(t,!0,n,null,s);break;case"together":ls(t,!1,null,null,void 0);break;default:t.memoizedState=null}return t.child}function pn(e,t,n){if(e!==null&&(t.dependencies=e.dependencies),Vn|=t.lanes,(n&t.childLanes)===0)if(e!==null){if(Va(e,t,n,!1),(n&t.childLanes)===0)return null}else return null;if(e!==null&&t.child!==e.child)throw Error(c(153));if(t.child!==null){for(e=t.child,n=un(e,e.pendingProps),t.child=n,n.return=t;e.sibling!==null;)e=e.sibling,n=n.sibling=un(e,e.pendingProps),n.return=t;n.sibling=null}return t.child}function as(e,t){return(e.lanes&t)!==0?!0:(e=e.dependencies,!!(e!==null&&oo(e)))}function G0(e,t,n){switch(t.tag){case 3:Me(t,t.stateNode.containerInfo),On(t,Ke,e.memoizedState.cache),Ga();break;case 27:case 5:Qt(t);break;case 4:Me(t,t.stateNode.containerInfo);break;case 10:On(t,t.type,t.memoizedProps.value);break;case 13:var l=t.memoizedState;if(l!==null)return l.dehydrated!==null?(Ln(t),t.flags|=128,null):(n&t.child.childLanes)!==0?Tm(e,t,n):(Ln(t),e=pn(e,t,n),e!==null?e.sibling:null);Ln(t);break;case 19:var r=(e.flags&128)!==0;if(l=(n&t.childLanes)!==0,l||(Va(e,t,n,!1),l=(n&t.childLanes)!==0),r){if(l)return _m(e,t,n);t.flags|=128}if(r=t.memoizedState,r!==null&&(r.rendering=null,r.tail=null,r.lastEffect=null),$(Pe,Pe.current),l)break;return null;case 22:case 23:return t.lanes=0,wm(e,t,n);case 24:On(t,Ke,e.memoizedState.cache)}return pn(e,t,n)}function Cm(e,t,n){if(e!==null)if(e.memoizedProps!==t.pendingProps)We=!0;else{if(!as(e,n)&&(t.flags&128)===0)return We=!1,G0(e,t,n);We=(e.flags&131072)!==0}else We=!1,Te&&(t.flags&1048576)!==0&&od(t,ro,t.index);switch(t.lanes=0,t.tag){case 16:e:{e=t.pendingProps;var l=t.elementType,r=l._init;if(l=r(l._payload),t.type=l,typeof l=="function")mc(l)?(e=yl(l,e),t.tag=1,t=Mm(null,t,l,e,n)):(t.tag=0,t=Wc(null,t,l,e,n));else{if(l!=null){if(r=l.$$typeof,r===Y){t.tag=11,t=bm(null,t,l,e,n);break e}else if(r===P){t.tag=14,t=xm(null,t,l,e,n);break e}}throw t=be(l)||l,Error(c(306,t,""))}}return t;case 0:return Wc(e,t,t.type,t.pendingProps,n);case 1:return l=t.type,r=yl(l,t.pendingProps),Mm(e,t,l,r,n);case 3:e:{if(Me(t,t.stateNode.containerInfo),e===null)throw Error(c(387));l=t.pendingProps;var s=t.memoizedState;r=s.element,Tc(e,t),Ja(t,l,null,n);var m=t.memoizedState;if(l=m.cache,On(t,Ke,l),l!==s.cache&&Sc(t,[Ke],n,!0),$a(),l=m.element,s.isDehydrated)if(s={element:l,isDehydrated:!1,cache:m.cache},t.updateQueue.baseState=s,t.memoizedState=s,t.flags&256){t=Nm(e,t,l,n);break e}else if(l!==r){r=jt(Error(c(424)),t),Ya(r),t=Nm(e,t,l,n);break e}else{switch(e=t.stateNode.containerInfo,e.nodeType){case 9:e=e.body;break;default:e=e.nodeName==="HTML"?e.ownerDocument.body:e}for(Be=Yt(e.firstChild),ct=t,Te=!0,dl=null,$t=!0,n=cm(t,null,l,n),t.child=n;n;)n.flags=n.flags&-3|4096,n=n.sibling}else{if(Ga(),l===r){t=pn(e,t,n);break e}et(e,t,l,n)}t=t.child}return t;case 26:return No(e,t),e===null?(n=zh(t.type,null,t.pendingProps,null))?t.memoizedState=n:Te||(n=t.type,e=t.pendingProps,l=Go(oe.current).createElement(n),l[lt]=t,l[ut]=e,nt(l,n,e),Je(l),t.stateNode=l):t.memoizedState=zh(t.type,e.memoizedProps,t.pendingProps,e.memoizedState),null;case 27:return Qt(t),e===null&&Te&&(l=t.stateNode=jh(t.type,t.pendingProps,oe.current),ct=t,$t=!0,r=Be,Kn(t.type)?(Hs=r,Be=Yt(l.firstChild)):Be=r),et(e,t,t.pendingProps.children,n),No(e,t),e===null&&(t.flags|=4194304),t.child;case 5:return e===null&&Te&&((r=l=Be)&&(l=vb(l,t.type,t.pendingProps,$t),l!==null?(t.stateNode=l,ct=t,Be=Yt(l.firstChild),$t=!1,r=!0):r=!1),r||ml(t)),Qt(t),r=t.type,s=t.pendingProps,m=e!==null?e.memoizedProps:null,l=s.children,zs(r,s)?l=null:m!==null&&zs(r,m)&&(t.flags|=32),t.memoizedState!==null&&(r=Dc(e,t,D0,null,null,n),yr._currentValue=r),No(e,t),et(e,t,l,n),t.child;case 6:return e===null&&Te&&((e=n=Be)&&(n=gb(n,t.pendingProps,$t),n!==null?(t.stateNode=n,ct=t,Be=null,e=!0):e=!1),e||ml(t)),null;case 13:return Tm(e,t,n);case 4:return Me(t,t.stateNode.containerInfo),l=t.pendingProps,e===null?t.child=Il(t,null,l,n):et(e,t,l,n),t.child;case 11:return bm(e,t,t.type,t.pendingProps,n);case 7:return et(e,t,t.pendingProps,n),t.child;case 8:return et(e,t,t.pendingProps.children,n),t.child;case 12:return et(e,t,t.pendingProps.children,n),t.child;case 10:return l=t.pendingProps,On(t,t.type,l.value),et(e,t,l.children,n),t.child;case 9:return r=t.type._context,l=t.pendingProps.children,vl(t),r=at(r),l=l(r),t.flags|=1,et(e,t,l,n),t.child;case 14:return xm(e,t,t.type,t.pendingProps,n);case 15:return Sm(e,t,t.type,t.pendingProps,n);case 19:return _m(e,t,n);case 31:return l=t.pendingProps,n=t.mode,l={mode:l.mode,children:l.children},e===null?(n=To(l,n),n.ref=t.ref,t.child=n,n.return=t,t=n):(n=un(e.child,l),n.ref=t.ref,t.child=n,n.return=t,t=n),t;case 22:return wm(e,t,n);case 24:return vl(t),l=at(Ke),e===null?(r=Ac(),r===null&&(r=Ue,s=wc(),r.pooledCache=s,s.refCount++,s!==null&&(r.pooledCacheLanes|=n),r=s),t.memoizedState={parent:l,cache:r},Nc(t),On(t,Ke,r)):((e.lanes&n)!==0&&(Tc(e,t),Ja(t,null,null,n),$a()),r=e.memoizedState,s=t.memoizedState,r.parent!==l?(r={parent:l,cache:l},t.memoizedState=r,t.lanes===0&&(t.memoizedState=t.updateQueue.baseState=r),On(t,Ke,l)):(l=s.cache,On(t,Ke,l),l!==r.cache&&Sc(t,[Ke],n,!0))),et(e,t,t.pendingProps.children,n),t.child;case 29:throw t.pendingProps}throw Error(c(156,t.tag))}function yn(e){e.flags|=4}function jm(e,t){if(t.type!=="stylesheet"||(t.state.loading&4)!==0)e.flags&=-16777217;else if(e.flags|=16777216,!Bh(t)){if(t=Ut.current,t!==null&&((Ae&4194048)===Ae?Jt!==null:(Ae&62914560)!==Ae&&(Ae&536870912)===0||t!==Jt))throw Ka=Mc,hd;e.flags|=8192}}function Ro(e,t){t!==null&&(e.flags|=4),e.flags&16384&&(t=e.tag!==22?sf():536870912,e.lanes|=t,la|=t)}function lr(e,t){if(!Te)switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;t!==null;)t.alternate!==null&&(n=t),t=t.sibling;n===null?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var l=null;n!==null;)n.alternate!==null&&(l=n),n=n.sibling;l===null?t||e.tail===null?e.tail=null:e.tail.sibling=null:l.sibling=null}}function He(e){var t=e.alternate!==null&&e.alternate.child===e.child,n=0,l=0;if(t)for(var r=e.child;r!==null;)n|=r.lanes|r.childLanes,l|=r.subtreeFlags&65011712,l|=r.flags&65011712,r.return=e,r=r.sibling;else for(r=e.child;r!==null;)n|=r.lanes|r.childLanes,l|=r.subtreeFlags,l|=r.flags,r.return=e,r=r.sibling;return e.subtreeFlags|=l,e.childLanes=n,t}function Y0(e,t,n){var l=t.pendingProps;switch(pc(t),t.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return He(t),null;case 1:return He(t),null;case 3:return n=t.stateNode,l=null,e!==null&&(l=e.memoizedState.cache),t.memoizedState.cache!==l&&(t.flags|=2048),hn(Ke),it(),n.pendingContext&&(n.context=n.pendingContext,n.pendingContext=null),(e===null||e.child===null)&&(qa(t)?yn(t):e===null||e.memoizedState.isDehydrated&&(t.flags&256)===0||(t.flags|=1024,sd())),He(t),null;case 26:return n=t.memoizedState,e===null?(yn(t),n!==null?(He(t),jm(t,n)):(He(t),t.flags&=-16777217)):n?n!==e.memoizedState?(yn(t),He(t),jm(t,n)):(He(t),t.flags&=-16777217):(e.memoizedProps!==l&&yn(t),He(t),t.flags&=-16777217),null;case 27:Zt(t),n=oe.current;var r=t.type;if(e!==null&&t.stateNode!=null)e.memoizedProps!==l&&yn(t);else{if(!l){if(t.stateNode===null)throw Error(c(166));return He(t),null}e=ee.current,qa(t)?id(t):(e=jh(r,l,n),t.stateNode=e,yn(t))}return He(t),null;case 5:if(Zt(t),n=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==l&&yn(t);else{if(!l){if(t.stateNode===null)throw Error(c(166));return He(t),null}if(e=ee.current,qa(t))id(t);else{switch(r=Go(oe.current),e){case 1:e=r.createElementNS("http://www.w3.org/2000/svg",n);break;case 2:e=r.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;default:switch(n){case"svg":e=r.createElementNS("http://www.w3.org/2000/svg",n);break;case"math":e=r.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;case"script":e=r.createElement("div"),e.innerHTML="<script><\/script>",e=e.removeChild(e.firstChild);break;case"select":e=typeof l.is=="string"?r.createElement("select",{is:l.is}):r.createElement("select"),l.multiple?e.multiple=!0:l.size&&(e.size=l.size);break;default:e=typeof l.is=="string"?r.createElement(n,{is:l.is}):r.createElement(n)}}e[lt]=t,e[ut]=l;e:for(r=t.child;r!==null;){if(r.tag===5||r.tag===6)e.appendChild(r.stateNode);else if(r.tag!==4&&r.tag!==27&&r.child!==null){r.child.return=r,r=r.child;continue}if(r===t)break e;for(;r.sibling===null;){if(r.return===null||r.return===t)break e;r=r.return}r.sibling.return=r.return,r=r.sibling}t.stateNode=e;e:switch(nt(e,n,l),n){case"button":case"input":case"select":case"textarea":e=!!l.autoFocus;break e;case"img":e=!0;break e;default:e=!1}e&&yn(t)}}return He(t),t.flags&=-16777217,null;case 6:if(e&&t.stateNode!=null)e.memoizedProps!==l&&yn(t);else{if(typeof l!="string"&&t.stateNode===null)throw Error(c(166));if(e=oe.current,qa(t)){if(e=t.stateNode,n=t.memoizedProps,l=null,r=ct,r!==null)switch(r.tag){case 27:case 5:l=r.memoizedProps}e[lt]=t,e=!!(e.nodeValue===n||l!==null&&l.suppressHydrationWarning===!0||Ah(e.nodeValue,n)),e||ml(t)}else e=Go(e).createTextNode(l),e[lt]=t,t.stateNode=e}return He(t),null;case 13:if(l=t.memoizedState,e===null||e.memoizedState!==null&&e.memoizedState.dehydrated!==null){if(r=qa(t),l!==null&&l.dehydrated!==null){if(e===null){if(!r)throw Error(c(318));if(r=t.memoizedState,r=r!==null?r.dehydrated:null,!r)throw Error(c(317));r[lt]=t}else Ga(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;He(t),r=!1}else r=sd(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=r),r=!0;if(!r)return t.flags&256?(gn(t),t):(gn(t),null)}if(gn(t),(t.flags&128)!==0)return t.lanes=n,t;if(n=l!==null,e=e!==null&&e.memoizedState!==null,n){l=t.child,r=null,l.alternate!==null&&l.alternate.memoizedState!==null&&l.alternate.memoizedState.cachePool!==null&&(r=l.alternate.memoizedState.cachePool.pool);var s=null;l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(s=l.memoizedState.cachePool.pool),s!==r&&(l.flags|=2048)}return n!==e&&n&&(t.child.flags|=8192),Ro(t,t.updateQueue),He(t),null;case 4:return it(),e===null&&_s(t.stateNode.containerInfo),He(t),null;case 10:return hn(t.type),He(t),null;case 19:if(F(Pe),r=t.memoizedState,r===null)return He(t),null;if(l=(t.flags&128)!==0,s=r.rendering,s===null)if(l)lr(r,!1);else{if(qe!==0||e!==null&&(e.flags&128)!==0)for(e=t.child;e!==null;){if(s=Eo(e),s!==null){for(t.flags|=128,lr(r,!1),e=s.updateQueue,t.updateQueue=e,Ro(t,e),t.subtreeFlags=0,e=n,n=t.child;n!==null;)rd(n,e),n=n.sibling;return $(Pe,Pe.current&1|2),t.child}e=e.sibling}r.tail!==null&&Pt()>jo&&(t.flags|=128,l=!0,lr(r,!1),t.lanes=4194304)}else{if(!l)if(e=Eo(s),e!==null){if(t.flags|=128,l=!0,e=e.updateQueue,t.updateQueue=e,Ro(t,e),lr(r,!0),r.tail===null&&r.tailMode==="hidden"&&!s.alternate&&!Te)return He(t),null}else 2*Pt()-r.renderingStartTime>jo&&n!==536870912&&(t.flags|=128,l=!0,lr(r,!1),t.lanes=4194304);r.isBackwards?(s.sibling=t.child,t.child=s):(e=r.last,e!==null?e.sibling=s:t.child=s,r.last=s)}return r.tail!==null?(t=r.tail,r.rendering=t,r.tail=t.sibling,r.renderingStartTime=Pt(),t.sibling=null,e=Pe.current,$(Pe,l?e&1|2:e&1),t):(He(t),null);case 22:case 23:return gn(t),jc(),l=t.memoizedState!==null,e!==null?e.memoizedState!==null!==l&&(t.flags|=8192):l&&(t.flags|=8192),l?(n&536870912)!==0&&(t.flags&128)===0&&(He(t),t.subtreeFlags&6&&(t.flags|=8192)):He(t),n=t.updateQueue,n!==null&&Ro(t,n.retryQueue),n=null,e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),l=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(l=t.memoizedState.cachePool.pool),l!==n&&(t.flags|=2048),e!==null&&F(gl),null;case 24:return n=null,e!==null&&(n=e.memoizedState.cache),t.memoizedState.cache!==n&&(t.flags|=2048),hn(Ke),He(t),null;case 25:return null;case 30:return null}throw Error(c(156,t.tag))}function V0(e,t){switch(pc(t),t.tag){case 1:return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 3:return hn(Ke),it(),e=t.flags,(e&65536)!==0&&(e&128)===0?(t.flags=e&-65537|128,t):null;case 26:case 27:case 5:return Zt(t),null;case 13:if(gn(t),e=t.memoizedState,e!==null&&e.dehydrated!==null){if(t.alternate===null)throw Error(c(340));Ga()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 19:return F(Pe),null;case 4:return it(),null;case 10:return hn(t.type),null;case 22:case 23:return gn(t),jc(),e!==null&&F(gl),e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 24:return hn(Ke),null;case 25:return null;default:return null}}function Om(e,t){switch(pc(t),t.tag){case 3:hn(Ke),it();break;case 26:case 27:case 5:Zt(t);break;case 4:it();break;case 13:gn(t);break;case 19:F(Pe);break;case 10:hn(t.type);break;case 22:case 23:gn(t),jc(),e!==null&&F(gl);break;case 24:hn(Ke)}}function ar(e,t){try{var n=t.updateQueue,l=n!==null?n.lastEffect:null;if(l!==null){var r=l.next;n=r;do{if((n.tag&e)===e){l=void 0;var s=n.create,m=n.inst;l=s(),m.destroy=l}n=n.next}while(n!==r)}}catch(y){De(t,t.return,y)}}function Bn(e,t,n){try{var l=t.updateQueue,r=l!==null?l.lastEffect:null;if(r!==null){var s=r.next;l=s;do{if((l.tag&e)===e){var m=l.inst,y=m.destroy;if(y!==void 0){m.destroy=void 0,r=t;var E=n,z=y;try{z()}catch(G){De(r,E,G)}}}l=l.next}while(l!==s)}}catch(G){De(t,t.return,G)}}function Dm(e){var t=e.updateQueue;if(t!==null){var n=e.stateNode;try{xd(t,n)}catch(l){De(e,e.return,l)}}}function zm(e,t,n){n.props=yl(e.type,e.memoizedProps),n.state=e.memoizedState;try{n.componentWillUnmount()}catch(l){De(e,t,l)}}function rr(e,t){try{var n=e.ref;if(n!==null){switch(e.tag){case 26:case 27:case 5:var l=e.stateNode;break;case 30:l=e.stateNode;break;default:l=e.stateNode}typeof n=="function"?e.refCleanup=n(l):n.current=l}}catch(r){De(e,t,r)}}function Wt(e,t){var n=e.ref,l=e.refCleanup;if(n!==null)if(typeof l=="function")try{l()}catch(r){De(e,t,r)}finally{e.refCleanup=null,e=e.alternate,e!=null&&(e.refCleanup=null)}else if(typeof n=="function")try{n(null)}catch(r){De(e,t,r)}else n.current=null}function Um(e){var t=e.type,n=e.memoizedProps,l=e.stateNode;try{e:switch(t){case"button":case"input":case"select":case"textarea":n.autoFocus&&l.focus();break e;case"img":n.src?l.src=n.src:n.srcSet&&(l.srcset=n.srcSet)}}catch(r){De(e,e.return,r)}}function rs(e,t,n){try{var l=e.stateNode;ub(l,e.type,n,t),l[ut]=t}catch(r){De(e,e.return,r)}}function km(e){return e.tag===5||e.tag===3||e.tag===26||e.tag===27&&Kn(e.type)||e.tag===4}function os(e){e:for(;;){for(;e.sibling===null;){if(e.return===null||km(e.return))return null;e=e.return}for(e.sibling.return=e.return,e=e.sibling;e.tag!==5&&e.tag!==6&&e.tag!==18;){if(e.tag===27&&Kn(e.type)||e.flags&2||e.child===null||e.tag===4)continue e;e.child.return=e,e=e.child}if(!(e.flags&2))return e.stateNode}}function is(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?(n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n).insertBefore(e,t):(t=n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n,t.appendChild(e),n=n._reactRootContainer,n!=null||t.onclick!==null||(t.onclick=qo));else if(l!==4&&(l===27&&Kn(e.type)&&(n=e.stateNode,t=null),e=e.child,e!==null))for(is(e,t,n),e=e.sibling;e!==null;)is(e,t,n),e=e.sibling}function _o(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?n.insertBefore(e,t):n.appendChild(e);else if(l!==4&&(l===27&&Kn(e.type)&&(n=e.stateNode),e=e.child,e!==null))for(_o(e,t,n),e=e.sibling;e!==null;)_o(e,t,n),e=e.sibling}function Lm(e){var t=e.stateNode,n=e.memoizedProps;try{for(var l=e.type,r=t.attributes;r.length;)t.removeAttributeNode(r[0]);nt(t,l,n),t[lt]=e,t[ut]=n}catch(s){De(e,e.return,s)}}var bn=!1,Ye=!1,cs=!1,Hm=typeof WeakSet=="function"?WeakSet:Set,Fe=null;function X0(e,t){if(e=e.containerInfo,Os=Ko,e=$f(e),oc(e)){if("selectionStart"in e)var n={start:e.selectionStart,end:e.selectionEnd};else e:{n=(n=e.ownerDocument)&&n.defaultView||window;var l=n.getSelection&&n.getSelection();if(l&&l.rangeCount!==0){n=l.anchorNode;var r=l.anchorOffset,s=l.focusNode;l=l.focusOffset;try{n.nodeType,s.nodeType}catch{n=null;break e}var m=0,y=-1,E=-1,z=0,G=0,Q=e,U=null;t:for(;;){for(var k;Q!==n||r!==0&&Q.nodeType!==3||(y=m+r),Q!==s||l!==0&&Q.nodeType!==3||(E=m+l),Q.nodeType===3&&(m+=Q.nodeValue.length),(k=Q.firstChild)!==null;)U=Q,Q=k;for(;;){if(Q===e)break t;if(U===n&&++z===r&&(y=m),U===s&&++G===l&&(E=m),(k=Q.nextSibling)!==null)break;Q=U,U=Q.parentNode}Q=k}n=y===-1||E===-1?null:{start:y,end:E}}else n=null}n=n||{start:0,end:0}}else n=null;for(Ds={focusedElem:e,selectionRange:n},Ko=!1,Fe=t;Fe!==null;)if(t=Fe,e=t.child,(t.subtreeFlags&1024)!==0&&e!==null)e.return=t,Fe=e;else for(;Fe!==null;){switch(t=Fe,s=t.alternate,e=t.flags,t.tag){case 0:break;case 11:case 15:break;case 1:if((e&1024)!==0&&s!==null){e=void 0,n=t,r=s.memoizedProps,s=s.memoizedState,l=n.stateNode;try{var fe=yl(n.type,r,n.elementType===n.type);e=l.getSnapshotBeforeUpdate(fe,s),l.__reactInternalSnapshotBeforeUpdate=e}catch(ce){De(n,n.return,ce)}}break;case 3:if((e&1024)!==0){if(e=t.stateNode.containerInfo,n=e.nodeType,n===9)ks(e);else if(n===1)switch(e.nodeName){case"HEAD":case"HTML":case"BODY":ks(e);break;default:e.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((e&1024)!==0)throw Error(c(163))}if(e=t.sibling,e!==null){e.return=t.return,Fe=e;break}Fe=t.return}}function Bm(e,t,n){var l=n.flags;switch(n.tag){case 0:case 11:case 15:qn(e,n),l&4&&ar(5,n);break;case 1:if(qn(e,n),l&4)if(e=n.stateNode,t===null)try{e.componentDidMount()}catch(m){De(n,n.return,m)}else{var r=yl(n.type,t.memoizedProps);t=t.memoizedState;try{e.componentDidUpdate(r,t,e.__reactInternalSnapshotBeforeUpdate)}catch(m){De(n,n.return,m)}}l&64&&Dm(n),l&512&&rr(n,n.return);break;case 3:if(qn(e,n),l&64&&(e=n.updateQueue,e!==null)){if(t=null,n.child!==null)switch(n.child.tag){case 27:case 5:t=n.child.stateNode;break;case 1:t=n.child.stateNode}try{xd(e,t)}catch(m){De(n,n.return,m)}}break;case 27:t===null&&l&4&&Lm(n);case 26:case 5:qn(e,n),t===null&&l&4&&Um(n),l&512&&rr(n,n.return);break;case 12:qn(e,n);break;case 13:qn(e,n),l&4&&Ym(e,n),l&64&&(e=n.memoizedState,e!==null&&(e=e.dehydrated,e!==null&&(n=I0.bind(null,n),pb(e,n))));break;case 22:if(l=n.memoizedState!==null||bn,!l){t=t!==null&&t.memoizedState!==null||Ye,r=bn;var s=Ye;bn=l,(Ye=t)&&!s?Gn(e,n,(n.subtreeFlags&8772)!==0):qn(e,n),bn=r,Ye=s}break;case 30:break;default:qn(e,n)}}function qm(e){var t=e.alternate;t!==null&&(e.alternate=null,qm(t)),e.child=null,e.deletions=null,e.sibling=null,e.tag===5&&(t=e.stateNode,t!==null&&Gi(t)),e.stateNode=null,e.return=null,e.dependencies=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.stateNode=null,e.updateQueue=null}var Le=null,mt=!1;function xn(e,t,n){for(n=n.child;n!==null;)Gm(e,t,n),n=n.sibling}function Gm(e,t,n){if(vt&&typeof vt.onCommitFiberUnmount=="function")try{vt.onCommitFiberUnmount(Na,n)}catch{}switch(n.tag){case 26:Ye||Wt(n,t),xn(e,t,n),n.memoizedState?n.memoizedState.count--:n.stateNode&&(n=n.stateNode,n.parentNode.removeChild(n));break;case 27:Ye||Wt(n,t);var l=Le,r=mt;Kn(n.type)&&(Le=n.stateNode,mt=!1),xn(e,t,n),hr(n.stateNode),Le=l,mt=r;break;case 5:Ye||Wt(n,t);case 6:if(l=Le,r=mt,Le=null,xn(e,t,n),Le=l,mt=r,Le!==null)if(mt)try{(Le.nodeType===9?Le.body:Le.nodeName==="HTML"?Le.ownerDocument.body:Le).removeChild(n.stateNode)}catch(s){De(n,t,s)}else try{Le.removeChild(n.stateNode)}catch(s){De(n,t,s)}break;case 18:Le!==null&&(mt?(e=Le,_h(e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e,n.stateNode),wr(e)):_h(Le,n.stateNode));break;case 4:l=Le,r=mt,Le=n.stateNode.containerInfo,mt=!0,xn(e,t,n),Le=l,mt=r;break;case 0:case 11:case 14:case 15:Ye||Bn(2,n,t),Ye||Bn(4,n,t),xn(e,t,n);break;case 1:Ye||(Wt(n,t),l=n.stateNode,typeof l.componentWillUnmount=="function"&&zm(n,t,l)),xn(e,t,n);break;case 21:xn(e,t,n);break;case 22:Ye=(l=Ye)||n.memoizedState!==null,xn(e,t,n),Ye=l;break;default:xn(e,t,n)}}function Ym(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null&&(e=e.dehydrated,e!==null))))try{wr(e)}catch(n){De(t,t.return,n)}}function Q0(e){switch(e.tag){case 13:case 19:var t=e.stateNode;return t===null&&(t=e.stateNode=new Hm),t;case 22:return e=e.stateNode,t=e._retryCache,t===null&&(t=e._retryCache=new Hm),t;default:throw Error(c(435,e.tag))}}function ss(e,t){var n=Q0(e);t.forEach(function(l){var r=eb.bind(null,e,l);n.has(l)||(n.add(l),l.then(r,r))})}function bt(e,t){var n=t.deletions;if(n!==null)for(var l=0;l<n.length;l++){var r=n[l],s=e,m=t,y=m;e:for(;y!==null;){switch(y.tag){case 27:if(Kn(y.type)){Le=y.stateNode,mt=!1;break e}break;case 5:Le=y.stateNode,mt=!1;break e;case 3:case 4:Le=y.stateNode.containerInfo,mt=!0;break e}y=y.return}if(Le===null)throw Error(c(160));Gm(s,m,r),Le=null,mt=!1,s=r.alternate,s!==null&&(s.return=null),r.return=null}if(t.subtreeFlags&13878)for(t=t.child;t!==null;)Vm(t,e),t=t.sibling}var Gt=null;function Vm(e,t){var n=e.alternate,l=e.flags;switch(e.tag){case 0:case 11:case 14:case 15:bt(t,e),xt(e),l&4&&(Bn(3,e,e.return),ar(3,e),Bn(5,e,e.return));break;case 1:bt(t,e),xt(e),l&512&&(Ye||n===null||Wt(n,n.return)),l&64&&bn&&(e=e.updateQueue,e!==null&&(l=e.callbacks,l!==null&&(n=e.shared.hiddenCallbacks,e.shared.hiddenCallbacks=n===null?l:n.concat(l))));break;case 26:var r=Gt;if(bt(t,e),xt(e),l&512&&(Ye||n===null||Wt(n,n.return)),l&4){var s=n!==null?n.memoizedState:null;if(l=e.memoizedState,n===null)if(l===null)if(e.stateNode===null){e:{l=e.type,n=e.memoizedProps,r=r.ownerDocument||r;t:switch(l){case"title":s=r.getElementsByTagName("title")[0],(!s||s[_a]||s[lt]||s.namespaceURI==="http://www.w3.org/2000/svg"||s.hasAttribute("itemprop"))&&(s=r.createElement(l),r.head.insertBefore(s,r.querySelector("head > title"))),nt(s,l,n),s[lt]=e,Je(s),l=s;break e;case"link":var m=Lh("link","href",r).get(l+(n.href||""));if(m){for(var y=0;y<m.length;y++)if(s=m[y],s.getAttribute("href")===(n.href==null||n.href===""?null:n.href)&&s.getAttribute("rel")===(n.rel==null?null:n.rel)&&s.getAttribute("title")===(n.title==null?null:n.title)&&s.getAttribute("crossorigin")===(n.crossOrigin==null?null:n.crossOrigin)){m.splice(y,1);break t}}s=r.createElement(l),nt(s,l,n),r.head.appendChild(s);break;case"meta":if(m=Lh("meta","content",r).get(l+(n.content||""))){for(y=0;y<m.length;y++)if(s=m[y],s.getAttribute("content")===(n.content==null?null:""+n.content)&&s.getAttribute("name")===(n.name==null?null:n.name)&&s.getAttribute("property")===(n.property==null?null:n.property)&&s.getAttribute("http-equiv")===(n.httpEquiv==null?null:n.httpEquiv)&&s.getAttribute("charset")===(n.charSet==null?null:n.charSet)){m.splice(y,1);break t}}s=r.createElement(l),nt(s,l,n),r.head.appendChild(s);break;default:throw Error(c(468,l))}s[lt]=e,Je(s),l=s}e.stateNode=l}else Hh(r,e.type,e.stateNode);else e.stateNode=kh(r,l,e.memoizedProps);else s!==l?(s===null?n.stateNode!==null&&(n=n.stateNode,n.parentNode.removeChild(n)):s.count--,l===null?Hh(r,e.type,e.stateNode):kh(r,l,e.memoizedProps)):l===null&&e.stateNode!==null&&rs(e,e.memoizedProps,n.memoizedProps)}break;case 27:bt(t,e),xt(e),l&512&&(Ye||n===null||Wt(n,n.return)),n!==null&&l&4&&rs(e,e.memoizedProps,n.memoizedProps);break;case 5:if(bt(t,e),xt(e),l&512&&(Ye||n===null||Wt(n,n.return)),e.flags&32){r=e.stateNode;try{Ul(r,"")}catch(k){De(e,e.return,k)}}l&4&&e.stateNode!=null&&(r=e.memoizedProps,rs(e,r,n!==null?n.memoizedProps:r)),l&1024&&(cs=!0);break;case 6:if(bt(t,e),xt(e),l&4){if(e.stateNode===null)throw Error(c(162));l=e.memoizedProps,n=e.stateNode;try{n.nodeValue=l}catch(k){De(e,e.return,k)}}break;case 3:if(Xo=null,r=Gt,Gt=Yo(t.containerInfo),bt(t,e),Gt=r,xt(e),l&4&&n!==null&&n.memoizedState.isDehydrated)try{wr(t.containerInfo)}catch(k){De(e,e.return,k)}cs&&(cs=!1,Xm(e));break;case 4:l=Gt,Gt=Yo(e.stateNode.containerInfo),bt(t,e),xt(e),Gt=l;break;case 12:bt(t,e),xt(e);break;case 13:bt(t,e),xt(e),e.child.flags&8192&&e.memoizedState!==null!=(n!==null&&n.memoizedState!==null)&&(vs=Pt()),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,ss(e,l)));break;case 22:r=e.memoizedState!==null;var E=n!==null&&n.memoizedState!==null,z=bn,G=Ye;if(bn=z||r,Ye=G||E,bt(t,e),Ye=G,bn=z,xt(e),l&8192)e:for(t=e.stateNode,t._visibility=r?t._visibility&-2:t._visibility|1,r&&(n===null||E||bn||Ye||bl(e)),n=null,t=e;;){if(t.tag===5||t.tag===26){if(n===null){E=n=t;try{if(s=E.stateNode,r)m=s.style,typeof m.setProperty=="function"?m.setProperty("display","none","important"):m.display="none";else{y=E.stateNode;var Q=E.memoizedProps.style,U=Q!=null&&Q.hasOwnProperty("display")?Q.display:null;y.style.display=U==null||typeof U=="boolean"?"":(""+U).trim()}}catch(k){De(E,E.return,k)}}}else if(t.tag===6){if(n===null){E=t;try{E.stateNode.nodeValue=r?"":E.memoizedProps}catch(k){De(E,E.return,k)}}}else if((t.tag!==22&&t.tag!==23||t.memoizedState===null||t===e)&&t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;n===t&&(n=null),t=t.return}n===t&&(n=null),t.sibling.return=t.return,t=t.sibling}l&4&&(l=e.updateQueue,l!==null&&(n=l.retryQueue,n!==null&&(l.retryQueue=null,ss(e,n))));break;case 19:bt(t,e),xt(e),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,ss(e,l)));break;case 30:break;case 21:break;default:bt(t,e),xt(e)}}function xt(e){var t=e.flags;if(t&2){try{for(var n,l=e.return;l!==null;){if(km(l)){n=l;break}l=l.return}if(n==null)throw Error(c(160));switch(n.tag){case 27:var r=n.stateNode,s=os(e);_o(e,s,r);break;case 5:var m=n.stateNode;n.flags&32&&(Ul(m,""),n.flags&=-33);var y=os(e);_o(e,y,m);break;case 3:case 4:var E=n.stateNode.containerInfo,z=os(e);is(e,z,E);break;default:throw Error(c(161))}}catch(G){De(e,e.return,G)}e.flags&=-3}t&4096&&(e.flags&=-4097)}function Xm(e){if(e.subtreeFlags&1024)for(e=e.child;e!==null;){var t=e;Xm(t),t.tag===5&&t.flags&1024&&t.stateNode.reset(),e=e.sibling}}function qn(e,t){if(t.subtreeFlags&8772)for(t=t.child;t!==null;)Bm(e,t.alternate,t),t=t.sibling}function bl(e){for(e=e.child;e!==null;){var t=e;switch(t.tag){case 0:case 11:case 14:case 15:Bn(4,t,t.return),bl(t);break;case 1:Wt(t,t.return);var n=t.stateNode;typeof n.componentWillUnmount=="function"&&zm(t,t.return,n),bl(t);break;case 27:hr(t.stateNode);case 26:case 5:Wt(t,t.return),bl(t);break;case 22:t.memoizedState===null&&bl(t);break;case 30:bl(t);break;default:bl(t)}e=e.sibling}}function Gn(e,t,n){for(n=n&&(t.subtreeFlags&8772)!==0,t=t.child;t!==null;){var l=t.alternate,r=e,s=t,m=s.flags;switch(s.tag){case 0:case 11:case 15:Gn(r,s,n),ar(4,s);break;case 1:if(Gn(r,s,n),l=s,r=l.stateNode,typeof r.componentDidMount=="function")try{r.componentDidMount()}catch(z){De(l,l.return,z)}if(l=s,r=l.updateQueue,r!==null){var y=l.stateNode;try{var E=r.shared.hiddenCallbacks;if(E!==null)for(r.shared.hiddenCallbacks=null,r=0;r<E.length;r++)bd(E[r],y)}catch(z){De(l,l.return,z)}}n&&m&64&&Dm(s),rr(s,s.return);break;case 27:Lm(s);case 26:case 5:Gn(r,s,n),n&&l===null&&m&4&&Um(s),rr(s,s.return);break;case 12:Gn(r,s,n);break;case 13:Gn(r,s,n),n&&m&4&&Ym(r,s);break;case 22:s.memoizedState===null&&Gn(r,s,n),rr(s,s.return);break;case 30:break;default:Gn(r,s,n)}t=t.sibling}}function us(e,t){var n=null;e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),e=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),e!==n&&(e!=null&&e.refCount++,n!=null&&Xa(n))}function fs(e,t){e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&Xa(e))}function Ft(e,t,n,l){if(t.subtreeFlags&10256)for(t=t.child;t!==null;)Qm(e,t,n,l),t=t.sibling}function Qm(e,t,n,l){var r=t.flags;switch(t.tag){case 0:case 11:case 15:Ft(e,t,n,l),r&2048&&ar(9,t);break;case 1:Ft(e,t,n,l);break;case 3:Ft(e,t,n,l),r&2048&&(e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&Xa(e)));break;case 12:if(r&2048){Ft(e,t,n,l),e=t.stateNode;try{var s=t.memoizedProps,m=s.id,y=s.onPostCommit;typeof y=="function"&&y(m,t.alternate===null?"mount":"update",e.passiveEffectDuration,-0)}catch(E){De(t,t.return,E)}}else Ft(e,t,n,l);break;case 13:Ft(e,t,n,l);break;case 23:break;case 22:s=t.stateNode,m=t.alternate,t.memoizedState!==null?s._visibility&2?Ft(e,t,n,l):or(e,t):s._visibility&2?Ft(e,t,n,l):(s._visibility|=2,ea(e,t,n,l,(t.subtreeFlags&10256)!==0)),r&2048&&us(m,t);break;case 24:Ft(e,t,n,l),r&2048&&fs(t.alternate,t);break;default:Ft(e,t,n,l)}}function ea(e,t,n,l,r){for(r=r&&(t.subtreeFlags&10256)!==0,t=t.child;t!==null;){var s=e,m=t,y=n,E=l,z=m.flags;switch(m.tag){case 0:case 11:case 15:ea(s,m,y,E,r),ar(8,m);break;case 23:break;case 22:var G=m.stateNode;m.memoizedState!==null?G._visibility&2?ea(s,m,y,E,r):or(s,m):(G._visibility|=2,ea(s,m,y,E,r)),r&&z&2048&&us(m.alternate,m);break;case 24:ea(s,m,y,E,r),r&&z&2048&&fs(m.alternate,m);break;default:ea(s,m,y,E,r)}t=t.sibling}}function or(e,t){if(t.subtreeFlags&10256)for(t=t.child;t!==null;){var n=e,l=t,r=l.flags;switch(l.tag){case 22:or(n,l),r&2048&&us(l.alternate,l);break;case 24:or(n,l),r&2048&&fs(l.alternate,l);break;default:or(n,l)}t=t.sibling}}var ir=8192;function ta(e){if(e.subtreeFlags&ir)for(e=e.child;e!==null;)Zm(e),e=e.sibling}function Zm(e){switch(e.tag){case 26:ta(e),e.flags&ir&&e.memoizedState!==null&&Cb(Gt,e.memoizedState,e.memoizedProps);break;case 5:ta(e);break;case 3:case 4:var t=Gt;Gt=Yo(e.stateNode.containerInfo),ta(e),Gt=t;break;case 22:e.memoizedState===null&&(t=e.alternate,t!==null&&t.memoizedState!==null?(t=ir,ir=16777216,ta(e),ir=t):ta(e));break;default:ta(e)}}function Km(e){var t=e.alternate;if(t!==null&&(e=t.child,e!==null)){t.child=null;do t=e.sibling,e.sibling=null,e=t;while(e!==null)}}function cr(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];Fe=l,$m(l,e)}Km(e)}if(e.subtreeFlags&10256)for(e=e.child;e!==null;)Pm(e),e=e.sibling}function Pm(e){switch(e.tag){case 0:case 11:case 15:cr(e),e.flags&2048&&Bn(9,e,e.return);break;case 3:cr(e);break;case 12:cr(e);break;case 22:var t=e.stateNode;e.memoizedState!==null&&t._visibility&2&&(e.return===null||e.return.tag!==13)?(t._visibility&=-3,Co(e)):cr(e);break;default:cr(e)}}function Co(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];Fe=l,$m(l,e)}Km(e)}for(e=e.child;e!==null;){switch(t=e,t.tag){case 0:case 11:case 15:Bn(8,t,t.return),Co(t);break;case 22:n=t.stateNode,n._visibility&2&&(n._visibility&=-3,Co(t));break;default:Co(t)}e=e.sibling}}function $m(e,t){for(;Fe!==null;){var n=Fe;switch(n.tag){case 0:case 11:case 15:Bn(8,n,t);break;case 23:case 22:if(n.memoizedState!==null&&n.memoizedState.cachePool!==null){var l=n.memoizedState.cachePool.pool;l!=null&&l.refCount++}break;case 24:Xa(n.memoizedState.cache)}if(l=n.child,l!==null)l.return=n,Fe=l;else e:for(n=e;Fe!==null;){l=Fe;var r=l.sibling,s=l.return;if(qm(l),l===n){Fe=null;break e}if(r!==null){r.return=s,Fe=r;break e}Fe=s}}}var Z0={getCacheForType:function(e){var t=at(Ke),n=t.data.get(e);return n===void 0&&(n=e(),t.data.set(e,n)),n}},K0=typeof WeakMap=="function"?WeakMap:Map,Re=0,Ue=null,Se=null,Ae=0,_e=0,St=null,Yn=!1,na=!1,ds=!1,Sn=0,qe=0,Vn=0,xl=0,ms=0,kt=0,la=0,sr=null,ht=null,hs=!1,vs=0,jo=1/0,Oo=null,Xn=null,tt=0,Qn=null,aa=null,ra=0,gs=0,ps=null,Jm=null,ur=0,ys=null;function wt(){if((Re&2)!==0&&Ae!==0)return Ae&-Ae;if(D.T!==null){var e=Zl;return e!==0?e:Ms()}return df()}function Wm(){kt===0&&(kt=(Ae&536870912)===0||Te?cf():536870912);var e=Ut.current;return e!==null&&(e.flags|=32),kt}function Et(e,t,n){(e===Ue&&(_e===2||_e===9)||e.cancelPendingCommit!==null)&&(oa(e,0),Zn(e,Ae,kt,!1)),Ra(e,n),((Re&2)===0||e!==Ue)&&(e===Ue&&((Re&2)===0&&(xl|=n),qe===4&&Zn(e,Ae,kt,!1)),It(e))}function Fm(e,t,n){if((Re&6)!==0)throw Error(c(327));var l=!n&&(t&124)===0&&(t&e.expiredLanes)===0||Ta(e,t),r=l?J0(e,t):Ss(e,t,!0),s=l;do{if(r===0){na&&!l&&Zn(e,t,0,!1);break}else{if(n=e.current.alternate,s&&!P0(n)){r=Ss(e,t,!1),s=!1;continue}if(r===2){if(s=t,e.errorRecoveryDisabledLanes&s)var m=0;else m=e.pendingLanes&-536870913,m=m!==0?m:m&536870912?536870912:0;if(m!==0){t=m;e:{var y=e;r=sr;var E=y.current.memoizedState.isDehydrated;if(E&&(oa(y,m).flags|=256),m=Ss(y,m,!1),m!==2){if(ds&&!E){y.errorRecoveryDisabledLanes|=s,xl|=s,r=4;break e}s=ht,ht=r,s!==null&&(ht===null?ht=s:ht.push.apply(ht,s))}r=m}if(s=!1,r!==2)continue}}if(r===1){oa(e,0),Zn(e,t,0,!0);break}e:{switch(l=e,s=r,s){case 0:case 1:throw Error(c(345));case 4:if((t&4194048)!==t)break;case 6:Zn(l,t,kt,!Yn);break e;case 2:ht=null;break;case 3:case 5:break;default:throw Error(c(329))}if((t&62914560)===t&&(r=vs+300-Pt(),10<r)){if(Zn(l,t,kt,!Yn),Yr(l,0,!0)!==0)break e;l.timeoutHandle=Th(Im.bind(null,l,n,ht,Oo,hs,t,kt,xl,la,Yn,s,2,-0,0),r);break e}Im(l,n,ht,Oo,hs,t,kt,xl,la,Yn,s,0,-0,0)}}break}while(!0);It(e)}function Im(e,t,n,l,r,s,m,y,E,z,G,Q,U,k){if(e.timeoutHandle=-1,Q=t.subtreeFlags,(Q&8192||(Q&16785408)===16785408)&&(pr={stylesheets:null,count:0,unsuspend:_b},Zm(t),Q=jb(),Q!==null)){e.cancelPendingCommit=Q(oh.bind(null,e,t,s,n,l,r,m,y,E,G,1,U,k)),Zn(e,s,m,!z);return}oh(e,t,s,n,l,r,m,y,E)}function P0(e){for(var t=e;;){var n=t.tag;if((n===0||n===11||n===15)&&t.flags&16384&&(n=t.updateQueue,n!==null&&(n=n.stores,n!==null)))for(var l=0;l<n.length;l++){var r=n[l],s=r.getSnapshot;r=r.value;try{if(!pt(s(),r))return!1}catch{return!1}}if(n=t.child,t.subtreeFlags&16384&&n!==null)n.return=t,t=n;else{if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return!0;t=t.return}t.sibling.return=t.return,t=t.sibling}}return!0}function Zn(e,t,n,l){t&=~ms,t&=~xl,e.suspendedLanes|=t,e.pingedLanes&=~t,l&&(e.warmLanes|=t),l=e.expirationTimes;for(var r=t;0<r;){var s=31-gt(r),m=1<<s;l[s]=-1,r&=~m}n!==0&&uf(e,n,t)}function Do(){return(Re&6)===0?(fr(0),!1):!0}function bs(){if(Se!==null){if(_e===0)var e=Se.return;else e=Se,mn=hl=null,kc(e),Fl=null,tr=0,e=Se;for(;e!==null;)Om(e.alternate,e),e=e.return;Se=null}}function oa(e,t){var n=e.timeoutHandle;n!==-1&&(e.timeoutHandle=-1,db(n)),n=e.cancelPendingCommit,n!==null&&(e.cancelPendingCommit=null,n()),bs(),Ue=e,Se=n=un(e.current,null),Ae=t,_e=0,St=null,Yn=!1,na=Ta(e,t),ds=!1,la=kt=ms=xl=Vn=qe=0,ht=sr=null,hs=!1,(t&8)!==0&&(t|=t&32);var l=e.entangledLanes;if(l!==0)for(e=e.entanglements,l&=t;0<l;){var r=31-gt(l),s=1<<r;t|=e[r],l&=~s}return Sn=t,eo(),n}function eh(e,t){pe=null,D.H=xo,t===Za||t===so?(t=pd(),_e=3):t===hd?(t=pd(),_e=4):_e=t===ym?8:t!==null&&typeof t=="object"&&typeof t.then=="function"?6:1,St=t,Se===null&&(qe=1,Mo(e,jt(t,e.current)))}function th(){var e=D.H;return D.H=xo,e===null?xo:e}function nh(){var e=D.A;return D.A=Z0,e}function xs(){qe=4,Yn||(Ae&4194048)!==Ae&&Ut.current!==null||(na=!0),(Vn&134217727)===0&&(xl&134217727)===0||Ue===null||Zn(Ue,Ae,kt,!1)}function Ss(e,t,n){var l=Re;Re|=2;var r=th(),s=nh();(Ue!==e||Ae!==t)&&(Oo=null,oa(e,t)),t=!1;var m=qe;e:do try{if(_e!==0&&Se!==null){var y=Se,E=St;switch(_e){case 8:bs(),m=6;break e;case 3:case 2:case 9:case 6:Ut.current===null&&(t=!0);var z=_e;if(_e=0,St=null,ia(e,y,E,z),n&&na){m=0;break e}break;default:z=_e,_e=0,St=null,ia(e,y,E,z)}}$0(),m=qe;break}catch(G){eh(e,G)}while(!0);return t&&e.shellSuspendCounter++,mn=hl=null,Re=l,D.H=r,D.A=s,Se===null&&(Ue=null,Ae=0,eo()),m}function $0(){for(;Se!==null;)lh(Se)}function J0(e,t){var n=Re;Re|=2;var l=th(),r=nh();Ue!==e||Ae!==t?(Oo=null,jo=Pt()+500,oa(e,t)):na=Ta(e,t);e:do try{if(_e!==0&&Se!==null){t=Se;var s=St;t:switch(_e){case 1:_e=0,St=null,ia(e,t,s,1);break;case 2:case 9:if(vd(s)){_e=0,St=null,ah(t);break}t=function(){_e!==2&&_e!==9||Ue!==e||(_e=7),It(e)},s.then(t,t);break e;case 3:_e=7;break e;case 4:_e=5;break e;case 7:vd(s)?(_e=0,St=null,ah(t)):(_e=0,St=null,ia(e,t,s,7));break;case 5:var m=null;switch(Se.tag){case 26:m=Se.memoizedState;case 5:case 27:var y=Se;if(!m||Bh(m)){_e=0,St=null;var E=y.sibling;if(E!==null)Se=E;else{var z=y.return;z!==null?(Se=z,zo(z)):Se=null}break t}}_e=0,St=null,ia(e,t,s,5);break;case 6:_e=0,St=null,ia(e,t,s,6);break;case 8:bs(),qe=6;break e;default:throw Error(c(462))}}W0();break}catch(G){eh(e,G)}while(!0);return mn=hl=null,D.H=l,D.A=r,Re=n,Se!==null?0:(Ue=null,Ae=0,eo(),qe)}function W0(){for(;Se!==null&&!by();)lh(Se)}function lh(e){var t=Cm(e.alternate,e,Sn);e.memoizedProps=e.pendingProps,t===null?zo(e):Se=t}function ah(e){var t=e,n=t.alternate;switch(t.tag){case 15:case 0:t=Am(n,t,t.pendingProps,t.type,void 0,Ae);break;case 11:t=Am(n,t,t.pendingProps,t.type.render,t.ref,Ae);break;case 5:kc(t);default:Om(n,t),t=Se=rd(t,Sn),t=Cm(n,t,Sn)}e.memoizedProps=e.pendingProps,t===null?zo(e):Se=t}function ia(e,t,n,l){mn=hl=null,kc(t),Fl=null,tr=0;var r=t.return;try{if(q0(e,r,t,n,Ae)){qe=1,Mo(e,jt(n,e.current)),Se=null;return}}catch(s){if(r!==null)throw Se=r,s;qe=1,Mo(e,jt(n,e.current)),Se=null;return}t.flags&32768?(Te||l===1?e=!0:na||(Ae&536870912)!==0?e=!1:(Yn=e=!0,(l===2||l===9||l===3||l===6)&&(l=Ut.current,l!==null&&l.tag===13&&(l.flags|=16384))),rh(t,e)):zo(t)}function zo(e){var t=e;do{if((t.flags&32768)!==0){rh(t,Yn);return}e=t.return;var n=Y0(t.alternate,t,Sn);if(n!==null){Se=n;return}if(t=t.sibling,t!==null){Se=t;return}Se=t=e}while(t!==null);qe===0&&(qe=5)}function rh(e,t){do{var n=V0(e.alternate,e);if(n!==null){n.flags&=32767,Se=n;return}if(n=e.return,n!==null&&(n.flags|=32768,n.subtreeFlags=0,n.deletions=null),!t&&(e=e.sibling,e!==null)){Se=e;return}Se=e=n}while(e!==null);qe=6,Se=null}function oh(e,t,n,l,r,s,m,y,E){e.cancelPendingCommit=null;do Uo();while(tt!==0);if((Re&6)!==0)throw Error(c(327));if(t!==null){if(t===e.current)throw Error(c(177));if(s=t.lanes|t.childLanes,s|=fc,_y(e,n,s,m,y,E),e===Ue&&(Se=Ue=null,Ae=0),aa=t,Qn=e,ra=n,gs=s,ps=r,Jm=l,(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?(e.callbackNode=null,e.callbackPriority=0,tb(Br,function(){return fh(),null})):(e.callbackNode=null,e.callbackPriority=0),l=(t.flags&13878)!==0,(t.subtreeFlags&13878)!==0||l){l=D.T,D.T=null,r=K.p,K.p=2,m=Re,Re|=4;try{X0(e,t,n)}finally{Re=m,K.p=r,D.T=l}}tt=1,ih(),ch(),sh()}}function ih(){if(tt===1){tt=0;var e=Qn,t=aa,n=(t.flags&13878)!==0;if((t.subtreeFlags&13878)!==0||n){n=D.T,D.T=null;var l=K.p;K.p=2;var r=Re;Re|=4;try{Vm(t,e);var s=Ds,m=$f(e.containerInfo),y=s.focusedElem,E=s.selectionRange;if(m!==y&&y&&y.ownerDocument&&Pf(y.ownerDocument.documentElement,y)){if(E!==null&&oc(y)){var z=E.start,G=E.end;if(G===void 0&&(G=z),"selectionStart"in y)y.selectionStart=z,y.selectionEnd=Math.min(G,y.value.length);else{var Q=y.ownerDocument||document,U=Q&&Q.defaultView||window;if(U.getSelection){var k=U.getSelection(),fe=y.textContent.length,ce=Math.min(E.start,fe),Oe=E.end===void 0?ce:Math.min(E.end,fe);!k.extend&&ce>Oe&&(m=Oe,Oe=ce,ce=m);var C=Kf(y,ce),R=Kf(y,Oe);if(C&&R&&(k.rangeCount!==1||k.anchorNode!==C.node||k.anchorOffset!==C.offset||k.focusNode!==R.node||k.focusOffset!==R.offset)){var O=Q.createRange();O.setStart(C.node,C.offset),k.removeAllRanges(),ce>Oe?(k.addRange(O),k.extend(R.node,R.offset)):(O.setEnd(R.node,R.offset),k.addRange(O))}}}}for(Q=[],k=y;k=k.parentNode;)k.nodeType===1&&Q.push({element:k,left:k.scrollLeft,top:k.scrollTop});for(typeof y.focus=="function"&&y.focus(),y=0;y<Q.length;y++){var V=Q[y];V.element.scrollLeft=V.left,V.element.scrollTop=V.top}}Ko=!!Os,Ds=Os=null}finally{Re=r,K.p=l,D.T=n}}e.current=t,tt=2}}function ch(){if(tt===2){tt=0;var e=Qn,t=aa,n=(t.flags&8772)!==0;if((t.subtreeFlags&8772)!==0||n){n=D.T,D.T=null;var l=K.p;K.p=2;var r=Re;Re|=4;try{Bm(e,t.alternate,t)}finally{Re=r,K.p=l,D.T=n}}tt=3}}function sh(){if(tt===4||tt===3){tt=0,xy();var e=Qn,t=aa,n=ra,l=Jm;(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?tt=5:(tt=0,aa=Qn=null,uh(e,e.pendingLanes));var r=e.pendingLanes;if(r===0&&(Xn=null),Bi(n),t=t.stateNode,vt&&typeof vt.onCommitFiberRoot=="function")try{vt.onCommitFiberRoot(Na,t,void 0,(t.current.flags&128)===128)}catch{}if(l!==null){t=D.T,r=K.p,K.p=2,D.T=null;try{for(var s=e.onRecoverableError,m=0;m<l.length;m++){var y=l[m];s(y.value,{componentStack:y.stack})}}finally{D.T=t,K.p=r}}(ra&3)!==0&&Uo(),It(e),r=e.pendingLanes,(n&4194090)!==0&&(r&42)!==0?e===ys?ur++:(ur=0,ys=e):ur=0,fr(0)}}function uh(e,t){(e.pooledCacheLanes&=t)===0&&(t=e.pooledCache,t!=null&&(e.pooledCache=null,Xa(t)))}function Uo(e){return ih(),ch(),sh(),fh()}function fh(){if(tt!==5)return!1;var e=Qn,t=gs;gs=0;var n=Bi(ra),l=D.T,r=K.p;try{K.p=32>n?32:n,D.T=null,n=ps,ps=null;var s=Qn,m=ra;if(tt=0,aa=Qn=null,ra=0,(Re&6)!==0)throw Error(c(331));var y=Re;if(Re|=4,Pm(s.current),Qm(s,s.current,m,n),Re=y,fr(0,!1),vt&&typeof vt.onPostCommitFiberRoot=="function")try{vt.onPostCommitFiberRoot(Na,s)}catch{}return!0}finally{K.p=r,D.T=l,uh(e,t)}}function dh(e,t,n){t=jt(n,t),t=Jc(e.stateNode,t,2),e=Un(e,t,2),e!==null&&(Ra(e,2),It(e))}function De(e,t,n){if(e.tag===3)dh(e,e,n);else for(;t!==null;){if(t.tag===3){dh(t,e,n);break}else if(t.tag===1){var l=t.stateNode;if(typeof t.type.getDerivedStateFromError=="function"||typeof l.componentDidCatch=="function"&&(Xn===null||!Xn.has(l))){e=jt(n,e),n=gm(2),l=Un(t,n,2),l!==null&&(pm(n,l,t,e),Ra(l,2),It(l));break}}t=t.return}}function ws(e,t,n){var l=e.pingCache;if(l===null){l=e.pingCache=new K0;var r=new Set;l.set(t,r)}else r=l.get(t),r===void 0&&(r=new Set,l.set(t,r));r.has(n)||(ds=!0,r.add(n),e=F0.bind(null,e,t,n),t.then(e,e))}function F0(e,t,n){var l=e.pingCache;l!==null&&l.delete(t),e.pingedLanes|=e.suspendedLanes&n,e.warmLanes&=~n,Ue===e&&(Ae&n)===n&&(qe===4||qe===3&&(Ae&62914560)===Ae&&300>Pt()-vs?(Re&2)===0&&oa(e,0):ms|=n,la===Ae&&(la=0)),It(e)}function mh(e,t){t===0&&(t=sf()),e=Yl(e,t),e!==null&&(Ra(e,t),It(e))}function I0(e){var t=e.memoizedState,n=0;t!==null&&(n=t.retryLane),mh(e,n)}function eb(e,t){var n=0;switch(e.tag){case 13:var l=e.stateNode,r=e.memoizedState;r!==null&&(n=r.retryLane);break;case 19:l=e.stateNode;break;case 22:l=e.stateNode._retryCache;break;default:throw Error(c(314))}l!==null&&l.delete(t),mh(e,n)}function tb(e,t){return Rn(e,t)}var ko=null,ca=null,Es=!1,Lo=!1,As=!1,Sl=0;function It(e){e!==ca&&e.next===null&&(ca===null?ko=ca=e:ca=ca.next=e),Lo=!0,Es||(Es=!0,lb())}function fr(e,t){if(!As&&Lo){As=!0;do for(var n=!1,l=ko;l!==null;){if(e!==0){var r=l.pendingLanes;if(r===0)var s=0;else{var m=l.suspendedLanes,y=l.pingedLanes;s=(1<<31-gt(42|e)+1)-1,s&=r&~(m&~y),s=s&201326741?s&201326741|1:s?s|2:0}s!==0&&(n=!0,ph(l,s))}else s=Ae,s=Yr(l,l===Ue?s:0,l.cancelPendingCommit!==null||l.timeoutHandle!==-1),(s&3)===0||Ta(l,s)||(n=!0,ph(l,s));l=l.next}while(n);As=!1}}function nb(){hh()}function hh(){Lo=Es=!1;var e=0;Sl!==0&&(fb()&&(e=Sl),Sl=0);for(var t=Pt(),n=null,l=ko;l!==null;){var r=l.next,s=vh(l,t);s===0?(l.next=null,n===null?ko=r:n.next=r,r===null&&(ca=n)):(n=l,(e!==0||(s&3)!==0)&&(Lo=!0)),l=r}fr(e)}function vh(e,t){for(var n=e.suspendedLanes,l=e.pingedLanes,r=e.expirationTimes,s=e.pendingLanes&-62914561;0<s;){var m=31-gt(s),y=1<<m,E=r[m];E===-1?((y&n)===0||(y&l)!==0)&&(r[m]=Ry(y,t)):E<=t&&(e.expiredLanes|=y),s&=~y}if(t=Ue,n=Ae,n=Yr(e,e===t?n:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),l=e.callbackNode,n===0||e===t&&(_e===2||_e===9)||e.cancelPendingCommit!==null)return l!==null&&l!==null&&ki(l),e.callbackNode=null,e.callbackPriority=0;if((n&3)===0||Ta(e,n)){if(t=n&-n,t===e.callbackPriority)return t;switch(l!==null&&ki(l),Bi(n)){case 2:case 8:n=rf;break;case 32:n=Br;break;case 268435456:n=of;break;default:n=Br}return l=gh.bind(null,e),n=Rn(n,l),e.callbackPriority=t,e.callbackNode=n,t}return l!==null&&l!==null&&ki(l),e.callbackPriority=2,e.callbackNode=null,2}function gh(e,t){if(tt!==0&&tt!==5)return e.callbackNode=null,e.callbackPriority=0,null;var n=e.callbackNode;if(Uo()&&e.callbackNode!==n)return null;var l=Ae;return l=Yr(e,e===Ue?l:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),l===0?null:(Fm(e,l,t),vh(e,Pt()),e.callbackNode!=null&&e.callbackNode===n?gh.bind(null,e):null)}function ph(e,t){if(Uo())return null;Fm(e,t,!0)}function lb(){mb(function(){(Re&6)!==0?Rn(af,nb):hh()})}function Ms(){return Sl===0&&(Sl=cf()),Sl}function yh(e){return e==null||typeof e=="symbol"||typeof e=="boolean"?null:typeof e=="function"?e:Kr(""+e)}function bh(e,t){var n=t.ownerDocument.createElement("input");return n.name=t.name,n.value=t.value,e.id&&n.setAttribute("form",e.id),t.parentNode.insertBefore(n,t),e=new FormData(e),n.parentNode.removeChild(n),e}function ab(e,t,n,l,r){if(t==="submit"&&n&&n.stateNode===r){var s=yh((r[ut]||null).action),m=l.submitter;m&&(t=(t=m[ut]||null)?yh(t.formAction):m.getAttribute("formAction"),t!==null&&(s=t,m=null));var y=new Wr("action","action",null,l,r);e.push({event:y,listeners:[{instance:null,listener:function(){if(l.defaultPrevented){if(Sl!==0){var E=m?bh(r,m):new FormData(r);Qc(n,{pending:!0,data:E,method:r.method,action:s},null,E)}}else typeof s=="function"&&(y.preventDefault(),E=m?bh(r,m):new FormData(r),Qc(n,{pending:!0,data:E,method:r.method,action:s},s,E))},currentTarget:r}]})}}for(var Ns=0;Ns<uc.length;Ns++){var Ts=uc[Ns],rb=Ts.toLowerCase(),ob=Ts[0].toUpperCase()+Ts.slice(1);qt(rb,"on"+ob)}qt(Ff,"onAnimationEnd"),qt(If,"onAnimationIteration"),qt(ed,"onAnimationStart"),qt("dblclick","onDoubleClick"),qt("focusin","onFocus"),qt("focusout","onBlur"),qt(E0,"onTransitionRun"),qt(A0,"onTransitionStart"),qt(M0,"onTransitionCancel"),qt(td,"onTransitionEnd"),Ol("onMouseEnter",["mouseout","mouseover"]),Ol("onMouseLeave",["mouseout","mouseover"]),Ol("onPointerEnter",["pointerout","pointerover"]),Ol("onPointerLeave",["pointerout","pointerover"]),rl("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),rl("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),rl("onBeforeInput",["compositionend","keypress","textInput","paste"]),rl("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),rl("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),rl("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var dr="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),ib=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(dr));function xh(e,t){t=(t&4)!==0;for(var n=0;n<e.length;n++){var l=e[n],r=l.event;l=l.listeners;e:{var s=void 0;if(t)for(var m=l.length-1;0<=m;m--){var y=l[m],E=y.instance,z=y.currentTarget;if(y=y.listener,E!==s&&r.isPropagationStopped())break e;s=y,r.currentTarget=z;try{s(r)}catch(G){Ao(G)}r.currentTarget=null,s=E}else for(m=0;m<l.length;m++){if(y=l[m],E=y.instance,z=y.currentTarget,y=y.listener,E!==s&&r.isPropagationStopped())break e;s=y,r.currentTarget=z;try{s(r)}catch(G){Ao(G)}r.currentTarget=null,s=E}}}}function we(e,t){var n=t[qi];n===void 0&&(n=t[qi]=new Set);var l=e+"__bubble";n.has(l)||(Sh(t,e,2,!1),n.add(l))}function Rs(e,t,n){var l=0;t&&(l|=4),Sh(n,e,l,t)}var Ho="_reactListening"+Math.random().toString(36).slice(2);function _s(e){if(!e[Ho]){e[Ho]=!0,hf.forEach(function(n){n!=="selectionchange"&&(ib.has(n)||Rs(n,!1,e),Rs(n,!0,e))});var t=e.nodeType===9?e:e.ownerDocument;t===null||t[Ho]||(t[Ho]=!0,Rs("selectionchange",!1,t))}}function Sh(e,t,n,l){switch(Qh(t)){case 2:var r=zb;break;case 8:r=Ub;break;default:r=Vs}n=r.bind(null,t,n,e),r=void 0,!Wi||t!=="touchstart"&&t!=="touchmove"&&t!=="wheel"||(r=!0),l?r!==void 0?e.addEventListener(t,n,{capture:!0,passive:r}):e.addEventListener(t,n,!0):r!==void 0?e.addEventListener(t,n,{passive:r}):e.addEventListener(t,n,!1)}function Cs(e,t,n,l,r){var s=l;if((t&1)===0&&(t&2)===0&&l!==null)e:for(;;){if(l===null)return;var m=l.tag;if(m===3||m===4){var y=l.stateNode.containerInfo;if(y===r)break;if(m===4)for(m=l.return;m!==null;){var E=m.tag;if((E===3||E===4)&&m.stateNode.containerInfo===r)return;m=m.return}for(;y!==null;){if(m=_l(y),m===null)return;if(E=m.tag,E===5||E===6||E===26||E===27){l=s=m;continue e}y=y.parentNode}}l=l.return}Rf(function(){var z=s,G=$i(n),Q=[];e:{var U=nd.get(e);if(U!==void 0){var k=Wr,fe=e;switch(e){case"keypress":if($r(n)===0)break e;case"keydown":case"keyup":k=t0;break;case"focusin":fe="focus",k=tc;break;case"focusout":fe="blur",k=tc;break;case"beforeblur":case"afterblur":k=tc;break;case"click":if(n.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":k=jf;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":k=Vy;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":k=a0;break;case Ff:case If:case ed:k=Zy;break;case td:k=o0;break;case"scroll":case"scrollend":k=Gy;break;case"wheel":k=c0;break;case"copy":case"cut":case"paste":k=Py;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":k=Df;break;case"toggle":case"beforetoggle":k=u0}var ce=(t&4)!==0,Oe=!ce&&(e==="scroll"||e==="scrollend"),C=ce?U!==null?U+"Capture":null:U;ce=[];for(var R=z,O;R!==null;){var V=R;if(O=V.stateNode,V=V.tag,V!==5&&V!==26&&V!==27||O===null||C===null||(V=ja(R,C),V!=null&&ce.push(mr(R,V,O))),Oe)break;R=R.return}0<ce.length&&(U=new k(U,fe,null,n,G),Q.push({event:U,listeners:ce}))}}if((t&7)===0){e:{if(U=e==="mouseover"||e==="pointerover",k=e==="mouseout"||e==="pointerout",U&&n!==Pi&&(fe=n.relatedTarget||n.fromElement)&&(_l(fe)||fe[Rl]))break e;if((k||U)&&(U=G.window===G?G:(U=G.ownerDocument)?U.defaultView||U.parentWindow:window,k?(fe=n.relatedTarget||n.toElement,k=z,fe=fe?_l(fe):null,fe!==null&&(Oe=f(fe),ce=fe.tag,fe!==Oe||ce!==5&&ce!==27&&ce!==6)&&(fe=null)):(k=null,fe=z),k!==fe)){if(ce=jf,V="onMouseLeave",C="onMouseEnter",R="mouse",(e==="pointerout"||e==="pointerover")&&(ce=Df,V="onPointerLeave",C="onPointerEnter",R="pointer"),Oe=k==null?U:Ca(k),O=fe==null?U:Ca(fe),U=new ce(V,R+"leave",k,n,G),U.target=Oe,U.relatedTarget=O,V=null,_l(G)===z&&(ce=new ce(C,R+"enter",fe,n,G),ce.target=O,ce.relatedTarget=Oe,V=ce),Oe=V,k&&fe)t:{for(ce=k,C=fe,R=0,O=ce;O;O=sa(O))R++;for(O=0,V=C;V;V=sa(V))O++;for(;0<R-O;)ce=sa(ce),R--;for(;0<O-R;)C=sa(C),O--;for(;R--;){if(ce===C||C!==null&&ce===C.alternate)break t;ce=sa(ce),C=sa(C)}ce=null}else ce=null;k!==null&&wh(Q,U,k,ce,!1),fe!==null&&Oe!==null&&wh(Q,Oe,fe,ce,!0)}}e:{if(U=z?Ca(z):window,k=U.nodeName&&U.nodeName.toLowerCase(),k==="select"||k==="input"&&U.type==="file")var te=Gf;else if(Bf(U))if(Yf)te=x0;else{te=y0;var xe=p0}else k=U.nodeName,!k||k.toLowerCase()!=="input"||U.type!=="checkbox"&&U.type!=="radio"?z&&Ki(z.elementType)&&(te=Gf):te=b0;if(te&&(te=te(e,z))){qf(Q,te,n,G);break e}xe&&xe(e,U,z),e==="focusout"&&z&&U.type==="number"&&z.memoizedProps.value!=null&&Zi(U,"number",U.value)}switch(xe=z?Ca(z):window,e){case"focusin":(Bf(xe)||xe.contentEditable==="true")&&(Bl=xe,ic=z,Ba=null);break;case"focusout":Ba=ic=Bl=null;break;case"mousedown":cc=!0;break;case"contextmenu":case"mouseup":case"dragend":cc=!1,Jf(Q,n,G);break;case"selectionchange":if(w0)break;case"keydown":case"keyup":Jf(Q,n,G)}var ae;if(lc)e:{switch(e){case"compositionstart":var se="onCompositionStart";break e;case"compositionend":se="onCompositionEnd";break e;case"compositionupdate":se="onCompositionUpdate";break e}se=void 0}else Hl?Lf(e,n)&&(se="onCompositionEnd"):e==="keydown"&&n.keyCode===229&&(se="onCompositionStart");se&&(zf&&n.locale!=="ko"&&(Hl||se!=="onCompositionStart"?se==="onCompositionEnd"&&Hl&&(ae=_f()):(jn=G,Fi="value"in jn?jn.value:jn.textContent,Hl=!0)),xe=Bo(z,se),0<xe.length&&(se=new Of(se,e,null,n,G),Q.push({event:se,listeners:xe}),ae?se.data=ae:(ae=Hf(n),ae!==null&&(se.data=ae)))),(ae=d0?m0(e,n):h0(e,n))&&(se=Bo(z,"onBeforeInput"),0<se.length&&(xe=new Of("onBeforeInput","beforeinput",null,n,G),Q.push({event:xe,listeners:se}),xe.data=ae)),ab(Q,e,z,n,G)}xh(Q,t)})}function mr(e,t,n){return{instance:e,listener:t,currentTarget:n}}function Bo(e,t){for(var n=t+"Capture",l=[];e!==null;){var r=e,s=r.stateNode;if(r=r.tag,r!==5&&r!==26&&r!==27||s===null||(r=ja(e,n),r!=null&&l.unshift(mr(e,r,s)),r=ja(e,t),r!=null&&l.push(mr(e,r,s))),e.tag===3)return l;e=e.return}return[]}function sa(e){if(e===null)return null;do e=e.return;while(e&&e.tag!==5&&e.tag!==27);return e||null}function wh(e,t,n,l,r){for(var s=t._reactName,m=[];n!==null&&n!==l;){var y=n,E=y.alternate,z=y.stateNode;if(y=y.tag,E!==null&&E===l)break;y!==5&&y!==26&&y!==27||z===null||(E=z,r?(z=ja(n,s),z!=null&&m.unshift(mr(n,z,E))):r||(z=ja(n,s),z!=null&&m.push(mr(n,z,E)))),n=n.return}m.length!==0&&e.push({event:t,listeners:m})}var cb=/\r\n?/g,sb=/\u0000|\uFFFD/g;function Eh(e){return(typeof e=="string"?e:""+e).replace(cb,`
`).replace(sb,"")}function Ah(e,t){return t=Eh(t),Eh(e)===t}function qo(){}function je(e,t,n,l,r,s){switch(n){case"children":typeof l=="string"?t==="body"||t==="textarea"&&l===""||Ul(e,l):(typeof l=="number"||typeof l=="bigint")&&t!=="body"&&Ul(e,""+l);break;case"className":Xr(e,"class",l);break;case"tabIndex":Xr(e,"tabindex",l);break;case"dir":case"role":case"viewBox":case"width":case"height":Xr(e,n,l);break;case"style":Nf(e,l,s);break;case"data":if(t!=="object"){Xr(e,"data",l);break}case"src":case"href":if(l===""&&(t!=="a"||n!=="href")){e.removeAttribute(n);break}if(l==null||typeof l=="function"||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=Kr(""+l),e.setAttribute(n,l);break;case"action":case"formAction":if(typeof l=="function"){e.setAttribute(n,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof s=="function"&&(n==="formAction"?(t!=="input"&&je(e,t,"name",r.name,r,null),je(e,t,"formEncType",r.formEncType,r,null),je(e,t,"formMethod",r.formMethod,r,null),je(e,t,"formTarget",r.formTarget,r,null)):(je(e,t,"encType",r.encType,r,null),je(e,t,"method",r.method,r,null),je(e,t,"target",r.target,r,null)));if(l==null||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=Kr(""+l),e.setAttribute(n,l);break;case"onClick":l!=null&&(e.onclick=qo);break;case"onScroll":l!=null&&we("scroll",e);break;case"onScrollEnd":l!=null&&we("scrollend",e);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(c(61));if(n=l.__html,n!=null){if(r.children!=null)throw Error(c(60));e.innerHTML=n}}break;case"multiple":e.multiple=l&&typeof l!="function"&&typeof l!="symbol";break;case"muted":e.muted=l&&typeof l!="function"&&typeof l!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(l==null||typeof l=="function"||typeof l=="boolean"||typeof l=="symbol"){e.removeAttribute("xlink:href");break}n=Kr(""+l),e.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",n);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""+l):e.removeAttribute(n);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":l&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""):e.removeAttribute(n);break;case"capture":case"download":l===!0?e.setAttribute(n,""):l!==!1&&l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,l):e.removeAttribute(n);break;case"cols":case"rows":case"size":case"span":l!=null&&typeof l!="function"&&typeof l!="symbol"&&!isNaN(l)&&1<=l?e.setAttribute(n,l):e.removeAttribute(n);break;case"rowSpan":case"start":l==null||typeof l=="function"||typeof l=="symbol"||isNaN(l)?e.removeAttribute(n):e.setAttribute(n,l);break;case"popover":we("beforetoggle",e),we("toggle",e),Vr(e,"popover",l);break;case"xlinkActuate":cn(e,"http://www.w3.org/1999/xlink","xlink:actuate",l);break;case"xlinkArcrole":cn(e,"http://www.w3.org/1999/xlink","xlink:arcrole",l);break;case"xlinkRole":cn(e,"http://www.w3.org/1999/xlink","xlink:role",l);break;case"xlinkShow":cn(e,"http://www.w3.org/1999/xlink","xlink:show",l);break;case"xlinkTitle":cn(e,"http://www.w3.org/1999/xlink","xlink:title",l);break;case"xlinkType":cn(e,"http://www.w3.org/1999/xlink","xlink:type",l);break;case"xmlBase":cn(e,"http://www.w3.org/XML/1998/namespace","xml:base",l);break;case"xmlLang":cn(e,"http://www.w3.org/XML/1998/namespace","xml:lang",l);break;case"xmlSpace":cn(e,"http://www.w3.org/XML/1998/namespace","xml:space",l);break;case"is":Vr(e,"is",l);break;case"innerText":case"textContent":break;default:(!(2<n.length)||n[0]!=="o"&&n[0]!=="O"||n[1]!=="n"&&n[1]!=="N")&&(n=By.get(n)||n,Vr(e,n,l))}}function js(e,t,n,l,r,s){switch(n){case"style":Nf(e,l,s);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(c(61));if(n=l.__html,n!=null){if(r.children!=null)throw Error(c(60));e.innerHTML=n}}break;case"children":typeof l=="string"?Ul(e,l):(typeof l=="number"||typeof l=="bigint")&&Ul(e,""+l);break;case"onScroll":l!=null&&we("scroll",e);break;case"onScrollEnd":l!=null&&we("scrollend",e);break;case"onClick":l!=null&&(e.onclick=qo);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!vf.hasOwnProperty(n))e:{if(n[0]==="o"&&n[1]==="n"&&(r=n.endsWith("Capture"),t=n.slice(2,r?n.length-7:void 0),s=e[ut]||null,s=s!=null?s[n]:null,typeof s=="function"&&e.removeEventListener(t,s,r),typeof l=="function")){typeof s!="function"&&s!==null&&(n in e?e[n]=null:e.hasAttribute(n)&&e.removeAttribute(n)),e.addEventListener(t,l,r);break e}n in e?e[n]=l:l===!0?e.setAttribute(n,""):Vr(e,n,l)}}}function nt(e,t,n){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":we("error",e),we("load",e);var l=!1,r=!1,s;for(s in n)if(n.hasOwnProperty(s)){var m=n[s];if(m!=null)switch(s){case"src":l=!0;break;case"srcSet":r=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(c(137,t));default:je(e,t,s,m,n,null)}}r&&je(e,t,"srcSet",n.srcSet,n,null),l&&je(e,t,"src",n.src,n,null);return;case"input":we("invalid",e);var y=s=m=r=null,E=null,z=null;for(l in n)if(n.hasOwnProperty(l)){var G=n[l];if(G!=null)switch(l){case"name":r=G;break;case"type":m=G;break;case"checked":E=G;break;case"defaultChecked":z=G;break;case"value":s=G;break;case"defaultValue":y=G;break;case"children":case"dangerouslySetInnerHTML":if(G!=null)throw Error(c(137,t));break;default:je(e,t,l,G,n,null)}}wf(e,s,y,E,z,m,r,!1),Qr(e);return;case"select":we("invalid",e),l=m=s=null;for(r in n)if(n.hasOwnProperty(r)&&(y=n[r],y!=null))switch(r){case"value":s=y;break;case"defaultValue":m=y;break;case"multiple":l=y;default:je(e,t,r,y,n,null)}t=s,n=m,e.multiple=!!l,t!=null?zl(e,!!l,t,!1):n!=null&&zl(e,!!l,n,!0);return;case"textarea":we("invalid",e),s=r=l=null;for(m in n)if(n.hasOwnProperty(m)&&(y=n[m],y!=null))switch(m){case"value":l=y;break;case"defaultValue":r=y;break;case"children":s=y;break;case"dangerouslySetInnerHTML":if(y!=null)throw Error(c(91));break;default:je(e,t,m,y,n,null)}Af(e,l,r,s),Qr(e);return;case"option":for(E in n)if(n.hasOwnProperty(E)&&(l=n[E],l!=null))switch(E){case"selected":e.selected=l&&typeof l!="function"&&typeof l!="symbol";break;default:je(e,t,E,l,n,null)}return;case"dialog":we("beforetoggle",e),we("toggle",e),we("cancel",e),we("close",e);break;case"iframe":case"object":we("load",e);break;case"video":case"audio":for(l=0;l<dr.length;l++)we(dr[l],e);break;case"image":we("error",e),we("load",e);break;case"details":we("toggle",e);break;case"embed":case"source":case"link":we("error",e),we("load",e);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(z in n)if(n.hasOwnProperty(z)&&(l=n[z],l!=null))switch(z){case"children":case"dangerouslySetInnerHTML":throw Error(c(137,t));default:je(e,t,z,l,n,null)}return;default:if(Ki(t)){for(G in n)n.hasOwnProperty(G)&&(l=n[G],l!==void 0&&js(e,t,G,l,n,void 0));return}}for(y in n)n.hasOwnProperty(y)&&(l=n[y],l!=null&&je(e,t,y,l,n,null))}function ub(e,t,n,l){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var r=null,s=null,m=null,y=null,E=null,z=null,G=null;for(k in n){var Q=n[k];if(n.hasOwnProperty(k)&&Q!=null)switch(k){case"checked":break;case"value":break;case"defaultValue":E=Q;default:l.hasOwnProperty(k)||je(e,t,k,null,l,Q)}}for(var U in l){var k=l[U];if(Q=n[U],l.hasOwnProperty(U)&&(k!=null||Q!=null))switch(U){case"type":s=k;break;case"name":r=k;break;case"checked":z=k;break;case"defaultChecked":G=k;break;case"value":m=k;break;case"defaultValue":y=k;break;case"children":case"dangerouslySetInnerHTML":if(k!=null)throw Error(c(137,t));break;default:k!==Q&&je(e,t,U,k,l,Q)}}Qi(e,m,y,E,z,G,s,r);return;case"select":k=m=y=U=null;for(s in n)if(E=n[s],n.hasOwnProperty(s)&&E!=null)switch(s){case"value":break;case"multiple":k=E;default:l.hasOwnProperty(s)||je(e,t,s,null,l,E)}for(r in l)if(s=l[r],E=n[r],l.hasOwnProperty(r)&&(s!=null||E!=null))switch(r){case"value":U=s;break;case"defaultValue":y=s;break;case"multiple":m=s;default:s!==E&&je(e,t,r,s,l,E)}t=y,n=m,l=k,U!=null?zl(e,!!n,U,!1):!!l!=!!n&&(t!=null?zl(e,!!n,t,!0):zl(e,!!n,n?[]:"",!1));return;case"textarea":k=U=null;for(y in n)if(r=n[y],n.hasOwnProperty(y)&&r!=null&&!l.hasOwnProperty(y))switch(y){case"value":break;case"children":break;default:je(e,t,y,null,l,r)}for(m in l)if(r=l[m],s=n[m],l.hasOwnProperty(m)&&(r!=null||s!=null))switch(m){case"value":U=r;break;case"defaultValue":k=r;break;case"children":break;case"dangerouslySetInnerHTML":if(r!=null)throw Error(c(91));break;default:r!==s&&je(e,t,m,r,l,s)}Ef(e,U,k);return;case"option":for(var fe in n)if(U=n[fe],n.hasOwnProperty(fe)&&U!=null&&!l.hasOwnProperty(fe))switch(fe){case"selected":e.selected=!1;break;default:je(e,t,fe,null,l,U)}for(E in l)if(U=l[E],k=n[E],l.hasOwnProperty(E)&&U!==k&&(U!=null||k!=null))switch(E){case"selected":e.selected=U&&typeof U!="function"&&typeof U!="symbol";break;default:je(e,t,E,U,l,k)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var ce in n)U=n[ce],n.hasOwnProperty(ce)&&U!=null&&!l.hasOwnProperty(ce)&&je(e,t,ce,null,l,U);for(z in l)if(U=l[z],k=n[z],l.hasOwnProperty(z)&&U!==k&&(U!=null||k!=null))switch(z){case"children":case"dangerouslySetInnerHTML":if(U!=null)throw Error(c(137,t));break;default:je(e,t,z,U,l,k)}return;default:if(Ki(t)){for(var Oe in n)U=n[Oe],n.hasOwnProperty(Oe)&&U!==void 0&&!l.hasOwnProperty(Oe)&&js(e,t,Oe,void 0,l,U);for(G in l)U=l[G],k=n[G],!l.hasOwnProperty(G)||U===k||U===void 0&&k===void 0||js(e,t,G,U,l,k);return}}for(var C in n)U=n[C],n.hasOwnProperty(C)&&U!=null&&!l.hasOwnProperty(C)&&je(e,t,C,null,l,U);for(Q in l)U=l[Q],k=n[Q],!l.hasOwnProperty(Q)||U===k||U==null&&k==null||je(e,t,Q,U,l,k)}var Os=null,Ds=null;function Go(e){return e.nodeType===9?e:e.ownerDocument}function Mh(e){switch(e){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function Nh(e,t){if(e===0)switch(t){case"svg":return 1;case"math":return 2;default:return 0}return e===1&&t==="foreignObject"?0:e}function zs(e,t){return e==="textarea"||e==="noscript"||typeof t.children=="string"||typeof t.children=="number"||typeof t.children=="bigint"||typeof t.dangerouslySetInnerHTML=="object"&&t.dangerouslySetInnerHTML!==null&&t.dangerouslySetInnerHTML.__html!=null}var Us=null;function fb(){var e=window.event;return e&&e.type==="popstate"?e===Us?!1:(Us=e,!0):(Us=null,!1)}var Th=typeof setTimeout=="function"?setTimeout:void 0,db=typeof clearTimeout=="function"?clearTimeout:void 0,Rh=typeof Promise=="function"?Promise:void 0,mb=typeof queueMicrotask=="function"?queueMicrotask:typeof Rh<"u"?function(e){return Rh.resolve(null).then(e).catch(hb)}:Th;function hb(e){setTimeout(function(){throw e})}function Kn(e){return e==="head"}function _h(e,t){var n=t,l=0,r=0;do{var s=n.nextSibling;if(e.removeChild(n),s&&s.nodeType===8)if(n=s.data,n==="/$"){if(0<l&&8>l){n=l;var m=e.ownerDocument;if(n&1&&hr(m.documentElement),n&2&&hr(m.body),n&4)for(n=m.head,hr(n),m=n.firstChild;m;){var y=m.nextSibling,E=m.nodeName;m[_a]||E==="SCRIPT"||E==="STYLE"||E==="LINK"&&m.rel.toLowerCase()==="stylesheet"||n.removeChild(m),m=y}}if(r===0){e.removeChild(s),wr(t);return}r--}else n==="$"||n==="$?"||n==="$!"?r++:l=n.charCodeAt(0)-48;else l=0;n=s}while(n);wr(t)}function ks(e){var t=e.firstChild;for(t&&t.nodeType===10&&(t=t.nextSibling);t;){var n=t;switch(t=t.nextSibling,n.nodeName){case"HTML":case"HEAD":case"BODY":ks(n),Gi(n);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(n.rel.toLowerCase()==="stylesheet")continue}e.removeChild(n)}}function vb(e,t,n,l){for(;e.nodeType===1;){var r=n;if(e.nodeName.toLowerCase()!==t.toLowerCase()){if(!l&&(e.nodeName!=="INPUT"||e.type!=="hidden"))break}else if(l){if(!e[_a])switch(t){case"meta":if(!e.hasAttribute("itemprop"))break;return e;case"link":if(s=e.getAttribute("rel"),s==="stylesheet"&&e.hasAttribute("data-precedence"))break;if(s!==r.rel||e.getAttribute("href")!==(r.href==null||r.href===""?null:r.href)||e.getAttribute("crossorigin")!==(r.crossOrigin==null?null:r.crossOrigin)||e.getAttribute("title")!==(r.title==null?null:r.title))break;return e;case"style":if(e.hasAttribute("data-precedence"))break;return e;case"script":if(s=e.getAttribute("src"),(s!==(r.src==null?null:r.src)||e.getAttribute("type")!==(r.type==null?null:r.type)||e.getAttribute("crossorigin")!==(r.crossOrigin==null?null:r.crossOrigin))&&s&&e.hasAttribute("async")&&!e.hasAttribute("itemprop"))break;return e;default:return e}}else if(t==="input"&&e.type==="hidden"){var s=r.name==null?null:""+r.name;if(r.type==="hidden"&&e.getAttribute("name")===s)return e}else return e;if(e=Yt(e.nextSibling),e===null)break}return null}function gb(e,t,n){if(t==="")return null;for(;e.nodeType!==3;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!n||(e=Yt(e.nextSibling),e===null))return null;return e}function Ls(e){return e.data==="$!"||e.data==="$?"&&e.ownerDocument.readyState==="complete"}function pb(e,t){var n=e.ownerDocument;if(e.data!=="$?"||n.readyState==="complete")t();else{var l=function(){t(),n.removeEventListener("DOMContentLoaded",l)};n.addEventListener("DOMContentLoaded",l),e._reactRetry=l}}function Yt(e){for(;e!=null;e=e.nextSibling){var t=e.nodeType;if(t===1||t===3)break;if(t===8){if(t=e.data,t==="$"||t==="$!"||t==="$?"||t==="F!"||t==="F")break;if(t==="/$")return null}}return e}var Hs=null;function Ch(e){e=e.previousSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="$"||n==="$!"||n==="$?"){if(t===0)return e;t--}else n==="/$"&&t++}e=e.previousSibling}return null}function jh(e,t,n){switch(t=Go(n),e){case"html":if(e=t.documentElement,!e)throw Error(c(452));return e;case"head":if(e=t.head,!e)throw Error(c(453));return e;case"body":if(e=t.body,!e)throw Error(c(454));return e;default:throw Error(c(451))}}function hr(e){for(var t=e.attributes;t.length;)e.removeAttributeNode(t[0]);Gi(e)}var Lt=new Map,Oh=new Set;function Yo(e){return typeof e.getRootNode=="function"?e.getRootNode():e.nodeType===9?e:e.ownerDocument}var wn=K.d;K.d={f:yb,r:bb,D:xb,C:Sb,L:wb,m:Eb,X:Mb,S:Ab,M:Nb};function yb(){var e=wn.f(),t=Do();return e||t}function bb(e){var t=Cl(e);t!==null&&t.tag===5&&t.type==="form"?Fd(t):wn.r(e)}var ua=typeof document>"u"?null:document;function Dh(e,t,n){var l=ua;if(l&&typeof t=="string"&&t){var r=Ct(t);r='link[rel="'+e+'"][href="'+r+'"]',typeof n=="string"&&(r+='[crossorigin="'+n+'"]'),Oh.has(r)||(Oh.add(r),e={rel:e,crossOrigin:n,href:t},l.querySelector(r)===null&&(t=l.createElement("link"),nt(t,"link",e),Je(t),l.head.appendChild(t)))}}function xb(e){wn.D(e),Dh("dns-prefetch",e,null)}function Sb(e,t){wn.C(e,t),Dh("preconnect",e,t)}function wb(e,t,n){wn.L(e,t,n);var l=ua;if(l&&e&&t){var r='link[rel="preload"][as="'+Ct(t)+'"]';t==="image"&&n&&n.imageSrcSet?(r+='[imagesrcset="'+Ct(n.imageSrcSet)+'"]',typeof n.imageSizes=="string"&&(r+='[imagesizes="'+Ct(n.imageSizes)+'"]')):r+='[href="'+Ct(e)+'"]';var s=r;switch(t){case"style":s=fa(e);break;case"script":s=da(e)}Lt.has(s)||(e=p({rel:"preload",href:t==="image"&&n&&n.imageSrcSet?void 0:e,as:t},n),Lt.set(s,e),l.querySelector(r)!==null||t==="style"&&l.querySelector(vr(s))||t==="script"&&l.querySelector(gr(s))||(t=l.createElement("link"),nt(t,"link",e),Je(t),l.head.appendChild(t)))}}function Eb(e,t){wn.m(e,t);var n=ua;if(n&&e){var l=t&&typeof t.as=="string"?t.as:"script",r='link[rel="modulepreload"][as="'+Ct(l)+'"][href="'+Ct(e)+'"]',s=r;switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":s=da(e)}if(!Lt.has(s)&&(e=p({rel:"modulepreload",href:e},t),Lt.set(s,e),n.querySelector(r)===null)){switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(n.querySelector(gr(s)))return}l=n.createElement("link"),nt(l,"link",e),Je(l),n.head.appendChild(l)}}}function Ab(e,t,n){wn.S(e,t,n);var l=ua;if(l&&e){var r=jl(l).hoistableStyles,s=fa(e);t=t||"default";var m=r.get(s);if(!m){var y={loading:0,preload:null};if(m=l.querySelector(vr(s)))y.loading=5;else{e=p({rel:"stylesheet",href:e,"data-precedence":t},n),(n=Lt.get(s))&&Bs(e,n);var E=m=l.createElement("link");Je(E),nt(E,"link",e),E._p=new Promise(function(z,G){E.onload=z,E.onerror=G}),E.addEventListener("load",function(){y.loading|=1}),E.addEventListener("error",function(){y.loading|=2}),y.loading|=4,Vo(m,t,l)}m={type:"stylesheet",instance:m,count:1,state:y},r.set(s,m)}}}function Mb(e,t){wn.X(e,t);var n=ua;if(n&&e){var l=jl(n).hoistableScripts,r=da(e),s=l.get(r);s||(s=n.querySelector(gr(r)),s||(e=p({src:e,async:!0},t),(t=Lt.get(r))&&qs(e,t),s=n.createElement("script"),Je(s),nt(s,"link",e),n.head.appendChild(s)),s={type:"script",instance:s,count:1,state:null},l.set(r,s))}}function Nb(e,t){wn.M(e,t);var n=ua;if(n&&e){var l=jl(n).hoistableScripts,r=da(e),s=l.get(r);s||(s=n.querySelector(gr(r)),s||(e=p({src:e,async:!0,type:"module"},t),(t=Lt.get(r))&&qs(e,t),s=n.createElement("script"),Je(s),nt(s,"link",e),n.head.appendChild(s)),s={type:"script",instance:s,count:1,state:null},l.set(r,s))}}function zh(e,t,n,l){var r=(r=oe.current)?Yo(r):null;if(!r)throw Error(c(446));switch(e){case"meta":case"title":return null;case"style":return typeof n.precedence=="string"&&typeof n.href=="string"?(t=fa(n.href),n=jl(r).hoistableStyles,l=n.get(t),l||(l={type:"style",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};case"link":if(n.rel==="stylesheet"&&typeof n.href=="string"&&typeof n.precedence=="string"){e=fa(n.href);var s=jl(r).hoistableStyles,m=s.get(e);if(m||(r=r.ownerDocument||r,m={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},s.set(e,m),(s=r.querySelector(vr(e)))&&!s._p&&(m.instance=s,m.state.loading=5),Lt.has(e)||(n={rel:"preload",as:"style",href:n.href,crossOrigin:n.crossOrigin,integrity:n.integrity,media:n.media,hrefLang:n.hrefLang,referrerPolicy:n.referrerPolicy},Lt.set(e,n),s||Tb(r,e,n,m.state))),t&&l===null)throw Error(c(528,""));return m}if(t&&l!==null)throw Error(c(529,""));return null;case"script":return t=n.async,n=n.src,typeof n=="string"&&t&&typeof t!="function"&&typeof t!="symbol"?(t=da(n),n=jl(r).hoistableScripts,l=n.get(t),l||(l={type:"script",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};default:throw Error(c(444,e))}}function fa(e){return'href="'+Ct(e)+'"'}function vr(e){return'link[rel="stylesheet"]['+e+"]"}function Uh(e){return p({},e,{"data-precedence":e.precedence,precedence:null})}function Tb(e,t,n,l){e.querySelector('link[rel="preload"][as="style"]['+t+"]")?l.loading=1:(t=e.createElement("link"),l.preload=t,t.addEventListener("load",function(){return l.loading|=1}),t.addEventListener("error",function(){return l.loading|=2}),nt(t,"link",n),Je(t),e.head.appendChild(t))}function da(e){return'[src="'+Ct(e)+'"]'}function gr(e){return"script[async]"+e}function kh(e,t,n){if(t.count++,t.instance===null)switch(t.type){case"style":var l=e.querySelector('style[data-href~="'+Ct(n.href)+'"]');if(l)return t.instance=l,Je(l),l;var r=p({},n,{"data-href":n.href,"data-precedence":n.precedence,href:null,precedence:null});return l=(e.ownerDocument||e).createElement("style"),Je(l),nt(l,"style",r),Vo(l,n.precedence,e),t.instance=l;case"stylesheet":r=fa(n.href);var s=e.querySelector(vr(r));if(s)return t.state.loading|=4,t.instance=s,Je(s),s;l=Uh(n),(r=Lt.get(r))&&Bs(l,r),s=(e.ownerDocument||e).createElement("link"),Je(s);var m=s;return m._p=new Promise(function(y,E){m.onload=y,m.onerror=E}),nt(s,"link",l),t.state.loading|=4,Vo(s,n.precedence,e),t.instance=s;case"script":return s=da(n.src),(r=e.querySelector(gr(s)))?(t.instance=r,Je(r),r):(l=n,(r=Lt.get(s))&&(l=p({},n),qs(l,r)),e=e.ownerDocument||e,r=e.createElement("script"),Je(r),nt(r,"link",l),e.head.appendChild(r),t.instance=r);case"void":return null;default:throw Error(c(443,t.type))}else t.type==="stylesheet"&&(t.state.loading&4)===0&&(l=t.instance,t.state.loading|=4,Vo(l,n.precedence,e));return t.instance}function Vo(e,t,n){for(var l=n.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),r=l.length?l[l.length-1]:null,s=r,m=0;m<l.length;m++){var y=l[m];if(y.dataset.precedence===t)s=y;else if(s!==r)break}s?s.parentNode.insertBefore(e,s.nextSibling):(t=n.nodeType===9?n.head:n,t.insertBefore(e,t.firstChild))}function Bs(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.title==null&&(e.title=t.title)}function qs(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.integrity==null&&(e.integrity=t.integrity)}var Xo=null;function Lh(e,t,n){if(Xo===null){var l=new Map,r=Xo=new Map;r.set(n,l)}else r=Xo,l=r.get(n),l||(l=new Map,r.set(n,l));if(l.has(e))return l;for(l.set(e,null),n=n.getElementsByTagName(e),r=0;r<n.length;r++){var s=n[r];if(!(s[_a]||s[lt]||e==="link"&&s.getAttribute("rel")==="stylesheet")&&s.namespaceURI!=="http://www.w3.org/2000/svg"){var m=s.getAttribute(t)||"";m=e+m;var y=l.get(m);y?y.push(s):l.set(m,[s])}}return l}function Hh(e,t,n){e=e.ownerDocument||e,e.head.insertBefore(n,t==="title"?e.querySelector("head > title"):null)}function Rb(e,t,n){if(n===1||t.itemProp!=null)return!1;switch(e){case"meta":case"title":return!0;case"style":if(typeof t.precedence!="string"||typeof t.href!="string"||t.href==="")break;return!0;case"link":if(typeof t.rel!="string"||typeof t.href!="string"||t.href===""||t.onLoad||t.onError)break;switch(t.rel){case"stylesheet":return e=t.disabled,typeof t.precedence=="string"&&e==null;default:return!0}case"script":if(t.async&&typeof t.async!="function"&&typeof t.async!="symbol"&&!t.onLoad&&!t.onError&&t.src&&typeof t.src=="string")return!0}return!1}function Bh(e){return!(e.type==="stylesheet"&&(e.state.loading&3)===0)}var pr=null;function _b(){}function Cb(e,t,n){if(pr===null)throw Error(c(475));var l=pr;if(t.type==="stylesheet"&&(typeof n.media!="string"||matchMedia(n.media).matches!==!1)&&(t.state.loading&4)===0){if(t.instance===null){var r=fa(n.href),s=e.querySelector(vr(r));if(s){e=s._p,e!==null&&typeof e=="object"&&typeof e.then=="function"&&(l.count++,l=Qo.bind(l),e.then(l,l)),t.state.loading|=4,t.instance=s,Je(s);return}s=e.ownerDocument||e,n=Uh(n),(r=Lt.get(r))&&Bs(n,r),s=s.createElement("link"),Je(s);var m=s;m._p=new Promise(function(y,E){m.onload=y,m.onerror=E}),nt(s,"link",n),t.instance=s}l.stylesheets===null&&(l.stylesheets=new Map),l.stylesheets.set(t,e),(e=t.state.preload)&&(t.state.loading&3)===0&&(l.count++,t=Qo.bind(l),e.addEventListener("load",t),e.addEventListener("error",t))}}function jb(){if(pr===null)throw Error(c(475));var e=pr;return e.stylesheets&&e.count===0&&Gs(e,e.stylesheets),0<e.count?function(t){var n=setTimeout(function(){if(e.stylesheets&&Gs(e,e.stylesheets),e.unsuspend){var l=e.unsuspend;e.unsuspend=null,l()}},6e4);return e.unsuspend=t,function(){e.unsuspend=null,clearTimeout(n)}}:null}function Qo(){if(this.count--,this.count===0){if(this.stylesheets)Gs(this,this.stylesheets);else if(this.unsuspend){var e=this.unsuspend;this.unsuspend=null,e()}}}var Zo=null;function Gs(e,t){e.stylesheets=null,e.unsuspend!==null&&(e.count++,Zo=new Map,t.forEach(Ob,e),Zo=null,Qo.call(e))}function Ob(e,t){if(!(t.state.loading&4)){var n=Zo.get(e);if(n)var l=n.get(null);else{n=new Map,Zo.set(e,n);for(var r=e.querySelectorAll("link[data-precedence],style[data-precedence]"),s=0;s<r.length;s++){var m=r[s];(m.nodeName==="LINK"||m.getAttribute("media")!=="not all")&&(n.set(m.dataset.precedence,m),l=m)}l&&n.set(null,l)}r=t.instance,m=r.getAttribute("data-precedence"),s=n.get(m)||l,s===l&&n.set(null,r),n.set(m,r),this.count++,l=Qo.bind(this),r.addEventListener("load",l),r.addEventListener("error",l),s?s.parentNode.insertBefore(r,s.nextSibling):(e=e.nodeType===9?e.head:e,e.insertBefore(r,e.firstChild)),t.state.loading|=4}}var yr={$$typeof:L,Provider:null,Consumer:null,_currentValue:H,_currentValue2:H,_threadCount:0};function Db(e,t,n,l,r,s,m,y){this.tag=1,this.containerInfo=e,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=Li(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=Li(0),this.hiddenUpdates=Li(null),this.identifierPrefix=l,this.onUncaughtError=r,this.onCaughtError=s,this.onRecoverableError=m,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=y,this.incompleteTransitions=new Map}function qh(e,t,n,l,r,s,m,y,E,z,G,Q){return e=new Db(e,t,n,m,y,E,z,Q),t=1,s===!0&&(t|=24),s=yt(3,null,null,t),e.current=s,s.stateNode=e,t=wc(),t.refCount++,e.pooledCache=t,t.refCount++,s.memoizedState={element:l,isDehydrated:n,cache:t},Nc(s),e}function Gh(e){return e?(e=Vl,e):Vl}function Yh(e,t,n,l,r,s){r=Gh(r),l.context===null?l.context=r:l.pendingContext=r,l=zn(t),l.payload={element:n},s=s===void 0?null:s,s!==null&&(l.callback=s),n=Un(e,l,t),n!==null&&(Et(n,e,t),Pa(n,e,t))}function Vh(e,t){if(e=e.memoizedState,e!==null&&e.dehydrated!==null){var n=e.retryLane;e.retryLane=n!==0&&n<t?n:t}}function Ys(e,t){Vh(e,t),(e=e.alternate)&&Vh(e,t)}function Xh(e){if(e.tag===13){var t=Yl(e,67108864);t!==null&&Et(t,e,67108864),Ys(e,67108864)}}var Ko=!0;function zb(e,t,n,l){var r=D.T;D.T=null;var s=K.p;try{K.p=2,Vs(e,t,n,l)}finally{K.p=s,D.T=r}}function Ub(e,t,n,l){var r=D.T;D.T=null;var s=K.p;try{K.p=8,Vs(e,t,n,l)}finally{K.p=s,D.T=r}}function Vs(e,t,n,l){if(Ko){var r=Xs(l);if(r===null)Cs(e,t,l,Po,n),Zh(e,l);else if(Lb(r,e,t,n,l))l.stopPropagation();else if(Zh(e,l),t&4&&-1<kb.indexOf(e)){for(;r!==null;){var s=Cl(r);if(s!==null)switch(s.tag){case 3:if(s=s.stateNode,s.current.memoizedState.isDehydrated){var m=al(s.pendingLanes);if(m!==0){var y=s;for(y.pendingLanes|=2,y.entangledLanes|=2;m;){var E=1<<31-gt(m);y.entanglements[1]|=E,m&=~E}It(s),(Re&6)===0&&(jo=Pt()+500,fr(0))}}break;case 13:y=Yl(s,2),y!==null&&Et(y,s,2),Do(),Ys(s,2)}if(s=Xs(l),s===null&&Cs(e,t,l,Po,n),s===r)break;r=s}r!==null&&l.stopPropagation()}else Cs(e,t,l,null,n)}}function Xs(e){return e=$i(e),Qs(e)}var Po=null;function Qs(e){if(Po=null,e=_l(e),e!==null){var t=f(e);if(t===null)e=null;else{var n=t.tag;if(n===13){if(e=h(t),e!==null)return e;e=null}else if(n===3){if(t.stateNode.current.memoizedState.isDehydrated)return t.tag===3?t.stateNode.containerInfo:null;e=null}else t!==e&&(e=null)}}return Po=e,null}function Qh(e){switch(e){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(Sy()){case af:return 2;case rf:return 8;case Br:case wy:return 32;case of:return 268435456;default:return 32}default:return 32}}var Zs=!1,Pn=null,$n=null,Jn=null,br=new Map,xr=new Map,Wn=[],kb="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function Zh(e,t){switch(e){case"focusin":case"focusout":Pn=null;break;case"dragenter":case"dragleave":$n=null;break;case"mouseover":case"mouseout":Jn=null;break;case"pointerover":case"pointerout":br.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":xr.delete(t.pointerId)}}function Sr(e,t,n,l,r,s){return e===null||e.nativeEvent!==s?(e={blockedOn:t,domEventName:n,eventSystemFlags:l,nativeEvent:s,targetContainers:[r]},t!==null&&(t=Cl(t),t!==null&&Xh(t)),e):(e.eventSystemFlags|=l,t=e.targetContainers,r!==null&&t.indexOf(r)===-1&&t.push(r),e)}function Lb(e,t,n,l,r){switch(t){case"focusin":return Pn=Sr(Pn,e,t,n,l,r),!0;case"dragenter":return $n=Sr($n,e,t,n,l,r),!0;case"mouseover":return Jn=Sr(Jn,e,t,n,l,r),!0;case"pointerover":var s=r.pointerId;return br.set(s,Sr(br.get(s)||null,e,t,n,l,r)),!0;case"gotpointercapture":return s=r.pointerId,xr.set(s,Sr(xr.get(s)||null,e,t,n,l,r)),!0}return!1}function Kh(e){var t=_l(e.target);if(t!==null){var n=f(t);if(n!==null){if(t=n.tag,t===13){if(t=h(n),t!==null){e.blockedOn=t,Cy(e.priority,function(){if(n.tag===13){var l=wt();l=Hi(l);var r=Yl(n,l);r!==null&&Et(r,n,l),Ys(n,l)}});return}}else if(t===3&&n.stateNode.current.memoizedState.isDehydrated){e.blockedOn=n.tag===3?n.stateNode.containerInfo:null;return}}}e.blockedOn=null}function $o(e){if(e.blockedOn!==null)return!1;for(var t=e.targetContainers;0<t.length;){var n=Xs(e.nativeEvent);if(n===null){n=e.nativeEvent;var l=new n.constructor(n.type,n);Pi=l,n.target.dispatchEvent(l),Pi=null}else return t=Cl(n),t!==null&&Xh(t),e.blockedOn=n,!1;t.shift()}return!0}function Ph(e,t,n){$o(e)&&n.delete(t)}function Hb(){Zs=!1,Pn!==null&&$o(Pn)&&(Pn=null),$n!==null&&$o($n)&&($n=null),Jn!==null&&$o(Jn)&&(Jn=null),br.forEach(Ph),xr.forEach(Ph)}function Jo(e,t){e.blockedOn===t&&(e.blockedOn=null,Zs||(Zs=!0,a.unstable_scheduleCallback(a.unstable_NormalPriority,Hb)))}var Wo=null;function $h(e){Wo!==e&&(Wo=e,a.unstable_scheduleCallback(a.unstable_NormalPriority,function(){Wo===e&&(Wo=null);for(var t=0;t<e.length;t+=3){var n=e[t],l=e[t+1],r=e[t+2];if(typeof l!="function"){if(Qs(l||n)===null)continue;break}var s=Cl(n);s!==null&&(e.splice(t,3),t-=3,Qc(s,{pending:!0,data:r,method:n.method,action:l},l,r))}}))}function wr(e){function t(E){return Jo(E,e)}Pn!==null&&Jo(Pn,e),$n!==null&&Jo($n,e),Jn!==null&&Jo(Jn,e),br.forEach(t),xr.forEach(t);for(var n=0;n<Wn.length;n++){var l=Wn[n];l.blockedOn===e&&(l.blockedOn=null)}for(;0<Wn.length&&(n=Wn[0],n.blockedOn===null);)Kh(n),n.blockedOn===null&&Wn.shift();if(n=(e.ownerDocument||e).$$reactFormReplay,n!=null)for(l=0;l<n.length;l+=3){var r=n[l],s=n[l+1],m=r[ut]||null;if(typeof s=="function")m||$h(n);else if(m){var y=null;if(s&&s.hasAttribute("formAction")){if(r=s,m=s[ut]||null)y=m.formAction;else if(Qs(r)!==null)continue}else y=m.action;typeof y=="function"?n[l+1]=y:(n.splice(l,3),l-=3),$h(n)}}}function Ks(e){this._internalRoot=e}Fo.prototype.render=Ks.prototype.render=function(e){var t=this._internalRoot;if(t===null)throw Error(c(409));var n=t.current,l=wt();Yh(n,l,e,t,null,null)},Fo.prototype.unmount=Ks.prototype.unmount=function(){var e=this._internalRoot;if(e!==null){this._internalRoot=null;var t=e.containerInfo;Yh(e.current,2,null,e,null,null),Do(),t[Rl]=null}};function Fo(e){this._internalRoot=e}Fo.prototype.unstable_scheduleHydration=function(e){if(e){var t=df();e={blockedOn:null,target:e,priority:t};for(var n=0;n<Wn.length&&t!==0&&t<Wn[n].priority;n++);Wn.splice(n,0,e),n===0&&Kh(e)}};var Jh=o.version;if(Jh!=="19.1.1")throw Error(c(527,Jh,"19.1.1"));K.findDOMNode=function(e){var t=e._reactInternals;if(t===void 0)throw typeof e.render=="function"?Error(c(188)):(e=Object.keys(e).join(","),Error(c(268,e)));return e=b(t),e=e!==null?g(e):null,e=e===null?null:e.stateNode,e};var Bb={bundleType:0,version:"19.1.1",rendererPackageName:"react-dom",currentDispatcherRef:D,reconcilerVersion:"19.1.1"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var Io=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!Io.isDisabled&&Io.supportsFiber)try{Na=Io.inject(Bb),vt=Io}catch{}}return Ar.createRoot=function(e,t){if(!u(e))throw Error(c(299));var n=!1,l="",r=dm,s=mm,m=hm,y=null;return t!=null&&(t.unstable_strictMode===!0&&(n=!0),t.identifierPrefix!==void 0&&(l=t.identifierPrefix),t.onUncaughtError!==void 0&&(r=t.onUncaughtError),t.onCaughtError!==void 0&&(s=t.onCaughtError),t.onRecoverableError!==void 0&&(m=t.onRecoverableError),t.unstable_transitionCallbacks!==void 0&&(y=t.unstable_transitionCallbacks)),t=qh(e,1,!1,null,null,n,l,r,s,m,y,null),e[Rl]=t.current,_s(e),new Ks(t)},Ar.hydrateRoot=function(e,t,n){if(!u(e))throw Error(c(299));var l=!1,r="",s=dm,m=mm,y=hm,E=null,z=null;return n!=null&&(n.unstable_strictMode===!0&&(l=!0),n.identifierPrefix!==void 0&&(r=n.identifierPrefix),n.onUncaughtError!==void 0&&(s=n.onUncaughtError),n.onCaughtError!==void 0&&(m=n.onCaughtError),n.onRecoverableError!==void 0&&(y=n.onRecoverableError),n.unstable_transitionCallbacks!==void 0&&(E=n.unstable_transitionCallbacks),n.formState!==void 0&&(z=n.formState)),t=qh(e,1,!0,t,n??null,l,r,s,m,y,E,z),t.context=Gh(null),n=t.current,l=wt(),l=Hi(l),r=zn(l),r.callback=null,Un(n,r,l),n=l,t.current.lanes=n,Ra(t,n),It(t),e[Rl]=t.current,_s(e),new Fo(t)},Ar.version="19.1.1",Ar}var ov;function $b(){if(ov)return Js.exports;ov=1;function a(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(a)}catch(o){console.error(o)}}return a(),Js.exports=Pb(),Js.exports}var Jb=$b();function iv(a,o){if(typeof a=="function")return a(o);a!=null&&(a.current=o)}function Si(...a){return o=>{let i=!1;const c=a.map(u=>{const f=iv(u,o);return!i&&typeof f=="function"&&(i=!0),f});if(i)return()=>{for(let u=0;u<c.length;u++){const f=c[u];typeof f=="function"?f():iv(a[u],null)}}}}function Qe(...a){return x.useCallback(Si(...a),a)}function Cr(a){const o=Fb(a),i=x.forwardRef((c,u)=>{const{children:f,...h}=c,v=x.Children.toArray(f),b=v.find(ex);if(b){const g=b.props.children,p=v.map(S=>S===b?x.Children.count(g)>1?x.Children.only(null):x.isValidElement(g)?g.props.children:null:S);return d.jsx(o,{...h,ref:u,children:x.isValidElement(g)?x.cloneElement(g,void 0,p):null})}return d.jsx(o,{...h,ref:u,children:f})});return i.displayName=`${a}.Slot`,i}var Wb=Cr("Slot");function Fb(a){const o=x.forwardRef((i,c)=>{const{children:u,...f}=i;if(x.isValidElement(u)){const h=nx(u),v=tx(f,u.props);return u.type!==x.Fragment&&(v.ref=c?Si(c,h):h),x.cloneElement(u,v)}return x.Children.count(u)>1?x.Children.only(null):null});return o.displayName=`${a}.SlotClone`,o}var Ib=Symbol("radix.slottable");function ex(a){return x.isValidElement(a)&&typeof a.type=="function"&&"__radixId"in a.type&&a.type.__radixId===Ib}function tx(a,o){const i={...o};for(const c in o){const u=a[c],f=o[c];/^on[A-Z]/.test(c)?u&&f?i[c]=(...v)=>{const b=f(...v);return u(...v),b}:u&&(i[c]=u):c==="style"?i[c]={...u,...f}:c==="className"&&(i[c]=[u,f].filter(Boolean).join(" "))}return{...a,...i}}function nx(a){let o=Object.getOwnPropertyDescriptor(a.props,"ref")?.get,i=o&&"isReactWarning"in o&&o.isReactWarning;return i?a.ref:(o=Object.getOwnPropertyDescriptor(a,"ref")?.get,i=o&&"isReactWarning"in o&&o.isReactWarning,i?a.props.ref:a.props.ref||a.ref)}function $v(a){var o,i,c="";if(typeof a=="string"||typeof a=="number")c+=a;else if(typeof a=="object")if(Array.isArray(a)){var u=a.length;for(o=0;o<u;o++)a[o]&&(i=$v(a[o]))&&(c&&(c+=" "),c+=i)}else for(i in a)a[i]&&(c&&(c+=" "),c+=i);return c}function Jv(){for(var a,o,i=0,c="",u=arguments.length;i<u;i++)(a=arguments[i])&&(o=$v(a))&&(c&&(c+=" "),c+=o);return c}const cv=a=>typeof a=="boolean"?`${a}`:a===0?"0":a,sv=Jv,Wv=(a,o)=>i=>{var c;if(o?.variants==null)return sv(a,i?.class,i?.className);const{variants:u,defaultVariants:f}=o,h=Object.keys(u).map(g=>{const p=i?.[g],S=f?.[g];if(p===null)return null;const w=cv(p)||cv(S);return u[g][w]}),v=i&&Object.entries(i).reduce((g,p)=>{let[S,w]=p;return w===void 0||(g[S]=w),g},{}),b=o==null||(c=o.compoundVariants)===null||c===void 0?void 0:c.reduce((g,p)=>{let{class:S,className:w,...M}=p;return Object.entries(M).every(T=>{let[N,_]=T;return Array.isArray(_)?_.includes({...f,...v}[N]):{...f,...v}[N]===_})?[...g,S,w]:g},[]);return sv(a,h,b,i?.class,i?.className)},Ou="-",lx=a=>{const o=rx(a),{conflictingClassGroups:i,conflictingClassGroupModifiers:c}=a;return{getClassGroupId:h=>{const v=h.split(Ou);return v[0]===""&&v.length!==1&&v.shift(),Fv(v,o)||ax(h)},getConflictingClassGroupIds:(h,v)=>{const b=i[h]||[];return v&&c[h]?[...b,...c[h]]:b}}},Fv=(a,o)=>{if(a.length===0)return o.classGroupId;const i=a[0],c=o.nextPart.get(i),u=c?Fv(a.slice(1),c):void 0;if(u)return u;if(o.validators.length===0)return;const f=a.join(Ou);return o.validators.find(({validator:h})=>h(f))?.classGroupId},uv=/^\[(.+)\]$/,ax=a=>{if(uv.test(a)){const o=uv.exec(a)[1],i=o?.substring(0,o.indexOf(":"));if(i)return"arbitrary.."+i}},rx=a=>{const{theme:o,classGroups:i}=a,c={nextPart:new Map,validators:[]};for(const u in i)vu(i[u],c,u,o);return c},vu=(a,o,i,c)=>{a.forEach(u=>{if(typeof u=="string"){const f=u===""?o:fv(o,u);f.classGroupId=i;return}if(typeof u=="function"){if(ox(u)){vu(u(c),o,i,c);return}o.validators.push({validator:u,classGroupId:i});return}Object.entries(u).forEach(([f,h])=>{vu(h,fv(o,f),i,c)})})},fv=(a,o)=>{let i=a;return o.split(Ou).forEach(c=>{i.nextPart.has(c)||i.nextPart.set(c,{nextPart:new Map,validators:[]}),i=i.nextPart.get(c)}),i},ox=a=>a.isThemeGetter,ix=a=>{if(a<1)return{get:()=>{},set:()=>{}};let o=0,i=new Map,c=new Map;const u=(f,h)=>{i.set(f,h),o++,o>a&&(o=0,c=i,i=new Map)};return{get(f){let h=i.get(f);if(h!==void 0)return h;if((h=c.get(f))!==void 0)return u(f,h),h},set(f,h){i.has(f)?i.set(f,h):u(f,h)}}},gu="!",pu=":",cx=pu.length,sx=a=>{const{prefix:o,experimentalParseClassName:i}=a;let c=u=>{const f=[];let h=0,v=0,b=0,g;for(let T=0;T<u.length;T++){let N=u[T];if(h===0&&v===0){if(N===pu){f.push(u.slice(b,T)),b=T+cx;continue}if(N==="/"){g=T;continue}}N==="["?h++:N==="]"?h--:N==="("?v++:N===")"&&v--}const p=f.length===0?u:u.substring(b),S=ux(p),w=S!==p,M=g&&g>b?g-b:void 0;return{modifiers:f,hasImportantModifier:w,baseClassName:S,maybePostfixModifierPosition:M}};if(o){const u=o+pu,f=c;c=h=>h.startsWith(u)?f(h.substring(u.length)):{isExternal:!0,modifiers:[],hasImportantModifier:!1,baseClassName:h,maybePostfixModifierPosition:void 0}}if(i){const u=c;c=f=>i({className:f,parseClassName:u})}return c},ux=a=>a.endsWith(gu)?a.substring(0,a.length-1):a.startsWith(gu)?a.substring(1):a,fx=a=>{const o=Object.fromEntries(a.orderSensitiveModifiers.map(c=>[c,!0]));return c=>{if(c.length<=1)return c;const u=[];let f=[];return c.forEach(h=>{h[0]==="["||o[h]?(u.push(...f.sort(),h),f=[]):f.push(h)}),u.push(...f.sort()),u}},dx=a=>({cache:ix(a.cacheSize),parseClassName:sx(a),sortModifiers:fx(a),...lx(a)}),mx=/\s+/,hx=(a,o)=>{const{parseClassName:i,getClassGroupId:c,getConflictingClassGroupIds:u,sortModifiers:f}=o,h=[],v=a.trim().split(mx);let b="";for(let g=v.length-1;g>=0;g-=1){const p=v[g],{isExternal:S,modifiers:w,hasImportantModifier:M,baseClassName:T,maybePostfixModifierPosition:N}=i(p);if(S){b=p+(b.length>0?" "+b:b);continue}let _=!!N,j=c(_?T.substring(0,N):T);if(!j){if(!_){b=p+(b.length>0?" "+b:b);continue}if(j=c(T),!j){b=p+(b.length>0?" "+b:b);continue}_=!1}const q=f(w).join(":"),L=M?q+gu:q,Y=L+j;if(h.includes(Y))continue;h.push(Y);const B=u(j,_);for(let J=0;J<B.length;++J){const P=B[J];h.push(L+P)}b=p+(b.length>0?" "+b:b)}return b};function vx(){let a=0,o,i,c="";for(;a<arguments.length;)(o=arguments[a++])&&(i=Iv(o))&&(c&&(c+=" "),c+=i);return c}const Iv=a=>{if(typeof a=="string")return a;let o,i="";for(let c=0;c<a.length;c++)a[c]&&(o=Iv(a[c]))&&(i&&(i+=" "),i+=o);return i};function gx(a,...o){let i,c,u,f=h;function h(b){const g=o.reduce((p,S)=>S(p),a());return i=dx(g),c=i.cache.get,u=i.cache.set,f=v,v(b)}function v(b){const g=c(b);if(g)return g;const p=hx(b,i);return u(b,p),p}return function(){return f(vx.apply(null,arguments))}}const $e=a=>{const o=i=>i[a]||[];return o.isThemeGetter=!0,o},eg=/^\[(?:(\w[\w-]*):)?(.+)\]$/i,tg=/^\((?:(\w[\w-]*):)?(.+)\)$/i,px=/^\d+\/\d+$/,yx=/^(\d+(\.\d+)?)?(xs|sm|md|lg|xl)$/,bx=/\d+(%|px|r?em|[sdl]?v([hwib]|min|max)|pt|pc|in|cm|mm|cap|ch|ex|r?lh|cq(w|h|i|b|min|max))|\b(calc|min|max|clamp)\(.+\)|^0$/,xx=/^(rgba?|hsla?|hwb|(ok)?(lab|lch)|color-mix)\(.+\)$/,Sx=/^(inset_)?-?((\d+)?\.?(\d+)[a-z]+|0)_-?((\d+)?\.?(\d+)[a-z]+|0)/,wx=/^(url|image|image-set|cross-fade|element|(repeating-)?(linear|radial|conic)-gradient)\(.+\)$/,ma=a=>px.test(a),ye=a=>!!a&&!Number.isNaN(Number(a)),In=a=>!!a&&Number.isInteger(Number(a)),eu=a=>a.endsWith("%")&&ye(a.slice(0,-1)),En=a=>yx.test(a),Ex=()=>!0,Ax=a=>bx.test(a)&&!xx.test(a),ng=()=>!1,Mx=a=>Sx.test(a),Nx=a=>wx.test(a),Tx=a=>!ne(a)&&!le(a),Rx=a=>wa(a,rg,ng),ne=a=>eg.test(a),wl=a=>wa(a,og,Ax),tu=a=>wa(a,Dx,ye),dv=a=>wa(a,lg,ng),_x=a=>wa(a,ag,Nx),ei=a=>wa(a,ig,Mx),le=a=>tg.test(a),Mr=a=>Ea(a,og),Cx=a=>Ea(a,zx),mv=a=>Ea(a,lg),jx=a=>Ea(a,rg),Ox=a=>Ea(a,ag),ti=a=>Ea(a,ig,!0),wa=(a,o,i)=>{const c=eg.exec(a);return c?c[1]?o(c[1]):i(c[2]):!1},Ea=(a,o,i=!1)=>{const c=tg.exec(a);return c?c[1]?o(c[1]):i:!1},lg=a=>a==="position"||a==="percentage",ag=a=>a==="image"||a==="url",rg=a=>a==="length"||a==="size"||a==="bg-size",og=a=>a==="length",Dx=a=>a==="number",zx=a=>a==="family-name",ig=a=>a==="shadow",Ux=()=>{const a=$e("color"),o=$e("font"),i=$e("text"),c=$e("font-weight"),u=$e("tracking"),f=$e("leading"),h=$e("breakpoint"),v=$e("container"),b=$e("spacing"),g=$e("radius"),p=$e("shadow"),S=$e("inset-shadow"),w=$e("text-shadow"),M=$e("drop-shadow"),T=$e("blur"),N=$e("perspective"),_=$e("aspect"),j=$e("ease"),q=$e("animate"),L=()=>["auto","avoid","all","avoid-page","page","left","right","column"],Y=()=>["center","top","bottom","left","right","top-left","left-top","top-right","right-top","bottom-right","right-bottom","bottom-left","left-bottom"],B=()=>[...Y(),le,ne],J=()=>["auto","hidden","clip","visible","scroll"],P=()=>["auto","contain","none"],Z=()=>[le,ne,b],I=()=>[ma,"full","auto",...Z()],re=()=>[In,"none","subgrid",le,ne],ue=()=>["auto",{span:["full",In,le,ne]},In,le,ne],de=()=>[In,"auto",le,ne],Ee=()=>["auto","min","max","fr",le,ne],be=()=>["start","end","center","between","around","evenly","stretch","baseline","center-safe","end-safe"],ve=()=>["start","end","center","stretch","center-safe","end-safe"],D=()=>["auto",...Z()],K=()=>[ma,"auto","full","dvw","dvh","lvw","lvh","svw","svh","min","max","fit",...Z()],H=()=>[a,le,ne],W=()=>[...Y(),mv,dv,{position:[le,ne]}],A=()=>["no-repeat",{repeat:["","x","y","space","round"]}],X=()=>["auto","cover","contain",jx,Rx,{size:[le,ne]}],F=()=>[eu,Mr,wl],$=()=>["","none","full",g,le,ne],ee=()=>["",ye,Mr,wl],he=()=>["solid","dashed","dotted","double"],oe=()=>["normal","multiply","screen","overlay","darken","lighten","color-dodge","color-burn","hard-light","soft-light","difference","exclusion","hue","saturation","color","luminosity"],ie=()=>[ye,eu,mv,dv],Me=()=>["","none",T,le,ne],it=()=>["none",ye,le,ne],Qt=()=>["none",ye,le,ne],Zt=()=>[ye,le,ne],Kt=()=>[ma,"full",...Z()];return{cacheSize:500,theme:{animate:["spin","ping","pulse","bounce"],aspect:["video"],blur:[En],breakpoint:[En],color:[Ex],container:[En],"drop-shadow":[En],ease:["in","out","in-out"],font:[Tx],"font-weight":["thin","extralight","light","normal","medium","semibold","bold","extrabold","black"],"inset-shadow":[En],leading:["none","tight","snug","normal","relaxed","loose"],perspective:["dramatic","near","normal","midrange","distant","none"],radius:[En],shadow:[En],spacing:["px",ye],text:[En],"text-shadow":[En],tracking:["tighter","tight","normal","wide","wider","widest"]},classGroups:{aspect:[{aspect:["auto","square",ma,ne,le,_]}],container:["container"],columns:[{columns:[ye,ne,le,v]}],"break-after":[{"break-after":L()}],"break-before":[{"break-before":L()}],"break-inside":[{"break-inside":["auto","avoid","avoid-page","avoid-column"]}],"box-decoration":[{"box-decoration":["slice","clone"]}],box:[{box:["border","content"]}],display:["block","inline-block","inline","flex","inline-flex","table","inline-table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row-group","table-row","flow-root","grid","inline-grid","contents","list-item","hidden"],sr:["sr-only","not-sr-only"],float:[{float:["right","left","none","start","end"]}],clear:[{clear:["left","right","both","none","start","end"]}],isolation:["isolate","isolation-auto"],"object-fit":[{object:["contain","cover","fill","none","scale-down"]}],"object-position":[{object:B()}],overflow:[{overflow:J()}],"overflow-x":[{"overflow-x":J()}],"overflow-y":[{"overflow-y":J()}],overscroll:[{overscroll:P()}],"overscroll-x":[{"overscroll-x":P()}],"overscroll-y":[{"overscroll-y":P()}],position:["static","fixed","absolute","relative","sticky"],inset:[{inset:I()}],"inset-x":[{"inset-x":I()}],"inset-y":[{"inset-y":I()}],start:[{start:I()}],end:[{end:I()}],top:[{top:I()}],right:[{right:I()}],bottom:[{bottom:I()}],left:[{left:I()}],visibility:["visible","invisible","collapse"],z:[{z:[In,"auto",le,ne]}],basis:[{basis:[ma,"full","auto",v,...Z()]}],"flex-direction":[{flex:["row","row-reverse","col","col-reverse"]}],"flex-wrap":[{flex:["nowrap","wrap","wrap-reverse"]}],flex:[{flex:[ye,ma,"auto","initial","none",ne]}],grow:[{grow:["",ye,le,ne]}],shrink:[{shrink:["",ye,le,ne]}],order:[{order:[In,"first","last","none",le,ne]}],"grid-cols":[{"grid-cols":re()}],"col-start-end":[{col:ue()}],"col-start":[{"col-start":de()}],"col-end":[{"col-end":de()}],"grid-rows":[{"grid-rows":re()}],"row-start-end":[{row:ue()}],"row-start":[{"row-start":de()}],"row-end":[{"row-end":de()}],"grid-flow":[{"grid-flow":["row","col","dense","row-dense","col-dense"]}],"auto-cols":[{"auto-cols":Ee()}],"auto-rows":[{"auto-rows":Ee()}],gap:[{gap:Z()}],"gap-x":[{"gap-x":Z()}],"gap-y":[{"gap-y":Z()}],"justify-content":[{justify:[...be(),"normal"]}],"justify-items":[{"justify-items":[...ve(),"normal"]}],"justify-self":[{"justify-self":["auto",...ve()]}],"align-content":[{content:["normal",...be()]}],"align-items":[{items:[...ve(),{baseline:["","last"]}]}],"align-self":[{self:["auto",...ve(),{baseline:["","last"]}]}],"place-content":[{"place-content":be()}],"place-items":[{"place-items":[...ve(),"baseline"]}],"place-self":[{"place-self":["auto",...ve()]}],p:[{p:Z()}],px:[{px:Z()}],py:[{py:Z()}],ps:[{ps:Z()}],pe:[{pe:Z()}],pt:[{pt:Z()}],pr:[{pr:Z()}],pb:[{pb:Z()}],pl:[{pl:Z()}],m:[{m:D()}],mx:[{mx:D()}],my:[{my:D()}],ms:[{ms:D()}],me:[{me:D()}],mt:[{mt:D()}],mr:[{mr:D()}],mb:[{mb:D()}],ml:[{ml:D()}],"space-x":[{"space-x":Z()}],"space-x-reverse":["space-x-reverse"],"space-y":[{"space-y":Z()}],"space-y-reverse":["space-y-reverse"],size:[{size:K()}],w:[{w:[v,"screen",...K()]}],"min-w":[{"min-w":[v,"screen","none",...K()]}],"max-w":[{"max-w":[v,"screen","none","prose",{screen:[h]},...K()]}],h:[{h:["screen","lh",...K()]}],"min-h":[{"min-h":["screen","lh","none",...K()]}],"max-h":[{"max-h":["screen","lh",...K()]}],"font-size":[{text:["base",i,Mr,wl]}],"font-smoothing":["antialiased","subpixel-antialiased"],"font-style":["italic","not-italic"],"font-weight":[{font:[c,le,tu]}],"font-stretch":[{"font-stretch":["ultra-condensed","extra-condensed","condensed","semi-condensed","normal","semi-expanded","expanded","extra-expanded","ultra-expanded",eu,ne]}],"font-family":[{font:[Cx,ne,o]}],"fvn-normal":["normal-nums"],"fvn-ordinal":["ordinal"],"fvn-slashed-zero":["slashed-zero"],"fvn-figure":["lining-nums","oldstyle-nums"],"fvn-spacing":["proportional-nums","tabular-nums"],"fvn-fraction":["diagonal-fractions","stacked-fractions"],tracking:[{tracking:[u,le,ne]}],"line-clamp":[{"line-clamp":[ye,"none",le,tu]}],leading:[{leading:[f,...Z()]}],"list-image":[{"list-image":["none",le,ne]}],"list-style-position":[{list:["inside","outside"]}],"list-style-type":[{list:["disc","decimal","none",le,ne]}],"text-alignment":[{text:["left","center","right","justify","start","end"]}],"placeholder-color":[{placeholder:H()}],"text-color":[{text:H()}],"text-decoration":["underline","overline","line-through","no-underline"],"text-decoration-style":[{decoration:[...he(),"wavy"]}],"text-decoration-thickness":[{decoration:[ye,"from-font","auto",le,wl]}],"text-decoration-color":[{decoration:H()}],"underline-offset":[{"underline-offset":[ye,"auto",le,ne]}],"text-transform":["uppercase","lowercase","capitalize","normal-case"],"text-overflow":["truncate","text-ellipsis","text-clip"],"text-wrap":[{text:["wrap","nowrap","balance","pretty"]}],indent:[{indent:Z()}],"vertical-align":[{align:["baseline","top","middle","bottom","text-top","text-bottom","sub","super",le,ne]}],whitespace:[{whitespace:["normal","nowrap","pre","pre-line","pre-wrap","break-spaces"]}],break:[{break:["normal","words","all","keep"]}],wrap:[{wrap:["break-word","anywhere","normal"]}],hyphens:[{hyphens:["none","manual","auto"]}],content:[{content:["none",le,ne]}],"bg-attachment":[{bg:["fixed","local","scroll"]}],"bg-clip":[{"bg-clip":["border","padding","content","text"]}],"bg-origin":[{"bg-origin":["border","padding","content"]}],"bg-position":[{bg:W()}],"bg-repeat":[{bg:A()}],"bg-size":[{bg:X()}],"bg-image":[{bg:["none",{linear:[{to:["t","tr","r","br","b","bl","l","tl"]},In,le,ne],radial:["",le,ne],conic:[In,le,ne]},Ox,_x]}],"bg-color":[{bg:H()}],"gradient-from-pos":[{from:F()}],"gradient-via-pos":[{via:F()}],"gradient-to-pos":[{to:F()}],"gradient-from":[{from:H()}],"gradient-via":[{via:H()}],"gradient-to":[{to:H()}],rounded:[{rounded:$()}],"rounded-s":[{"rounded-s":$()}],"rounded-e":[{"rounded-e":$()}],"rounded-t":[{"rounded-t":$()}],"rounded-r":[{"rounded-r":$()}],"rounded-b":[{"rounded-b":$()}],"rounded-l":[{"rounded-l":$()}],"rounded-ss":[{"rounded-ss":$()}],"rounded-se":[{"rounded-se":$()}],"rounded-ee":[{"rounded-ee":$()}],"rounded-es":[{"rounded-es":$()}],"rounded-tl":[{"rounded-tl":$()}],"rounded-tr":[{"rounded-tr":$()}],"rounded-br":[{"rounded-br":$()}],"rounded-bl":[{"rounded-bl":$()}],"border-w":[{border:ee()}],"border-w-x":[{"border-x":ee()}],"border-w-y":[{"border-y":ee()}],"border-w-s":[{"border-s":ee()}],"border-w-e":[{"border-e":ee()}],"border-w-t":[{"border-t":ee()}],"border-w-r":[{"border-r":ee()}],"border-w-b":[{"border-b":ee()}],"border-w-l":[{"border-l":ee()}],"divide-x":[{"divide-x":ee()}],"divide-x-reverse":["divide-x-reverse"],"divide-y":[{"divide-y":ee()}],"divide-y-reverse":["divide-y-reverse"],"border-style":[{border:[...he(),"hidden","none"]}],"divide-style":[{divide:[...he(),"hidden","none"]}],"border-color":[{border:H()}],"border-color-x":[{"border-x":H()}],"border-color-y":[{"border-y":H()}],"border-color-s":[{"border-s":H()}],"border-color-e":[{"border-e":H()}],"border-color-t":[{"border-t":H()}],"border-color-r":[{"border-r":H()}],"border-color-b":[{"border-b":H()}],"border-color-l":[{"border-l":H()}],"divide-color":[{divide:H()}],"outline-style":[{outline:[...he(),"none","hidden"]}],"outline-offset":[{"outline-offset":[ye,le,ne]}],"outline-w":[{outline:["",ye,Mr,wl]}],"outline-color":[{outline:H()}],shadow:[{shadow:["","none",p,ti,ei]}],"shadow-color":[{shadow:H()}],"inset-shadow":[{"inset-shadow":["none",S,ti,ei]}],"inset-shadow-color":[{"inset-shadow":H()}],"ring-w":[{ring:ee()}],"ring-w-inset":["ring-inset"],"ring-color":[{ring:H()}],"ring-offset-w":[{"ring-offset":[ye,wl]}],"ring-offset-color":[{"ring-offset":H()}],"inset-ring-w":[{"inset-ring":ee()}],"inset-ring-color":[{"inset-ring":H()}],"text-shadow":[{"text-shadow":["none",w,ti,ei]}],"text-shadow-color":[{"text-shadow":H()}],opacity:[{opacity:[ye,le,ne]}],"mix-blend":[{"mix-blend":[...oe(),"plus-darker","plus-lighter"]}],"bg-blend":[{"bg-blend":oe()}],"mask-clip":[{"mask-clip":["border","padding","content","fill","stroke","view"]},"mask-no-clip"],"mask-composite":[{mask:["add","subtract","intersect","exclude"]}],"mask-image-linear-pos":[{"mask-linear":[ye]}],"mask-image-linear-from-pos":[{"mask-linear-from":ie()}],"mask-image-linear-to-pos":[{"mask-linear-to":ie()}],"mask-image-linear-from-color":[{"mask-linear-from":H()}],"mask-image-linear-to-color":[{"mask-linear-to":H()}],"mask-image-t-from-pos":[{"mask-t-from":ie()}],"mask-image-t-to-pos":[{"mask-t-to":ie()}],"mask-image-t-from-color":[{"mask-t-from":H()}],"mask-image-t-to-color":[{"mask-t-to":H()}],"mask-image-r-from-pos":[{"mask-r-from":ie()}],"mask-image-r-to-pos":[{"mask-r-to":ie()}],"mask-image-r-from-color":[{"mask-r-from":H()}],"mask-image-r-to-color":[{"mask-r-to":H()}],"mask-image-b-from-pos":[{"mask-b-from":ie()}],"mask-image-b-to-pos":[{"mask-b-to":ie()}],"mask-image-b-from-color":[{"mask-b-from":H()}],"mask-image-b-to-color":[{"mask-b-to":H()}],"mask-image-l-from-pos":[{"mask-l-from":ie()}],"mask-image-l-to-pos":[{"mask-l-to":ie()}],"mask-image-l-from-color":[{"mask-l-from":H()}],"mask-image-l-to-color":[{"mask-l-to":H()}],"mask-image-x-from-pos":[{"mask-x-from":ie()}],"mask-image-x-to-pos":[{"mask-x-to":ie()}],"mask-image-x-from-color":[{"mask-x-from":H()}],"mask-image-x-to-color":[{"mask-x-to":H()}],"mask-image-y-from-pos":[{"mask-y-from":ie()}],"mask-image-y-to-pos":[{"mask-y-to":ie()}],"mask-image-y-from-color":[{"mask-y-from":H()}],"mask-image-y-to-color":[{"mask-y-to":H()}],"mask-image-radial":[{"mask-radial":[le,ne]}],"mask-image-radial-from-pos":[{"mask-radial-from":ie()}],"mask-image-radial-to-pos":[{"mask-radial-to":ie()}],"mask-image-radial-from-color":[{"mask-radial-from":H()}],"mask-image-radial-to-color":[{"mask-radial-to":H()}],"mask-image-radial-shape":[{"mask-radial":["circle","ellipse"]}],"mask-image-radial-size":[{"mask-radial":[{closest:["side","corner"],farthest:["side","corner"]}]}],"mask-image-radial-pos":[{"mask-radial-at":Y()}],"mask-image-conic-pos":[{"mask-conic":[ye]}],"mask-image-conic-from-pos":[{"mask-conic-from":ie()}],"mask-image-conic-to-pos":[{"mask-conic-to":ie()}],"mask-image-conic-from-color":[{"mask-conic-from":H()}],"mask-image-conic-to-color":[{"mask-conic-to":H()}],"mask-mode":[{mask:["alpha","luminance","match"]}],"mask-origin":[{"mask-origin":["border","padding","content","fill","stroke","view"]}],"mask-position":[{mask:W()}],"mask-repeat":[{mask:A()}],"mask-size":[{mask:X()}],"mask-type":[{"mask-type":["alpha","luminance"]}],"mask-image":[{mask:["none",le,ne]}],filter:[{filter:["","none",le,ne]}],blur:[{blur:Me()}],brightness:[{brightness:[ye,le,ne]}],contrast:[{contrast:[ye,le,ne]}],"drop-shadow":[{"drop-shadow":["","none",M,ti,ei]}],"drop-shadow-color":[{"drop-shadow":H()}],grayscale:[{grayscale:["",ye,le,ne]}],"hue-rotate":[{"hue-rotate":[ye,le,ne]}],invert:[{invert:["",ye,le,ne]}],saturate:[{saturate:[ye,le,ne]}],sepia:[{sepia:["",ye,le,ne]}],"backdrop-filter":[{"backdrop-filter":["","none",le,ne]}],"backdrop-blur":[{"backdrop-blur":Me()}],"backdrop-brightness":[{"backdrop-brightness":[ye,le,ne]}],"backdrop-contrast":[{"backdrop-contrast":[ye,le,ne]}],"backdrop-grayscale":[{"backdrop-grayscale":["",ye,le,ne]}],"backdrop-hue-rotate":[{"backdrop-hue-rotate":[ye,le,ne]}],"backdrop-invert":[{"backdrop-invert":["",ye,le,ne]}],"backdrop-opacity":[{"backdrop-opacity":[ye,le,ne]}],"backdrop-saturate":[{"backdrop-saturate":[ye,le,ne]}],"backdrop-sepia":[{"backdrop-sepia":["",ye,le,ne]}],"border-collapse":[{border:["collapse","separate"]}],"border-spacing":[{"border-spacing":Z()}],"border-spacing-x":[{"border-spacing-x":Z()}],"border-spacing-y":[{"border-spacing-y":Z()}],"table-layout":[{table:["auto","fixed"]}],caption:[{caption:["top","bottom"]}],transition:[{transition:["","all","colors","opacity","shadow","transform","none",le,ne]}],"transition-behavior":[{transition:["normal","discrete"]}],duration:[{duration:[ye,"initial",le,ne]}],ease:[{ease:["linear","initial",j,le,ne]}],delay:[{delay:[ye,le,ne]}],animate:[{animate:["none",q,le,ne]}],backface:[{backface:["hidden","visible"]}],perspective:[{perspective:[N,le,ne]}],"perspective-origin":[{"perspective-origin":B()}],rotate:[{rotate:it()}],"rotate-x":[{"rotate-x":it()}],"rotate-y":[{"rotate-y":it()}],"rotate-z":[{"rotate-z":it()}],scale:[{scale:Qt()}],"scale-x":[{"scale-x":Qt()}],"scale-y":[{"scale-y":Qt()}],"scale-z":[{"scale-z":Qt()}],"scale-3d":["scale-3d"],skew:[{skew:Zt()}],"skew-x":[{"skew-x":Zt()}],"skew-y":[{"skew-y":Zt()}],transform:[{transform:[le,ne,"","none","gpu","cpu"]}],"transform-origin":[{origin:B()}],"transform-style":[{transform:["3d","flat"]}],translate:[{translate:Kt()}],"translate-x":[{"translate-x":Kt()}],"translate-y":[{"translate-y":Kt()}],"translate-z":[{"translate-z":Kt()}],"translate-none":["translate-none"],accent:[{accent:H()}],appearance:[{appearance:["none","auto"]}],"caret-color":[{caret:H()}],"color-scheme":[{scheme:["normal","dark","light","light-dark","only-dark","only-light"]}],cursor:[{cursor:["auto","default","pointer","wait","text","move","help","not-allowed","none","context-menu","progress","cell","crosshair","vertical-text","alias","copy","no-drop","grab","grabbing","all-scroll","col-resize","row-resize","n-resize","e-resize","s-resize","w-resize","ne-resize","nw-resize","se-resize","sw-resize","ew-resize","ns-resize","nesw-resize","nwse-resize","zoom-in","zoom-out",le,ne]}],"field-sizing":[{"field-sizing":["fixed","content"]}],"pointer-events":[{"pointer-events":["auto","none"]}],resize:[{resize:["none","","y","x"]}],"scroll-behavior":[{scroll:["auto","smooth"]}],"scroll-m":[{"scroll-m":Z()}],"scroll-mx":[{"scroll-mx":Z()}],"scroll-my":[{"scroll-my":Z()}],"scroll-ms":[{"scroll-ms":Z()}],"scroll-me":[{"scroll-me":Z()}],"scroll-mt":[{"scroll-mt":Z()}],"scroll-mr":[{"scroll-mr":Z()}],"scroll-mb":[{"scroll-mb":Z()}],"scroll-ml":[{"scroll-ml":Z()}],"scroll-p":[{"scroll-p":Z()}],"scroll-px":[{"scroll-px":Z()}],"scroll-py":[{"scroll-py":Z()}],"scroll-ps":[{"scroll-ps":Z()}],"scroll-pe":[{"scroll-pe":Z()}],"scroll-pt":[{"scroll-pt":Z()}],"scroll-pr":[{"scroll-pr":Z()}],"scroll-pb":[{"scroll-pb":Z()}],"scroll-pl":[{"scroll-pl":Z()}],"snap-align":[{snap:["start","end","center","align-none"]}],"snap-stop":[{snap:["normal","always"]}],"snap-type":[{snap:["none","x","y","both"]}],"snap-strictness":[{snap:["mandatory","proximity"]}],touch:[{touch:["auto","none","manipulation"]}],"touch-x":[{"touch-pan":["x","left","right"]}],"touch-y":[{"touch-pan":["y","up","down"]}],"touch-pz":["touch-pinch-zoom"],select:[{select:["none","text","all","auto"]}],"will-change":[{"will-change":["auto","scroll","contents","transform",le,ne]}],fill:[{fill:["none",...H()]}],"stroke-w":[{stroke:[ye,Mr,wl,tu]}],stroke:[{stroke:["none",...H()]}],"forced-color-adjust":[{"forced-color-adjust":["auto","none"]}]},conflictingClassGroups:{overflow:["overflow-x","overflow-y"],overscroll:["overscroll-x","overscroll-y"],inset:["inset-x","inset-y","start","end","top","right","bottom","left"],"inset-x":["right","left"],"inset-y":["top","bottom"],flex:["basis","grow","shrink"],gap:["gap-x","gap-y"],p:["px","py","ps","pe","pt","pr","pb","pl"],px:["pr","pl"],py:["pt","pb"],m:["mx","my","ms","me","mt","mr","mb","ml"],mx:["mr","ml"],my:["mt","mb"],size:["w","h"],"font-size":["leading"],"fvn-normal":["fvn-ordinal","fvn-slashed-zero","fvn-figure","fvn-spacing","fvn-fraction"],"fvn-ordinal":["fvn-normal"],"fvn-slashed-zero":["fvn-normal"],"fvn-figure":["fvn-normal"],"fvn-spacing":["fvn-normal"],"fvn-fraction":["fvn-normal"],"line-clamp":["display","overflow"],rounded:["rounded-s","rounded-e","rounded-t","rounded-r","rounded-b","rounded-l","rounded-ss","rounded-se","rounded-ee","rounded-es","rounded-tl","rounded-tr","rounded-br","rounded-bl"],"rounded-s":["rounded-ss","rounded-es"],"rounded-e":["rounded-se","rounded-ee"],"rounded-t":["rounded-tl","rounded-tr"],"rounded-r":["rounded-tr","rounded-br"],"rounded-b":["rounded-br","rounded-bl"],"rounded-l":["rounded-tl","rounded-bl"],"border-spacing":["border-spacing-x","border-spacing-y"],"border-w":["border-w-x","border-w-y","border-w-s","border-w-e","border-w-t","border-w-r","border-w-b","border-w-l"],"border-w-x":["border-w-r","border-w-l"],"border-w-y":["border-w-t","border-w-b"],"border-color":["border-color-x","border-color-y","border-color-s","border-color-e","border-color-t","border-color-r","border-color-b","border-color-l"],"border-color-x":["border-color-r","border-color-l"],"border-color-y":["border-color-t","border-color-b"],translate:["translate-x","translate-y","translate-none"],"translate-none":["translate","translate-x","translate-y","translate-z"],"scroll-m":["scroll-mx","scroll-my","scroll-ms","scroll-me","scroll-mt","scroll-mr","scroll-mb","scroll-ml"],"scroll-mx":["scroll-mr","scroll-ml"],"scroll-my":["scroll-mt","scroll-mb"],"scroll-p":["scroll-px","scroll-py","scroll-ps","scroll-pe","scroll-pt","scroll-pr","scroll-pb","scroll-pl"],"scroll-px":["scroll-pr","scroll-pl"],"scroll-py":["scroll-pt","scroll-pb"],touch:["touch-x","touch-y","touch-pz"],"touch-x":["touch"],"touch-y":["touch"],"touch-pz":["touch"]},conflictingClassGroupModifiers:{"font-size":["leading"]},orderSensitiveModifiers:["*","**","after","backdrop","before","details-content","file","first-letter","first-line","marker","placeholder","selection"]}},kx=gx(Ux);function Ze(...a){return kx(Jv(a))}const Lx=Wv("inline-flex items-center justify-center gap-2 whitespace-nowrap rounded-md text-sm font-medium transition-all disabled:pointer-events-none disabled:opacity-50 [&_svg]:pointer-events-none [&_svg:not([class*='size-'])]:size-4 shrink-0 [&_svg]:shrink-0 outline-none focus-visible:border-ring focus-visible:ring-ring/50 focus-visible:ring-[3px] aria-invalid:ring-destructive/20 dark:aria-invalid:ring-destructive/40 aria-invalid:border-destructive",{variants:{variant:{default:"bg-primary text-primary-foreground shadow-xs hover:bg-primary/90",destructive:"bg-destructive text-white shadow-xs hover:bg-destructive/90 focus-visible:ring-destructive/20 dark:focus-visible:ring-destructive/40 dark:bg-destructive/60",outline:"border bg-background shadow-xs hover:bg-accent hover:text-accent-foreground dark:bg-input/30 dark:border-input dark:hover:bg-input/50",secondary:"bg-secondary text-secondary-foreground shadow-xs hover:bg-secondary/80",ghost:"hover:bg-accent hover:text-accent-foreground dark:hover:bg-accent/50",link:"text-primary underline-offset-4 hover:underline"},size:{default:"h-9 px-4 py-2 has-[>svg]:px-3",sm:"h-8 rounded-md gap-1.5 px-3 has-[>svg]:px-2.5",lg:"h-10 rounded-md px-6 has-[>svg]:px-4",icon:"size-9"}},defaultVariants:{variant:"default",size:"default"}});function An({className:a,variant:o,size:i,asChild:c=!1,...u}){const f=c?Wb:"button";return d.jsx(f,{"data-slot":"button",className:Ze(Lx({variant:o,size:i,className:a})),...u})}function me(a,o,{checkForDefaultPrevented:i=!0}={}){return function(u){if(a?.(u),i===!1||!u.defaultPrevented)return o?.(u)}}function Ml(a,o=[]){let i=[];function c(f,h){const v=x.createContext(h),b=i.length;i=[...i,h];const g=S=>{const{scope:w,children:M,...T}=S,N=w?.[a]?.[b]||v,_=x.useMemo(()=>T,Object.values(T));return d.jsx(N.Provider,{value:_,children:M})};g.displayName=f+"Provider";function p(S,w){const M=w?.[a]?.[b]||v,T=x.useContext(M);if(T)return T;if(h!==void 0)return h;throw new Error(`\`${S}\` must be used within \`${f}\``)}return[g,p]}const u=()=>{const f=i.map(h=>x.createContext(h));return function(v){const b=v?.[a]||f;return x.useMemo(()=>({[`__scope${a}`]:{...v,[a]:b}}),[v,b])}};return u.scopeName=a,[c,Hx(u,...o)]}function Hx(...a){const o=a[0];if(a.length===1)return o;const i=()=>{const c=a.map(u=>({useScope:u(),scopeName:u.scopeName}));return function(f){const h=c.reduce((v,{useScope:b,scopeName:g})=>{const S=b(f)[`__scope${g}`];return{...v,...S}},{});return x.useMemo(()=>({[`__scope${o.scopeName}`]:h}),[h])}};return i.scopeName=o.scopeName,i}var Mn=globalThis?.document?x.useLayoutEffect:()=>{},Bx=Kv[" useInsertionEffect ".trim().toString()]||Mn;function Du({prop:a,defaultProp:o,onChange:i=()=>{},caller:c}){const[u,f,h]=qx({defaultProp:o,onChange:i}),v=a!==void 0,b=v?a:u;{const p=x.useRef(a!==void 0);x.useEffect(()=>{const S=p.current;S!==v&&console.warn(`${c} is changing from ${S?"controlled":"uncontrolled"} to ${v?"controlled":"uncontrolled"}. Components should not switch from controlled to uncontrolled (or vice versa). Decide between using a controlled or uncontrolled value for the lifetime of the component.`),p.current=v},[v,c])}const g=x.useCallback(p=>{if(v){const S=Gx(p)?p(a):p;S!==a&&h.current?.(S)}else f(p)},[v,a,f,h]);return[b,g]}function qx({defaultProp:a,onChange:o}){const[i,c]=x.useState(a),u=x.useRef(i),f=x.useRef(o);return Bx(()=>{f.current=o},[o]),x.useEffect(()=>{u.current!==i&&(f.current?.(i),u.current=i)},[i,u]),[i,c,f]}function Gx(a){return typeof a=="function"}var zu=Pv();const Yx=Zv(zu);var Vx=["a","button","div","form","h2","h3","img","input","label","li","nav","ol","p","select","span","svg","ul"],ke=Vx.reduce((a,o)=>{const i=Cr(`Primitive.${o}`),c=x.forwardRef((u,f)=>{const{asChild:h,...v}=u,b=h?i:o;return typeof window<"u"&&(window[Symbol.for("radix-ui")]=!0),d.jsx(b,{...v,ref:f})});return c.displayName=`Primitive.${o}`,{...a,[o]:c}},{});function cg(a,o){a&&zu.flushSync(()=>a.dispatchEvent(o))}function sg(a){const o=a+"CollectionProvider",[i,c]=Ml(o),[u,f]=i(o,{collectionRef:{current:null},itemMap:new Map}),h=N=>{const{scope:_,children:j}=N,q=el.useRef(null),L=el.useRef(new Map).current;return d.jsx(u,{scope:_,itemMap:L,collectionRef:q,children:j})};h.displayName=o;const v=a+"CollectionSlot",b=Cr(v),g=el.forwardRef((N,_)=>{const{scope:j,children:q}=N,L=f(v,j),Y=Qe(_,L.collectionRef);return d.jsx(b,{ref:Y,children:q})});g.displayName=v;const p=a+"CollectionItemSlot",S="data-radix-collection-item",w=Cr(p),M=el.forwardRef((N,_)=>{const{scope:j,children:q,...L}=N,Y=el.useRef(null),B=Qe(_,Y),J=f(p,j);return el.useEffect(()=>(J.itemMap.set(Y,{ref:Y,...L}),()=>void J.itemMap.delete(Y))),d.jsx(w,{[S]:"",ref:B,children:q})});M.displayName=p;function T(N){const _=f(a+"CollectionConsumer",N);return el.useCallback(()=>{const q=_.collectionRef.current;if(!q)return[];const L=Array.from(q.querySelectorAll(`[${S}]`));return Array.from(_.itemMap.values()).sort((J,P)=>L.indexOf(J.ref.current)-L.indexOf(P.ref.current))},[_.collectionRef,_.itemMap])}return[{Provider:h,Slot:g,ItemSlot:M},T,c]}var Xx=x.createContext(void 0);function wi(a){const o=x.useContext(Xx);return a||o||"ltr"}function ot(a){const o=x.useRef(a);return x.useEffect(()=>{o.current=a}),x.useMemo(()=>(...i)=>o.current?.(...i),[])}function Qx(a,o=globalThis?.document){const i=ot(a);x.useEffect(()=>{const c=u=>{u.key==="Escape"&&i(u)};return o.addEventListener("keydown",c,{capture:!0}),()=>o.removeEventListener("keydown",c,{capture:!0})},[i,o])}var Zx="DismissableLayer",yu="dismissableLayer.update",Kx="dismissableLayer.pointerDownOutside",Px="dismissableLayer.focusOutside",hv,ug=x.createContext({layers:new Set,layersWithOutsidePointerEventsDisabled:new Set,branches:new Set}),fg=x.forwardRef((a,o)=>{const{disableOutsidePointerEvents:i=!1,onEscapeKeyDown:c,onPointerDownOutside:u,onFocusOutside:f,onInteractOutside:h,onDismiss:v,...b}=a,g=x.useContext(ug),[p,S]=x.useState(null),w=p?.ownerDocument??globalThis?.document,[,M]=x.useState({}),T=Qe(o,P=>S(P)),N=Array.from(g.layers),[_]=[...g.layersWithOutsidePointerEventsDisabled].slice(-1),j=N.indexOf(_),q=p?N.indexOf(p):-1,L=g.layersWithOutsidePointerEventsDisabled.size>0,Y=q>=j,B=Wx(P=>{const Z=P.target,I=[...g.branches].some(re=>re.contains(Z));!Y||I||(u?.(P),h?.(P),P.defaultPrevented||v?.())},w),J=Fx(P=>{const Z=P.target;[...g.branches].some(re=>re.contains(Z))||(f?.(P),h?.(P),P.defaultPrevented||v?.())},w);return Qx(P=>{q===g.layers.size-1&&(c?.(P),!P.defaultPrevented&&v&&(P.preventDefault(),v()))},w),x.useEffect(()=>{if(p)return i&&(g.layersWithOutsidePointerEventsDisabled.size===0&&(hv=w.body.style.pointerEvents,w.body.style.pointerEvents="none"),g.layersWithOutsidePointerEventsDisabled.add(p)),g.layers.add(p),vv(),()=>{i&&g.layersWithOutsidePointerEventsDisabled.size===1&&(w.body.style.pointerEvents=hv)}},[p,w,i,g]),x.useEffect(()=>()=>{p&&(g.layers.delete(p),g.layersWithOutsidePointerEventsDisabled.delete(p),vv())},[p,g]),x.useEffect(()=>{const P=()=>M({});return document.addEventListener(yu,P),()=>document.removeEventListener(yu,P)},[]),d.jsx(ke.div,{...b,ref:T,style:{pointerEvents:L?Y?"auto":"none":void 0,...a.style},onFocusCapture:me(a.onFocusCapture,J.onFocusCapture),onBlurCapture:me(a.onBlurCapture,J.onBlurCapture),onPointerDownCapture:me(a.onPointerDownCapture,B.onPointerDownCapture)})});fg.displayName=Zx;var $x="DismissableLayerBranch",Jx=x.forwardRef((a,o)=>{const i=x.useContext(ug),c=x.useRef(null),u=Qe(o,c);return x.useEffect(()=>{const f=c.current;if(f)return i.branches.add(f),()=>{i.branches.delete(f)}},[i.branches]),d.jsx(ke.div,{...a,ref:u})});Jx.displayName=$x;function Wx(a,o=globalThis?.document){const i=ot(a),c=x.useRef(!1),u=x.useRef(()=>{});return x.useEffect(()=>{const f=v=>{if(v.target&&!c.current){let b=function(){dg(Kx,i,g,{discrete:!0})};const g={originalEvent:v};v.pointerType==="touch"?(o.removeEventListener("click",u.current),u.current=b,o.addEventListener("click",u.current,{once:!0})):b()}else o.removeEventListener("click",u.current);c.current=!1},h=window.setTimeout(()=>{o.addEventListener("pointerdown",f)},0);return()=>{window.clearTimeout(h),o.removeEventListener("pointerdown",f),o.removeEventListener("click",u.current)}},[o,i]),{onPointerDownCapture:()=>c.current=!0}}function Fx(a,o=globalThis?.document){const i=ot(a),c=x.useRef(!1);return x.useEffect(()=>{const u=f=>{f.target&&!c.current&&dg(Px,i,{originalEvent:f},{discrete:!1})};return o.addEventListener("focusin",u),()=>o.removeEventListener("focusin",u)},[o,i]),{onFocusCapture:()=>c.current=!0,onBlurCapture:()=>c.current=!1}}function vv(){const a=new CustomEvent(yu);document.dispatchEvent(a)}function dg(a,o,i,{discrete:c}){const u=i.originalEvent.target,f=new CustomEvent(a,{bubbles:!1,cancelable:!0,detail:i});o&&u.addEventListener(a,o,{once:!0}),c?cg(u,f):u.dispatchEvent(f)}var nu=0;function Ix(){x.useEffect(()=>{const a=document.querySelectorAll("[data-radix-focus-guard]");return document.body.insertAdjacentElement("afterbegin",a[0]??gv()),document.body.insertAdjacentElement("beforeend",a[1]??gv()),nu++,()=>{nu===1&&document.querySelectorAll("[data-radix-focus-guard]").forEach(o=>o.remove()),nu--}},[])}function gv(){const a=document.createElement("span");return a.setAttribute("data-radix-focus-guard",""),a.tabIndex=0,a.style.outline="none",a.style.opacity="0",a.style.position="fixed",a.style.pointerEvents="none",a}var lu="focusScope.autoFocusOnMount",au="focusScope.autoFocusOnUnmount",pv={bubbles:!1,cancelable:!0},e1="FocusScope",mg=x.forwardRef((a,o)=>{const{loop:i=!1,trapped:c=!1,onMountAutoFocus:u,onUnmountAutoFocus:f,...h}=a,[v,b]=x.useState(null),g=ot(u),p=ot(f),S=x.useRef(null),w=Qe(o,N=>b(N)),M=x.useRef({paused:!1,pause(){this.paused=!0},resume(){this.paused=!1}}).current;x.useEffect(()=>{if(c){let N=function(L){if(M.paused||!v)return;const Y=L.target;v.contains(Y)?S.current=Y:tl(S.current,{select:!0})},_=function(L){if(M.paused||!v)return;const Y=L.relatedTarget;Y!==null&&(v.contains(Y)||tl(S.current,{select:!0}))},j=function(L){if(document.activeElement===document.body)for(const B of L)B.removedNodes.length>0&&tl(v)};document.addEventListener("focusin",N),document.addEventListener("focusout",_);const q=new MutationObserver(j);return v&&q.observe(v,{childList:!0,subtree:!0}),()=>{document.removeEventListener("focusin",N),document.removeEventListener("focusout",_),q.disconnect()}}},[c,v,M.paused]),x.useEffect(()=>{if(v){bv.add(M);const N=document.activeElement;if(!v.contains(N)){const j=new CustomEvent(lu,pv);v.addEventListener(lu,g),v.dispatchEvent(j),j.defaultPrevented||(t1(o1(hg(v)),{select:!0}),document.activeElement===N&&tl(v))}return()=>{v.removeEventListener(lu,g),setTimeout(()=>{const j=new CustomEvent(au,pv);v.addEventListener(au,p),v.dispatchEvent(j),j.defaultPrevented||tl(N??document.body,{select:!0}),v.removeEventListener(au,p),bv.remove(M)},0)}}},[v,g,p,M]);const T=x.useCallback(N=>{if(!i&&!c||M.paused)return;const _=N.key==="Tab"&&!N.altKey&&!N.ctrlKey&&!N.metaKey,j=document.activeElement;if(_&&j){const q=N.currentTarget,[L,Y]=n1(q);L&&Y?!N.shiftKey&&j===Y?(N.preventDefault(),i&&tl(L,{select:!0})):N.shiftKey&&j===L&&(N.preventDefault(),i&&tl(Y,{select:!0})):j===q&&N.preventDefault()}},[i,c,M.paused]);return d.jsx(ke.div,{tabIndex:-1,...h,ref:w,onKeyDown:T})});mg.displayName=e1;function t1(a,{select:o=!1}={}){const i=document.activeElement;for(const c of a)if(tl(c,{select:o}),document.activeElement!==i)return}function n1(a){const o=hg(a),i=yv(o,a),c=yv(o.reverse(),a);return[i,c]}function hg(a){const o=[],i=document.createTreeWalker(a,NodeFilter.SHOW_ELEMENT,{acceptNode:c=>{const u=c.tagName==="INPUT"&&c.type==="hidden";return c.disabled||c.hidden||u?NodeFilter.FILTER_SKIP:c.tabIndex>=0?NodeFilter.FILTER_ACCEPT:NodeFilter.FILTER_SKIP}});for(;i.nextNode();)o.push(i.currentNode);return o}function yv(a,o){for(const i of a)if(!l1(i,{upTo:o}))return i}function l1(a,{upTo:o}){if(getComputedStyle(a).visibility==="hidden")return!0;for(;a;){if(o!==void 0&&a===o)return!1;if(getComputedStyle(a).display==="none")return!0;a=a.parentElement}return!1}function a1(a){return a instanceof HTMLInputElement&&"select"in a}function tl(a,{select:o=!1}={}){if(a&&a.focus){const i=document.activeElement;a.focus({preventScroll:!0}),a!==i&&a1(a)&&o&&a.select()}}var bv=r1();function r1(){let a=[];return{add(o){const i=a[0];o!==i&&i?.pause(),a=xv(a,o),a.unshift(o)},remove(o){a=xv(a,o),a[0]?.resume()}}}function xv(a,o){const i=[...a],c=i.indexOf(o);return c!==-1&&i.splice(c,1),i}function o1(a){return a.filter(o=>o.tagName!=="A")}var i1=Kv[" useId ".trim().toString()]||(()=>{}),c1=0;function fi(a){const[o,i]=x.useState(i1());return Mn(()=>{i(c=>c??String(c1++))},[a]),o?`radix-${o}`:""}const s1=["top","right","bottom","left"],nl=Math.min,At=Math.max,di=Math.round,ni=Math.floor,nn=a=>({x:a,y:a}),u1={left:"right",right:"left",bottom:"top",top:"bottom"},f1={start:"end",end:"start"};function bu(a,o,i){return At(a,nl(o,i))}function Nn(a,o){return typeof a=="function"?a(o):a}function Tn(a){return a.split("-")[0]}function Aa(a){return a.split("-")[1]}function Uu(a){return a==="x"?"y":"x"}function ku(a){return a==="y"?"height":"width"}const d1=new Set(["top","bottom"]);function tn(a){return d1.has(Tn(a))?"y":"x"}function Lu(a){return Uu(tn(a))}function m1(a,o,i){i===void 0&&(i=!1);const c=Aa(a),u=Lu(a),f=ku(u);let h=u==="x"?c===(i?"end":"start")?"right":"left":c==="start"?"bottom":"top";return o.reference[f]>o.floating[f]&&(h=mi(h)),[h,mi(h)]}function h1(a){const o=mi(a);return[xu(a),o,xu(o)]}function xu(a){return a.replace(/start|end/g,o=>f1[o])}const Sv=["left","right"],wv=["right","left"],v1=["top","bottom"],g1=["bottom","top"];function p1(a,o,i){switch(a){case"top":case"bottom":return i?o?wv:Sv:o?Sv:wv;case"left":case"right":return o?v1:g1;default:return[]}}function y1(a,o,i,c){const u=Aa(a);let f=p1(Tn(a),i==="start",c);return u&&(f=f.map(h=>h+"-"+u),o&&(f=f.concat(f.map(xu)))),f}function mi(a){return a.replace(/left|right|bottom|top/g,o=>u1[o])}function b1(a){return{top:0,right:0,bottom:0,left:0,...a}}function vg(a){return typeof a!="number"?b1(a):{top:a,right:a,bottom:a,left:a}}function hi(a){const{x:o,y:i,width:c,height:u}=a;return{width:c,height:u,top:i,left:o,right:o+c,bottom:i+u,x:o,y:i}}function Ev(a,o,i){let{reference:c,floating:u}=a;const f=tn(o),h=Lu(o),v=ku(h),b=Tn(o),g=f==="y",p=c.x+c.width/2-u.width/2,S=c.y+c.height/2-u.height/2,w=c[v]/2-u[v]/2;let M;switch(b){case"top":M={x:p,y:c.y-u.height};break;case"bottom":M={x:p,y:c.y+c.height};break;case"right":M={x:c.x+c.width,y:S};break;case"left":M={x:c.x-u.width,y:S};break;default:M={x:c.x,y:c.y}}switch(Aa(o)){case"start":M[h]-=w*(i&&g?-1:1);break;case"end":M[h]+=w*(i&&g?-1:1);break}return M}const x1=async(a,o,i)=>{const{placement:c="bottom",strategy:u="absolute",middleware:f=[],platform:h}=i,v=f.filter(Boolean),b=await(h.isRTL==null?void 0:h.isRTL(o));let g=await h.getElementRects({reference:a,floating:o,strategy:u}),{x:p,y:S}=Ev(g,c,b),w=c,M={},T=0;for(let N=0;N<v.length;N++){const{name:_,fn:j}=v[N],{x:q,y:L,data:Y,reset:B}=await j({x:p,y:S,initialPlacement:c,placement:w,strategy:u,middlewareData:M,rects:g,platform:h,elements:{reference:a,floating:o}});p=q??p,S=L??S,M={...M,[_]:{...M[_],...Y}},B&&T<=50&&(T++,typeof B=="object"&&(B.placement&&(w=B.placement),B.rects&&(g=B.rects===!0?await h.getElementRects({reference:a,floating:o,strategy:u}):B.rects),{x:p,y:S}=Ev(g,w,b)),N=-1)}return{x:p,y:S,placement:w,strategy:u,middlewareData:M}};async function jr(a,o){var i;o===void 0&&(o={});const{x:c,y:u,platform:f,rects:h,elements:v,strategy:b}=a,{boundary:g="clippingAncestors",rootBoundary:p="viewport",elementContext:S="floating",altBoundary:w=!1,padding:M=0}=Nn(o,a),T=vg(M),_=v[w?S==="floating"?"reference":"floating":S],j=hi(await f.getClippingRect({element:(i=await(f.isElement==null?void 0:f.isElement(_)))==null||i?_:_.contextElement||await(f.getDocumentElement==null?void 0:f.getDocumentElement(v.floating)),boundary:g,rootBoundary:p,strategy:b})),q=S==="floating"?{x:c,y:u,width:h.floating.width,height:h.floating.height}:h.reference,L=await(f.getOffsetParent==null?void 0:f.getOffsetParent(v.floating)),Y=await(f.isElement==null?void 0:f.isElement(L))?await(f.getScale==null?void 0:f.getScale(L))||{x:1,y:1}:{x:1,y:1},B=hi(f.convertOffsetParentRelativeRectToViewportRelativeRect?await f.convertOffsetParentRelativeRectToViewportRelativeRect({elements:v,rect:q,offsetParent:L,strategy:b}):q);return{top:(j.top-B.top+T.top)/Y.y,bottom:(B.bottom-j.bottom+T.bottom)/Y.y,left:(j.left-B.left+T.left)/Y.x,right:(B.right-j.right+T.right)/Y.x}}const S1=a=>({name:"arrow",options:a,async fn(o){const{x:i,y:c,placement:u,rects:f,platform:h,elements:v,middlewareData:b}=o,{element:g,padding:p=0}=Nn(a,o)||{};if(g==null)return{};const S=vg(p),w={x:i,y:c},M=Lu(u),T=ku(M),N=await h.getDimensions(g),_=M==="y",j=_?"top":"left",q=_?"bottom":"right",L=_?"clientHeight":"clientWidth",Y=f.reference[T]+f.reference[M]-w[M]-f.floating[T],B=w[M]-f.reference[M],J=await(h.getOffsetParent==null?void 0:h.getOffsetParent(g));let P=J?J[L]:0;(!P||!await(h.isElement==null?void 0:h.isElement(J)))&&(P=v.floating[L]||f.floating[T]);const Z=Y/2-B/2,I=P/2-N[T]/2-1,re=nl(S[j],I),ue=nl(S[q],I),de=re,Ee=P-N[T]-ue,be=P/2-N[T]/2+Z,ve=bu(de,be,Ee),D=!b.arrow&&Aa(u)!=null&&be!==ve&&f.reference[T]/2-(be<de?re:ue)-N[T]/2<0,K=D?be<de?be-de:be-Ee:0;return{[M]:w[M]+K,data:{[M]:ve,centerOffset:be-ve-K,...D&&{alignmentOffset:K}},reset:D}}}),w1=function(a){return a===void 0&&(a={}),{name:"flip",options:a,async fn(o){var i,c;const{placement:u,middlewareData:f,rects:h,initialPlacement:v,platform:b,elements:g}=o,{mainAxis:p=!0,crossAxis:S=!0,fallbackPlacements:w,fallbackStrategy:M="bestFit",fallbackAxisSideDirection:T="none",flipAlignment:N=!0,..._}=Nn(a,o);if((i=f.arrow)!=null&&i.alignmentOffset)return{};const j=Tn(u),q=tn(v),L=Tn(v)===v,Y=await(b.isRTL==null?void 0:b.isRTL(g.floating)),B=w||(L||!N?[mi(v)]:h1(v)),J=T!=="none";!w&&J&&B.push(...y1(v,N,T,Y));const P=[v,...B],Z=await jr(o,_),I=[];let re=((c=f.flip)==null?void 0:c.overflows)||[];if(p&&I.push(Z[j]),S){const be=m1(u,h,Y);I.push(Z[be[0]],Z[be[1]])}if(re=[...re,{placement:u,overflows:I}],!I.every(be=>be<=0)){var ue,de;const be=(((ue=f.flip)==null?void 0:ue.index)||0)+1,ve=P[be];if(ve&&(!(S==="alignment"?q!==tn(ve):!1)||re.every(H=>tn(H.placement)===q?H.overflows[0]>0:!0)))return{data:{index:be,overflows:re},reset:{placement:ve}};let D=(de=re.filter(K=>K.overflows[0]<=0).sort((K,H)=>K.overflows[1]-H.overflows[1])[0])==null?void 0:de.placement;if(!D)switch(M){case"bestFit":{var Ee;const K=(Ee=re.filter(H=>{if(J){const W=tn(H.placement);return W===q||W==="y"}return!0}).map(H=>[H.placement,H.overflows.filter(W=>W>0).reduce((W,A)=>W+A,0)]).sort((H,W)=>H[1]-W[1])[0])==null?void 0:Ee[0];K&&(D=K);break}case"initialPlacement":D=v;break}if(u!==D)return{reset:{placement:D}}}return{}}}};function Av(a,o){return{top:a.top-o.height,right:a.right-o.width,bottom:a.bottom-o.height,left:a.left-o.width}}function Mv(a){return s1.some(o=>a[o]>=0)}const E1=function(a){return a===void 0&&(a={}),{name:"hide",options:a,async fn(o){const{rects:i}=o,{strategy:c="referenceHidden",...u}=Nn(a,o);switch(c){case"referenceHidden":{const f=await jr(o,{...u,elementContext:"reference"}),h=Av(f,i.reference);return{data:{referenceHiddenOffsets:h,referenceHidden:Mv(h)}}}case"escaped":{const f=await jr(o,{...u,altBoundary:!0}),h=Av(f,i.floating);return{data:{escapedOffsets:h,escaped:Mv(h)}}}default:return{}}}}},gg=new Set(["left","top"]);async function A1(a,o){const{placement:i,platform:c,elements:u}=a,f=await(c.isRTL==null?void 0:c.isRTL(u.floating)),h=Tn(i),v=Aa(i),b=tn(i)==="y",g=gg.has(h)?-1:1,p=f&&b?-1:1,S=Nn(o,a);let{mainAxis:w,crossAxis:M,alignmentAxis:T}=typeof S=="number"?{mainAxis:S,crossAxis:0,alignmentAxis:null}:{mainAxis:S.mainAxis||0,crossAxis:S.crossAxis||0,alignmentAxis:S.alignmentAxis};return v&&typeof T=="number"&&(M=v==="end"?T*-1:T),b?{x:M*p,y:w*g}:{x:w*g,y:M*p}}const M1=function(a){return a===void 0&&(a=0),{name:"offset",options:a,async fn(o){var i,c;const{x:u,y:f,placement:h,middlewareData:v}=o,b=await A1(o,a);return h===((i=v.offset)==null?void 0:i.placement)&&(c=v.arrow)!=null&&c.alignmentOffset?{}:{x:u+b.x,y:f+b.y,data:{...b,placement:h}}}}},N1=function(a){return a===void 0&&(a={}),{name:"shift",options:a,async fn(o){const{x:i,y:c,placement:u}=o,{mainAxis:f=!0,crossAxis:h=!1,limiter:v={fn:_=>{let{x:j,y:q}=_;return{x:j,y:q}}},...b}=Nn(a,o),g={x:i,y:c},p=await jr(o,b),S=tn(Tn(u)),w=Uu(S);let M=g[w],T=g[S];if(f){const _=w==="y"?"top":"left",j=w==="y"?"bottom":"right",q=M+p[_],L=M-p[j];M=bu(q,M,L)}if(h){const _=S==="y"?"top":"left",j=S==="y"?"bottom":"right",q=T+p[_],L=T-p[j];T=bu(q,T,L)}const N=v.fn({...o,[w]:M,[S]:T});return{...N,data:{x:N.x-i,y:N.y-c,enabled:{[w]:f,[S]:h}}}}}},T1=function(a){return a===void 0&&(a={}),{options:a,fn(o){const{x:i,y:c,placement:u,rects:f,middlewareData:h}=o,{offset:v=0,mainAxis:b=!0,crossAxis:g=!0}=Nn(a,o),p={x:i,y:c},S=tn(u),w=Uu(S);let M=p[w],T=p[S];const N=Nn(v,o),_=typeof N=="number"?{mainAxis:N,crossAxis:0}:{mainAxis:0,crossAxis:0,...N};if(b){const L=w==="y"?"height":"width",Y=f.reference[w]-f.floating[L]+_.mainAxis,B=f.reference[w]+f.reference[L]-_.mainAxis;M<Y?M=Y:M>B&&(M=B)}if(g){var j,q;const L=w==="y"?"width":"height",Y=gg.has(Tn(u)),B=f.reference[S]-f.floating[L]+(Y&&((j=h.offset)==null?void 0:j[S])||0)+(Y?0:_.crossAxis),J=f.reference[S]+f.reference[L]+(Y?0:((q=h.offset)==null?void 0:q[S])||0)-(Y?_.crossAxis:0);T<B?T=B:T>J&&(T=J)}return{[w]:M,[S]:T}}}},R1=function(a){return a===void 0&&(a={}),{name:"size",options:a,async fn(o){var i,c;const{placement:u,rects:f,platform:h,elements:v}=o,{apply:b=()=>{},...g}=Nn(a,o),p=await jr(o,g),S=Tn(u),w=Aa(u),M=tn(u)==="y",{width:T,height:N}=f.floating;let _,j;S==="top"||S==="bottom"?(_=S,j=w===(await(h.isRTL==null?void 0:h.isRTL(v.floating))?"start":"end")?"left":"right"):(j=S,_=w==="end"?"top":"bottom");const q=N-p.top-p.bottom,L=T-p.left-p.right,Y=nl(N-p[_],q),B=nl(T-p[j],L),J=!o.middlewareData.shift;let P=Y,Z=B;if((i=o.middlewareData.shift)!=null&&i.enabled.x&&(Z=L),(c=o.middlewareData.shift)!=null&&c.enabled.y&&(P=q),J&&!w){const re=At(p.left,0),ue=At(p.right,0),de=At(p.top,0),Ee=At(p.bottom,0);M?Z=T-2*(re!==0||ue!==0?re+ue:At(p.left,p.right)):P=N-2*(de!==0||Ee!==0?de+Ee:At(p.top,p.bottom))}await b({...o,availableWidth:Z,availableHeight:P});const I=await h.getDimensions(v.floating);return T!==I.width||N!==I.height?{reset:{rects:!0}}:{}}}};function Ei(){return typeof window<"u"}function Ma(a){return pg(a)?(a.nodeName||"").toLowerCase():"#document"}function Rt(a){var o;return(a==null||(o=a.ownerDocument)==null?void 0:o.defaultView)||window}function an(a){var o;return(o=(pg(a)?a.ownerDocument:a.document)||window.document)==null?void 0:o.documentElement}function pg(a){return Ei()?a instanceof Node||a instanceof Rt(a).Node:!1}function Vt(a){return Ei()?a instanceof Element||a instanceof Rt(a).Element:!1}function ln(a){return Ei()?a instanceof HTMLElement||a instanceof Rt(a).HTMLElement:!1}function Nv(a){return!Ei()||typeof ShadowRoot>"u"?!1:a instanceof ShadowRoot||a instanceof Rt(a).ShadowRoot}const _1=new Set(["inline","contents"]);function Ur(a){const{overflow:o,overflowX:i,overflowY:c,display:u}=Xt(a);return/auto|scroll|overlay|hidden|clip/.test(o+c+i)&&!_1.has(u)}const C1=new Set(["table","td","th"]);function j1(a){return C1.has(Ma(a))}const O1=[":popover-open",":modal"];function Ai(a){return O1.some(o=>{try{return a.matches(o)}catch{return!1}})}const D1=["transform","translate","scale","rotate","perspective"],z1=["transform","translate","scale","rotate","perspective","filter"],U1=["paint","layout","strict","content"];function Hu(a){const o=Bu(),i=Vt(a)?Xt(a):a;return D1.some(c=>i[c]?i[c]!=="none":!1)||(i.containerType?i.containerType!=="normal":!1)||!o&&(i.backdropFilter?i.backdropFilter!=="none":!1)||!o&&(i.filter?i.filter!=="none":!1)||z1.some(c=>(i.willChange||"").includes(c))||U1.some(c=>(i.contain||"").includes(c))}function k1(a){let o=ll(a);for(;ln(o)&&!ba(o);){if(Hu(o))return o;if(Ai(o))return null;o=ll(o)}return null}function Bu(){return typeof CSS>"u"||!CSS.supports?!1:CSS.supports("-webkit-backdrop-filter","none")}const L1=new Set(["html","body","#document"]);function ba(a){return L1.has(Ma(a))}function Xt(a){return Rt(a).getComputedStyle(a)}function Mi(a){return Vt(a)?{scrollLeft:a.scrollLeft,scrollTop:a.scrollTop}:{scrollLeft:a.scrollX,scrollTop:a.scrollY}}function ll(a){if(Ma(a)==="html")return a;const o=a.assignedSlot||a.parentNode||Nv(a)&&a.host||an(a);return Nv(o)?o.host:o}function yg(a){const o=ll(a);return ba(o)?a.ownerDocument?a.ownerDocument.body:a.body:ln(o)&&Ur(o)?o:yg(o)}function Or(a,o,i){var c;o===void 0&&(o=[]),i===void 0&&(i=!0);const u=yg(a),f=u===((c=a.ownerDocument)==null?void 0:c.body),h=Rt(u);if(f){const v=Su(h);return o.concat(h,h.visualViewport||[],Ur(u)?u:[],v&&i?Or(v):[])}return o.concat(u,Or(u,[],i))}function Su(a){return a.parent&&Object.getPrototypeOf(a.parent)?a.frameElement:null}function bg(a){const o=Xt(a);let i=parseFloat(o.width)||0,c=parseFloat(o.height)||0;const u=ln(a),f=u?a.offsetWidth:i,h=u?a.offsetHeight:c,v=di(i)!==f||di(c)!==h;return v&&(i=f,c=h),{width:i,height:c,$:v}}function qu(a){return Vt(a)?a:a.contextElement}function pa(a){const o=qu(a);if(!ln(o))return nn(1);const i=o.getBoundingClientRect(),{width:c,height:u,$:f}=bg(o);let h=(f?di(i.width):i.width)/c,v=(f?di(i.height):i.height)/u;return(!h||!Number.isFinite(h))&&(h=1),(!v||!Number.isFinite(v))&&(v=1),{x:h,y:v}}const H1=nn(0);function xg(a){const o=Rt(a);return!Bu()||!o.visualViewport?H1:{x:o.visualViewport.offsetLeft,y:o.visualViewport.offsetTop}}function B1(a,o,i){return o===void 0&&(o=!1),!i||o&&i!==Rt(a)?!1:o}function El(a,o,i,c){o===void 0&&(o=!1),i===void 0&&(i=!1);const u=a.getBoundingClientRect(),f=qu(a);let h=nn(1);o&&(c?Vt(c)&&(h=pa(c)):h=pa(a));const v=B1(f,i,c)?xg(f):nn(0);let b=(u.left+v.x)/h.x,g=(u.top+v.y)/h.y,p=u.width/h.x,S=u.height/h.y;if(f){const w=Rt(f),M=c&&Vt(c)?Rt(c):c;let T=w,N=Su(T);for(;N&&c&&M!==T;){const _=pa(N),j=N.getBoundingClientRect(),q=Xt(N),L=j.left+(N.clientLeft+parseFloat(q.paddingLeft))*_.x,Y=j.top+(N.clientTop+parseFloat(q.paddingTop))*_.y;b*=_.x,g*=_.y,p*=_.x,S*=_.y,b+=L,g+=Y,T=Rt(N),N=Su(T)}}return hi({width:p,height:S,x:b,y:g})}function Ni(a,o){const i=Mi(a).scrollLeft;return o?o.left+i:El(an(a)).left+i}function Sg(a,o){const i=a.getBoundingClientRect(),c=i.left+o.scrollLeft-Ni(a,i),u=i.top+o.scrollTop;return{x:c,y:u}}function q1(a){let{elements:o,rect:i,offsetParent:c,strategy:u}=a;const f=u==="fixed",h=an(c),v=o?Ai(o.floating):!1;if(c===h||v&&f)return i;let b={scrollLeft:0,scrollTop:0},g=nn(1);const p=nn(0),S=ln(c);if((S||!S&&!f)&&((Ma(c)!=="body"||Ur(h))&&(b=Mi(c)),ln(c))){const M=El(c);g=pa(c),p.x=M.x+c.clientLeft,p.y=M.y+c.clientTop}const w=h&&!S&&!f?Sg(h,b):nn(0);return{width:i.width*g.x,height:i.height*g.y,x:i.x*g.x-b.scrollLeft*g.x+p.x+w.x,y:i.y*g.y-b.scrollTop*g.y+p.y+w.y}}function G1(a){return Array.from(a.getClientRects())}function Y1(a){const o=an(a),i=Mi(a),c=a.ownerDocument.body,u=At(o.scrollWidth,o.clientWidth,c.scrollWidth,c.clientWidth),f=At(o.scrollHeight,o.clientHeight,c.scrollHeight,c.clientHeight);let h=-i.scrollLeft+Ni(a);const v=-i.scrollTop;return Xt(c).direction==="rtl"&&(h+=At(o.clientWidth,c.clientWidth)-u),{width:u,height:f,x:h,y:v}}const Tv=25;function V1(a,o){const i=Rt(a),c=an(a),u=i.visualViewport;let f=c.clientWidth,h=c.clientHeight,v=0,b=0;if(u){f=u.width,h=u.height;const p=Bu();(!p||p&&o==="fixed")&&(v=u.offsetLeft,b=u.offsetTop)}const g=Ni(c);if(g<=0){const p=c.ownerDocument,S=p.body,w=getComputedStyle(S),M=p.compatMode==="CSS1Compat"&&parseFloat(w.marginLeft)+parseFloat(w.marginRight)||0,T=Math.abs(c.clientWidth-S.clientWidth-M);T<=Tv&&(f-=T)}else g<=Tv&&(f+=g);return{width:f,height:h,x:v,y:b}}const X1=new Set(["absolute","fixed"]);function Q1(a,o){const i=El(a,!0,o==="fixed"),c=i.top+a.clientTop,u=i.left+a.clientLeft,f=ln(a)?pa(a):nn(1),h=a.clientWidth*f.x,v=a.clientHeight*f.y,b=u*f.x,g=c*f.y;return{width:h,height:v,x:b,y:g}}function Rv(a,o,i){let c;if(o==="viewport")c=V1(a,i);else if(o==="document")c=Y1(an(a));else if(Vt(o))c=Q1(o,i);else{const u=xg(a);c={x:o.x-u.x,y:o.y-u.y,width:o.width,height:o.height}}return hi(c)}function wg(a,o){const i=ll(a);return i===o||!Vt(i)||ba(i)?!1:Xt(i).position==="fixed"||wg(i,o)}function Z1(a,o){const i=o.get(a);if(i)return i;let c=Or(a,[],!1).filter(v=>Vt(v)&&Ma(v)!=="body"),u=null;const f=Xt(a).position==="fixed";let h=f?ll(a):a;for(;Vt(h)&&!ba(h);){const v=Xt(h),b=Hu(h);!b&&v.position==="fixed"&&(u=null),(f?!b&&!u:!b&&v.position==="static"&&!!u&&X1.has(u.position)||Ur(h)&&!b&&wg(a,h))?c=c.filter(p=>p!==h):u=v,h=ll(h)}return o.set(a,c),c}function K1(a){let{element:o,boundary:i,rootBoundary:c,strategy:u}=a;const h=[...i==="clippingAncestors"?Ai(o)?[]:Z1(o,this._c):[].concat(i),c],v=h[0],b=h.reduce((g,p)=>{const S=Rv(o,p,u);return g.top=At(S.top,g.top),g.right=nl(S.right,g.right),g.bottom=nl(S.bottom,g.bottom),g.left=At(S.left,g.left),g},Rv(o,v,u));return{width:b.right-b.left,height:b.bottom-b.top,x:b.left,y:b.top}}function P1(a){const{width:o,height:i}=bg(a);return{width:o,height:i}}function $1(a,o,i){const c=ln(o),u=an(o),f=i==="fixed",h=El(a,!0,f,o);let v={scrollLeft:0,scrollTop:0};const b=nn(0);function g(){b.x=Ni(u)}if(c||!c&&!f)if((Ma(o)!=="body"||Ur(u))&&(v=Mi(o)),c){const M=El(o,!0,f,o);b.x=M.x+o.clientLeft,b.y=M.y+o.clientTop}else u&&g();f&&!c&&u&&g();const p=u&&!c&&!f?Sg(u,v):nn(0),S=h.left+v.scrollLeft-b.x-p.x,w=h.top+v.scrollTop-b.y-p.y;return{x:S,y:w,width:h.width,height:h.height}}function ru(a){return Xt(a).position==="static"}function _v(a,o){if(!ln(a)||Xt(a).position==="fixed")return null;if(o)return o(a);let i=a.offsetParent;return an(a)===i&&(i=i.ownerDocument.body),i}function Eg(a,o){const i=Rt(a);if(Ai(a))return i;if(!ln(a)){let u=ll(a);for(;u&&!ba(u);){if(Vt(u)&&!ru(u))return u;u=ll(u)}return i}let c=_v(a,o);for(;c&&j1(c)&&ru(c);)c=_v(c,o);return c&&ba(c)&&ru(c)&&!Hu(c)?i:c||k1(a)||i}const J1=async function(a){const o=this.getOffsetParent||Eg,i=this.getDimensions,c=await i(a.floating);return{reference:$1(a.reference,await o(a.floating),a.strategy),floating:{x:0,y:0,width:c.width,height:c.height}}};function W1(a){return Xt(a).direction==="rtl"}const F1={convertOffsetParentRelativeRectToViewportRelativeRect:q1,getDocumentElement:an,getClippingRect:K1,getOffsetParent:Eg,getElementRects:J1,getClientRects:G1,getDimensions:P1,getScale:pa,isElement:Vt,isRTL:W1};function Ag(a,o){return a.x===o.x&&a.y===o.y&&a.width===o.width&&a.height===o.height}function I1(a,o){let i=null,c;const u=an(a);function f(){var v;clearTimeout(c),(v=i)==null||v.disconnect(),i=null}function h(v,b){v===void 0&&(v=!1),b===void 0&&(b=1),f();const g=a.getBoundingClientRect(),{left:p,top:S,width:w,height:M}=g;if(v||o(),!w||!M)return;const T=ni(S),N=ni(u.clientWidth-(p+w)),_=ni(u.clientHeight-(S+M)),j=ni(p),L={rootMargin:-T+"px "+-N+"px "+-_+"px "+-j+"px",threshold:At(0,nl(1,b))||1};let Y=!0;function B(J){const P=J[0].intersectionRatio;if(P!==b){if(!Y)return h();P?h(!1,P):c=setTimeout(()=>{h(!1,1e-7)},1e3)}P===1&&!Ag(g,a.getBoundingClientRect())&&h(),Y=!1}try{i=new IntersectionObserver(B,{...L,root:u.ownerDocument})}catch{i=new IntersectionObserver(B,L)}i.observe(a)}return h(!0),f}function eS(a,o,i,c){c===void 0&&(c={});const{ancestorScroll:u=!0,ancestorResize:f=!0,elementResize:h=typeof ResizeObserver=="function",layoutShift:v=typeof IntersectionObserver=="function",animationFrame:b=!1}=c,g=qu(a),p=u||f?[...g?Or(g):[],...Or(o)]:[];p.forEach(j=>{u&&j.addEventListener("scroll",i,{passive:!0}),f&&j.addEventListener("resize",i)});const S=g&&v?I1(g,i):null;let w=-1,M=null;h&&(M=new ResizeObserver(j=>{let[q]=j;q&&q.target===g&&M&&(M.unobserve(o),cancelAnimationFrame(w),w=requestAnimationFrame(()=>{var L;(L=M)==null||L.observe(o)})),i()}),g&&!b&&M.observe(g),M.observe(o));let T,N=b?El(a):null;b&&_();function _(){const j=El(a);N&&!Ag(N,j)&&i(),N=j,T=requestAnimationFrame(_)}return i(),()=>{var j;p.forEach(q=>{u&&q.removeEventListener("scroll",i),f&&q.removeEventListener("resize",i)}),S?.(),(j=M)==null||j.disconnect(),M=null,b&&cancelAnimationFrame(T)}}const tS=M1,nS=N1,lS=w1,aS=R1,rS=E1,Cv=S1,oS=T1,iS=(a,o,i)=>{const c=new Map,u={platform:F1,...i},f={...u.platform,_c:c};return x1(a,o,{...u,platform:f})};var cS=typeof document<"u",sS=function(){},ci=cS?x.useLayoutEffect:sS;function vi(a,o){if(a===o)return!0;if(typeof a!=typeof o)return!1;if(typeof a=="function"&&a.toString()===o.toString())return!0;let i,c,u;if(a&&o&&typeof a=="object"){if(Array.isArray(a)){if(i=a.length,i!==o.length)return!1;for(c=i;c--!==0;)if(!vi(a[c],o[c]))return!1;return!0}if(u=Object.keys(a),i=u.length,i!==Object.keys(o).length)return!1;for(c=i;c--!==0;)if(!{}.hasOwnProperty.call(o,u[c]))return!1;for(c=i;c--!==0;){const f=u[c];if(!(f==="_owner"&&a.$$typeof)&&!vi(a[f],o[f]))return!1}return!0}return a!==a&&o!==o}function Mg(a){return typeof window>"u"?1:(a.ownerDocument.defaultView||window).devicePixelRatio||1}function jv(a,o){const i=Mg(a);return Math.round(o*i)/i}function ou(a){const o=x.useRef(a);return ci(()=>{o.current=a}),o}function uS(a){a===void 0&&(a={});const{placement:o="bottom",strategy:i="absolute",middleware:c=[],platform:u,elements:{reference:f,floating:h}={},transform:v=!0,whileElementsMounted:b,open:g}=a,[p,S]=x.useState({x:0,y:0,strategy:i,placement:o,middlewareData:{},isPositioned:!1}),[w,M]=x.useState(c);vi(w,c)||M(c);const[T,N]=x.useState(null),[_,j]=x.useState(null),q=x.useCallback(H=>{H!==J.current&&(J.current=H,N(H))},[]),L=x.useCallback(H=>{H!==P.current&&(P.current=H,j(H))},[]),Y=f||T,B=h||_,J=x.useRef(null),P=x.useRef(null),Z=x.useRef(p),I=b!=null,re=ou(b),ue=ou(u),de=ou(g),Ee=x.useCallback(()=>{if(!J.current||!P.current)return;const H={placement:o,strategy:i,middleware:w};ue.current&&(H.platform=ue.current),iS(J.current,P.current,H).then(W=>{const A={...W,isPositioned:de.current!==!1};be.current&&!vi(Z.current,A)&&(Z.current=A,zu.flushSync(()=>{S(A)}))})},[w,o,i,ue,de]);ci(()=>{g===!1&&Z.current.isPositioned&&(Z.current.isPositioned=!1,S(H=>({...H,isPositioned:!1})))},[g]);const be=x.useRef(!1);ci(()=>(be.current=!0,()=>{be.current=!1}),[]),ci(()=>{if(Y&&(J.current=Y),B&&(P.current=B),Y&&B){if(re.current)return re.current(Y,B,Ee);Ee()}},[Y,B,Ee,re,I]);const ve=x.useMemo(()=>({reference:J,floating:P,setReference:q,setFloating:L}),[q,L]),D=x.useMemo(()=>({reference:Y,floating:B}),[Y,B]),K=x.useMemo(()=>{const H={position:i,left:0,top:0};if(!D.floating)return H;const W=jv(D.floating,p.x),A=jv(D.floating,p.y);return v?{...H,transform:"translate("+W+"px, "+A+"px)",...Mg(D.floating)>=1.5&&{willChange:"transform"}}:{position:i,left:W,top:A}},[i,v,D.floating,p.x,p.y]);return x.useMemo(()=>({...p,update:Ee,refs:ve,elements:D,floatingStyles:K}),[p,Ee,ve,D,K])}const fS=a=>{function o(i){return{}.hasOwnProperty.call(i,"current")}return{name:"arrow",options:a,fn(i){const{element:c,padding:u}=typeof a=="function"?a(i):a;return c&&o(c)?c.current!=null?Cv({element:c.current,padding:u}).fn(i):{}:c?Cv({element:c,padding:u}).fn(i):{}}}},dS=(a,o)=>({...tS(a),options:[a,o]}),mS=(a,o)=>({...nS(a),options:[a,o]}),hS=(a,o)=>({...oS(a),options:[a,o]}),vS=(a,o)=>({...lS(a),options:[a,o]}),gS=(a,o)=>({...aS(a),options:[a,o]}),pS=(a,o)=>({...rS(a),options:[a,o]}),yS=(a,o)=>({...fS(a),options:[a,o]});var bS="Arrow",Ng=x.forwardRef((a,o)=>{const{children:i,width:c=10,height:u=5,...f}=a;return d.jsx(ke.svg,{...f,ref:o,width:c,height:u,viewBox:"0 0 30 10",preserveAspectRatio:"none",children:a.asChild?i:d.jsx("polygon",{points:"0,0 30,0 15,10"})})});Ng.displayName=bS;var xS=Ng;function SS(a){const[o,i]=x.useState(void 0);return Mn(()=>{if(a){i({width:a.offsetWidth,height:a.offsetHeight});const c=new ResizeObserver(u=>{if(!Array.isArray(u)||!u.length)return;const f=u[0];let h,v;if("borderBoxSize"in f){const b=f.borderBoxSize,g=Array.isArray(b)?b[0]:b;h=g.inlineSize,v=g.blockSize}else h=a.offsetWidth,v=a.offsetHeight;i({width:h,height:v})});return c.observe(a,{box:"border-box"}),()=>c.unobserve(a)}else i(void 0)},[a]),o}var Gu="Popper",[Tg,Rg]=Ml(Gu),[wS,_g]=Tg(Gu),Cg=a=>{const{__scopePopper:o,children:i}=a,[c,u]=x.useState(null);return d.jsx(wS,{scope:o,anchor:c,onAnchorChange:u,children:i})};Cg.displayName=Gu;var jg="PopperAnchor",Og=x.forwardRef((a,o)=>{const{__scopePopper:i,virtualRef:c,...u}=a,f=_g(jg,i),h=x.useRef(null),v=Qe(o,h),b=x.useRef(null);return x.useEffect(()=>{const g=b.current;b.current=c?.current||h.current,g!==b.current&&f.onAnchorChange(b.current)}),c?null:d.jsx(ke.div,{...u,ref:v})});Og.displayName=jg;var Yu="PopperContent",[ES,AS]=Tg(Yu),Dg=x.forwardRef((a,o)=>{const{__scopePopper:i,side:c="bottom",sideOffset:u=0,align:f="center",alignOffset:h=0,arrowPadding:v=0,avoidCollisions:b=!0,collisionBoundary:g=[],collisionPadding:p=0,sticky:S="partial",hideWhenDetached:w=!1,updatePositionStrategy:M="optimized",onPlaced:T,...N}=a,_=_g(Yu,i),[j,q]=x.useState(null),L=Qe(o,ie=>q(ie)),[Y,B]=x.useState(null),J=SS(Y),P=J?.width??0,Z=J?.height??0,I=c+(f!=="center"?"-"+f:""),re=typeof p=="number"?p:{top:0,right:0,bottom:0,left:0,...p},ue=Array.isArray(g)?g:[g],de=ue.length>0,Ee={padding:re,boundary:ue.filter(NS),altBoundary:de},{refs:be,floatingStyles:ve,placement:D,isPositioned:K,middlewareData:H}=uS({strategy:"fixed",placement:I,whileElementsMounted:(...ie)=>eS(...ie,{animationFrame:M==="always"}),elements:{reference:_.anchor},middleware:[dS({mainAxis:u+Z,alignmentAxis:h}),b&&mS({mainAxis:!0,crossAxis:!1,limiter:S==="partial"?hS():void 0,...Ee}),b&&vS({...Ee}),gS({...Ee,apply:({elements:ie,rects:Me,availableWidth:it,availableHeight:Qt})=>{const{width:Zt,height:Kt}=Me.reference,Rn=ie.floating.style;Rn.setProperty("--radix-popper-available-width",`${it}px`),Rn.setProperty("--radix-popper-available-height",`${Qt}px`),Rn.setProperty("--radix-popper-anchor-width",`${Zt}px`),Rn.setProperty("--radix-popper-anchor-height",`${Kt}px`)}}),Y&&yS({element:Y,padding:v}),TS({arrowWidth:P,arrowHeight:Z}),w&&pS({strategy:"referenceHidden",...Ee})]}),[W,A]=kg(D),X=ot(T);Mn(()=>{K&&X?.()},[K,X]);const F=H.arrow?.x,$=H.arrow?.y,ee=H.arrow?.centerOffset!==0,[he,oe]=x.useState();return Mn(()=>{j&&oe(window.getComputedStyle(j).zIndex)},[j]),d.jsx("div",{ref:be.setFloating,"data-radix-popper-content-wrapper":"",style:{...ve,transform:K?ve.transform:"translate(0, -200%)",minWidth:"max-content",zIndex:he,"--radix-popper-transform-origin":[H.transformOrigin?.x,H.transformOrigin?.y].join(" "),...H.hide?.referenceHidden&&{visibility:"hidden",pointerEvents:"none"}},dir:a.dir,children:d.jsx(ES,{scope:i,placedSide:W,onArrowChange:B,arrowX:F,arrowY:$,shouldHideArrow:ee,children:d.jsx(ke.div,{"data-side":W,"data-align":A,...N,ref:L,style:{...N.style,animation:K?void 0:"none"}})})})});Dg.displayName=Yu;var zg="PopperArrow",MS={top:"bottom",right:"left",bottom:"top",left:"right"},Ug=x.forwardRef(function(o,i){const{__scopePopper:c,...u}=o,f=AS(zg,c),h=MS[f.placedSide];return d.jsx("span",{ref:f.onArrowChange,style:{position:"absolute",left:f.arrowX,top:f.arrowY,[h]:0,transformOrigin:{top:"",right:"0 0",bottom:"center 0",left:"100% 0"}[f.placedSide],transform:{top:"translateY(100%)",right:"translateY(50%) rotate(90deg) translateX(-50%)",bottom:"rotate(180deg)",left:"translateY(50%) rotate(-90deg) translateX(50%)"}[f.placedSide],visibility:f.shouldHideArrow?"hidden":void 0},children:d.jsx(xS,{...u,ref:i,style:{...u.style,display:"block"}})})});Ug.displayName=zg;function NS(a){return a!==null}var TS=a=>({name:"transformOrigin",options:a,fn(o){const{placement:i,rects:c,middlewareData:u}=o,h=u.arrow?.centerOffset!==0,v=h?0:a.arrowWidth,b=h?0:a.arrowHeight,[g,p]=kg(i),S={start:"0%",center:"50%",end:"100%"}[p],w=(u.arrow?.x??0)+v/2,M=(u.arrow?.y??0)+b/2;let T="",N="";return g==="bottom"?(T=h?S:`${w}px`,N=`${-b}px`):g==="top"?(T=h?S:`${w}px`,N=`${c.floating.height+b}px`):g==="right"?(T=`${-b}px`,N=h?S:`${M}px`):g==="left"&&(T=`${c.floating.width+b}px`,N=h?S:`${M}px`),{data:{x:T,y:N}}}});function kg(a){const[o,i="center"]=a.split("-");return[o,i]}var RS=Cg,_S=Og,CS=Dg,jS=Ug,OS="Portal",Lg=x.forwardRef((a,o)=>{const{container:i,...c}=a,[u,f]=x.useState(!1);Mn(()=>f(!0),[]);const h=i||u&&globalThis?.document?.body;return h?Yx.createPortal(d.jsx(ke.div,{...c,ref:o}),h):null});Lg.displayName=OS;function DS(a,o){return x.useReducer((i,c)=>o[i][c]??i,a)}var rn=a=>{const{present:o,children:i}=a,c=zS(o),u=typeof i=="function"?i({present:c.isPresent}):x.Children.only(i),f=Qe(c.ref,US(u));return typeof i=="function"||c.isPresent?x.cloneElement(u,{ref:f}):null};rn.displayName="Presence";function zS(a){const[o,i]=x.useState(),c=x.useRef(null),u=x.useRef(a),f=x.useRef("none"),h=a?"mounted":"unmounted",[v,b]=DS(h,{mounted:{UNMOUNT:"unmounted",ANIMATION_OUT:"unmountSuspended"},unmountSuspended:{MOUNT:"mounted",ANIMATION_END:"unmounted"},unmounted:{MOUNT:"mounted"}});return x.useEffect(()=>{const g=li(c.current);f.current=v==="mounted"?g:"none"},[v]),Mn(()=>{const g=c.current,p=u.current;if(p!==a){const w=f.current,M=li(g);a?b("MOUNT"):M==="none"||g?.display==="none"?b("UNMOUNT"):b(p&&w!==M?"ANIMATION_OUT":"UNMOUNT"),u.current=a}},[a,b]),Mn(()=>{if(o){let g;const p=o.ownerDocument.defaultView??window,S=M=>{const N=li(c.current).includes(CSS.escape(M.animationName));if(M.target===o&&N&&(b("ANIMATION_END"),!u.current)){const _=o.style.animationFillMode;o.style.animationFillMode="forwards",g=p.setTimeout(()=>{o.style.animationFillMode==="forwards"&&(o.style.animationFillMode=_)})}},w=M=>{M.target===o&&(f.current=li(c.current))};return o.addEventListener("animationstart",w),o.addEventListener("animationcancel",S),o.addEventListener("animationend",S),()=>{p.clearTimeout(g),o.removeEventListener("animationstart",w),o.removeEventListener("animationcancel",S),o.removeEventListener("animationend",S)}}else b("ANIMATION_END")},[o,b]),{isPresent:["mounted","unmountSuspended"].includes(v),ref:x.useCallback(g=>{c.current=g?getComputedStyle(g):null,i(g)},[])}}function li(a){return a?.animationName||"none"}function US(a){let o=Object.getOwnPropertyDescriptor(a.props,"ref")?.get,i=o&&"isReactWarning"in o&&o.isReactWarning;return i?a.ref:(o=Object.getOwnPropertyDescriptor(a,"ref")?.get,i=o&&"isReactWarning"in o&&o.isReactWarning,i?a.props.ref:a.props.ref||a.ref)}var iu="rovingFocusGroup.onEntryFocus",kS={bubbles:!1,cancelable:!0},kr="RovingFocusGroup",[wu,Hg,LS]=sg(kr),[HS,Ti]=Ml(kr,[LS]),[BS,qS]=HS(kr),Bg=x.forwardRef((a,o)=>d.jsx(wu.Provider,{scope:a.__scopeRovingFocusGroup,children:d.jsx(wu.Slot,{scope:a.__scopeRovingFocusGroup,children:d.jsx(GS,{...a,ref:o})})}));Bg.displayName=kr;var GS=x.forwardRef((a,o)=>{const{__scopeRovingFocusGroup:i,orientation:c,loop:u=!1,dir:f,currentTabStopId:h,defaultCurrentTabStopId:v,onCurrentTabStopIdChange:b,onEntryFocus:g,preventScrollOnEntryFocus:p=!1,...S}=a,w=x.useRef(null),M=Qe(o,w),T=wi(f),[N,_]=Du({prop:h,defaultProp:v??null,onChange:b,caller:kr}),[j,q]=x.useState(!1),L=ot(g),Y=Hg(i),B=x.useRef(!1),[J,P]=x.useState(0);return x.useEffect(()=>{const Z=w.current;if(Z)return Z.addEventListener(iu,L),()=>Z.removeEventListener(iu,L)},[L]),d.jsx(BS,{scope:i,orientation:c,dir:T,loop:u,currentTabStopId:N,onItemFocus:x.useCallback(Z=>_(Z),[_]),onItemShiftTab:x.useCallback(()=>q(!0),[]),onFocusableItemAdd:x.useCallback(()=>P(Z=>Z+1),[]),onFocusableItemRemove:x.useCallback(()=>P(Z=>Z-1),[]),children:d.jsx(ke.div,{tabIndex:j||J===0?-1:0,"data-orientation":c,...S,ref:M,style:{outline:"none",...a.style},onMouseDown:me(a.onMouseDown,()=>{B.current=!0}),onFocus:me(a.onFocus,Z=>{const I=!B.current;if(Z.target===Z.currentTarget&&I&&!j){const re=new CustomEvent(iu,kS);if(Z.currentTarget.dispatchEvent(re),!re.defaultPrevented){const ue=Y().filter(D=>D.focusable),de=ue.find(D=>D.active),Ee=ue.find(D=>D.id===N),ve=[de,Ee,...ue].filter(Boolean).map(D=>D.ref.current);Yg(ve,p)}}B.current=!1}),onBlur:me(a.onBlur,()=>q(!1))})})}),qg="RovingFocusGroupItem",Gg=x.forwardRef((a,o)=>{const{__scopeRovingFocusGroup:i,focusable:c=!0,active:u=!1,tabStopId:f,children:h,...v}=a,b=fi(),g=f||b,p=qS(qg,i),S=p.currentTabStopId===g,w=Hg(i),{onFocusableItemAdd:M,onFocusableItemRemove:T,currentTabStopId:N}=p;return x.useEffect(()=>{if(c)return M(),()=>T()},[c,M,T]),d.jsx(wu.ItemSlot,{scope:i,id:g,focusable:c,active:u,children:d.jsx(ke.span,{tabIndex:S?0:-1,"data-orientation":p.orientation,...v,ref:o,onMouseDown:me(a.onMouseDown,_=>{c?p.onItemFocus(g):_.preventDefault()}),onFocus:me(a.onFocus,()=>p.onItemFocus(g)),onKeyDown:me(a.onKeyDown,_=>{if(_.key==="Tab"&&_.shiftKey){p.onItemShiftTab();return}if(_.target!==_.currentTarget)return;const j=XS(_,p.orientation,p.dir);if(j!==void 0){if(_.metaKey||_.ctrlKey||_.altKey||_.shiftKey)return;_.preventDefault();let L=w().filter(Y=>Y.focusable).map(Y=>Y.ref.current);if(j==="last")L.reverse();else if(j==="prev"||j==="next"){j==="prev"&&L.reverse();const Y=L.indexOf(_.currentTarget);L=p.loop?QS(L,Y+1):L.slice(Y+1)}setTimeout(()=>Yg(L))}}),children:typeof h=="function"?h({isCurrentTabStop:S,hasTabStop:N!=null}):h})})});Gg.displayName=qg;var YS={ArrowLeft:"prev",ArrowUp:"prev",ArrowRight:"next",ArrowDown:"next",PageUp:"first",Home:"first",PageDown:"last",End:"last"};function VS(a,o){return o!=="rtl"?a:a==="ArrowLeft"?"ArrowRight":a==="ArrowRight"?"ArrowLeft":a}function XS(a,o,i){const c=VS(a.key,i);if(!(o==="vertical"&&["ArrowLeft","ArrowRight"].includes(c))&&!(o==="horizontal"&&["ArrowUp","ArrowDown"].includes(c)))return YS[c]}function Yg(a,o=!1){const i=document.activeElement;for(const c of a)if(c===i||(c.focus({preventScroll:o}),document.activeElement!==i))return}function QS(a,o){return a.map((i,c)=>a[(o+c)%a.length])}var Vg=Bg,Xg=Gg,ZS=function(a){if(typeof document>"u")return null;var o=Array.isArray(a)?a[0]:a;return o.ownerDocument.body},ha=new WeakMap,ai=new WeakMap,ri={},cu=0,Qg=function(a){return a&&(a.host||Qg(a.parentNode))},KS=function(a,o){return o.map(function(i){if(a.contains(i))return i;var c=Qg(i);return c&&a.contains(c)?c:(console.error("aria-hidden",i,"in not contained inside",a,". Doing nothing"),null)}).filter(function(i){return!!i})},PS=function(a,o,i,c){var u=KS(o,Array.isArray(a)?a:[a]);ri[i]||(ri[i]=new WeakMap);var f=ri[i],h=[],v=new Set,b=new Set(u),g=function(S){!S||v.has(S)||(v.add(S),g(S.parentNode))};u.forEach(g);var p=function(S){!S||b.has(S)||Array.prototype.forEach.call(S.children,function(w){if(v.has(w))p(w);else try{var M=w.getAttribute(c),T=M!==null&&M!=="false",N=(ha.get(w)||0)+1,_=(f.get(w)||0)+1;ha.set(w,N),f.set(w,_),h.push(w),N===1&&T&&ai.set(w,!0),_===1&&w.setAttribute(i,"true"),T||w.setAttribute(c,"true")}catch(j){console.error("aria-hidden: cannot operate on ",w,j)}})};return p(o),v.clear(),cu++,function(){h.forEach(function(S){var w=ha.get(S)-1,M=f.get(S)-1;ha.set(S,w),f.set(S,M),w||(ai.has(S)||S.removeAttribute(c),ai.delete(S)),M||S.removeAttribute(i)}),cu--,cu||(ha=new WeakMap,ha=new WeakMap,ai=new WeakMap,ri={})}},$S=function(a,o,i){i===void 0&&(i="data-aria-hidden");var c=Array.from(Array.isArray(a)?a:[a]),u=ZS(a);return u?(c.push.apply(c,Array.from(u.querySelectorAll("[aria-live], script"))),PS(c,u,i,"aria-hidden")):function(){return null}},en=function(){return en=Object.assign||function(o){for(var i,c=1,u=arguments.length;c<u;c++){i=arguments[c];for(var f in i)Object.prototype.hasOwnProperty.call(i,f)&&(o[f]=i[f])}return o},en.apply(this,arguments)};function Zg(a,o){var i={};for(var c in a)Object.prototype.hasOwnProperty.call(a,c)&&o.indexOf(c)<0&&(i[c]=a[c]);if(a!=null&&typeof Object.getOwnPropertySymbols=="function")for(var u=0,c=Object.getOwnPropertySymbols(a);u<c.length;u++)o.indexOf(c[u])<0&&Object.prototype.propertyIsEnumerable.call(a,c[u])&&(i[c[u]]=a[c[u]]);return i}function JS(a,o,i){if(i||arguments.length===2)for(var c=0,u=o.length,f;c<u;c++)(f||!(c in o))&&(f||(f=Array.prototype.slice.call(o,0,c)),f[c]=o[c]);return a.concat(f||Array.prototype.slice.call(o))}var si="right-scroll-bar-position",ui="width-before-scroll-bar",WS="with-scroll-bars-hidden",FS="--removed-body-scroll-bar-size";function su(a,o){return typeof a=="function"?a(o):a&&(a.current=o),a}function IS(a,o){var i=x.useState(function(){return{value:a,callback:o,facade:{get current(){return i.value},set current(c){var u=i.value;u!==c&&(i.value=c,i.callback(c,u))}}}})[0];return i.callback=o,i.facade}var ew=typeof window<"u"?x.useLayoutEffect:x.useEffect,Ov=new WeakMap;function tw(a,o){var i=IS(null,function(c){return a.forEach(function(u){return su(u,c)})});return ew(function(){var c=Ov.get(i);if(c){var u=new Set(c),f=new Set(a),h=i.current;u.forEach(function(v){f.has(v)||su(v,null)}),f.forEach(function(v){u.has(v)||su(v,h)})}Ov.set(i,a)},[a]),i}function nw(a){return a}function lw(a,o){o===void 0&&(o=nw);var i=[],c=!1,u={read:function(){if(c)throw new Error("Sidecar: could not `read` from an `assigned` medium. `read` could be used only with `useMedium`.");return i.length?i[i.length-1]:a},useMedium:function(f){var h=o(f,c);return i.push(h),function(){i=i.filter(function(v){return v!==h})}},assignSyncMedium:function(f){for(c=!0;i.length;){var h=i;i=[],h.forEach(f)}i={push:function(v){return f(v)},filter:function(){return i}}},assignMedium:function(f){c=!0;var h=[];if(i.length){var v=i;i=[],v.forEach(f),h=i}var b=function(){var p=h;h=[],p.forEach(f)},g=function(){return Promise.resolve().then(b)};g(),i={push:function(p){h.push(p),g()},filter:function(p){return h=h.filter(p),i}}}};return u}function aw(a){a===void 0&&(a={});var o=lw(null);return o.options=en({async:!0,ssr:!1},a),o}var Kg=function(a){var o=a.sideCar,i=Zg(a,["sideCar"]);if(!o)throw new Error("Sidecar: please provide `sideCar` property to import the right car");var c=o.read();if(!c)throw new Error("Sidecar medium not found");return x.createElement(c,en({},i))};Kg.isSideCarExport=!0;function rw(a,o){return a.useMedium(o),Kg}var Pg=aw(),uu=function(){},Ri=x.forwardRef(function(a,o){var i=x.useRef(null),c=x.useState({onScrollCapture:uu,onWheelCapture:uu,onTouchMoveCapture:uu}),u=c[0],f=c[1],h=a.forwardProps,v=a.children,b=a.className,g=a.removeScrollBar,p=a.enabled,S=a.shards,w=a.sideCar,M=a.noRelative,T=a.noIsolation,N=a.inert,_=a.allowPinchZoom,j=a.as,q=j===void 0?"div":j,L=a.gapMode,Y=Zg(a,["forwardProps","children","className","removeScrollBar","enabled","shards","sideCar","noRelative","noIsolation","inert","allowPinchZoom","as","gapMode"]),B=w,J=tw([i,o]),P=en(en({},Y),u);return x.createElement(x.Fragment,null,p&&x.createElement(B,{sideCar:Pg,removeScrollBar:g,shards:S,noRelative:M,noIsolation:T,inert:N,setCallbacks:f,allowPinchZoom:!!_,lockRef:i,gapMode:L}),h?x.cloneElement(x.Children.only(v),en(en({},P),{ref:J})):x.createElement(q,en({},P,{className:b,ref:J}),v))});Ri.defaultProps={enabled:!0,removeScrollBar:!0,inert:!1};Ri.classNames={fullWidth:ui,zeroRight:si};var ow=function(){if(typeof __webpack_nonce__<"u")return __webpack_nonce__};function iw(){if(!document)return null;var a=document.createElement("style");a.type="text/css";var o=ow();return o&&a.setAttribute("nonce",o),a}function cw(a,o){a.styleSheet?a.styleSheet.cssText=o:a.appendChild(document.createTextNode(o))}function sw(a){var o=document.head||document.getElementsByTagName("head")[0];o.appendChild(a)}var uw=function(){var a=0,o=null;return{add:function(i){a==0&&(o=iw())&&(cw(o,i),sw(o)),a++},remove:function(){a--,!a&&o&&(o.parentNode&&o.parentNode.removeChild(o),o=null)}}},fw=function(){var a=uw();return function(o,i){x.useEffect(function(){return a.add(o),function(){a.remove()}},[o&&i])}},$g=function(){var a=fw(),o=function(i){var c=i.styles,u=i.dynamic;return a(c,u),null};return o},dw={left:0,top:0,right:0,gap:0},fu=function(a){return parseInt(a||"",10)||0},mw=function(a){var o=window.getComputedStyle(document.body),i=o[a==="padding"?"paddingLeft":"marginLeft"],c=o[a==="padding"?"paddingTop":"marginTop"],u=o[a==="padding"?"paddingRight":"marginRight"];return[fu(i),fu(c),fu(u)]},hw=function(a){if(a===void 0&&(a="margin"),typeof window>"u")return dw;var o=mw(a),i=document.documentElement.clientWidth,c=window.innerWidth;return{left:o[0],top:o[1],right:o[2],gap:Math.max(0,c-i+o[2]-o[0])}},vw=$g(),ya="data-scroll-locked",gw=function(a,o,i,c){var u=a.left,f=a.top,h=a.right,v=a.gap;return i===void 0&&(i="margin"),`
  .`.concat(WS,` {
   overflow: hidden `).concat(c,`;
   padding-right: `).concat(v,"px ").concat(c,`;
  }
  body[`).concat(ya,`] {
    overflow: hidden `).concat(c,`;
    overscroll-behavior: contain;
    `).concat([o&&"position: relative ".concat(c,";"),i==="margin"&&`
    padding-left: `.concat(u,`px;
    padding-top: `).concat(f,`px;
    padding-right: `).concat(h,`px;
    margin-left:0;
    margin-top:0;
    margin-right: `).concat(v,"px ").concat(c,`;
    `),i==="padding"&&"padding-right: ".concat(v,"px ").concat(c,";")].filter(Boolean).join(""),`
  }
  
  .`).concat(si,` {
    right: `).concat(v,"px ").concat(c,`;
  }
  
  .`).concat(ui,` {
    margin-right: `).concat(v,"px ").concat(c,`;
  }
  
  .`).concat(si," .").concat(si,` {
    right: 0 `).concat(c,`;
  }
  
  .`).concat(ui," .").concat(ui,` {
    margin-right: 0 `).concat(c,`;
  }
  
  body[`).concat(ya,`] {
    `).concat(FS,": ").concat(v,`px;
  }
`)},Dv=function(){var a=parseInt(document.body.getAttribute(ya)||"0",10);return isFinite(a)?a:0},pw=function(){x.useEffect(function(){return document.body.setAttribute(ya,(Dv()+1).toString()),function(){var a=Dv()-1;a<=0?document.body.removeAttribute(ya):document.body.setAttribute(ya,a.toString())}},[])},yw=function(a){var o=a.noRelative,i=a.noImportant,c=a.gapMode,u=c===void 0?"margin":c;pw();var f=x.useMemo(function(){return hw(u)},[u]);return x.createElement(vw,{styles:gw(f,!o,u,i?"":"!important")})},Eu=!1;if(typeof window<"u")try{var oi=Object.defineProperty({},"passive",{get:function(){return Eu=!0,!0}});window.addEventListener("test",oi,oi),window.removeEventListener("test",oi,oi)}catch{Eu=!1}var va=Eu?{passive:!1}:!1,bw=function(a){return a.tagName==="TEXTAREA"},Jg=function(a,o){if(!(a instanceof Element))return!1;var i=window.getComputedStyle(a);return i[o]!=="hidden"&&!(i.overflowY===i.overflowX&&!bw(a)&&i[o]==="visible")},xw=function(a){return Jg(a,"overflowY")},Sw=function(a){return Jg(a,"overflowX")},zv=function(a,o){var i=o.ownerDocument,c=o;do{typeof ShadowRoot<"u"&&c instanceof ShadowRoot&&(c=c.host);var u=Wg(a,c);if(u){var f=Fg(a,c),h=f[1],v=f[2];if(h>v)return!0}c=c.parentNode}while(c&&c!==i.body);return!1},ww=function(a){var o=a.scrollTop,i=a.scrollHeight,c=a.clientHeight;return[o,i,c]},Ew=function(a){var o=a.scrollLeft,i=a.scrollWidth,c=a.clientWidth;return[o,i,c]},Wg=function(a,o){return a==="v"?xw(o):Sw(o)},Fg=function(a,o){return a==="v"?ww(o):Ew(o)},Aw=function(a,o){return a==="h"&&o==="rtl"?-1:1},Mw=function(a,o,i,c,u){var f=Aw(a,window.getComputedStyle(o).direction),h=f*c,v=i.target,b=o.contains(v),g=!1,p=h>0,S=0,w=0;do{if(!v)break;var M=Fg(a,v),T=M[0],N=M[1],_=M[2],j=N-_-f*T;(T||j)&&Wg(a,v)&&(S+=j,w+=T);var q=v.parentNode;v=q&&q.nodeType===Node.DOCUMENT_FRAGMENT_NODE?q.host:q}while(!b&&v!==document.body||b&&(o.contains(v)||o===v));return(p&&Math.abs(S)<1||!p&&Math.abs(w)<1)&&(g=!0),g},ii=function(a){return"changedTouches"in a?[a.changedTouches[0].clientX,a.changedTouches[0].clientY]:[0,0]},Uv=function(a){return[a.deltaX,a.deltaY]},kv=function(a){return a&&"current"in a?a.current:a},Nw=function(a,o){return a[0]===o[0]&&a[1]===o[1]},Tw=function(a){return`
  .block-interactivity-`.concat(a,` {pointer-events: none;}
  .allow-interactivity-`).concat(a,` {pointer-events: all;}
`)},Rw=0,ga=[];function _w(a){var o=x.useRef([]),i=x.useRef([0,0]),c=x.useRef(),u=x.useState(Rw++)[0],f=x.useState($g)[0],h=x.useRef(a);x.useEffect(function(){h.current=a},[a]),x.useEffect(function(){if(a.inert){document.body.classList.add("block-interactivity-".concat(u));var N=JS([a.lockRef.current],(a.shards||[]).map(kv),!0).filter(Boolean);return N.forEach(function(_){return _.classList.add("allow-interactivity-".concat(u))}),function(){document.body.classList.remove("block-interactivity-".concat(u)),N.forEach(function(_){return _.classList.remove("allow-interactivity-".concat(u))})}}},[a.inert,a.lockRef.current,a.shards]);var v=x.useCallback(function(N,_){if("touches"in N&&N.touches.length===2||N.type==="wheel"&&N.ctrlKey)return!h.current.allowPinchZoom;var j=ii(N),q=i.current,L="deltaX"in N?N.deltaX:q[0]-j[0],Y="deltaY"in N?N.deltaY:q[1]-j[1],B,J=N.target,P=Math.abs(L)>Math.abs(Y)?"h":"v";if("touches"in N&&P==="h"&&J.type==="range")return!1;var Z=zv(P,J);if(!Z)return!0;if(Z?B=P:(B=P==="v"?"h":"v",Z=zv(P,J)),!Z)return!1;if(!c.current&&"changedTouches"in N&&(L||Y)&&(c.current=B),!B)return!0;var I=c.current||B;return Mw(I,_,N,I==="h"?L:Y)},[]),b=x.useCallback(function(N){var _=N;if(!(!ga.length||ga[ga.length-1]!==f)){var j="deltaY"in _?Uv(_):ii(_),q=o.current.filter(function(B){return B.name===_.type&&(B.target===_.target||_.target===B.shadowParent)&&Nw(B.delta,j)})[0];if(q&&q.should){_.cancelable&&_.preventDefault();return}if(!q){var L=(h.current.shards||[]).map(kv).filter(Boolean).filter(function(B){return B.contains(_.target)}),Y=L.length>0?v(_,L[0]):!h.current.noIsolation;Y&&_.cancelable&&_.preventDefault()}}},[]),g=x.useCallback(function(N,_,j,q){var L={name:N,delta:_,target:j,should:q,shadowParent:Cw(j)};o.current.push(L),setTimeout(function(){o.current=o.current.filter(function(Y){return Y!==L})},1)},[]),p=x.useCallback(function(N){i.current=ii(N),c.current=void 0},[]),S=x.useCallback(function(N){g(N.type,Uv(N),N.target,v(N,a.lockRef.current))},[]),w=x.useCallback(function(N){g(N.type,ii(N),N.target,v(N,a.lockRef.current))},[]);x.useEffect(function(){return ga.push(f),a.setCallbacks({onScrollCapture:S,onWheelCapture:S,onTouchMoveCapture:w}),document.addEventListener("wheel",b,va),document.addEventListener("touchmove",b,va),document.addEventListener("touchstart",p,va),function(){ga=ga.filter(function(N){return N!==f}),document.removeEventListener("wheel",b,va),document.removeEventListener("touchmove",b,va),document.removeEventListener("touchstart",p,va)}},[]);var M=a.removeScrollBar,T=a.inert;return x.createElement(x.Fragment,null,T?x.createElement(f,{styles:Tw(u)}):null,M?x.createElement(yw,{noRelative:a.noRelative,gapMode:a.gapMode}):null)}function Cw(a){for(var o=null;a!==null;)a instanceof ShadowRoot&&(o=a.host,a=a.host),a=a.parentNode;return o}const jw=rw(Pg,_w);var Ig=x.forwardRef(function(a,o){return x.createElement(Ri,en({},a,{ref:o,sideCar:jw}))});Ig.classNames=Ri.classNames;var Au=["Enter"," "],Ow=["ArrowDown","PageUp","Home"],ep=["ArrowUp","PageDown","End"],Dw=[...Ow,...ep],zw={ltr:[...Au,"ArrowRight"],rtl:[...Au,"ArrowLeft"]},Uw={ltr:["ArrowLeft"],rtl:["ArrowRight"]},Lr="Menu",[Dr,kw,Lw]=sg(Lr),[Nl,tp]=Ml(Lr,[Lw,Rg,Ti]),_i=Rg(),np=Ti(),[Hw,Tl]=Nl(Lr),[Bw,Hr]=Nl(Lr),lp=a=>{const{__scopeMenu:o,open:i=!1,children:c,dir:u,onOpenChange:f,modal:h=!0}=a,v=_i(o),[b,g]=x.useState(null),p=x.useRef(!1),S=ot(f),w=wi(u);return x.useEffect(()=>{const M=()=>{p.current=!0,document.addEventListener("pointerdown",T,{capture:!0,once:!0}),document.addEventListener("pointermove",T,{capture:!0,once:!0})},T=()=>p.current=!1;return document.addEventListener("keydown",M,{capture:!0}),()=>{document.removeEventListener("keydown",M,{capture:!0}),document.removeEventListener("pointerdown",T,{capture:!0}),document.removeEventListener("pointermove",T,{capture:!0})}},[]),d.jsx(RS,{...v,children:d.jsx(Hw,{scope:o,open:i,onOpenChange:S,content:b,onContentChange:g,children:d.jsx(Bw,{scope:o,onClose:x.useCallback(()=>S(!1),[S]),isUsingKeyboardRef:p,dir:w,modal:h,children:c})})})};lp.displayName=Lr;var qw="MenuAnchor",Vu=x.forwardRef((a,o)=>{const{__scopeMenu:i,...c}=a,u=_i(i);return d.jsx(_S,{...u,...c,ref:o})});Vu.displayName=qw;var Xu="MenuPortal",[Gw,ap]=Nl(Xu,{forceMount:void 0}),rp=a=>{const{__scopeMenu:o,forceMount:i,children:c,container:u}=a,f=Tl(Xu,o);return d.jsx(Gw,{scope:o,forceMount:i,children:d.jsx(rn,{present:i||f.open,children:d.jsx(Lg,{asChild:!0,container:u,children:c})})})};rp.displayName=Xu;var Ht="MenuContent",[Yw,Qu]=Nl(Ht),op=x.forwardRef((a,o)=>{const i=ap(Ht,a.__scopeMenu),{forceMount:c=i.forceMount,...u}=a,f=Tl(Ht,a.__scopeMenu),h=Hr(Ht,a.__scopeMenu);return d.jsx(Dr.Provider,{scope:a.__scopeMenu,children:d.jsx(rn,{present:c||f.open,children:d.jsx(Dr.Slot,{scope:a.__scopeMenu,children:h.modal?d.jsx(Vw,{...u,ref:o}):d.jsx(Xw,{...u,ref:o})})})})}),Vw=x.forwardRef((a,o)=>{const i=Tl(Ht,a.__scopeMenu),c=x.useRef(null),u=Qe(o,c);return x.useEffect(()=>{const f=c.current;if(f)return $S(f)},[]),d.jsx(Zu,{...a,ref:u,trapFocus:i.open,disableOutsidePointerEvents:i.open,disableOutsideScroll:!0,onFocusOutside:me(a.onFocusOutside,f=>f.preventDefault(),{checkForDefaultPrevented:!1}),onDismiss:()=>i.onOpenChange(!1)})}),Xw=x.forwardRef((a,o)=>{const i=Tl(Ht,a.__scopeMenu);return d.jsx(Zu,{...a,ref:o,trapFocus:!1,disableOutsidePointerEvents:!1,disableOutsideScroll:!1,onDismiss:()=>i.onOpenChange(!1)})}),Qw=Cr("MenuContent.ScrollLock"),Zu=x.forwardRef((a,o)=>{const{__scopeMenu:i,loop:c=!1,trapFocus:u,onOpenAutoFocus:f,onCloseAutoFocus:h,disableOutsidePointerEvents:v,onEntryFocus:b,onEscapeKeyDown:g,onPointerDownOutside:p,onFocusOutside:S,onInteractOutside:w,onDismiss:M,disableOutsideScroll:T,...N}=a,_=Tl(Ht,i),j=Hr(Ht,i),q=_i(i),L=np(i),Y=kw(i),[B,J]=x.useState(null),P=x.useRef(null),Z=Qe(o,P,_.onContentChange),I=x.useRef(0),re=x.useRef(""),ue=x.useRef(0),de=x.useRef(null),Ee=x.useRef("right"),be=x.useRef(0),ve=T?Ig:x.Fragment,D=T?{as:Qw,allowPinchZoom:!0}:void 0,K=W=>{const A=re.current+W,X=Y().filter(ie=>!ie.disabled),F=document.activeElement,$=X.find(ie=>ie.ref.current===F)?.textValue,ee=X.map(ie=>ie.textValue),he=l2(ee,A,$),oe=X.find(ie=>ie.textValue===he)?.ref.current;(function ie(Me){re.current=Me,window.clearTimeout(I.current),Me!==""&&(I.current=window.setTimeout(()=>ie(""),1e3))})(A),oe&&setTimeout(()=>oe.focus())};x.useEffect(()=>()=>window.clearTimeout(I.current),[]),Ix();const H=x.useCallback(W=>Ee.current===de.current?.side&&r2(W,de.current?.area),[]);return d.jsx(Yw,{scope:i,searchRef:re,onItemEnter:x.useCallback(W=>{H(W)&&W.preventDefault()},[H]),onItemLeave:x.useCallback(W=>{H(W)||(P.current?.focus(),J(null))},[H]),onTriggerLeave:x.useCallback(W=>{H(W)&&W.preventDefault()},[H]),pointerGraceTimerRef:ue,onPointerGraceIntentChange:x.useCallback(W=>{de.current=W},[]),children:d.jsx(ve,{...D,children:d.jsx(mg,{asChild:!0,trapped:u,onMountAutoFocus:me(f,W=>{W.preventDefault(),P.current?.focus({preventScroll:!0})}),onUnmountAutoFocus:h,children:d.jsx(fg,{asChild:!0,disableOutsidePointerEvents:v,onEscapeKeyDown:g,onPointerDownOutside:p,onFocusOutside:S,onInteractOutside:w,onDismiss:M,children:d.jsx(Vg,{asChild:!0,...L,dir:j.dir,orientation:"vertical",loop:c,currentTabStopId:B,onCurrentTabStopIdChange:J,onEntryFocus:me(b,W=>{j.isUsingKeyboardRef.current||W.preventDefault()}),preventScrollOnEntryFocus:!0,children:d.jsx(CS,{role:"menu","aria-orientation":"vertical","data-state":wp(_.open),"data-radix-menu-content":"",dir:j.dir,...q,...N,ref:Z,style:{outline:"none",...N.style},onKeyDown:me(N.onKeyDown,W=>{const X=W.target.closest("[data-radix-menu-content]")===W.currentTarget,F=W.ctrlKey||W.altKey||W.metaKey,$=W.key.length===1;X&&(W.key==="Tab"&&W.preventDefault(),!F&&$&&K(W.key));const ee=P.current;if(W.target!==ee||!Dw.includes(W.key))return;W.preventDefault();const oe=Y().filter(ie=>!ie.disabled).map(ie=>ie.ref.current);ep.includes(W.key)&&oe.reverse(),t2(oe)}),onBlur:me(a.onBlur,W=>{W.currentTarget.contains(W.target)||(window.clearTimeout(I.current),re.current="")}),onPointerMove:me(a.onPointerMove,zr(W=>{const A=W.target,X=be.current!==W.clientX;if(W.currentTarget.contains(A)&&X){const F=W.clientX>be.current?"right":"left";Ee.current=F,be.current=W.clientX}}))})})})})})})});op.displayName=Ht;var Zw="MenuGroup",Ku=x.forwardRef((a,o)=>{const{__scopeMenu:i,...c}=a;return d.jsx(ke.div,{role:"group",...c,ref:o})});Ku.displayName=Zw;var Kw="MenuLabel",ip=x.forwardRef((a,o)=>{const{__scopeMenu:i,...c}=a;return d.jsx(ke.div,{...c,ref:o})});ip.displayName=Kw;var gi="MenuItem",Lv="menu.itemSelect",Ci=x.forwardRef((a,o)=>{const{disabled:i=!1,onSelect:c,...u}=a,f=x.useRef(null),h=Hr(gi,a.__scopeMenu),v=Qu(gi,a.__scopeMenu),b=Qe(o,f),g=x.useRef(!1),p=()=>{const S=f.current;if(!i&&S){const w=new CustomEvent(Lv,{bubbles:!0,cancelable:!0});S.addEventListener(Lv,M=>c?.(M),{once:!0}),cg(S,w),w.defaultPrevented?g.current=!1:h.onClose()}};return d.jsx(cp,{...u,ref:b,disabled:i,onClick:me(a.onClick,p),onPointerDown:S=>{a.onPointerDown?.(S),g.current=!0},onPointerUp:me(a.onPointerUp,S=>{g.current||S.currentTarget?.click()}),onKeyDown:me(a.onKeyDown,S=>{const w=v.searchRef.current!=="";i||w&&S.key===" "||Au.includes(S.key)&&(S.currentTarget.click(),S.preventDefault())})})});Ci.displayName=gi;var cp=x.forwardRef((a,o)=>{const{__scopeMenu:i,disabled:c=!1,textValue:u,...f}=a,h=Qu(gi,i),v=np(i),b=x.useRef(null),g=Qe(o,b),[p,S]=x.useState(!1),[w,M]=x.useState("");return x.useEffect(()=>{const T=b.current;T&&M((T.textContent??"").trim())},[f.children]),d.jsx(Dr.ItemSlot,{scope:i,disabled:c,textValue:u??w,children:d.jsx(Xg,{asChild:!0,...v,focusable:!c,children:d.jsx(ke.div,{role:"menuitem","data-highlighted":p?"":void 0,"aria-disabled":c||void 0,"data-disabled":c?"":void 0,...f,ref:g,onPointerMove:me(a.onPointerMove,zr(T=>{c?h.onItemLeave(T):(h.onItemEnter(T),T.defaultPrevented||T.currentTarget.focus({preventScroll:!0}))})),onPointerLeave:me(a.onPointerLeave,zr(T=>h.onItemLeave(T))),onFocus:me(a.onFocus,()=>S(!0)),onBlur:me(a.onBlur,()=>S(!1))})})})}),Pw="MenuCheckboxItem",sp=x.forwardRef((a,o)=>{const{checked:i=!1,onCheckedChange:c,...u}=a;return d.jsx(hp,{scope:a.__scopeMenu,checked:i,children:d.jsx(Ci,{role:"menuitemcheckbox","aria-checked":pi(i)?"mixed":i,...u,ref:o,"data-state":$u(i),onSelect:me(u.onSelect,()=>c?.(pi(i)?!0:!i),{checkForDefaultPrevented:!1})})})});sp.displayName=Pw;var up="MenuRadioGroup",[$w,Jw]=Nl(up,{value:void 0,onValueChange:()=>{}}),fp=x.forwardRef((a,o)=>{const{value:i,onValueChange:c,...u}=a,f=ot(c);return d.jsx($w,{scope:a.__scopeMenu,value:i,onValueChange:f,children:d.jsx(Ku,{...u,ref:o})})});fp.displayName=up;var dp="MenuRadioItem",mp=x.forwardRef((a,o)=>{const{value:i,...c}=a,u=Jw(dp,a.__scopeMenu),f=i===u.value;return d.jsx(hp,{scope:a.__scopeMenu,checked:f,children:d.jsx(Ci,{role:"menuitemradio","aria-checked":f,...c,ref:o,"data-state":$u(f),onSelect:me(c.onSelect,()=>u.onValueChange?.(i),{checkForDefaultPrevented:!1})})})});mp.displayName=dp;var Pu="MenuItemIndicator",[hp,Ww]=Nl(Pu,{checked:!1}),vp=x.forwardRef((a,o)=>{const{__scopeMenu:i,forceMount:c,...u}=a,f=Ww(Pu,i);return d.jsx(rn,{present:c||pi(f.checked)||f.checked===!0,children:d.jsx(ke.span,{...u,ref:o,"data-state":$u(f.checked)})})});vp.displayName=Pu;var Fw="MenuSeparator",gp=x.forwardRef((a,o)=>{const{__scopeMenu:i,...c}=a;return d.jsx(ke.div,{role:"separator","aria-orientation":"horizontal",...c,ref:o})});gp.displayName=Fw;var Iw="MenuArrow",pp=x.forwardRef((a,o)=>{const{__scopeMenu:i,...c}=a,u=_i(i);return d.jsx(jS,{...u,...c,ref:o})});pp.displayName=Iw;var e2="MenuSub",[HA,yp]=Nl(e2),Nr="MenuSubTrigger",bp=x.forwardRef((a,o)=>{const i=Tl(Nr,a.__scopeMenu),c=Hr(Nr,a.__scopeMenu),u=yp(Nr,a.__scopeMenu),f=Qu(Nr,a.__scopeMenu),h=x.useRef(null),{pointerGraceTimerRef:v,onPointerGraceIntentChange:b}=f,g={__scopeMenu:a.__scopeMenu},p=x.useCallback(()=>{h.current&&window.clearTimeout(h.current),h.current=null},[]);return x.useEffect(()=>p,[p]),x.useEffect(()=>{const S=v.current;return()=>{window.clearTimeout(S),b(null)}},[v,b]),d.jsx(Vu,{asChild:!0,...g,children:d.jsx(cp,{id:u.triggerId,"aria-haspopup":"menu","aria-expanded":i.open,"aria-controls":u.contentId,"data-state":wp(i.open),...a,ref:Si(o,u.onTriggerChange),onClick:S=>{a.onClick?.(S),!(a.disabled||S.defaultPrevented)&&(S.currentTarget.focus(),i.open||i.onOpenChange(!0))},onPointerMove:me(a.onPointerMove,zr(S=>{f.onItemEnter(S),!S.defaultPrevented&&!a.disabled&&!i.open&&!h.current&&(f.onPointerGraceIntentChange(null),h.current=window.setTimeout(()=>{i.onOpenChange(!0),p()},100))})),onPointerLeave:me(a.onPointerLeave,zr(S=>{p();const w=i.content?.getBoundingClientRect();if(w){const M=i.content?.dataset.side,T=M==="right",N=T?-5:5,_=w[T?"left":"right"],j=w[T?"right":"left"];f.onPointerGraceIntentChange({area:[{x:S.clientX+N,y:S.clientY},{x:_,y:w.top},{x:j,y:w.top},{x:j,y:w.bottom},{x:_,y:w.bottom}],side:M}),window.clearTimeout(v.current),v.current=window.setTimeout(()=>f.onPointerGraceIntentChange(null),300)}else{if(f.onTriggerLeave(S),S.defaultPrevented)return;f.onPointerGraceIntentChange(null)}})),onKeyDown:me(a.onKeyDown,S=>{const w=f.searchRef.current!=="";a.disabled||w&&S.key===" "||zw[c.dir].includes(S.key)&&(i.onOpenChange(!0),i.content?.focus(),S.preventDefault())})})})});bp.displayName=Nr;var xp="MenuSubContent",Sp=x.forwardRef((a,o)=>{const i=ap(Ht,a.__scopeMenu),{forceMount:c=i.forceMount,...u}=a,f=Tl(Ht,a.__scopeMenu),h=Hr(Ht,a.__scopeMenu),v=yp(xp,a.__scopeMenu),b=x.useRef(null),g=Qe(o,b);return d.jsx(Dr.Provider,{scope:a.__scopeMenu,children:d.jsx(rn,{present:c||f.open,children:d.jsx(Dr.Slot,{scope:a.__scopeMenu,children:d.jsx(Zu,{id:v.contentId,"aria-labelledby":v.triggerId,...u,ref:g,align:"start",side:h.dir==="rtl"?"left":"right",disableOutsidePointerEvents:!1,disableOutsideScroll:!1,trapFocus:!1,onOpenAutoFocus:p=>{h.isUsingKeyboardRef.current&&b.current?.focus(),p.preventDefault()},onCloseAutoFocus:p=>p.preventDefault(),onFocusOutside:me(a.onFocusOutside,p=>{p.target!==v.trigger&&f.onOpenChange(!1)}),onEscapeKeyDown:me(a.onEscapeKeyDown,p=>{h.onClose(),p.preventDefault()}),onKeyDown:me(a.onKeyDown,p=>{const S=p.currentTarget.contains(p.target),w=Uw[h.dir].includes(p.key);S&&w&&(f.onOpenChange(!1),v.trigger?.focus(),p.preventDefault())})})})})})});Sp.displayName=xp;function wp(a){return a?"open":"closed"}function pi(a){return a==="indeterminate"}function $u(a){return pi(a)?"indeterminate":a?"checked":"unchecked"}function t2(a){const o=document.activeElement;for(const i of a)if(i===o||(i.focus(),document.activeElement!==o))return}function n2(a,o){return a.map((i,c)=>a[(o+c)%a.length])}function l2(a,o,i){const u=o.length>1&&Array.from(o).every(g=>g===o[0])?o[0]:o,f=i?a.indexOf(i):-1;let h=n2(a,Math.max(f,0));u.length===1&&(h=h.filter(g=>g!==i));const b=h.find(g=>g.toLowerCase().startsWith(u.toLowerCase()));return b!==i?b:void 0}function a2(a,o){const{x:i,y:c}=a;let u=!1;for(let f=0,h=o.length-1;f<o.length;h=f++){const v=o[f],b=o[h],g=v.x,p=v.y,S=b.x,w=b.y;p>c!=w>c&&i<(S-g)*(c-p)/(w-p)+g&&(u=!u)}return u}function r2(a,o){if(!o)return!1;const i={x:a.clientX,y:a.clientY};return a2(i,o)}function zr(a){return o=>o.pointerType==="mouse"?a(o):void 0}var o2=lp,i2=Vu,c2=rp,s2=op,u2=Ku,f2=ip,d2=Ci,m2=sp,h2=fp,v2=mp,g2=vp,p2=gp,y2=pp,b2=bp,x2=Sp,ji="DropdownMenu",[S2,BA]=Ml(ji,[tp]),st=tp(),[w2,Ep]=S2(ji),Ap=a=>{const{__scopeDropdownMenu:o,children:i,dir:c,open:u,defaultOpen:f,onOpenChange:h,modal:v=!0}=a,b=st(o),g=x.useRef(null),[p,S]=Du({prop:u,defaultProp:f??!1,onChange:h,caller:ji});return d.jsx(w2,{scope:o,triggerId:fi(),triggerRef:g,contentId:fi(),open:p,onOpenChange:S,onOpenToggle:x.useCallback(()=>S(w=>!w),[S]),modal:v,children:d.jsx(o2,{...b,open:p,onOpenChange:S,dir:c,modal:v,children:i})})};Ap.displayName=ji;var Mp="DropdownMenuTrigger",Np=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,disabled:c=!1,...u}=a,f=Ep(Mp,i),h=st(i);return d.jsx(i2,{asChild:!0,...h,children:d.jsx(ke.button,{type:"button",id:f.triggerId,"aria-haspopup":"menu","aria-expanded":f.open,"aria-controls":f.open?f.contentId:void 0,"data-state":f.open?"open":"closed","data-disabled":c?"":void 0,disabled:c,...u,ref:Si(o,f.triggerRef),onPointerDown:me(a.onPointerDown,v=>{!c&&v.button===0&&v.ctrlKey===!1&&(f.onOpenToggle(),f.open||v.preventDefault())}),onKeyDown:me(a.onKeyDown,v=>{c||(["Enter"," "].includes(v.key)&&f.onOpenToggle(),v.key==="ArrowDown"&&f.onOpenChange(!0),["Enter"," ","ArrowDown"].includes(v.key)&&v.preventDefault())})})})});Np.displayName=Mp;var E2="DropdownMenuPortal",Tp=a=>{const{__scopeDropdownMenu:o,...i}=a,c=st(o);return d.jsx(c2,{...c,...i})};Tp.displayName=E2;var Rp="DropdownMenuContent",_p=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=Ep(Rp,i),f=st(i),h=x.useRef(!1);return d.jsx(s2,{id:u.contentId,"aria-labelledby":u.triggerId,...f,...c,ref:o,onCloseAutoFocus:me(a.onCloseAutoFocus,v=>{h.current||u.triggerRef.current?.focus(),h.current=!1,v.preventDefault()}),onInteractOutside:me(a.onInteractOutside,v=>{const b=v.detail.originalEvent,g=b.button===0&&b.ctrlKey===!0,p=b.button===2||g;(!u.modal||p)&&(h.current=!0)}),style:{...a.style,"--radix-dropdown-menu-content-transform-origin":"var(--radix-popper-transform-origin)","--radix-dropdown-menu-content-available-width":"var(--radix-popper-available-width)","--radix-dropdown-menu-content-available-height":"var(--radix-popper-available-height)","--radix-dropdown-menu-trigger-width":"var(--radix-popper-anchor-width)","--radix-dropdown-menu-trigger-height":"var(--radix-popper-anchor-height)"}})});_p.displayName=Rp;var A2="DropdownMenuGroup",M2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(u2,{...u,...c,ref:o})});M2.displayName=A2;var N2="DropdownMenuLabel",Cp=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(f2,{...u,...c,ref:o})});Cp.displayName=N2;var T2="DropdownMenuItem",jp=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(d2,{...u,...c,ref:o})});jp.displayName=T2;var R2="DropdownMenuCheckboxItem",_2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(m2,{...u,...c,ref:o})});_2.displayName=R2;var C2="DropdownMenuRadioGroup",j2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(h2,{...u,...c,ref:o})});j2.displayName=C2;var O2="DropdownMenuRadioItem",D2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(v2,{...u,...c,ref:o})});D2.displayName=O2;var z2="DropdownMenuItemIndicator",U2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(g2,{...u,...c,ref:o})});U2.displayName=z2;var k2="DropdownMenuSeparator",Op=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(p2,{...u,...c,ref:o})});Op.displayName=k2;var L2="DropdownMenuArrow",H2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(y2,{...u,...c,ref:o})});H2.displayName=L2;var B2="DropdownMenuSubTrigger",q2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(b2,{...u,...c,ref:o})});q2.displayName=B2;var G2="DropdownMenuSubContent",Y2=x.forwardRef((a,o)=>{const{__scopeDropdownMenu:i,...c}=a,u=st(i);return d.jsx(x2,{...u,...c,ref:o,style:{...a.style,"--radix-dropdown-menu-content-transform-origin":"var(--radix-popper-transform-origin)","--radix-dropdown-menu-content-available-width":"var(--radix-popper-available-width)","--radix-dropdown-menu-content-available-height":"var(--radix-popper-available-height)","--radix-dropdown-menu-trigger-width":"var(--radix-popper-anchor-width)","--radix-dropdown-menu-trigger-height":"var(--radix-popper-anchor-height)"}})});Y2.displayName=G2;var V2=Ap,X2=Np,Q2=Tp,Z2=_p,K2=Cp,P2=jp,$2=Op;/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const J2=a=>a.replace(/([a-z0-9])([A-Z])/g,"$1-$2").toLowerCase(),W2=a=>a.replace(/^([A-Z])|[\s-_]+(\w)/g,(o,i,c)=>c?c.toUpperCase():i.toLowerCase()),Hv=a=>{const o=W2(a);return o.charAt(0).toUpperCase()+o.slice(1)},Dp=(...a)=>a.filter((o,i,c)=>!!o&&o.trim()!==""&&c.indexOf(o)===i).join(" ").trim(),F2=a=>{for(const o in a)if(o.startsWith("aria-")||o==="role"||o==="title")return!0};/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */var I2={xmlns:"http://www.w3.org/2000/svg",width:24,height:24,viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:2,strokeLinecap:"round",strokeLinejoin:"round"};/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const eE=x.forwardRef(({color:a="currentColor",size:o=24,strokeWidth:i=2,absoluteStrokeWidth:c,className:u="",children:f,iconNode:h,...v},b)=>x.createElement("svg",{ref:b,...I2,width:o,height:o,stroke:a,strokeWidth:c?Number(i)*24/Number(o):i,className:Dp("lucide",u),...!f&&!F2(v)&&{"aria-hidden":"true"},...v},[...h.map(([g,p])=>x.createElement(g,p)),...Array.isArray(f)?f:[f]]));/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const ze=(a,o)=>{const i=x.forwardRef(({className:c,...u},f)=>x.createElement(eE,{ref:f,iconNode:o,className:Dp(`lucide-${J2(Hv(a))}`,`lucide-${a}`,c),...u}));return i.displayName=Hv(a),i};/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const tE=[["path",{d:"M22 12h-2.48a2 2 0 0 0-1.93 1.46l-2.35 8.36a.25.25 0 0 1-.48 0L9.24 2.18a.25.25 0 0 0-.48 0l-2.35 8.36A2 2 0 0 1 4.49 12H2",key:"169zse"}]],Mu=ze("activity",tE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const nE=[["path",{d:"M5 12h14",key:"1ays0h"}],["path",{d:"m12 5 7 7-7 7",key:"xquz4c"}]],lE=ze("arrow-right",nE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const aE=[["path",{d:"M12 8V4H8",key:"hb8ula"}],["rect",{width:"16",height:"12",x:"4",y:"8",rx:"2",key:"enze0r"}],["path",{d:"M2 14h2",key:"vft8re"}],["path",{d:"M20 14h2",key:"4cs60a"}],["path",{d:"M15 13v2",key:"1xurst"}],["path",{d:"M9 13v2",key:"rq6x2g"}]],Al=ze("bot",aE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const rE=[["path",{d:"M12 18V5",key:"adv99a"}],["path",{d:"M15 13a4.17 4.17 0 0 1-3-4 4.17 4.17 0 0 1-3 4",key:"1e3is1"}],["path",{d:"M17.598 6.5A3 3 0 1 0 12 5a3 3 0 1 0-5.598 1.5",key:"1gqd8o"}],["path",{d:"M17.997 5.125a4 4 0 0 1 2.526 5.77",key:"iwvgf7"}],["path",{d:"M18 18a4 4 0 0 0 2-7.464",key:"efp6ie"}],["path",{d:"M19.967 17.483A4 4 0 1 1 12 18a4 4 0 1 1-7.967-.517",key:"1gq6am"}],["path",{d:"M6 18a4 4 0 0 1-2-7.464",key:"k1g0md"}],["path",{d:"M6.003 5.125a4 4 0 0 0-2.526 5.77",key:"q97ue3"}]],oE=ze("brain",rE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const iE=[["path",{d:"m8 2 1.88 1.88",key:"fmnt4t"}],["path",{d:"M14.12 3.88 16 2",key:"qol33r"}],["path",{d:"M9 7.13v-1a3.003 3.003 0 1 1 6 0v1",key:"d7y7pr"}],["path",{d:"M12 20c-3.3 0-6-2.7-6-6v-3a4 4 0 0 1 4-4h4a4 4 0 0 1 4 4v3c0 3.3-2.7 6-6 6",key:"xs1cw7"}],["path",{d:"M12 20v-9",key:"1qisl0"}],["path",{d:"M6.53 9C4.6 8.8 3 7.1 3 5",key:"32zzws"}],["path",{d:"M6 13H2",key:"82j7cp"}],["path",{d:"M3 21c0-2.1 1.7-3.9 3.8-4",key:"4p0ekp"}],["path",{d:"M20.97 5c0 2.1-1.6 3.8-3.5 4",key:"18gb23"}],["path",{d:"M22 13h-4",key:"1jl80f"}],["path",{d:"M17.2 17c2.1.1 3.8 1.9 3.8 4",key:"k3fwyw"}]],Bv=ze("bug",iE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const cE=[["path",{d:"m6 9 6 6 6-6",key:"qrunsl"}]],sE=ze("chevron-down",cE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const uE=[["path",{d:"m15 18-6-6 6-6",key:"1wnfg3"}]],fE=ze("chevron-left",uE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const dE=[["circle",{cx:"12",cy:"12",r:"10",key:"1mglay"}],["line",{x1:"12",x2:"12",y1:"8",y2:"12",key:"1pkeuh"}],["line",{x1:"12",x2:"12.01",y1:"16",y2:"16",key:"4dfq90"}]],Nu=ze("circle-alert",dE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const mE=[["path",{d:"M21.801 10A10 10 0 1 1 17 3.335",key:"yps3ct"}],["path",{d:"m9 11 3 3L22 4",key:"1pflzl"}]],Tu=ze("circle-check-big",mE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const hE=[["circle",{cx:"12",cy:"12",r:"10",key:"1mglay"}],["rect",{x:"9",y:"9",width:"6",height:"6",rx:"1",key:"1ssd4o"}]],vE=ze("circle-stop",hE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const gE=[["path",{d:"M12 6v6l4 2",key:"mmk7yg"}],["circle",{cx:"12",cy:"12",r:"10",key:"1mglay"}]],Ru=ze("clock",gE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const pE=[["ellipse",{cx:"12",cy:"5",rx:"9",ry:"3",key:"msslwz"}],["path",{d:"M3 5V19A9 3 0 0 0 21 19V5",key:"1wlel7"}],["path",{d:"M3 12A9 3 0 0 0 21 12",key:"mv7ke4"}]],zp=ze("database",pE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const yE=[["path",{d:"M10 12.5 8 15l2 2.5",key:"1tg20x"}],["path",{d:"m14 12.5 2 2.5-2 2.5",key:"yinavb"}],["path",{d:"M14 2v4a2 2 0 0 0 2 2h4",key:"tnqrlb"}],["path",{d:"M15 2H6a2 2 0 0 0-2 2v16a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V7z",key:"1mlx9k"}]],bE=ze("file-code",yE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const xE=[["path",{d:"m6 14 1.5-2.9A2 2 0 0 1 9.24 10H20a2 2 0 0 1 1.94 2.5l-1.54 6a2 2 0 0 1-1.95 1.5H4a2 2 0 0 1-2-2V5a2 2 0 0 1 2-2h3.9a2 2 0 0 1 1.69.9l.81 1.2a2 2 0 0 0 1.67.9H18a2 2 0 0 1 2 2v2",key:"usdka0"}]],SE=ze("folder-open",xE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const wE=[["circle",{cx:"12",cy:"12",r:"10",key:"1mglay"}],["path",{d:"M12 16v-4",key:"1dtifu"}],["path",{d:"M12 8h.01",key:"e9boi3"}]],EE=ze("info",wE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const AE=[["path",{d:"M21 12a9 9 0 1 1-6.219-8.56",key:"13zald"}]],ME=ze("loader-circle",AE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const NE=[["path",{d:"M22 17a2 2 0 0 1-2 2H6.828a2 2 0 0 0-1.414.586l-2.202 2.202A.71.71 0 0 1 2 21.286V5a2 2 0 0 1 2-2h16a2 2 0 0 1 2 2z",key:"18887p"}]],Up=ze("message-square",NE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const TE=[["path",{d:"M20.985 12.486a9 9 0 1 1-9.473-9.472c.405-.022.617.46.402.803a6 6 0 0 0 8.268 8.268c.344-.215.825-.004.803.401",key:"kfwtm"}]],RE=ze("moon",TE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const _E=[["path",{d:"M5 5a2 2 0 0 1 3.008-1.728l11.997 6.998a2 2 0 0 1 .003 3.458l-12 7A2 2 0 0 1 5 19z",key:"10ikf1"}]],CE=ze("play",_E);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const jE=[["path",{d:"M14.536 21.686a.5.5 0 0 0 .937-.024l6.5-19a.496.496 0 0 0-.635-.635l-19 6.5a.5.5 0 0 0-.024.937l7.93 3.18a2 2 0 0 1 1.112 1.11z",key:"1ffxy3"}],["path",{d:"m21.854 2.147-10.94 10.939",key:"12cjpa"}]],OE=ze("send",jE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const DE=[["path",{d:"M9.671 4.136a2.34 2.34 0 0 1 4.659 0 2.34 2.34 0 0 0 3.319 1.915 2.34 2.34 0 0 1 2.33 4.033 2.34 2.34 0 0 0 0 3.831 2.34 2.34 0 0 1-2.33 4.033 2.34 2.34 0 0 0-3.319 1.915 2.34 2.34 0 0 1-4.659 0 2.34 2.34 0 0 0-3.32-1.915 2.34 2.34 0 0 1-2.33-4.033 2.34 2.34 0 0 0 0-3.831A2.34 2.34 0 0 1 6.35 6.051a2.34 2.34 0 0 0 3.319-1.915",key:"1i5ecw"}],["circle",{cx:"12",cy:"12",r:"3",key:"1v7zrd"}]],kp=ze("settings",DE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const zE=[["circle",{cx:"12",cy:"12",r:"4",key:"4exip2"}],["path",{d:"M12 2v2",key:"tus03m"}],["path",{d:"M12 20v2",key:"1lh1kg"}],["path",{d:"m4.93 4.93 1.41 1.41",key:"149t6j"}],["path",{d:"m17.66 17.66 1.41 1.41",key:"ptbguv"}],["path",{d:"M2 12h2",key:"1t8f8n"}],["path",{d:"M20 12h2",key:"1q8mjw"}],["path",{d:"m6.34 17.66-1.41 1.41",key:"1m8zz5"}],["path",{d:"m19.07 4.93-1.41 1.41",key:"1shlcs"}]],UE=ze("sun",zE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const kE=[["path",{d:"M10 11v6",key:"nco0om"}],["path",{d:"M14 11v6",key:"outv1u"}],["path",{d:"M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6",key:"miytrc"}],["path",{d:"M3 6h18",key:"d0wm0j"}],["path",{d:"M8 6V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2",key:"e791ji"}]],LE=ze("trash-2",kE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const HE=[["path",{d:"M19 21v-2a4 4 0 0 0-4-4H9a4 4 0 0 0-4 4v2",key:"975kel"}],["circle",{cx:"12",cy:"7",r:"4",key:"17ys0d"}]],BE=ze("user",HE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const qE=[["path",{d:"M16 21v-2a4 4 0 0 0-4-4H6a4 4 0 0 0-4 4v2",key:"1yyitq"}],["path",{d:"M16 3.128a4 4 0 0 1 0 7.744",key:"16gr8j"}],["path",{d:"M22 21v-2a4 4 0 0 0-3-3.87",key:"kshegd"}],["circle",{cx:"9",cy:"7",r:"4",key:"nufk8"}]],Ju=ze("users",qE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const GE=[["rect",{width:"8",height:"8",x:"3",y:"3",rx:"2",key:"by2w9f"}],["path",{d:"M7 11v4a2 2 0 0 0 2 2h4",key:"xkn7yn"}],["rect",{width:"8",height:"8",x:"13",y:"13",rx:"2",key:"1cgmvn"}]],yi=ze("workflow",GE);/**
 * @license lucide-react v0.540.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const YE=[["path",{d:"M14.7 6.3a1 1 0 0 0 0 1.4l1.6 1.6a1 1 0 0 0 1.4 0l3.106-3.105c.32-.322.863-.22.983.218a6 6 0 0 1-8.259 7.057l-7.91 7.91a1 1 0 0 1-2.999-3l7.91-7.91a6 6 0 0 1 7.057-8.259c.438.12.54.662.219.984z",key:"1ngwbx"}]],qv=ze("wrench",YE);function Lp({...a}){return d.jsx(V2,{"data-slot":"dropdown-menu",...a})}function Hp({...a}){return d.jsx(X2,{"data-slot":"dropdown-menu-trigger",...a})}function Bp({className:a,sideOffset:o=4,...i}){return d.jsx(Q2,{children:d.jsx(Z2,{"data-slot":"dropdown-menu-content",sideOffset:o,className:Ze("bg-popover text-popover-foreground data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 data-[state=closed]:zoom-out-95 data-[state=open]:zoom-in-95 data-[side=bottom]:slide-in-from-top-2 data-[side=left]:slide-in-from-right-2 data-[side=right]:slide-in-from-left-2 data-[side=top]:slide-in-from-bottom-2 z-50 max-h-(--radix-dropdown-menu-content-available-height) min-w-[8rem] origin-(--radix-dropdown-menu-content-transform-origin) overflow-x-hidden overflow-y-auto rounded-md border p-1 shadow-md",a),...i})})}function _r({className:a,inset:o,variant:i="default",...c}){return d.jsx(P2,{"data-slot":"dropdown-menu-item","data-inset":o,"data-variant":i,className:Ze("focus:bg-accent focus:text-accent-foreground data-[variant=destructive]:text-destructive data-[variant=destructive]:focus:bg-destructive/10 dark:data-[variant=destructive]:focus:bg-destructive/20 data-[variant=destructive]:focus:text-destructive data-[variant=destructive]:*:[svg]:!text-destructive [&_svg:not([class*='text-'])]:text-muted-foreground relative flex cursor-default items-center gap-2 rounded-sm px-2 py-1.5 text-sm outline-hidden select-none data-[disabled]:pointer-events-none data-[disabled]:opacity-50 data-[inset]:pl-8 [&_svg]:pointer-events-none [&_svg]:shrink-0 [&_svg:not([class*='size-'])]:size-4",a),...c})}function du({className:a,inset:o,...i}){return d.jsx(K2,{"data-slot":"dropdown-menu-label","data-inset":o,className:Ze("px-2 py-1.5 text-sm font-medium data-[inset]:pl-8",a),...i})}function Gv({className:a,...o}){return d.jsx($2,{"data-slot":"dropdown-menu-separator",className:Ze("bg-border -mx-1 my-1 h-px",a),...o})}const VE=Wv("inline-flex items-center rounded-md border px-2.5 py-0.5 text-xs font-semibold transition-colors focus:outline-none focus:ring-2 focus:ring-ring focus:ring-offset-2",{variants:{variant:{default:"border-transparent bg-primary text-primary-foreground shadow hover:bg-primary/80",secondary:"border-transparent bg-secondary text-secondary-foreground hover:bg-secondary/80",destructive:"border-transparent bg-destructive text-destructive-foreground shadow hover:bg-destructive/80",outline:"text-foreground"}},defaultVariants:{variant:"default"}});function Tt({className:a,variant:o,...i}){return d.jsx("div",{className:Ze(VE({variant:o}),a),...i})}function Wu({size:a="md",className:o}){return d.jsx(ME,{className:Ze("animate-spin",{"h-4 w-4":a==="sm","h-6 w-6":a==="md","h-8 w-8":a==="lg"},o)})}const Yv=a=>{switch(a){case"agent":return Al;case"orchestrator":return Ju;case"workflow":return yi;default:return kp}},XE=a=>a==="directory"?SE:zp,Vv=a=>{switch(a.type){case"agent":const o=a;return{count:o.tools.length,label:"tools",secondaryInfo:o.model?`Model: ${o.model}`:void 0};case"orchestrator":const i=a;return{count:i.agents.length,label:"agents",secondaryInfo:`Type: ${i.orchestrator_type}`};case"workflow":const c=a;return{count:c.steps.length,label:"steps",secondaryInfo:c.start_step?`Start: ${c.start_step}`:void 0};default:return{count:0,label:"items"}}};function QE({entities:a,selectedEntity:o,onSelect:i,isLoading:c=!1}){const[u,f]=x.useState(!1),h=a.filter(T=>T.type==="agent"),v=a.filter(T=>T.type==="orchestrator"),b=a.filter(T=>T.type==="workflow"),g=T=>{i(T),f(!1)},p=o?Yv(o.type):Al,S=o?.name||o?.id||"Select Entity",w=o?Vv(o):null,M=T=>{const N=Yv(T.type),_=XE(T.source),j=Vv(T);return d.jsx(_r,{onClick:()=>g(T),className:"cursor-pointer",children:d.jsxs("div",{className:"flex items-center justify-between w-full",children:[d.jsxs("div",{className:"flex items-center gap-2 min-w-0",children:[d.jsx(N,{className:"h-4 w-4 flex-shrink-0"}),d.jsxs("div",{className:"min-w-0",children:[d.jsx("div",{className:"truncate font-medium",children:T.name||T.id}),T.description&&d.jsx("div",{className:"text-xs text-muted-foreground truncate",children:T.description}),j.secondaryInfo&&d.jsx("div",{className:"text-xs text-muted-foreground truncate",children:j.secondaryInfo})]})]}),d.jsxs("div",{className:"flex items-center gap-1 flex-shrink-0",children:[d.jsx(_,{className:"h-3 w-3 opacity-60"}),d.jsx(Tt,{variant:"outline",className:"text-xs",children:j.count})]})]})},T.id)};return d.jsxs(Lp,{open:u,onOpenChange:f,children:[d.jsx(Hp,{asChild:!0,children:d.jsx(An,{variant:"outline",className:"w-64 justify-between font-mono text-sm",disabled:c,children:c?d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx(Wu,{size:"sm"}),d.jsx("span",{className:"text-muted-foreground",children:"Loading..."})]}):d.jsxs(d.Fragment,{children:[d.jsxs("div",{className:"flex items-center gap-2 min-w-0",children:[d.jsx(p,{className:"h-4 w-4 flex-shrink-0"}),d.jsx("span",{className:"truncate",children:S}),o&&w&&d.jsxs(Tt,{variant:"secondary",className:"ml-auto flex-shrink-0",children:[w.count," ",w.label]})]}),d.jsx(sE,{className:"h-4 w-4 opacity-50"})]})})}),d.jsxs(Bp,{className:"w-80 font-mono",children:[h.length>0&&d.jsxs(d.Fragment,{children:[d.jsxs(du,{className:"flex items-center gap-2",children:[d.jsx(Al,{className:"h-4 w-4"}),"Agents (",h.length,")"]}),h.map(M)]}),v.length>0&&d.jsxs(d.Fragment,{children:[h.length>0&&d.jsx(Gv,{}),d.jsxs(du,{className:"flex items-center gap-2",children:[d.jsx(Ju,{className:"h-4 w-4"}),"Orchestrators (",v.length,")"]}),v.map(M)]}),b.length>0&&d.jsxs(d.Fragment,{children:[(h.length>0||v.length>0)&&d.jsx(Gv,{}),d.jsxs(du,{className:"flex items-center gap-2",children:[d.jsx(yi,{className:"h-4 w-4"}),"Workflows (",b.length,")"]}),b.map(M)]}),a.length===0&&d.jsx(_r,{disabled:!0,children:d.jsx("div",{className:"text-center text-muted-foreground py-2",children:c?"Loading entities...":"No entities found"})})]})]})}var ZE=(a,o,i,c,u,f,h,v)=>{let b=document.documentElement,g=["light","dark"];function p(M){(Array.isArray(a)?a:[a]).forEach(T=>{let N=T==="class",_=N&&f?u.map(j=>f[j]||j):u;N?(b.classList.remove(..._),b.classList.add(f&&f[M]?f[M]:M)):b.setAttribute(T,M)}),S(M)}function S(M){v&&g.includes(M)&&(b.style.colorScheme=M)}function w(){return window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light"}if(c)p(c);else try{let M=localStorage.getItem(o)||i,T=h&&M==="system"?w():M;p(T)}catch{}},Xv=["light","dark"],qp="(prefers-color-scheme: dark)",KE=typeof window>"u",Fu=x.createContext(void 0),PE={setTheme:a=>{},themes:[]},$E=()=>{var a;return(a=x.useContext(Fu))!=null?a:PE},JE=a=>x.useContext(Fu)?x.createElement(x.Fragment,null,a.children):x.createElement(FE,{...a}),WE=["light","dark"],FE=({forcedTheme:a,disableTransitionOnChange:o=!1,enableSystem:i=!0,enableColorScheme:c=!0,storageKey:u="theme",themes:f=WE,defaultTheme:h=i?"system":"light",attribute:v="data-theme",value:b,children:g,nonce:p,scriptProps:S})=>{let[w,M]=x.useState(()=>eA(u,h)),[T,N]=x.useState(()=>w==="system"?mu():w),_=b?Object.values(b):f,j=x.useCallback(B=>{let J=B;if(!J)return;B==="system"&&i&&(J=mu());let P=b?b[J]:J,Z=o?tA(p):null,I=document.documentElement,re=ue=>{ue==="class"?(I.classList.remove(..._),P&&I.classList.add(P)):ue.startsWith("data-")&&(P?I.setAttribute(ue,P):I.removeAttribute(ue))};if(Array.isArray(v)?v.forEach(re):re(v),c){let ue=Xv.includes(h)?h:null,de=Xv.includes(J)?J:ue;I.style.colorScheme=de}Z?.()},[p]),q=x.useCallback(B=>{let J=typeof B=="function"?B(w):B;M(J);try{localStorage.setItem(u,J)}catch{}},[w]),L=x.useCallback(B=>{let J=mu(B);N(J),w==="system"&&i&&!a&&j("system")},[w,a]);x.useEffect(()=>{let B=window.matchMedia(qp);return B.addListener(L),L(B),()=>B.removeListener(L)},[L]),x.useEffect(()=>{let B=J=>{J.key===u&&(J.newValue?M(J.newValue):q(h))};return window.addEventListener("storage",B),()=>window.removeEventListener("storage",B)},[q]),x.useEffect(()=>{j(a??w)},[a,w]);let Y=x.useMemo(()=>({theme:w,setTheme:q,forcedTheme:a,resolvedTheme:w==="system"?T:w,themes:i?[...f,"system"]:f,systemTheme:i?T:void 0}),[w,q,a,T,i,f]);return x.createElement(Fu.Provider,{value:Y},x.createElement(IE,{forcedTheme:a,storageKey:u,attribute:v,enableSystem:i,enableColorScheme:c,defaultTheme:h,value:b,themes:f,nonce:p,scriptProps:S}),g)},IE=x.memo(({forcedTheme:a,storageKey:o,attribute:i,enableSystem:c,enableColorScheme:u,defaultTheme:f,value:h,themes:v,nonce:b,scriptProps:g})=>{let p=JSON.stringify([i,o,f,a,v,h,c,u]).slice(1,-1);return x.createElement("script",{...g,suppressHydrationWarning:!0,nonce:typeof window>"u"?b:"",dangerouslySetInnerHTML:{__html:`(${ZE.toString()})(${p})`}})}),eA=(a,o)=>{if(KE)return;let i;try{i=localStorage.getItem(a)||void 0}catch{}return i||o},tA=a=>{let o=document.createElement("style");return a&&o.setAttribute("nonce",a),o.appendChild(document.createTextNode("*,*::before,*::after{-webkit-transition:none!important;-moz-transition:none!important;-o-transition:none!important;-ms-transition:none!important;transition:none!important}")),document.head.appendChild(o),()=>{window.getComputedStyle(document.body),setTimeout(()=>{document.head.removeChild(o)},1)}},mu=a=>(a||(a=window.matchMedia(qp)),a.matches?"dark":"light");function nA(){const{setTheme:a}=$E();return d.jsxs(Lp,{children:[d.jsx(Hp,{asChild:!0,children:d.jsxs(An,{variant:"ghost",size:"sm",children:[d.jsx(UE,{className:"h-[1.2rem] w-[1.2rem] rotate-0 scale-100 transition-all dark:-rotate-90 dark:scale-0"}),d.jsx(RE,{className:"absolute h-[1.2rem] w-[1.2rem] rotate-90 scale-0 transition-all dark:rotate-0 dark:scale-100"}),d.jsx("span",{className:"sr-only",children:"Toggle theme"})]})}),d.jsxs(Bp,{align:"end",children:[d.jsx(_r,{onClick:()=>a("light"),children:"Light"}),d.jsx(_r,{onClick:()=>a("dark"),children:"Dark"}),d.jsx(_r,{onClick:()=>a("system"),children:"System"})]})]})}function hu({entities:a,selectedEntity:o,onSelect:i,isLoading:c=!1}){return d.jsxs("header",{className:"flex h-14 items-center gap-4 border-b px-4",children:[d.jsx("div",{className:"font-semibold",children:"PicoAgents WebUI"}),d.jsx(QE,{entities:a,selectedEntity:o,onSelect:i,isLoading:c}),d.jsxs("div",{className:"flex items-center gap-2 ml-auto",children:[d.jsx(nA,{}),d.jsx(An,{variant:"ghost",size:"sm",children:d.jsx(kp,{className:"h-4 w-4"})})]})]})}function lA(a,[o,i]){return Math.min(i,Math.max(o,a))}function aA(a,o){return x.useReducer((i,c)=>o[i][c]??i,a)}var Iu="ScrollArea",[Gp,qA]=Ml(Iu),[rA,Bt]=Gp(Iu),Yp=x.forwardRef((a,o)=>{const{__scopeScrollArea:i,type:c="hover",dir:u,scrollHideDelay:f=600,...h}=a,[v,b]=x.useState(null),[g,p]=x.useState(null),[S,w]=x.useState(null),[M,T]=x.useState(null),[N,_]=x.useState(null),[j,q]=x.useState(0),[L,Y]=x.useState(0),[B,J]=x.useState(!1),[P,Z]=x.useState(!1),I=Qe(o,ue=>b(ue)),re=wi(u);return d.jsx(rA,{scope:i,type:c,dir:re,scrollHideDelay:f,scrollArea:v,viewport:g,onViewportChange:p,content:S,onContentChange:w,scrollbarX:M,onScrollbarXChange:T,scrollbarXEnabled:B,onScrollbarXEnabledChange:J,scrollbarY:N,onScrollbarYChange:_,scrollbarYEnabled:P,onScrollbarYEnabledChange:Z,onCornerWidthChange:q,onCornerHeightChange:Y,children:d.jsx(ke.div,{dir:re,...h,ref:I,style:{position:"relative","--radix-scroll-area-corner-width":j+"px","--radix-scroll-area-corner-height":L+"px",...a.style}})})});Yp.displayName=Iu;var Vp="ScrollAreaViewport",Xp=x.forwardRef((a,o)=>{const{__scopeScrollArea:i,children:c,nonce:u,...f}=a,h=Bt(Vp,i),v=x.useRef(null),b=Qe(o,v,h.onViewportChange);return d.jsxs(d.Fragment,{children:[d.jsx("style",{dangerouslySetInnerHTML:{__html:"[data-radix-scroll-area-viewport]{scrollbar-width:none;-ms-overflow-style:none;-webkit-overflow-scrolling:touch;}[data-radix-scroll-area-viewport]::-webkit-scrollbar{display:none}"},nonce:u}),d.jsx(ke.div,{"data-radix-scroll-area-viewport":"",...f,ref:b,style:{overflowX:h.scrollbarXEnabled?"scroll":"hidden",overflowY:h.scrollbarYEnabled?"scroll":"hidden",...a.style},children:d.jsx("div",{ref:h.onContentChange,style:{minWidth:"100%",display:"table"},children:c})})]})});Xp.displayName=Vp;var on="ScrollAreaScrollbar",ef=x.forwardRef((a,o)=>{const{forceMount:i,...c}=a,u=Bt(on,a.__scopeScrollArea),{onScrollbarXEnabledChange:f,onScrollbarYEnabledChange:h}=u,v=a.orientation==="horizontal";return x.useEffect(()=>(v?f(!0):h(!0),()=>{v?f(!1):h(!1)}),[v,f,h]),u.type==="hover"?d.jsx(oA,{...c,ref:o,forceMount:i}):u.type==="scroll"?d.jsx(iA,{...c,ref:o,forceMount:i}):u.type==="auto"?d.jsx(Qp,{...c,ref:o,forceMount:i}):u.type==="always"?d.jsx(tf,{...c,ref:o}):null});ef.displayName=on;var oA=x.forwardRef((a,o)=>{const{forceMount:i,...c}=a,u=Bt(on,a.__scopeScrollArea),[f,h]=x.useState(!1);return x.useEffect(()=>{const v=u.scrollArea;let b=0;if(v){const g=()=>{window.clearTimeout(b),h(!0)},p=()=>{b=window.setTimeout(()=>h(!1),u.scrollHideDelay)};return v.addEventListener("pointerenter",g),v.addEventListener("pointerleave",p),()=>{window.clearTimeout(b),v.removeEventListener("pointerenter",g),v.removeEventListener("pointerleave",p)}}},[u.scrollArea,u.scrollHideDelay]),d.jsx(rn,{present:i||f,children:d.jsx(Qp,{"data-state":f?"visible":"hidden",...c,ref:o})})}),iA=x.forwardRef((a,o)=>{const{forceMount:i,...c}=a,u=Bt(on,a.__scopeScrollArea),f=a.orientation==="horizontal",h=Di(()=>b("SCROLL_END"),100),[v,b]=aA("hidden",{hidden:{SCROLL:"scrolling"},scrolling:{SCROLL_END:"idle",POINTER_ENTER:"interacting"},interacting:{SCROLL:"interacting",POINTER_LEAVE:"idle"},idle:{HIDE:"hidden",SCROLL:"scrolling",POINTER_ENTER:"interacting"}});return x.useEffect(()=>{if(v==="idle"){const g=window.setTimeout(()=>b("HIDE"),u.scrollHideDelay);return()=>window.clearTimeout(g)}},[v,u.scrollHideDelay,b]),x.useEffect(()=>{const g=u.viewport,p=f?"scrollLeft":"scrollTop";if(g){let S=g[p];const w=()=>{const M=g[p];S!==M&&(b("SCROLL"),h()),S=M};return g.addEventListener("scroll",w),()=>g.removeEventListener("scroll",w)}},[u.viewport,f,b,h]),d.jsx(rn,{present:i||v!=="hidden",children:d.jsx(tf,{"data-state":v==="hidden"?"hidden":"visible",...c,ref:o,onPointerEnter:me(a.onPointerEnter,()=>b("POINTER_ENTER")),onPointerLeave:me(a.onPointerLeave,()=>b("POINTER_LEAVE"))})})}),Qp=x.forwardRef((a,o)=>{const i=Bt(on,a.__scopeScrollArea),{forceMount:c,...u}=a,[f,h]=x.useState(!1),v=a.orientation==="horizontal",b=Di(()=>{if(i.viewport){const g=i.viewport.offsetWidth<i.viewport.scrollWidth,p=i.viewport.offsetHeight<i.viewport.scrollHeight;h(v?g:p)}},10);return xa(i.viewport,b),xa(i.content,b),d.jsx(rn,{present:c||f,children:d.jsx(tf,{"data-state":f?"visible":"hidden",...u,ref:o})})}),tf=x.forwardRef((a,o)=>{const{orientation:i="vertical",...c}=a,u=Bt(on,a.__scopeScrollArea),f=x.useRef(null),h=x.useRef(0),[v,b]=x.useState({content:0,viewport:0,scrollbar:{size:0,paddingStart:0,paddingEnd:0}}),g=Jp(v.viewport,v.content),p={...c,sizes:v,onSizesChange:b,hasThumb:g>0&&g<1,onThumbChange:w=>f.current=w,onThumbPointerUp:()=>h.current=0,onThumbPointerDown:w=>h.current=w};function S(w,M){return mA(w,h.current,v,M)}return i==="horizontal"?d.jsx(cA,{...p,ref:o,onThumbPositionChange:()=>{if(u.viewport&&f.current){const w=u.viewport.scrollLeft,M=Qv(w,v,u.dir);f.current.style.transform=`translate3d(${M}px, 0, 0)`}},onWheelScroll:w=>{u.viewport&&(u.viewport.scrollLeft=w)},onDragScroll:w=>{u.viewport&&(u.viewport.scrollLeft=S(w,u.dir))}}):i==="vertical"?d.jsx(sA,{...p,ref:o,onThumbPositionChange:()=>{if(u.viewport&&f.current){const w=u.viewport.scrollTop,M=Qv(w,v);f.current.style.transform=`translate3d(0, ${M}px, 0)`}},onWheelScroll:w=>{u.viewport&&(u.viewport.scrollTop=w)},onDragScroll:w=>{u.viewport&&(u.viewport.scrollTop=S(w))}}):null}),cA=x.forwardRef((a,o)=>{const{sizes:i,onSizesChange:c,...u}=a,f=Bt(on,a.__scopeScrollArea),[h,v]=x.useState(),b=x.useRef(null),g=Qe(o,b,f.onScrollbarXChange);return x.useEffect(()=>{b.current&&v(getComputedStyle(b.current))},[b]),d.jsx(Kp,{"data-orientation":"horizontal",...u,ref:g,sizes:i,style:{bottom:0,left:f.dir==="rtl"?"var(--radix-scroll-area-corner-width)":0,right:f.dir==="ltr"?"var(--radix-scroll-area-corner-width)":0,"--radix-scroll-area-thumb-width":Oi(i)+"px",...a.style},onThumbPointerDown:p=>a.onThumbPointerDown(p.x),onDragScroll:p=>a.onDragScroll(p.x),onWheelScroll:(p,S)=>{if(f.viewport){const w=f.viewport.scrollLeft+p.deltaX;a.onWheelScroll(w),Fp(w,S)&&p.preventDefault()}},onResize:()=>{b.current&&f.viewport&&h&&c({content:f.viewport.scrollWidth,viewport:f.viewport.offsetWidth,scrollbar:{size:b.current.clientWidth,paddingStart:xi(h.paddingLeft),paddingEnd:xi(h.paddingRight)}})}})}),sA=x.forwardRef((a,o)=>{const{sizes:i,onSizesChange:c,...u}=a,f=Bt(on,a.__scopeScrollArea),[h,v]=x.useState(),b=x.useRef(null),g=Qe(o,b,f.onScrollbarYChange);return x.useEffect(()=>{b.current&&v(getComputedStyle(b.current))},[b]),d.jsx(Kp,{"data-orientation":"vertical",...u,ref:g,sizes:i,style:{top:0,right:f.dir==="ltr"?0:void 0,left:f.dir==="rtl"?0:void 0,bottom:"var(--radix-scroll-area-corner-height)","--radix-scroll-area-thumb-height":Oi(i)+"px",...a.style},onThumbPointerDown:p=>a.onThumbPointerDown(p.y),onDragScroll:p=>a.onDragScroll(p.y),onWheelScroll:(p,S)=>{if(f.viewport){const w=f.viewport.scrollTop+p.deltaY;a.onWheelScroll(w),Fp(w,S)&&p.preventDefault()}},onResize:()=>{b.current&&f.viewport&&h&&c({content:f.viewport.scrollHeight,viewport:f.viewport.offsetHeight,scrollbar:{size:b.current.clientHeight,paddingStart:xi(h.paddingTop),paddingEnd:xi(h.paddingBottom)}})}})}),[uA,Zp]=Gp(on),Kp=x.forwardRef((a,o)=>{const{__scopeScrollArea:i,sizes:c,hasThumb:u,onThumbChange:f,onThumbPointerUp:h,onThumbPointerDown:v,onThumbPositionChange:b,onDragScroll:g,onWheelScroll:p,onResize:S,...w}=a,M=Bt(on,i),[T,N]=x.useState(null),_=Qe(o,I=>N(I)),j=x.useRef(null),q=x.useRef(""),L=M.viewport,Y=c.content-c.viewport,B=ot(p),J=ot(b),P=Di(S,10);function Z(I){if(j.current){const re=I.clientX-j.current.left,ue=I.clientY-j.current.top;g({x:re,y:ue})}}return x.useEffect(()=>{const I=re=>{const ue=re.target;T?.contains(ue)&&B(re,Y)};return document.addEventListener("wheel",I,{passive:!1}),()=>document.removeEventListener("wheel",I,{passive:!1})},[L,T,Y,B]),x.useEffect(J,[c,J]),xa(T,P),xa(M.content,P),d.jsx(uA,{scope:i,scrollbar:T,hasThumb:u,onThumbChange:ot(f),onThumbPointerUp:ot(h),onThumbPositionChange:J,onThumbPointerDown:ot(v),children:d.jsx(ke.div,{...w,ref:_,style:{position:"absolute",...w.style},onPointerDown:me(a.onPointerDown,I=>{I.button===0&&(I.target.setPointerCapture(I.pointerId),j.current=T.getBoundingClientRect(),q.current=document.body.style.webkitUserSelect,document.body.style.webkitUserSelect="none",M.viewport&&(M.viewport.style.scrollBehavior="auto"),Z(I))}),onPointerMove:me(a.onPointerMove,Z),onPointerUp:me(a.onPointerUp,I=>{const re=I.target;re.hasPointerCapture(I.pointerId)&&re.releasePointerCapture(I.pointerId),document.body.style.webkitUserSelect=q.current,M.viewport&&(M.viewport.style.scrollBehavior=""),j.current=null})})})}),bi="ScrollAreaThumb",Pp=x.forwardRef((a,o)=>{const{forceMount:i,...c}=a,u=Zp(bi,a.__scopeScrollArea);return d.jsx(rn,{present:i||u.hasThumb,children:d.jsx(fA,{ref:o,...c})})}),fA=x.forwardRef((a,o)=>{const{__scopeScrollArea:i,style:c,...u}=a,f=Bt(bi,i),h=Zp(bi,i),{onThumbPositionChange:v}=h,b=Qe(o,S=>h.onThumbChange(S)),g=x.useRef(void 0),p=Di(()=>{g.current&&(g.current(),g.current=void 0)},100);return x.useEffect(()=>{const S=f.viewport;if(S){const w=()=>{if(p(),!g.current){const M=hA(S,v);g.current=M,v()}};return v(),S.addEventListener("scroll",w),()=>S.removeEventListener("scroll",w)}},[f.viewport,p,v]),d.jsx(ke.div,{"data-state":h.hasThumb?"visible":"hidden",...u,ref:b,style:{width:"var(--radix-scroll-area-thumb-width)",height:"var(--radix-scroll-area-thumb-height)",...c},onPointerDownCapture:me(a.onPointerDownCapture,S=>{const M=S.target.getBoundingClientRect(),T=S.clientX-M.left,N=S.clientY-M.top;h.onThumbPointerDown({x:T,y:N})}),onPointerUp:me(a.onPointerUp,h.onThumbPointerUp)})});Pp.displayName=bi;var nf="ScrollAreaCorner",$p=x.forwardRef((a,o)=>{const i=Bt(nf,a.__scopeScrollArea),c=!!(i.scrollbarX&&i.scrollbarY);return i.type!=="scroll"&&c?d.jsx(dA,{...a,ref:o}):null});$p.displayName=nf;var dA=x.forwardRef((a,o)=>{const{__scopeScrollArea:i,...c}=a,u=Bt(nf,i),[f,h]=x.useState(0),[v,b]=x.useState(0),g=!!(f&&v);return xa(u.scrollbarX,()=>{const p=u.scrollbarX?.offsetHeight||0;u.onCornerHeightChange(p),b(p)}),xa(u.scrollbarY,()=>{const p=u.scrollbarY?.offsetWidth||0;u.onCornerWidthChange(p),h(p)}),g?d.jsx(ke.div,{...c,ref:o,style:{width:f,height:v,position:"absolute",right:u.dir==="ltr"?0:void 0,left:u.dir==="rtl"?0:void 0,bottom:0,...a.style}}):null});function xi(a){return a?parseInt(a,10):0}function Jp(a,o){const i=a/o;return isNaN(i)?0:i}function Oi(a){const o=Jp(a.viewport,a.content),i=a.scrollbar.paddingStart+a.scrollbar.paddingEnd,c=(a.scrollbar.size-i)*o;return Math.max(c,18)}function mA(a,o,i,c="ltr"){const u=Oi(i),f=u/2,h=o||f,v=u-h,b=i.scrollbar.paddingStart+h,g=i.scrollbar.size-i.scrollbar.paddingEnd-v,p=i.content-i.viewport,S=c==="ltr"?[0,p]:[p*-1,0];return Wp([b,g],S)(a)}function Qv(a,o,i="ltr"){const c=Oi(o),u=o.scrollbar.paddingStart+o.scrollbar.paddingEnd,f=o.scrollbar.size-u,h=o.content-o.viewport,v=f-c,b=i==="ltr"?[0,h]:[h*-1,0],g=lA(a,b);return Wp([0,h],[0,v])(g)}function Wp(a,o){return i=>{if(a[0]===a[1]||o[0]===o[1])return o[0];const c=(o[1]-o[0])/(a[1]-a[0]);return o[0]+c*(i-a[0])}}function Fp(a,o){return a>0&&a<o}var hA=(a,o=()=>{})=>{let i={left:a.scrollLeft,top:a.scrollTop},c=0;return(function u(){const f={left:a.scrollLeft,top:a.scrollTop},h=i.left!==f.left,v=i.top!==f.top;(h||v)&&o(),i=f,c=window.requestAnimationFrame(u)})(),()=>window.cancelAnimationFrame(c)};function Di(a,o){const i=ot(a),c=x.useRef(0);return x.useEffect(()=>()=>window.clearTimeout(c.current),[]),x.useCallback(()=>{window.clearTimeout(c.current),c.current=window.setTimeout(i,o)},[i,o])}function xa(a,o){const i=ot(o);Mn(()=>{let c=0;if(a){const u=new ResizeObserver(()=>{cancelAnimationFrame(c),c=window.requestAnimationFrame(i)});return u.observe(a),()=>{window.cancelAnimationFrame(c),u.unobserve(a)}}},[a,i])}var Ip=Yp,vA=Xp,gA=$p;const Sa=x.forwardRef(({className:a,children:o,...i},c)=>d.jsxs(Ip,{ref:c,className:Ze("relative overflow-hidden",a),...i,children:[d.jsx(vA,{className:"h-full w-full rounded-[inherit]",children:o}),d.jsx(ey,{}),d.jsx(gA,{})]}));Sa.displayName=Ip.displayName;const ey=x.forwardRef(({className:a,orientation:o="vertical",...i},c)=>d.jsx(ef,{ref:c,orientation:o,className:Ze("flex touch-none select-none transition-colors",o==="vertical"&&"h-full w-2.5 border-l border-l-transparent p-[1px]",o==="horizontal"&&"h-2.5 flex-col border-t border-t-transparent p-[1px]",a),...i,children:d.jsx(Pp,{className:"relative flex-1 rounded-full bg-border"})}));ey.displayName=ef.displayName;var zi="Tabs",[pA,GA]=Ml(zi,[Ti]),ty=Ti(),[yA,lf]=pA(zi),ny=x.forwardRef((a,o)=>{const{__scopeTabs:i,value:c,onValueChange:u,defaultValue:f,orientation:h="horizontal",dir:v,activationMode:b="automatic",...g}=a,p=wi(v),[S,w]=Du({prop:c,onChange:u,defaultProp:f??"",caller:zi});return d.jsx(yA,{scope:i,baseId:fi(),value:S,onValueChange:w,orientation:h,dir:p,activationMode:b,children:d.jsx(ke.div,{dir:p,"data-orientation":h,...g,ref:o})})});ny.displayName=zi;var ly="TabsList",ay=x.forwardRef((a,o)=>{const{__scopeTabs:i,loop:c=!0,...u}=a,f=lf(ly,i),h=ty(i);return d.jsx(Vg,{asChild:!0,...h,orientation:f.orientation,dir:f.dir,loop:c,children:d.jsx(ke.div,{role:"tablist","aria-orientation":f.orientation,...u,ref:o})})});ay.displayName=ly;var ry="TabsTrigger",oy=x.forwardRef((a,o)=>{const{__scopeTabs:i,value:c,disabled:u=!1,...f}=a,h=lf(ry,i),v=ty(i),b=sy(h.baseId,c),g=uy(h.baseId,c),p=c===h.value;return d.jsx(Xg,{asChild:!0,...v,focusable:!u,active:p,children:d.jsx(ke.button,{type:"button",role:"tab","aria-selected":p,"aria-controls":g,"data-state":p?"active":"inactive","data-disabled":u?"":void 0,disabled:u,id:b,...f,ref:o,onMouseDown:me(a.onMouseDown,S=>{!u&&S.button===0&&S.ctrlKey===!1?h.onValueChange(c):S.preventDefault()}),onKeyDown:me(a.onKeyDown,S=>{[" ","Enter"].includes(S.key)&&h.onValueChange(c)}),onFocus:me(a.onFocus,()=>{const S=h.activationMode!=="manual";!p&&!u&&S&&h.onValueChange(c)})})})});oy.displayName=ry;var iy="TabsContent",cy=x.forwardRef((a,o)=>{const{__scopeTabs:i,value:c,forceMount:u,children:f,...h}=a,v=lf(iy,i),b=sy(v.baseId,c),g=uy(v.baseId,c),p=c===v.value,S=x.useRef(p);return x.useEffect(()=>{const w=requestAnimationFrame(()=>S.current=!1);return()=>cancelAnimationFrame(w)},[]),d.jsx(rn,{present:u||p,children:({present:w})=>d.jsx(ke.div,{"data-state":p?"active":"inactive","data-orientation":v.orientation,role:"tabpanel","aria-labelledby":b,hidden:!w,id:g,tabIndex:0,...h,ref:o,style:{...a.style,animationDuration:S.current?"0s":void 0},children:w&&f})})});cy.displayName=iy;function sy(a,o){return`${a}-trigger-${o}`}function uy(a,o){return`${a}-content-${o}`}var bA=ny,fy=ay,dy=oy,my=cy;const xA=bA,hy=x.forwardRef(({className:a,...o},i)=>d.jsx(fy,{ref:i,className:Ze("inline-flex h-9 items-center justify-center rounded-lg bg-muted p-1 text-muted-foreground",a),...o}));hy.displayName=fy.displayName;const _u=x.forwardRef(({className:a,...o},i)=>d.jsx(dy,{ref:i,className:Ze("inline-flex items-center justify-center whitespace-nowrap rounded-md px-3 py-1 text-sm font-medium ring-offset-background transition-all focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2 disabled:pointer-events-none disabled:opacity-50 data-[state=active]:bg-background data-[state=active]:text-foreground data-[state=active]:shadow",a),...o}));_u.displayName=dy.displayName;const Cu=x.forwardRef(({className:a,...o},i)=>d.jsx(my,{ref:i,className:Ze("mt-2 ring-offset-background focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2",a),...o}));Cu.displayName=my.displayName;function Mt({className:a,...o}){return d.jsx("div",{"data-slot":"card",className:Ze("bg-card text-card-foreground flex flex-col gap-6 rounded border py-6 shadow-sm",a),...o})}function Tr({className:a,...o}){return d.jsx("div",{"data-slot":"card-header",className:Ze("@container/card-header grid auto-rows-min grid-rows-[auto_auto] items-start gap-1.5 px-6 has-data-[slot=card-action]:grid-cols-[1fr_auto] [.border-b]:pb-6",a),...o})}function Rr({className:a,...o}){return d.jsx("div",{"data-slot":"card-title",className:Ze("leading-none font-semibold",a),...o})}function Nt({className:a,...o}){return d.jsx("div",{"data-slot":"card-content",className:Ze("px-6",a),...o})}const vy=a=>{switch(a){case"message":return d.jsx(Up,{className:"h-4 w-4"});case"tool_call":return d.jsx(Mu,{className:"h-4 w-4"});case"error":return d.jsx(Nu,{className:"h-4 w-4 text-red-500"});case"complete":return d.jsx(Tu,{className:"h-4 w-4 text-green-500"});case"workflow_started":case"workflow_completed":return d.jsx(Mu,{className:"h-4 w-4 text-blue-500"});default:return d.jsx(EE,{className:"h-4 w-4"})}},SA=a=>{switch(a){case"error":return"destructive";case"complete":case"workflow_completed":return"default";case"message":case"tool_call":return"secondary";default:return"outline"}};function wA({event:a}){const[o,i]=x.useState(!1);return d.jsxs(Mt,{className:"mb-2",children:[d.jsx(Tr,{className:"pb-2",children:d.jsxs("div",{className:"flex items-center justify-between",children:[d.jsxs("div",{className:"flex items-center gap-2",children:[vy(a.type),d.jsx(Rr,{className:"text-sm",children:a.type}),d.jsx(Tt,{variant:SA(a.type),className:"text-xs",children:a.type})]}),d.jsxs("div",{className:"flex items-center gap-2 text-xs text-muted-foreground",children:[d.jsx(Ru,{className:"h-3 w-3"}),new Date(a.timestamp).toLocaleTimeString()]})]})}),d.jsxs(Nt,{className:"pt-0",children:[a.data&&d.jsxs("div",{children:[d.jsxs("button",{onClick:()=>i(!o),className:"text-xs text-muted-foreground hover:text-foreground mb-2",children:[o?"Hide":"Show"," data"]}),o&&d.jsx(Sa,{className:"h-24",children:d.jsx("pre",{className:"text-xs bg-muted p-2 rounded overflow-auto",children:JSON.stringify(a.data,null,2)})})]}),a.session_id&&d.jsxs("div",{className:"text-xs text-muted-foreground mt-1",children:["Session: ",a.session_id]})]})]})}function EA({events:a}){const o=a.reduce((i,c)=>(i[c.type]=(i[c.type]||0)+1,i),{});return d.jsx("div",{className:"grid grid-cols-2 gap-2 mb-4",children:Object.entries(o).map(([i,c])=>d.jsx(Mt,{children:d.jsx(Nt,{className:"p-3",children:d.jsxs("div",{className:"flex items-center gap-2",children:[vy(i),d.jsxs("div",{children:[d.jsx("div",{className:"text-sm font-medium",children:c}),d.jsx("div",{className:"text-xs text-muted-foreground",children:i})]})]})})},i))})}function AA({events:a,isStreaming:o}){const i=a.slice(-50);return d.jsxs("div",{className:"h-full flex flex-col border-l bg-background",children:[d.jsxs("div",{className:"p-4 border-b",children:[d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx(Bv,{className:"h-5 w-5"}),d.jsx("h3",{className:"font-semibold",children:"Debug Panel"}),o&&d.jsxs(Tt,{variant:"secondary",className:"gap-1",children:[d.jsx("div",{className:"h-2 w-2 bg-green-500 rounded-full animate-pulse"}),"Live"]})]}),d.jsxs("div",{className:"text-xs text-muted-foreground mt-1",children:[a.length," events total"]})]}),d.jsx("div",{className:"flex-1 min-h-0",children:d.jsxs(xA,{defaultValue:"events",className:"h-full flex flex-col",children:[d.jsxs(hy,{className:"grid w-full grid-cols-2 mx-4 mt-4",children:[d.jsx(_u,{value:"events",children:"Events"}),d.jsx(_u,{value:"summary",children:"Summary"})]}),d.jsx(Cu,{value:"events",className:"flex-1 m-4 mt-2",children:d.jsx(Sa,{className:"h-full",children:i.length===0?d.jsxs("div",{className:"flex flex-col items-center justify-center h-32 text-center",children:[d.jsx(Bv,{className:"h-8 w-8 text-muted-foreground mb-2"}),d.jsx("div",{className:"text-sm text-muted-foreground",children:"No debug events yet"}),d.jsx("div",{className:"text-xs text-muted-foreground",children:"Events will appear here during execution"})]}):d.jsx("div",{className:"space-y-2",children:i.map((c,u)=>d.jsx(wA,{event:c},u))})})}),d.jsx(Cu,{value:"summary",className:"flex-1 m-4 mt-2",children:d.jsx(Sa,{className:"h-full",children:a.length===0?d.jsxs("div",{className:"flex flex-col items-center justify-center h-32 text-center",children:[d.jsx(Mu,{className:"h-8 w-8 text-muted-foreground mb-2"}),d.jsx("div",{className:"text-sm text-muted-foreground",children:"No events to summarize"})]}):d.jsx(EA,{events:a})})})]})})]})}function gy({className:a,type:o,...i}){return d.jsx("input",{type:o,"data-slot":"input",className:Ze("file:text-foreground placeholder:text-muted-foreground selection:bg-primary selection:text-primary-foreground dark:bg-input/30 border-input flex h-9 w-full min-w-0 rounded-md border bg-transparent px-3 py-1 text-base shadow-xs transition-[color,box-shadow] outline-none file:inline-flex file:h-7 file:border-0 file:bg-transparent file:text-sm file:font-medium disabled:pointer-events-none disabled:cursor-not-allowed disabled:opacity-50 md:text-sm","focus-visible:border-ring focus-visible:ring-ring/50 focus-visible:ring-[3px]","aria-invalid:ring-destructive/20 dark:aria-invalid:ring-destructive/40 aria-invalid:border-destructive",a),...i})}function MA({message:a}){const o=a.role==="user",i=o?BE:Al;return d.jsxs("div",{className:`flex gap-3 ${o?"flex-row-reverse":""}`,children:[d.jsx("div",{className:`flex h-8 w-8 shrink-0 select-none items-center justify-center rounded-md border ${o?"bg-primary text-primary-foreground":"bg-muted"}`,children:d.jsx(i,{className:"h-4 w-4"})}),d.jsx("div",{className:`flex flex-col space-y-1 ${o?"items-end":"items-start"} max-w-[80%]`,children:d.jsx("div",{className:`rounded px-3 py-2 text-sm whitespace-pre-wrap ${o?"bg-primary text-primary-foreground":"bg-muted"}`,children:a.content})})]})}function NA(){return d.jsxs("div",{className:"flex gap-3",children:[d.jsx("div",{className:"flex h-8 w-8 shrink-0 select-none items-center justify-center rounded-md border bg-muted",children:d.jsx(Al,{className:"h-4 w-4"})}),d.jsx("div",{className:"flex items-center space-x-1 rounded bg-muted px-3 py-2",children:d.jsxs("div",{className:"flex space-x-1",children:[d.jsx("div",{className:"h-2 w-2 animate-bounce rounded-full bg-current [animation-delay:-0.3s]"}),d.jsx("div",{className:"h-2 w-2 animate-bounce rounded-full bg-current [animation-delay:-0.15s]"}),d.jsx("div",{className:"h-2 w-2 animate-bounce rounded-full bg-current"})]})})]})}function py({messages:a,onSendMessage:o,onClearMessages:i,isStreaming:c,placeholder:u="Type a message...",emptyStateTitle:f="Start a conversation",emptyStateDescription:h="Type a message below to begin"}){const[v,b]=x.useState(""),[g,p]=x.useState(!1),S=x.useRef(null),w=x.useRef(null);x.useEffect(()=>{w.current?.scrollIntoView({behavior:"smooth"})},[a,c]);const M=async N=>{if(N.preventDefault(),!v.trim()||g||c)return;p(!0);const _=v.trim();b("");try{await o(_)}catch(j){console.error("Failed to send message:",j)}finally{p(!1)}},T=v.trim()&&!g&&!c;return d.jsxs("div",{className:"flex h-full flex-col",children:[d.jsx(Sa,{className:"flex-1 p-4 h-0",ref:S,children:d.jsxs("div",{className:"space-y-4",children:[a.length===0?d.jsxs("div",{className:"flex flex-col items-center justify-center h-32 text-center",children:[d.jsx("div",{className:"text-muted-foreground text-sm",children:f}),d.jsx("div",{className:"text-xs text-muted-foreground mt-1",children:h})]}):d.jsxs(d.Fragment,{children:[a.map((N,_)=>d.jsx(MA,{message:N},_)),c&&d.jsx(NA,{})]}),d.jsx("div",{ref:w})]})}),d.jsx("div",{className:"border-t flex-shrink-0",children:d.jsxs("div",{className:"p-4",children:[d.jsx("div",{className:"flex items-center gap-2 mb-2",children:a.length>0&&d.jsxs(An,{variant:"outline",size:"sm",onClick:i,disabled:c,className:"gap-1",children:[d.jsx(LE,{className:"h-3 w-3"}),"Clear"]})}),d.jsxs("form",{onSubmit:M,className:"flex gap-2",children:[d.jsx(gy,{value:v,onChange:N=>b(N.target.value),placeholder:u,disabled:g||c,className:"flex-1"}),d.jsx(An,{type:"submit",size:"icon",disabled:!T,className:"shrink-0",children:g?d.jsx(Wu,{size:"sm"}):d.jsx(OE,{className:"h-4 w-4"})})]})]})})]})}const TA="http://localhost:8080";class RA{baseUrl;constructor(o=TA){this.baseUrl=o}async request(o,i={}){const c=`${this.baseUrl}${o}`,u=await fetch(c,{headers:{"Content-Type":"application/json",...i.headers},...i});if(!u.ok)throw new Error(`API request failed: ${u.status} ${u.statusText}`);return u.json()}async getHealth(){return this.request("/api/health")}async getEntities(){return this.request("/api/entities")}async getEntity(o){return this.request(`/api/entities/${o}`)}async getAgents(){return(await this.getEntities()).filter(i=>i.type==="agent")}async getOrchestrators(){return(await this.getEntities()).filter(i=>i.type==="orchestrator")}async getWorkflows(){return(await this.getEntities()).filter(i=>i.type==="workflow")}async getSessions(o){const i=o?`?entity_id=${encodeURIComponent(o)}`:"";return this.request(`/api/sessions${i}`)}async getSession(o){return this.request(`/api/sessions/${o}`)}async getSessionMessages(o){return this.request(`/api/sessions/${o}/messages`)}async deleteSession(o){return this.request(`/api/sessions/${o}`,{method:"DELETE"})}async*streamEntityExecution(o,i){const c=`/api/entities/${o}/run/stream`,u=await fetch(`${this.baseUrl}${c}`,{method:"POST",headers:{"Content-Type":"application/json",Accept:"text/event-stream"},body:JSON.stringify(i)});if(!u.ok)throw new Error(`Streaming request failed: ${u.status}`);const f=u.body?.getReader();if(!f)throw new Error("Response body is not readable");const h=new TextDecoder;let v="";try{for(;;){const{done:b,value:g}=await f.read();if(b)break;v+=h.decode(g,{stream:!0});const p=v.split(`
`);v=p.pop()||"";for(const S of p)if(S.startsWith("data: "))try{yield JSON.parse(S.slice(6))}catch(w){console.error("Failed to parse SSE event:",w)}}}finally{f.releaseLock()}}async runEntity(o,i){return this.request(`/api/entities/${o}/run`,{method:"POST",body:JSON.stringify(i)})}async clearCache(){return this.request("/api/cache/clear",{method:"POST"})}async getStats(){return this.request("/api/stats")}async getAgentInfo(o){const i=await this.getEntity(o);if(i.type!=="agent")throw new Error(`Entity ${o} is not an agent`);return i}async getWorkflowInfo(o){const i=await this.getEntity(o);if(i.type!=="workflow")throw new Error(`Entity ${o} is not a workflow`);return i}async*streamAgentExecution(o,i){yield*this.streamEntityExecution(o,i)}async*streamWorkflowExecution(o,i){yield*this.streamEntityExecution(o,i)}async runAgent(o,i){return this.runEntity(o,i)}async runWorkflow(o,i){return this.runEntity(o,i)}}const Ui=new RA;function _A({selectedAgent:a,onDebugEvent:o}){const[i,c]=x.useState([]),[u,f]=x.useState(!1),h=x.useCallback(async b=>{const g={role:"user",content:b};c(p=>[...p,g]),f(!0);try{const p={messages:[...i,g]};let S={role:"assistant",content:""};c(w=>[...w,S]);for await(const w of Ui.streamEntityExecution(a.id,p))if(o(w),w.type==="message")w.data?.content&&(S={...S,content:S.content+w.data.content},c(M=>[...M.slice(0,-1),S]));else if(w.type==="complete")break}catch(p){console.error("Failed to send message:",p);const S={role:"assistant",content:`Error: ${p instanceof Error?p.message:"Unknown error"}`};c(w=>[...w.slice(0,-1),S])}finally{f(!1)}},[a.id,i,o]),v=x.useCallback(()=>{c([])},[]);return d.jsxs("div",{className:"flex flex-col h-full",children:[d.jsxs("div",{className:"border-b p-4 space-y-4",children:[d.jsx("div",{className:"flex items-center justify-between",children:d.jsxs("div",{className:"flex items-center gap-3",children:[d.jsx(Al,{className:"h-6 w-6 text-blue-600"}),d.jsxs("div",{children:[d.jsx("h2",{className:"text-lg font-semibold",children:a.name||a.id}),a.description&&d.jsx("p",{className:"text-sm text-muted-foreground",children:a.description})]})]})}),d.jsxs("div",{className:"grid grid-cols-2 md:grid-cols-4 gap-4",children:[a.model&&d.jsx(Mt,{children:d.jsx(Nt,{className:"p-3",children:d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx(oE,{className:"h-4 w-4 text-purple-600"}),d.jsxs("div",{children:[d.jsx("div",{className:"text-xs text-muted-foreground",children:"Model"}),d.jsx("div",{className:"text-sm font-medium",children:a.model})]})]})})}),d.jsx(Mt,{children:d.jsx(Nt,{className:"p-3",children:d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx(qv,{className:"h-4 w-4 text-orange-600"}),d.jsxs("div",{children:[d.jsx("div",{className:"text-xs text-muted-foreground",children:"Tools"}),d.jsxs("div",{className:"text-sm font-medium",children:[a.tools.length," available"]})]})]})})}),a.memory_type&&d.jsx(Mt,{children:d.jsx(Nt,{className:"p-3",children:d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx(zp,{className:"h-4 w-4 text-green-600"}),d.jsxs("div",{children:[d.jsx("div",{className:"text-xs text-muted-foreground",children:"Memory"}),d.jsx("div",{className:"text-sm font-medium",children:a.memory_type})]})]})})}),d.jsx(Mt,{children:d.jsx(Nt,{className:"p-3",children:d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx("div",{className:"h-4 w-4 rounded-full bg-gray-600"}),d.jsxs("div",{children:[d.jsx("div",{className:"text-xs text-muted-foreground",children:"Source"}),d.jsx("div",{className:"text-sm font-medium",children:a.source})]})]})})})]}),a.tools.length>0&&d.jsxs("div",{className:"space-y-2",children:[d.jsxs("div",{className:"flex items-center gap-2 text-sm text-muted-foreground",children:[d.jsx(qv,{className:"h-4 w-4"}),d.jsxs("span",{children:["Available Tools (",a.tools.length,")"]})]}),d.jsx("div",{className:"flex flex-wrap gap-2",children:a.tools.map(b=>d.jsx(Tt,{variant:"outline",children:b},b))})]})]}),d.jsx("div",{className:"flex-1 min-h-0",children:d.jsx(py,{messages:i,onSendMessage:h,onClearMessages:v,isStreaming:u,placeholder:`Chat with ${a.name||a.id}...`,emptyStateTitle:"Agent Chat",emptyStateDescription:`Start a conversation with this agent. It has access to ${a.tools.length} tools and can help you with various tasks.`})})]})}function CA({selectedOrchestrator:a,onDebugEvent:o}){const[i,c]=x.useState([]),[u,f]=x.useState(!1),[h,v]=x.useState(null),b=x.useCallback(async S=>{const w={role:"user",content:S};c(M=>[...M,w]),f(!0),v(null);try{const M={messages:[...i,w]};let T={role:"assistant",content:""};c(N=>[...N,T]);for await(const N of Ui.streamEntityExecution(a.id,M))if(o(N),N.type==="message")N.data?.content&&(T={...T,content:T.content+N.data.content},c(_=>[..._.slice(0,-1),T])),N.data?.name&&v(N.data.name);else if(N.type==="complete"){v(null);break}}catch(M){console.error("Failed to send message:",M);const T={role:"assistant",content:`Error: ${M instanceof Error?M.message:"Unknown error"}`};c(N=>[...N.slice(0,-1),T])}finally{f(!1),v(null)}},[a.id,i,o]),g=x.useCallback(()=>{c([]),v(null)},[]),p=S=>{switch(S.toLowerCase()){case"round_robin":return"bg-blue-100 text-blue-800 dark:bg-blue-900 dark:text-blue-300";case"ai":return"bg-purple-100 text-purple-800 dark:bg-purple-900 dark:text-purple-300";case"plan":return"bg-green-100 text-green-800 dark:bg-green-900 dark:text-green-300";default:return"bg-gray-100 text-gray-800 dark:bg-gray-900 dark:text-gray-300"}};return d.jsxs("div",{className:"flex flex-col h-full",children:[d.jsxs("div",{className:"border-b p-4 space-y-4",children:[d.jsxs("div",{className:"flex items-center justify-between",children:[d.jsxs("div",{className:"flex items-center gap-3",children:[d.jsx(Ju,{className:"h-6 w-6 text-blue-600"}),d.jsxs("div",{children:[d.jsx("h2",{className:"text-lg font-semibold",children:a.name||a.id}),a.description&&d.jsx("p",{className:"text-sm text-muted-foreground",children:a.description})]})]}),d.jsx(Tt,{className:p(a.orchestrator_type),children:a.orchestrator_type})]}),d.jsxs("div",{className:"space-y-2",children:[d.jsxs("div",{className:"flex items-center gap-2 text-sm text-muted-foreground",children:[d.jsx(Al,{className:"h-4 w-4"}),d.jsxs("span",{children:["Participating Agents (",a.agents.length,")"]})]}),d.jsx("div",{className:"flex flex-wrap gap-2",children:a.agents.map(S=>d.jsxs(Tt,{variant:h===S?"default":"outline",className:h===S?"animate-pulse":"",children:[S,h===S&&d.jsx(Up,{className:"h-3 w-3 ml-1"})]},S))})]}),a.termination_conditions.length>0&&d.jsxs("div",{className:"space-y-2",children:[d.jsxs("div",{className:"flex items-center gap-2 text-sm text-muted-foreground",children:[d.jsx(vE,{className:"h-4 w-4"}),d.jsx("span",{children:"Termination Conditions"})]}),d.jsx("div",{className:"flex flex-wrap gap-2",children:a.termination_conditions.map(S=>d.jsx(Tt,{variant:"secondary",children:S},S))})]}),h&&d.jsx(Mt,{className:"bg-blue-50 dark:bg-blue-950 border-blue-200 dark:border-blue-800",children:d.jsx(Nt,{className:"p-3",children:d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx("div",{className:"h-2 w-2 bg-blue-600 rounded-full animate-pulse"}),d.jsxs("span",{className:"text-sm font-medium",children:[h," is responding..."]})]})})})]}),d.jsx("div",{className:"flex-1 min-h-0",children:d.jsx(py,{messages:i,onSendMessage:b,onClearMessages:g,isStreaming:u,placeholder:`Start a conversation with ${a.agents.length} agents via ${a.orchestrator_type} orchestration...`,emptyStateTitle:"Multi-Agent Orchestration",emptyStateDescription:`This orchestrator will coordinate conversations between ${a.agents.join(", ")} using ${a.orchestrator_type} pattern.`})})]})}var jA="Label",yy=x.forwardRef((a,o)=>d.jsx(ke.label,{...a,ref:o,onMouseDown:i=>{i.target.closest("button, input, select, textarea")||(a.onMouseDown?.(i),!i.defaultPrevented&&i.detail>1&&i.preventDefault())}}));yy.displayName=jA;var OA=yy;function DA({className:a,...o}){return d.jsx(OA,{"data-slot":"label",className:Ze("flex items-center gap-2 text-sm leading-none font-medium select-none group-data-[disabled=true]:pointer-events-none group-data-[disabled=true]:opacity-50 peer-disabled:cursor-not-allowed peer-disabled:opacity-50",a),...o})}function zA({selectedWorkflow:a,onDebugEvent:o}){const[i,c]=x.useState({}),[u,f]=x.useState(!1),[h,v]=x.useState({status:"pending",steps_completed:[]}),[b,g]=x.useState(null),[p,S]=x.useState(null),w=(j,q)=>{c(L=>({...L,[j]:q}))},M=x.useCallback(async()=>{f(!0),S(null),g(null),v({status:"running",steps_completed:[]});try{const j={input_data:i};for await(const q of Ui.streamEntityExecution(a.id,j))o(q),q.type==="workflow_started"?v(L=>({...L,status:"running"})):q.type==="executor_invoke"?v(L=>({...L,current_step:q.data?.executor_id})):q.type==="executor_result"?v(L=>({...L,steps_completed:[...L.steps_completed,q.data?.executor_id||"unknown"]})):q.type==="workflow_completed"?(v(L=>({...L,status:"completed"})),g(q.data)):q.type==="workflow_error"&&(v(L=>({...L,status:"failed",error:q.data?.error||"Unknown error"})),S(q.data?.error||"Workflow execution failed"))}catch(j){console.error("Workflow execution error:",j),S(j instanceof Error?j.message:"Execution failed"),v(q=>({...q,status:"failed",error:j instanceof Error?j.message:"Unknown error"}))}finally{f(!1)}},[a.id,i,o]),T=j=>h.steps_completed.includes(j)?"completed":h.current_step===j?"running":"pending",N=j=>{switch(j){case"completed":return d.jsx(Tu,{className:"h-4 w-4 text-green-600"});case"running":return d.jsx(Ru,{className:"h-4 w-4 text-blue-600 animate-spin"});case"failed":return d.jsx(Nu,{className:"h-4 w-4 text-red-600"});default:return d.jsx("div",{className:"h-4 w-4 rounded-full border-2 border-gray-300"})}},_=()=>{if(!a.input_schema)return d.jsx(Mt,{children:d.jsx(Nt,{className:"p-4",children:d.jsx("p",{className:"text-sm text-muted-foreground",children:"No input schema defined for this workflow."})})});const j=a.input_schema,q=j.properties||{};return d.jsxs(Mt,{children:[d.jsx(Tr,{children:d.jsx(Rr,{className:"text-sm",children:"Workflow Input"})}),d.jsx(Nt,{className:"space-y-4",children:Object.entries(q).map(([L,Y])=>{const B=Y;return d.jsxs("div",{className:"space-y-2",children:[d.jsxs(DA,{htmlFor:L,className:"text-sm font-medium",children:[L,j.required?.includes(L)&&d.jsx("span",{className:"text-red-500 ml-1",children:"*"})]}),d.jsx(gy,{id:L,type:B.type==="number"?"number":"text",placeholder:B.description||`Enter ${L}`,value:i[L]||"",onChange:J=>w(L,J.target.value),disabled:u}),B.description&&d.jsx("p",{className:"text-xs text-muted-foreground",children:B.description})]},L)})})]})};return d.jsxs("div",{className:"flex flex-col h-full",children:[d.jsxs("div",{className:"border-b p-4 space-y-4",children:[d.jsxs("div",{className:"flex items-center justify-between",children:[d.jsxs("div",{className:"flex items-center gap-3",children:[d.jsx(yi,{className:"h-6 w-6 text-green-600"}),d.jsxs("div",{children:[d.jsx("h2",{className:"text-lg font-semibold",children:a.name||a.id}),a.description&&d.jsx("p",{className:"text-sm text-muted-foreground",children:a.description})]})]}),d.jsxs(Tt,{variant:"outline",className:"gap-1",children:[d.jsx(bE,{className:"h-3 w-3"}),a.steps.length," steps"]})]}),d.jsxs("div",{className:"space-y-2",children:[d.jsxs("div",{className:"flex items-center gap-2 text-sm text-muted-foreground",children:[d.jsx(yi,{className:"h-4 w-4"}),d.jsx("span",{children:"Workflow Steps"})]}),d.jsx("div",{className:"flex items-center gap-2 flex-wrap",children:a.steps.map((j,q)=>d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsxs(Tt,{variant:T(j)==="completed"?"default":T(j)==="running"?"secondary":"outline",className:"gap-1",children:[N(T(j)),j]}),q<a.steps.length-1&&d.jsx(lE,{className:"h-3 w-3 text-muted-foreground"})]},j))})]}),a.start_step&&d.jsxs("div",{className:"text-sm",children:[d.jsx("span",{className:"text-muted-foreground",children:"Entry point: "}),d.jsx(Tt,{variant:"outline",children:a.start_step})]})]}),d.jsxs("div",{className:"flex-1 p-4 space-y-4 overflow-auto",children:[_(),d.jsx(Mt,{children:d.jsx(Nt,{className:"p-4",children:d.jsxs("div",{className:"flex items-center justify-between",children:[d.jsxs("div",{children:[d.jsx("h3",{className:"font-medium",children:"Execute Workflow"}),d.jsx("p",{className:"text-sm text-muted-foreground",children:"Run the workflow with the provided input"})]}),d.jsxs(An,{onClick:M,disabled:u,className:"gap-2",children:[u?d.jsx(Ru,{className:"h-4 w-4 animate-spin"}):d.jsx(CE,{className:"h-4 w-4"}),u?"Executing...":"Execute"]})]})})}),h.status!=="pending"&&d.jsxs(Mt,{children:[d.jsx(Tr,{children:d.jsxs(Rr,{className:"text-sm flex items-center gap-2",children:[N(h.status),"Execution Status"]})}),d.jsxs(Nt,{className:"space-y-2",children:[d.jsxs("div",{className:"flex items-center gap-2",children:[d.jsx("span",{className:"text-sm font-medium",children:"Status:"}),d.jsx(Tt,{variant:h.status==="completed"?"default":h.status==="running"?"secondary":"destructive",children:h.status})]}),d.jsxs("div",{className:"text-sm",children:[d.jsx("span",{className:"font-medium",children:"Steps completed:"})," ",h.steps_completed.length," of ",a.steps.length]}),h.current_step&&d.jsxs("div",{className:"text-sm",children:[d.jsx("span",{className:"font-medium",children:"Current step:"})," ",d.jsx(Tt,{variant:"outline",children:h.current_step})]})]})]}),b&&d.jsxs(Mt,{children:[d.jsx(Tr,{children:d.jsxs(Rr,{className:"text-sm flex items-center gap-2",children:[d.jsx(Tu,{className:"h-4 w-4 text-green-600"}),"Execution Result"]})}),d.jsx(Nt,{children:d.jsx(Sa,{className:"h-32",children:d.jsx("pre",{className:"text-xs bg-muted p-2 rounded overflow-auto",children:JSON.stringify(b,null,2)})})})]}),p&&d.jsxs(Mt,{className:"border-destructive",children:[d.jsx(Tr,{children:d.jsxs(Rr,{className:"text-sm flex items-center gap-2 text-destructive",children:[d.jsx(Nu,{className:"h-4 w-4"}),"Execution Error"]})}),d.jsx(Nt,{children:d.jsx("p",{className:"text-sm text-destructive",children:p})})]})]})]})}function UA({message:a="Loading...",description:o,size:i="md",className:c,fullPage:u=!1}){const f=d.jsxs("div",{className:Ze("flex flex-col items-center justify-center gap-3",u?"min-h-[50vh]":"py-8",c),children:[d.jsx(Wu,{size:i,className:"text-muted-foreground"}),d.jsxs("div",{className:"text-center space-y-1",children:[d.jsx("p",{className:Ze("font-medium text-muted-foreground",i==="sm"&&"text-sm",i==="lg"&&"text-lg"),children:a}),o&&d.jsx("p",{className:"text-sm text-muted-foreground/80",children:o})]})]});return u?d.jsx("div",{className:"flex items-center justify-center min-h-screen bg-background",children:f}):f}function kA(){const[a,o]=x.useState({entities:[],agents:[],orchestrators:[],workflows:[],isLoading:!0}),[i,c]=x.useState([]),[u,f]=x.useState(!0),[h,v]=x.useState(()=>{const N=localStorage.getItem("debugPanelWidth");return N?parseInt(N,10):320}),[b,g]=x.useState(!1);x.useEffect(()=>{(async()=>{try{const _=await Ui.getEntities(),j=_.filter(Y=>Y.type==="agent"),q=_.filter(Y=>Y.type==="orchestrator"),L=_.filter(Y=>Y.type==="workflow");o(Y=>({...Y,entities:_,agents:j,orchestrators:q,workflows:L,selectedEntity:_.length>0?_[0]:void 0,isLoading:!1}))}catch(_){console.error("Failed to load entities:",_),o(j=>({...j,error:_ instanceof Error?_.message:"Failed to load entities",isLoading:!1}))}})()},[]),x.useEffect(()=>{localStorage.setItem("debugPanelWidth",h.toString())},[h]);const p=x.useCallback(N=>{N.preventDefault(),g(!0);const _=N.clientX,j=h,q=Y=>{const B=_-Y.clientX,J=Math.max(200,Math.min(window.innerWidth*.5,j+B));v(J)},L=()=>{g(!1),document.removeEventListener("mousemove",q),document.removeEventListener("mouseup",L)};document.addEventListener("mousemove",q),document.addEventListener("mouseup",L)},[h]),S=x.useCallback(()=>{f(!1)},[]),w=x.useCallback(N=>{o(_=>({..._,selectedEntity:N,currentSession:void 0})),c([])},[]),M=x.useCallback(N=>{c(_=>[..._,N])},[]);if(a.isLoading)return d.jsxs("div",{className:"h-screen flex flex-col bg-background",children:[d.jsxs("header",{className:"flex h-14 items-center gap-4 border-b px-4",children:[d.jsx("div",{className:"w-64 h-9 bg-muted animate-pulse rounded-md"}),d.jsxs("div",{className:"flex items-center gap-2 ml-auto",children:[d.jsx("div",{className:"w-8 h-8 bg-muted animate-pulse rounded-md"}),d.jsx("div",{className:"w-8 h-8 bg-muted animate-pulse rounded-md"})]})]}),d.jsx(UA,{message:"Initializing PicoAgents WebUI...",description:"Discovering agents, orchestrators, and workflows",fullPage:!0})]});if(a.error)return d.jsxs("div",{className:"h-screen flex flex-col bg-background",children:[d.jsx(hu,{entities:[],selectedEntity:void 0,onSelect:()=>{},isLoading:!1}),d.jsx("div",{className:"flex-1 flex items-center justify-center",children:d.jsxs("div",{className:"text-center space-y-4 max-w-md",children:[d.jsx("div",{className:"text-destructive text-lg font-medium",children:"Failed to load entities"}),d.jsx("p",{className:"text-muted-foreground text-sm",children:a.error}),d.jsx(An,{onClick:()=>window.location.reload(),variant:"outline",children:"Retry"})]})})]});if(!a.isLoading&&a.entities.length===0)return d.jsxs("div",{className:"h-screen flex flex-col bg-background",children:[d.jsx(hu,{entities:[],selectedEntity:void 0,onSelect:()=>{},isLoading:!1}),d.jsx("div",{className:"flex-1 flex items-center justify-center",children:d.jsxs("div",{className:"text-center space-y-4 max-w-md",children:[d.jsx("div",{className:"text-lg font-medium",children:"No entities found"}),d.jsx("p",{className:"text-muted-foreground text-sm",children:"No agents, orchestrators, or workflows were discovered. Please check your configuration and ensure entities are properly set up."}),d.jsx(An,{onClick:()=>window.location.reload(),variant:"outline",children:"Retry"})]})})]});const T=()=>{if(!a.selectedEntity)return d.jsx("div",{className:"flex-1 flex items-center justify-center text-muted-foreground",children:"Select an entity to get started."});switch(a.selectedEntity.type){case"agent":return d.jsx(_A,{selectedAgent:a.selectedEntity,onDebugEvent:M});case"orchestrator":return d.jsx(CA,{selectedOrchestrator:a.selectedEntity,onDebugEvent:M});case"workflow":return d.jsx(zA,{selectedWorkflow:a.selectedEntity,onDebugEvent:M});default:return d.jsxs("div",{className:"flex-1 flex items-center justify-center text-muted-foreground",children:["Unknown entity type: ",a.selectedEntity.type]})}};return d.jsxs("div",{className:"h-screen flex flex-col bg-background max-h-screen",children:[d.jsx(hu,{entities:a.entities,selectedEntity:a.selectedEntity,onSelect:w,isLoading:a.isLoading}),d.jsxs("div",{className:"flex flex-1 overflow-hidden",children:[d.jsx("div",{className:"flex-1 min-w-0",children:T()}),u&&d.jsx("div",{className:`w-1 bg-border hover:bg-accent cursor-col-resize flex-shrink-0 relative group ${b?"bg-accent":""}`,onMouseDown:p,onDoubleClick:S,children:d.jsx("div",{className:"absolute inset-y-0 -left-1 -right-1 flex items-center justify-center",children:d.jsx("div",{className:"h-12 rounded-lg bg-primary w-2"})})}),!u&&d.jsx("div",{className:"flex-shrink-0",children:d.jsx(An,{variant:"ghost",size:"sm",onClick:()=>f(!0),className:"h-full w-8 rounded-none border-l",children:d.jsx(fE,{className:"h-4 w-4"})})}),u&&d.jsx("div",{className:"flex-shrink-0",style:{width:`${h}px`},children:d.jsx(AA,{events:i,isStreaming:!1})})]})]})}function LA({children:a,attribute:o="class",defaultTheme:i="system",enableSystem:c=!0,disableTransitionOnChange:u=!0,...f}){return d.jsx(JE,{attribute:o,defaultTheme:i,enableSystem:c,disableTransitionOnChange:u,...f,children:a})}Jb.createRoot(document.getElementById("root")).render(d.jsx(x.StrictMode,{children:d.jsx(LA,{attribute:"class",defaultTheme:"system",enableSystem:!0,disableTransitionOnChange:!0,children:d.jsx(kA,{})})}));
